/*
 * Copyright (C) 2012 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this software; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

#include "memtools_register_info.h"

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark AIPSTZ
#endif

// Bitfields in register AIPSTZ_MPR.
static const field_t hw_aipstz_mpr[] =
{
    {
        "MPROT3",
        "Master 3 Priviledge, Buffer, Read, Write Control.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MPROT1",
        "Master 1 Priviledge, Buffer, Read, Write Control",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MPROT0",
        "Master 0 Priviledge, Buffer, Read, Write Control",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AIPSTZ_OPACR.
static const field_t hw_aipstz_opacr[] =
{
    {
        "OPAC7",
        "Off-platform Peripheral Access Control 7",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC6",
        "Off-platform Peripheral Access Control 6",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC5",
        "Off-platform Peripheral Access Control 5",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC4",
        "Off-platform Peripheral Access Control 4",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC3",
        "Off-platform Peripheral Access Control 3",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC2",
        "Off-platform Peripheral Access Control 2",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC1",
        "Off-platform Peripheral Access Control 1",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC0",
        "Off-platform Peripheral Access Control 0",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AIPSTZ_OPACR1.
static const field_t hw_aipstz_opacr1[] =
{
    {
        "OPAC15",
        "Off-platform Peripheral Access Control 15",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC14",
        "Off-platform Peripheral Access Control 14",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC13",
        "Off-platform Peripheral Access Control 13",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC12",
        "Off-platform Peripheral Access Control 12",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC11",
        "Off-platform Peripheral Access Control 11",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC10",
        "Off-platform Peripheral Access Control 10",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC9",
        "Off-platform Peripheral Access Control 9",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC8",
        "Off-platform Peripheral Access Control 8",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AIPSTZ_OPACR2.
static const field_t hw_aipstz_opacr2[] =
{
    {
        "OPAC23",
        "Off-platform Peripheral Access Control 23",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC22",
        "Off-platform Peripheral Access Control 22",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC21",
        "Off-platform Peripheral Access Control 21",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC20",
        "Off-platform Peripheral Access Control 20",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC19",
        "Off-platform Peripheral Access Control 19",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC18",
        "Off-platform Peripheral Access Control 18",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC17",
        "Off-platform Peripheral Access Control 17",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC16",
        "Off-platform Peripheral Access Control 16",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AIPSTZ_OPACR3.
static const field_t hw_aipstz_opacr3[] =
{
    {
        "OPAC31",
        "Off-platform Peripheral Access Control 31",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC30",
        "Off-platform Peripheral Access Control 30",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC29",
        "Off-platform Peripheral Access Control 29",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC28",
        "Off-platform Peripheral Access Control 28",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC27",
        "Off-platform Peripheral Access Control 27",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC26",
        "Off-platform Peripheral Access Control 26",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC25",
        "Off-platform Peripheral Access Control 25",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC24",
        "Off-platform Peripheral Access Control 24",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AIPSTZ_OPACR4.
static const field_t hw_aipstz_opacr4[] =
{
    {
        "OPAC33",
        "Off-platform Peripheral Access Control 33",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPAC32",
        "Off-platform Peripheral Access Control 32",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a AIPSTZ module.
static const reg_t hw_aipstz[] =
{
    {
        "MPR",
        "Each AIPSTZ_MPR specifies 16 4-bit fields defining the access privileg"
        "e level associated with a bus master in the platform, as well as speci"
        "fying whether write accesses from this master are bufferable shown in "
        " The registers provide one field per bus master, where field 15 corres"
        "ponds to master 15, field 14 to master 14,...",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_aipstz_mpr
    },
    {
        "OPACR",
        "Each of the off-platform peripherals have an Off-platform Peripheral A"
        "ccess Control Register (AIPSTZ_OPACR) which defines the access levels "
        "supported by the given block.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_aipstz_opacr
    },
    {
        "OPACR1",
        "Each of the off-platform peripherals have an Off-platform Peripheral A"
        "ccess Control Register (AIPSTZ_OPACR) which defines the access levels "
        "supported by the given block.",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_aipstz_opacr1
    },
    {
        "OPACR2",
        "Each of the off-platform peripherals have an Off-platform Peripheral A"
        "ccess Control Register (AIPSTZ_OPACR) which defines the access levels "
        "supported by the given block.",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_aipstz_opacr2
    },
    {
        "OPACR3",
        "Each of the off-platform peripherals have an Off-platform Peripheral A"
        "ccess Control Register (AIPSTZ_OPACR) which defines the access levels "
        "supported by the given block.",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_aipstz_opacr3
    },
    {
        "OPACR4",
        "Each of the off-platform peripherals have an Off-platform Peripheral A"
        "ccess Control Register (AIPSTZ_OPACR) which defines the access levels "
        "supported by the given block.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_aipstz_opacr4
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark ARMGLOBALTIMER
#endif

// Bitfields in register ARMGLOBALTIMER_COUNTER0.
static const field_t hw_armglobaltimer_counter0[] =
{
    {
        "VALUE",
        "32-bits of the counter value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ARMGLOBALTIMER_COUNTER1.
static const field_t hw_armglobaltimer_counter1[] =
{
    {
        "VALUE",
        "32-bits of the counter value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ARMGLOBALTIMER_CONTROL.
static const field_t hw_armglobaltimer_control[] =
{
    {
        "TIMER_ENABLE",
        "Timer enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COMP_ENABLE",
        "This bit is banked per Cortex-A9 processor.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRQ_ENABLE",
        "This bit is banked per Cortex-A9 processor.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUTO_INCREMENT",
        "This bit is banked per Cortex-A9 processor.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRESCALER",
        "The prescaler modifies the clock period for the decrementing event for"
        " the Counter Register.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ARMGLOBALTIMER_IRQSTATUS.
static const field_t hw_armglobaltimer_irqstatus[] =
{
    {
        "EVENT_FLAG",
        "The event flag is a sticky bit that is automatically set when the Coun"
        "ter Register reaches the Comparator Register value.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ARMGLOBALTIMER_COMPARATOR0.
static const field_t hw_armglobaltimer_comparator0[] =
{
    {
        "VALUE",
        "32-bits of the comparator value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ARMGLOBALTIMER_COMPARATOR1.
static const field_t hw_armglobaltimer_comparator1[] =
{
    {
        "VALUE",
        "32-bits of the comparator value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ARMGLOBALTIMER_AUTOINCREMENT.
static const field_t hw_armglobaltimer_autoincrement[] =
{
    {
        "VALUE",
        "32-bit auto-increment value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a ARMGLOBALTIMER module.
static const reg_t hw_armglobaltimer[] =
{
    {
        "COUNTER0",
        "There are two timer counter registers.",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_armglobaltimer_counter0
    },
    {
        "COUNTER1",
        "There are two timer counter registers.",
        4, // Width in bytes
        0x00000204, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_armglobaltimer_counter1
    },
    {
        "CONTROL",
        "Configuration and control of the Global Timer.",
        4, // Width in bytes
        0x00000208, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_armglobaltimer_control
    },
    {
        "IRQSTATUS",
        "This is a banked register for all Cortex-A9 processors present.",
        4, // Width in bytes
        0x0000020c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_armglobaltimer_irqstatus
    },
    {
        "COMPARATOR0",
        "There are two timer counter registers.",
        4, // Width in bytes
        0x00000210, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_armglobaltimer_comparator0
    },
    {
        "COMPARATOR1",
        "There are two timer counter registers.",
        4, // Width in bytes
        0x00000214, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_armglobaltimer_comparator1
    },
    {
        "AUTOINCREMENT",
        "This 32-bit register gives the increment value of the Comparator Regis"
        "ter when the Auto-increment bit is set in the Timer Control Register.",
        4, // Width in bytes
        0x00000218, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_armglobaltimer_autoincrement
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark AUDMUX
#endif

// Bitfields in register AUDMUX_PTCR1.
static const field_t hw_audmux_ptcr1[] =
{
    {
        "SYN",
        "Synchronous/Asynchronous Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSEL",
        "Receive Clock Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCLKDIR",
        "Receive Clock Direction Control.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSEL",
        "Receive Frame Sync Select.",
        17, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFS_DIR",
        "Receive Frame Sync Direction Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEL",
        "Transmit Clock Select.",
        22, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCLKDIR",
        "Transmit Clock Direction Control.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSEL",
        "Transmit Frame Sync Select.",
        27, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFS_DIR",
        "Transmit Frame Sync Direction Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PDCR1.
static const field_t hw_audmux_pdcr1[] =
{
    {
        "INMMASK",
        "Internal Network Mode Mask.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE",
        "Mode Select.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRXEN",
        "Transmit/Receive Switch Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEL",
        "Receive Data Select.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PTCR2.
static const field_t hw_audmux_ptcr2[] =
{
    {
        "SYN",
        "Synchronous/Asynchronous Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSEL",
        "Receive Clock Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCLKDIR",
        "Receive Clock Direction Control.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSEL",
        "Receive Frame Sync Select.",
        17, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFS_DIR",
        "Receive Frame Sync Direction Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEL",
        "Transmit Clock Select.",
        22, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCLKDIR",
        "Transmit Clock Direction Control.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSEL",
        "Transmit Frame Sync Select.",
        27, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFS_DIR",
        "Transmit Frame Sync Direction Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PDCR2.
static const field_t hw_audmux_pdcr2[] =
{
    {
        "INMMASK",
        "Internal Network Mode Mask.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE",
        "Mode Select.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRXEN",
        "Transmit/Receive Switch Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEL",
        "Receive Data Select.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PTCR3.
static const field_t hw_audmux_ptcr3[] =
{
    {
        "SYN",
        "Synchronous/Asynchronous Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSEL",
        "Receive Clock Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCLKDIR",
        "Receive Clock Direction Control.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSEL",
        "Receive Frame Sync Select.",
        17, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFS_DIR",
        "Receive Frame Sync Direction Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEL",
        "Transmit Clock Select.",
        22, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCLKDIR",
        "Transmit Clock Direction Control.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSEL",
        "Transmit Frame Sync Select.",
        27, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFS_DIR",
        "Transmit Frame Sync Direction Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PDCR3.
static const field_t hw_audmux_pdcr3[] =
{
    {
        "INMMASK",
        "Internal Network Mode Mask.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE",
        "Mode Select.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRXEN",
        "Transmit/Receive Switch Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEL",
        "Receive Data Select.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PTCR4.
static const field_t hw_audmux_ptcr4[] =
{
    {
        "SYN",
        "Synchronous/Asynchronous Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSEL",
        "Receive Clock Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCLKDIR",
        "Receive Clock Direction Control.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSEL",
        "Receive Frame Sync Select.",
        17, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFS_DIR",
        "Receive Frame Sync Direction Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEL",
        "Transmit Clock Select.",
        22, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCLKDIR",
        "Transmit Clock Direction Control.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSEL",
        "Transmit Frame Sync Select.",
        27, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFS_DIR",
        "Transmit Frame Sync Direction Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PDCR4.
static const field_t hw_audmux_pdcr4[] =
{
    {
        "INMMASK",
        "Internal Network Mode Mask.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE",
        "Mode Select.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRXEN",
        "Transmit/Receive Switch Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEL",
        "Receive Data Select.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PTCR5.
static const field_t hw_audmux_ptcr5[] =
{
    {
        "SYN",
        "Synchronous/Asynchronous Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSEL",
        "Receive Clock Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCLKDIR",
        "Receive Clock Direction Control.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSEL",
        "Receive Frame Sync Select.",
        17, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFS_DIR",
        "Receive Frame Sync Direction Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEL",
        "Transmit Clock Select.",
        22, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCLKDIR",
        "Transmit Clock Direction Control.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSEL",
        "Transmit Frame Sync Select.",
        27, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFS_DIR",
        "Transmit Frame Sync Direction Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PDCR5.
static const field_t hw_audmux_pdcr5[] =
{
    {
        "INMMASK",
        "Internal Network Mode Mask.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE",
        "Mode Select.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRXEN",
        "Transmit/Receive Switch Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEL",
        "Receive Data Select.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PTCR6.
static const field_t hw_audmux_ptcr6[] =
{
    {
        "SYN",
        "Synchronous/Asynchronous Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSEL",
        "Receive Clock Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCLKDIR",
        "Receive Clock Direction Control.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSEL",
        "Receive Frame Sync Select.",
        17, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFS_DIR",
        "Receive Frame Sync Direction Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEL",
        "Transmit Clock Select.",
        22, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCLKDIR",
        "Transmit Clock Direction Control.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSEL",
        "Transmit Frame Sync Select.",
        27, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFS_DIR",
        "Transmit Frame Sync Direction Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PDCR6.
static const field_t hw_audmux_pdcr6[] =
{
    {
        "INMMASK",
        "Internal Network Mode Mask.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE",
        "Mode Select.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRXEN",
        "Transmit/Receive Switch Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEL",
        "Receive Data Select.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PTCR7.
static const field_t hw_audmux_ptcr7[] =
{
    {
        "SYN",
        "Synchronous/Asynchronous Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSEL",
        "Receive Clock Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCLKDIR",
        "Receive Clock Direction Control.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSEL",
        "Receive Frame Sync Select.",
        17, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFS_DIR",
        "Receive Frame Sync Direction Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEL",
        "Transmit Clock Select.",
        22, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCLKDIR",
        "Transmit Clock Direction Control.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSEL",
        "Transmit Frame Sync Select.",
        27, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFS_DIR",
        "Transmit Frame Sync Direction Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register AUDMUX_PDCR7.
static const field_t hw_audmux_pdcr7[] =
{
    {
        "INMMASK",
        "Internal Network Mode Mask.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE",
        "Mode Select.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRXEN",
        "Transmit/Receive Switch Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEL",
        "Receive Data Select.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a AUDMUX module.
static const reg_t hw_audmux[] =
{
    {
        "PTCR1",
        "PTCR1 is the Port Timing Control Register for Port 1.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_audmux_ptcr1
    },
    {
        "PDCR1",
        "PDCR1 is the Port Data Control Register for Port 1.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_audmux_pdcr1
    },
    {
        "PTCR2",
        "PTCR2 is the Port Timing Control Register for Port 2.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_audmux_ptcr2
    },
    {
        "PDCR2",
        "PDCR2 is the Port Data Control Register for Port 2.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_audmux_pdcr2
    },
    {
        "PTCR3",
        "PTCR3 is the Port Timing Control Register for Port 3.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_audmux_ptcr3
    },
    {
        "PDCR3",
        "PDCR3 is the Port Data Control Register for Port 3.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_audmux_pdcr3
    },
    {
        "PTCR4",
        "Port Timing Control Register for Port 4",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_audmux_ptcr4
    },
    {
        "PDCR4",
        "PDCR4 is the Port Data Control Register for Port 4.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_audmux_pdcr4
    },
    {
        "PTCR5",
        "Port Timing Control Register for Port 5",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_audmux_ptcr5
    },
    {
        "PDCR5",
        "PDCR5 is the Port Data Control Register for Port 5.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_audmux_pdcr5
    },
    {
        "PTCR6",
        "Port Timing Control Register for Port 6",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_audmux_ptcr6
    },
    {
        "PDCR6",
        "PDCR6 is the Port Data Control Register for Port 6.",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_audmux_pdcr6
    },
    {
        "PTCR7",
        "Port Timing Control Register for Port 7",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_audmux_ptcr7
    },
    {
        "PDCR7",
        "PDCR7 is the Port Data Control Register for Port 7 .",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_audmux_pdcr7
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark CCM
#endif

// Bitfields in register CCM_CCR.
static const field_t hw_ccm_ccr[] =
{
    {
        "OSCNT",
        "Oscillator ready counter value.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COSC_EN",
        "On chip oscillator enable bit - this bit value is reflected on the out"
        "put cosc_en.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WB_COUNT",
        "Well Bias counter.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG_BYPASS_COUNT",
        "Counter for analog_reg_bypass signal assertion after standby voltage r"
        "equest by pmic_vstby_req.",
        21, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBC_EN",
        "Enable for REG_BYPASS_COUNTER.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCDR.
static const field_t hw_ccm_ccdr[] =
{
    {
        "MMDC_ROOT_MASK",
        "During divider ratio mmdc_root_axi_podf change or sync mux periph2_clk"
        "_sel change (but not jtag) or SRC request during warm reset, mask hand"
        "shake with mmdc_root module.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MMDC_CH0_MASK",
        "During divider ratio mmdc_ch0_axi_podf change or sync mux periph_clk_s"
        "el change (but not jtag) or SRC request during warm reset, mask handsh"
        "ake with mmdc_ch0 module.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CSR.
static const field_t hw_ccm_csr[] =
{
    {
        "REF_EN_B",
        "Status of the value of ref_en_b output of ccm",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "COSC_READY",
        "Status indication of on board oscillator.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCSR.
static const field_t hw_ccm_ccsr[] =
{
    {
        "PLL3_SW_CLK_SEL",
        "Selects source to generate pll3_sw_clk.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PLL2_SW_CLK_SEL",
        "Selects source to generate pll2_sw_clk.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PLL1_SW_CLK_SEL",
        "Selects source to generate pll1_sw_clk.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STEP_SEL",
        "Selects the option to be chosen for the step frequency when shifting A"
        "RM frequency.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDF_396M_DIS_MASK",
        "Mask of 396M PFD auto-disable.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDF_352M_DIS_MASK",
        "Mask of 352M PFD auto-disable.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDF_594M_DIS_MASK",
        "Mask of 594M PFD auto-disable.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDF_508M_DIS_MASK",
        "Mask of 508M PFD auto-disable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDF_454M_DIS_MASK",
        "Mask of 454M PFD auto-disable.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDF_720M_DIS_MASK",
        "Mask of 720M PFD auto-disable.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDF_540M_DIS_MASK",
        "Mask of 540M PFD auto-disable.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CACRR.
static const field_t hw_ccm_cacrr[] =
{
    {
        "ARM_PODF",
        "Divider for ARM clock root.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CBCDR.
static const field_t hw_ccm_cbcdr[] =
{
    {
        "PERIPH2_CLK2_PODF",
        "Divider for periph2_clk2 podf.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FABRIC_MMDC_PODF",
        "Post divider for fabric_mmdc clock.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_CLK_SEL",
        "OCRAM clock source select",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_ALT_CLK_SEL",
        "OCRAM alternative clock select",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IPG_PODF",
        "Divider for ipg podf.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AHB_PODF",
        "Divider for ahb podf.",
        10, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_PODF",
        "Post divider for ocram clock.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERIPH_CLK_SEL",
        "Selector for peripheral main clock (source of mmdc_ch0_axi_clk_root).",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERIPH2_CLK_SEL",
        "Selector for peripheral2 main clock (source of mmdc_root_axi_clk_root "
        ").",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERIPH_CLK2_PODF",
        "Divider for periph2 clock podf.",
        27, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CBCMR.
static const field_t hw_ccm_cbcmr[] =
{
    {
        "GPU2D_OVG_CORE_CLK_SEL",
        "Selector for gpu2d_ovg_core clock multiplexer",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU2D_CORE_CLK_SEL",
        "Selector for gpu2d_core clock multiplexer",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VDOAXI_CLK_SEL",
        "Selector for vdoaxi clock multiplexer",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERIPH_CLK2_SEL",
        "Selector for peripheral clk2 clock multiplexer",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRE_PERIPH_CLK_SEL",
        "Selector for pre_periph clock multiplexer",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERIPH2_CLK2_SEL",
        "Selector for periph2_clk2 clock multiplexer",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRE_PERIPH2_CLK_SEL",
        "Selector for pre_periph2 clock multiplexer",
        21, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_PIX_PODF",
        "Post divider for EPDC_PIX.",
        23, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU2D_OVG_CORE_PODF",
        "Divider for gpu3d_core clock.",
        26, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU2D_CORE_PODF",
        "Post divider for gpu2d_core clock.",
        29, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CSCMR1.
static const field_t hw_ccm_cscmr1[] =
{
    {
        "PERCLK_PODF",
        "Divider for perclk podf.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERCLK_CLK_SEL",
        "Selector for the perclk clock multiplexor",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSI1_CLK_SEL",
        "Selector for ssi1 clock multiplexer",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSI2_CLK_SEL",
        "Selector for ssi2 clock multiplexer",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSI3_CLK_SEL",
        "Selector for ssi3 clock multiplexer",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC1_CLK_SEL",
        "Selector for usdhc1 clock multiplexer",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC2_CLK_SEL",
        "Selector for usdhc2 clock multiplexer",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC3_CLK_SEL",
        "Selector for usdhc3 clock multiplexer",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC4_CLK_SEL",
        "Selector for usdhc4 clock multiplexer",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_PIX_PODF",
        "Post divider for LCDIF_PIX.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACLK_EMI_SLOW_PODF",
        "Divider for aclk_emi_slow clock root.",
        23, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACLK_EMI_SLOW_SEL",
        "Selector for aclk_emi_slow root clock multiplexer",
        29, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CSCMR2.
static const field_t hw_ccm_cscmr2[] =
{
    {
        "EXTERN_AUDIO_CLK_SEL",
        "Selector for external audio clock multiplexer",
        19, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CSCDR1.
static const field_t hw_ccm_cscdr1[] =
{
    {
        "UART_CLK_PODF",
        "Divider for uart clock podf.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UART_CLK_SEL",
        "Selector for the UART clock multiplexor",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC1_PODF",
        "Divider for usdhc1 clock podf.",
        11, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC2_PODF",
        "Divider for usdhc2 clock.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC3_PODF",
        "Divider for usdhc3 clock podf.",
        19, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHC4_PODF",
        "Divider for esdhc4 clock pred.",
        22, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CS1CDR.
static const field_t hw_ccm_cs1cdr[] =
{
    {
        "SSI1_CLK_PODF",
        "Divider for ssi1 clock podf.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSI1_CLK_PRED",
        "Divider for ssi1 clock pred.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EXTERN_AUDIO_CLK_PRED",
        "Divider for external audio clock pred.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSI3_CLK_PODF",
        "Divider for ssi3 clock podf.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSI3_CLK_PRED",
        "Divider for ssi3 clock pred.",
        22, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EXTERN_AUDIO_CLK_PODF",
        "Divider for external audio clock podf.",
        25, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CS2CDR.
static const field_t hw_ccm_cs2cdr[] =
{
    {
        "SSI2_CLK_PODF",
        "Divider for ssi2 clock podf.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSI2_CLK_PRED",
        "Divider for ssi2 clock pred.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LDB_DI0_CLK_SEL",
        "Selector for ldb_di1 clock multiplexer",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LDB_DI1_CLK_SEL",
        "Selector for ldb_di1 clock multiplexer",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CDCDR.
static const field_t hw_ccm_cdcdr[] =
{
    {
        "SPDIF1_CLK_SEL",
        "Selector for spdif1 clock multiplexer",
        7, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDIF1_CLK_PODF",
        "Divider for spdif1 clock podf.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDIF1_CLK_PRED",
        "Divider for spdif1 clock pred.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDIF0_CLK_SEL",
        "Selector for spdif0 clock multiplexer",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDIF0_CLK_PODF",
        "Divider for spdif0 clock podf.",
        22, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDIF0_CLK_PRED",
        "Divider for spdif0 clock pred.",
        25, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CHSCCDR.
static const field_t hw_ccm_chsccdr[] =
{
    {
        "PXP_AXI_PODF",
        "Divider for pxp_axi clock divider.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_AXI_CLK_SEL",
        "Selector for pxp_axi root clock multiplexer",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_AXI_PODF",
        "Divider for epdc_axi (SiPix) clock divider.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_AXI_CLK_SEL",
        "Selector for epdc_axi (SiPix) root clock multiplexer",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CSCDR2.
static const field_t hw_ccm_cscdr2[] =
{
    {
        "LCDIF_PIX_PRED",
        "Divider for lcdif_pix clock divider.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_PIX_CLK_SEL",
        "Selector for lcdif_pix root clock multiplexer",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_PIX_PRED",
        "Divider for epdc_pix clock divider.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_PIX_CLK_SEL",
        "Selector for epdc_pix root clock pre-multiplexer",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ECSPI_CLK_SEL",
        "Selector for the ECSPI clock multiplexor",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ECSPI_CLK_PODF",
        "Divider for ecspi clock podf.",
        19, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CSCDR3.
static const field_t hw_ccm_cscdr3[] =
{
    {
        "CSI_CORE_CLK_SEL",
        "Selector for csi_core clock multiplexer",
        9, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSI_CORE_PODF",
        "Post divider for csi_core clock.",
        11, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_AXI_CLK_SEL",
        "Selector for lcdif_axi clock multiplexer",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_AXI_PODF",
        "Divider for lcdif_axi clock.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CWDR.
static const field_t hw_ccm_cwdr[] =
{
    { 0 } // Terminator
};

// Bitfields in register CCM_CDHIPR.
static const field_t hw_ccm_cdhipr[] =
{
    {
        "AXI_PODF_BUSY",
        "Busy indicator for axi_podf.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AHB_PODF_BUSY",
        "Busy indicator for ahb_podf.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MMDC_ROOT_PODF_BUSY",
        "Busy indicator for mmdc_root_axi_podf.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PERIPH2_CLK_SEL_BUSY",
        "Busy indicator for periph2_clk_sel mux control.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MMDC_CH0_PODF_BUSY",
        "Busy indicator for mmdc_ch0_axi_podf.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PERIPH_CLK_SEL_BUSY",
        "Busy indicator for periph_clk_sel mux control.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ARM_PODF_BUSY",
        "Busy indicator for arm_podf.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CTOR.
static const field_t hw_ccm_ctor[] =
{
    {
        "OBS_SPARE_OUTPUT_2_SEL",
        "Selection of the signal to be generated on obs_output_2 (output of CCM"
        ") for observability on the pads.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OBS_SPARE_OUTPUT_1_SEL",
        "Selection of the signal to be generated on obs_output_1 (output of CCM"
        ") for observability on the pads.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OBS_SPARE_OUTPUT_0_SEL",
        "Selection of the signal to be generated on obs_output_0 (output of CCM"
        ") for observability on the pads.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OBS_EN",
        "observability enable bit.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CLPCR.
static const field_t hw_ccm_clpcr[] =
{
    {
        "LPM",
        "Setting the low power mode that system will enter on next assertion of"
        " dsm_request signal.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_PMIC_VFUNCTIONAL_READY",
        "By asserting this bit CCM will bypass waiting for pmic_vfunctional_rea"
        "dy signal when coming out of STOP mode.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARM_CLK_DIS_ON_LPM",
        "Define if ARM clocks (arm_clk, soc_mxclk, soc_pclk, soc_dbg_pclk, vl_w"
        "rck) will be disabled on wait mode.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SBYOS",
        "Standby clock oscillator bit.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIS_REF_OSC",
        "dis_ref_osc - in run mode, software can manually control closing of ex"
        "ternal reference oscillator clock, i.e.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSTBY",
        "Voltage standby request bit.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STBY_COUNT",
        "Standby counter definition.",
        9, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COSC_PWRDOWN",
        "In run mode, software can manually control powering down of on chip os"
        "cillator, i.e.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WB_PER_AT_LPM",
        "Enable periphery charge pump for well biasing at low power mode (stop "
        "or wait)",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_MMDC_CH0_LPM_HS",
        "Bypass handshake with mmdc_ch0 on next entrance to low power mode (wai"
        "t or stop mode).",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_MMDC_ROOT_LPM_HS",
        "Bypass handshake with mmdc_root on next entrance to low power mode (wa"
        "it or stop mode).",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CORE0_WFI",
        "Mask WFI of core0 for entering low power mode  Note: assertion of all "
        "bits[27:22] will generate low power mode request",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_SCU_IDLE",
        "Mask SCU IDLE for entering low power mode  Note: assertion of all bits"
        "[27:22] will generate low power mode request",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_L2CC_IDLE",
        "Mask L2CC IDLE for entering low power mode.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CISR.
static const field_t hw_ccm_cisr[] =
{
    {
        "LRF_PLL",
        "Interrupt ipi_int_2 generated due to lock of all enabled and not bypas"
        "eed pll's",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COSC_READY",
        "Interrupt ipi_int_2 generated due to on board oscillator ready, i.e.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AXI_PODF_LOADED",
        "Interrupt ipi_int_1 generated due to frequency change of axi_a_podf",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AXI_B_PODF_LOADED",
        "Interrupt ipi_int_1 generated due to frequency change of axi_b_podf",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PERIPH2_CLK_SEL_LOADED",
        "Interrupt ipi_int_1 generated due to frequency change of periph2_clk_s"
        "el",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AHB_PODF_LOADED",
        "Interrupt ipi_int_1 generated due to frequency change of ahb_podf",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MMDC_ROOT_PODF_LOADED",
        "Interrupt ipi_int_1 generated due to frequency change of mmdc_ch0_podf"
        "_ loaded",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERIPH_CLK_SEL_LOADED",
        "Interrupt ipi_int_1 generated due to update of periph_clk_sel.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MMDC_CH0_PODF_LOADED",
        "Interrupt ipi_int_1 generated due to update of mmdc_ch0_axi_podf.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARM_PODF_LOADED",
        "Interrupt ipi_int_1 generated due to frequency change of arm_podf.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CIMR.
static const field_t hw_ccm_cimr[] =
{
    {
        "MASK_LRF_PLL",
        "mask interrupt generation due to lrf of pll's",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_COSC_READY",
        "mask interrupt generation due to on board oscillator ready",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_AXI_PODF_LOADED",
        "mask interrupt generation due to frequency change of axi_podf",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_AXI_B_PODF_LOADED",
        "mask interrupt generation due to frequency change of axi_b_podf",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MASK_PERIPH2_CLK_SEL_LOADED",
        "mask interrupt generation due to update of periph2_clk_sel.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_AHB_PODF_LOADED",
        "mask interrupt generation due to frequency change of ahb_podf",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_MMDC_ROOT_PODF_LOADED",
        "mask interrupt generation due to update of mask_mmdc_root_podf",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_PERIPH_CLK_SEL_LOADED",
        "mask interrupt generation due to update of periph_clk_sel.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_MMDC_CH0_PODF_LOADED",
        "mask interrupt generation due to update of mask_mmdc_ch0_podf",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARM_PODF_LOADED",
        "mask interrupt generation due to frequency change of arm_podf",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCOSR.
static const field_t hw_ccm_ccosr[] =
{
    {
        "CLKO1_SEL",
        "Selection of the clock to be generated on clko1",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKO1_DIV",
        "Setting the divider of CLKO1",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKO1_EN",
        "Enable of CLKO1 clock",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKO1_CLKO2_SEL",
        "CLKO1 output to reflect clko1 or clko2 clocks",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKO2_SEL",
        "Selection of the clock to be generated on clko2",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKO2_DIV",
        "Setting the divider of CLKO2",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKO2_EN",
        "Enable of CLKO2 clock",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CGPR.
static const field_t hw_ccm_cgpr[] =
{
    {
        "PMIC_DELAY_SCALER",
        "Defines clock dividion of clock for stby_count (pmic delay counter)",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MEM_IGP_STOP_MASK",
        "Clear this bit to disallow putting the ARM cache into low power mode.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MMDC_EXT_CLK_DIS",
        "Disable external clock driver of MMDC during STOP mode",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EFUSE_PROG_SUPPLY_GATE",
        "Defines the value of the output signal cgpr_dout[4].",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FPL",
        "",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCGR0.
static const field_t hw_ccm_ccgr0[] =
{
    {
        "CG0",
        "aips_tz1 clocks (aips_tz1_clk_enable)",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG1",
        "aips_tz2 clocks (aips_tz2_clk_enable)",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG2",
        "Reserved",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG3",
        "Reserved",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG4",
        "Reserved",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG5",
        "Reserved",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG6",
        "Reserved",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG7",
        "Reserved",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG8",
        "Reserved",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG9",
        "Reserved",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG10",
        "Reserved",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG11",
        "CPU debug clocks (cheetah_dbg_clk_enable)",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG12",
        "Reserved",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG13",
        "Reserved",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG14",
        "Reserved",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG15",
        "Reserved",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCGR1.
static const field_t hw_ccm_ccgr1[] =
{
    {
        "CG0",
        "ecspi1 clocks (ecspi1_clk_enable)",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG1",
        "ecspi2 clocks (ecspi2_clk_enable)",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG2",
        "ecspi3 clocks (ecspi3_clk_enable)",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG3",
        "ecspi4 clocks (ecspi4_clk_enable)",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG4",
        "Reserved",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG5",
        "Reserved",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG6",
        "epit1 clocks (epit1_clk_enable)",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG7",
        "epit2 clocks (epit2_clk_enable)",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG8",
        "esai clocks (extern_audio_clk_enable)",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG9",
        "Reserved",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG10",
        "gpt bus clock (gpt_clk_enable)",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG11",
        "gpt serial clock (gpt_serial_clk_enable)",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG12",
        "Reserved",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG13",
        "gpu2d ovg clock (gpu2d_ovg_core_clk_enable)",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG14",
        "Reserved",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG15",
        "Reserved",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCGR2.
static const field_t hw_ccm_ccgr2[] =
{
    {
        "CG0",
        "Reserved",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG1",
        "Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG2",
        "Reserved",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG3",
        "i2c1_serial clock (i2c1_serial_clk_enable)",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG4",
        "i2c2_serial clock (i2c2_serial_clk_enable)",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG5",
        "i2c3_serial clock (i2c3_serial_clk_enable)",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG6",
        "OCOTP_CTRL clock (iim_clk_enable)",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG7",
        "iomux_ipt_clk_io clock (iomux_ipt_clk_io_enable)",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG8",
        "ipmux1 clock (ipmux1_clk_enable)",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG9",
        "ipmux2 clock (ipmux2_clk_enable)",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG10",
        "ipmux3 clock (ipmux3_clk_enable)",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG11",
        "ipsync_ip2apb_tzasc1_ipg clocks (ipsync_ip2apb_tzasc1_ipg_master_clk_e"
        "nable)",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG12",
        "Reserved",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG13",
        "ipsync_vdoa_ipg clocks (ipsync_vdoa_ipg_master_clk_enable)",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG14",
        "Reserved",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG15",
        "Reserved",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCGR3.
static const field_t hw_ccm_ccgr3[] =
{
    {
        "CG0",
        "csi core clock (csi_core_clk_enable)",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG1",
        "pxp axi clock (pxp_axi_clk_enable)",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG2",
        "epdc axi clock (epdc_axi_clk_enable)",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG3",
        "lcdif axi clock (lcdif_axi_clk_enable)",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG4",
        "ipu2_ipu_di0 clock (ipu2_ipu_di0_clk_enable)  lcdif_pix clock (lcdif_p"
        "ix_clk_enable)",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG5",
        "epdc_pix clock (epdc_pix_clk_enable)",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG6",
        "Reserved",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG7",
        "Reserved",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG8",
        "Reserved",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG9",
        "Reserved",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG10",
        "mmdc_core_aclk_fast_core_p0 clock (mmdc_core_aclk_fast_core_p0_enable)",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG11",
        "Reserved",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG12",
        "mmdc_core_ipg_clk_p0 clock (mmdc_core_ipg_clk_p0_enable)",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG13",
        "mmdc_core_ipg_clk_p1 clock (mmdc_core_ipg_clk_p1_enable)",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG14",
        "ocram clock (ocram_clk_enable)",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG15",
        "Reserved",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCGR4.
static const field_t hw_ccm_ccgr4[] =
{
    {
        "CG0",
        "Reserved",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG1",
        "Reserved.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG2",
        "Reserved.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG3",
        "Reserved.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG4",
        "pl301_mx6qfast1_s133 clock (pl301_mx6qfast1_s133clk_enable)",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG5",
        "Reserved",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG6",
        "Reserved",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG7",
        "pl301_mx6qper2_mainclk_enable (pl301_mx6qper2_mainclk_enable)",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG8",
        "pwm1 clocks (pwm1_clk_enable)",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG9",
        "pwm2 clocks (pwm2_clk_enable)",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG10",
        "pwm3 clocks (pwm3_clk_enable)",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG11",
        "pwm4 clocks (pwm4_clk_enable)",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG12",
        "rawnand_u_bch_input_apb clock (rawnand_u_bch_input_apb_clk_enable)",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG13",
        "rawnand_u_gpmi_bch_input_bch clock (rawnand_u_gpmi_bch_input_bch_clk_e"
        "nable)",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG14",
        "rawnand_u_gpmi_bch_input_gpmi_io clock (rawnand_u_gpmi_bch_input_gpmi_"
        "io_clk_enable)",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG15",
        "rawnand_u_gpmi_input_apb clock rawnand_u_gpmi_input_apb_clk_enable)",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCGR5.
static const field_t hw_ccm_ccgr5[] =
{
    {
        "CG0",
        "rom clock (rom_clk_enable)",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG1",
        "Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG2",
        "100M clock (100M_clk_enable)",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG3",
        "sdma clock (sdma_clk_enable)",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG4",
        "Reserved",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG5",
        "Reserved",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG6",
        "spba clock (spba_clk_enable)",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG7",
        "spdif clock (spdif_clk_enable)",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG8",
        "Reserved",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG9",
        "ssi1 clocks (ssi1_clk_enable)",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG10",
        "ssi2 clocks (ssi2_clk_enable)",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG11",
        "ssi3 clocks (ssi3_clk_enable)",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG12",
        "uart clock (uart_clk_enable)",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG13",
        "uart_serial clock (uart_serial_clk_enable)",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG14",
        "Reserved",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG15",
        "Reserved",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CCGR6.
static const field_t hw_ccm_ccgr6[] =
{
    {
        "CG0",
        "usboh3 clock (usboh3_clk_enable)",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG1",
        "usdhc1 clocks (usdhc1_clk_enable)",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG2",
        "usdhc2 clocks (usdhc2_clk_enable)",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG3",
        "usdhc3 clocks (usdhc3_clk_enable)",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG4",
        "usdhc4 clocks (usdhc4_clk_enable)",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG5",
        "emi_slow clocks (emi_slow_clk_enable)",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG6",
        "vdoaxiclk root clock (vdoaxiclk_clk_enable)",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG7",
        "vpu clocks (vpu_clk_enable)",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG8",
        "Reserved",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG9",
        "Reserved",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG10",
        "Reserved",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG11",
        "Reserved",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG12",
        "Reserved",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG13",
        "Reserved",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG14",
        "Reserved",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CG15",
        "Reserved",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_CMEOR.
static const field_t hw_ccm_cmeor[] =
{
    {
        "MOD_EN_OV_VDOA",
        "overide clock enable signal from vdoa - clock will not be gated based "
        "on vdoa signal.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_GPT",
        "overide clock enable signal from gpt - clock will not be gated based o"
        "n gpt's signal 'ipg_enable_clk' .",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_EPIT",
        "overide clock enable signal from epit - clock will not be gated based "
        "on epit's signal 'ipg_enable_clk' .",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_USDHC",
        "overide clock enable signal from usdhc.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_DAP",
        "overide clock enable signal from dap- clock will not be gated based on"
        " dap's signal 'dap_dbgen' .",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_VPU",
        "overide clock enable signal from vpu- clock will not be gated based on"
        " vpu's signal 'vpu_idle' .",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_GPU2D",
        "overide clock enable signal from gpu2d - clock will not be gated based"
        " on gpu2d's signal 'gpu2d_busy' .",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_GPU3D",
        "overide clock enable signal from gpu3d - clock will not be gated based"
        " on gpu3d's signal.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_CAN2_CPI",
        "overide clock enable signal from can2 - clock will not be gated based "
        "on can's signal 'enable_clk_cpi'.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MOD_EN_OV_CAN1_CPI",
        "overide clock enable signal from can1 - clock will not be gated based "
        "on can's signal 'enable_clk_cpi'.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a CCM module.
static const reg_t hw_ccm[] =
{
    {
        "CCR",
        "The figure below represents the CCM Control Register (CCR), which cont"
        "ains bits to control general operation of CCM.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_ccm_ccr
    },
    {
        "CCDR",
        "The figure below represents the CCM Control Divider Register (CCDR), w"
        "hich contains bits that control the loading of the dividers that need "
        "handshake with the modules they affect.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_ccm_ccdr
    },
    {
        "CSR",
        "The figure below represents the CCM status Register (CSR).",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_ccm_csr
    },
    {
        "CCSR",
        "The figure below represents the CCM Clock Switcher register (CCSR).",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_ccm_ccsr
    },
    {
        "CACRR",
        "The figure below represents the CCM Arm Clock Root register (CACRR).",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ccm_cacrr
    },
    {
        "CBCDR",
        "The figure below represents the CCM Bus Clock Divider Register (CBCDR)"
        ".",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_ccm_cbcdr
    },
    {
        "CBCMR",
        "The figure below represents the CCM Bus Clock Multiplexer Register (CB"
        "CMR).",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_ccm_cbcmr
    },
    {
        "CSCMR1",
        "The figure below represents the CCM Serial Clock Multiplexer Register "
        "1 (CSCMR1).",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_ccm_cscmr1
    },
    {
        "CSCMR2",
        "The figure below represents the CCM Serial Clock Multiplexer Register "
        "2 (CSCMR2).",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ccm_cscmr2
    },
    {
        "CSCDR1",
        "The figure below represents the CCM Serial Clock Divider Register 1 (C"
        "SCDR1).",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_ccm_cscdr1
    },
    {
        "CS1CDR",
        "The figure below represents the CCM SSI1, SSI3, ESAI Clock Divider Reg"
        "ister (CS1CDR).",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_ccm_cs1cdr
    },
    {
        "CS2CDR",
        "The figure below represents the CCM SSI2, LDB Clock Divider Register ("
        "CS2CDR).",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_ccm_cs2cdr
    },
    {
        "CDCDR",
        "The figure below represents the CCM DI Clock Divider Register (CDCDR).",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_ccm_cdcdr
    },
    {
        "CHSCCDR",
        "The figure below represents the CCM IPU1 DI Clock Divider Register (CH"
        "SCCDR).",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_ccm_chsccdr
    },
    {
        "CSCDR2",
        "The figure below represents the CCM Serial Clock Divider Register 2(CS"
        "CDR2).",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_ccm_cscdr2
    },
    {
        "CSCDR3",
        "The figure below represents the CCM Serial Clock Divider Register 3(CS"
        "CDR3).",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_ccm_cscdr3
    },
    {
        "CWDR",
        "The figure below represents the CCM Wakeup Detector Register (CWDR).",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        0, // Number of bitfields
        hw_ccm_cwdr
    },
    {
        "CDHIPR",
        "The figure below represents the CCM Divider Handshake In-Process Regis"
        "ter (CDHIPR).",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_ccm_cdhipr
    },
    {
        "CTOR",
        "The figure below represents the CCM Testing Observability Register (CT"
        "OR).",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_ccm_ctor
    },
    {
        "CLPCR",
        "The figure below represents the CCM Low Power Control Register (CLPCR)"
        ".",
        4, // Width in bytes
        0x00000054, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_ccm_clpcr
    },
    {
        "CISR",
        "The figure below represents the CCM Interrupt Status Register (CISR).",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_ccm_cisr
    },
    {
        "CIMR",
        "The figure below represents the CCM Interrupt Mask Register (CIMR).",
        4, // Width in bytes
        0x0000005c, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_ccm_cimr
    },
    {
        "CCOSR",
        "The figure below represents the CCM Clock Output Source Register (CCOS"
        "R).",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_ccm_ccosr
    },
    {
        "CGPR",
        "Fast PLL enable.",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_ccm_cgpr
    },
    {
        "CCGR0",
        "CG(i) bits CCGR 0-7  These bits are used to turn on/off the clock to e"
        "ach module independently.The following table details the possible cloc"
        "k activity conditions for each module         CGR value    Clock Activ"
        "ity Description        00    clock is off during all modes.",
        4, // Width in bytes
        0x00000068, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_ccm_ccgr0
    },
    {
        "CCGR1",
        "The figure below represents the CCM Clock Gating Register 1(CCM_CCGR1)"
        ".",
        4, // Width in bytes
        0x0000006c, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_ccm_ccgr1
    },
    {
        "CCGR2",
        "The figure below represents the CCM Clock Gating Register 2 (CCM_CCGR2"
        ").",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_ccm_ccgr2
    },
    {
        "CCGR3",
        "The figure below represents the CCM Clock Gating Register 3 (CCM_CCGR3"
        ").",
        4, // Width in bytes
        0x00000074, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_ccm_ccgr3
    },
    {
        "CCGR4",
        "The figure below represents the CCM Clock Gating Register 4 (CCM_CCGR4"
        ").",
        4, // Width in bytes
        0x00000078, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_ccm_ccgr4
    },
    {
        "CCGR5",
        "The figure below represents the CCM Clock Gating Register 5 (CCM_CCGR5"
        ").",
        4, // Width in bytes
        0x0000007c, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_ccm_ccgr5
    },
    {
        "CCGR6",
        "The figure below represents the CCM Clock Gating Register 6 (CCM_CCGR6"
        ").",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_ccm_ccgr6
    },
    {
        "CMEOR",
        "The follow figure represents the CCM Module Enable Override Register ("
        "CMEOR).",
        4, // Width in bytes
        0x00000088, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_ccm_cmeor
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark CCM_ANALOG
#endif

// Bitfields in register CCM_ANALOG_PLL_ARM.
static const field_t hw_ccm_analog_pll_arm[] =
{
    {
        "DIV_SELECT",
        "This field controls the pll loop divider.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_LF",
        "Reserved by Freescale.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_LF",
        "Reserved by Freescale.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_CP",
        "Reserved by Freescale.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_CP",
        "Reserved by Freescale.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HOLD_RING_OFF",
        "Analog debug bit.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWERDOWN",
        "Powers down the PLL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE",
        "Enable the clock output.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_CLK_SRC",
        "Determines the bypass source.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the pll.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVDS_SEL",
        "Analog Debug Bit",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVDS_24MHZ_SEL",
        "Analog Debug Bit",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PLL_SEL",
        "Reserved",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "1 - PLL is currently locked.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_USB1.
static const field_t hw_ccm_analog_pll_usb1[] =
{
    {
        "DIV_SELECT",
        "This field controls the pll loop divider.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_USB_CLKS",
        "Powers the 9-phase PLL outputs for USBPHYn.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWER",
        "Powers up the PLL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE",
        "Enable the PLL clock output.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_CLK_SRC",
        "Determines the bypass source.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the pll.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "1 - PLL is currently locked.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_USB2.
static const field_t hw_ccm_analog_pll_usb2[] =
{
    {
        "DIV_SELECT",
        "This field controls the pll loop divider.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_USB_CLKS",
        "0: 8-phase PLL outputs for USBPHY1 are powered down.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWER",
        "Powers up the PLL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE",
        "Enable the PLL clock output.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_CLK_SRC",
        "Determines the bypass source.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the pll.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "1 - PLL is currently locked.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_SYS.
static const field_t hw_ccm_analog_pll_sys[] =
{
    {
        "DIV_SELECT",
        "This field controls the pll loop divider.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_LF",
        "Reserved by Freescale",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_LF",
        "Reserved by Freescale",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_CP",
        "Reserved by Freescale",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_CP",
        "Reserved by Freescale",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HOLD_RING_OFF",
        "Analog Debug bit.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWERDOWN",
        "Powers down the PLL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE",
        "Enable PLL output",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_CLK_SRC",
        "Determines the bypass source.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the pll.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD_OFFSET_EN",
        "Enables an offset in the phase frequency detector.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "1 - PLL is currently locked; 0 - PLL is not currently locked.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_AUDIO.
static const field_t hw_ccm_analog_pll_audio[] =
{
    {
        "DIV_SELECT",
        "This field controls the pll loop divider.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_LF",
        "Reserved by Freescale.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_LF",
        "Reserved by Freescale.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_CP",
        "Reserved by Freescale.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_CP",
        "Reserved by Freescale.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HOLD_RING_OFF",
        "Analog debug Bit",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWERDOWN",
        "Powers down the PLL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE",
        "Enable PLL output",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_CLK_SRC",
        "Determines the bypass source.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the pll.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD_OFFSET_EN",
        "Enables an offset in the phase frequency detector.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POST_DIV_SELECT",
        "These bits implement a divider after the PLL, but before the enable an"
        "d bypass mux.",
        19, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSC_EN",
        "Reserved Bit",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "1 - PLL is currently locked.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_AUDIO_NUM.
static const field_t hw_ccm_analog_pll_audio_num[] =
{
    {
        "A",
        "30 bit numerator of fractional loop divider.",
        0, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_AUDIO_DENOM.
static const field_t hw_ccm_analog_pll_audio_denom[] =
{
    {
        "B",
        "30 bit Denominator of fractional loop divider.",
        0, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_VIDEO.
static const field_t hw_ccm_analog_pll_video[] =
{
    {
        "DIV_SELECT",
        "This field controls the pll loop divider.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_LF",
        "Reserved by Freescale.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_LF",
        "Reserved by Freescale.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_CP",
        "Reserved by Freescale.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_CP",
        "Reserved by Freescale.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HOLD_RING_OFF",
        "Analog Debug bit.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWERDOWN",
        "Powers down the PLL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE",
        "Enalbe PLL output",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_CLK_SRC",
        "Determines the bypass source.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the pll.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD_OFFSET_EN",
        "Enables an offset in the phase frequency detector.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POST_DIV_SELECT",
        "These bits implement a divider after the PLL, but before the enable an"
        "d bypass mux.",
        19, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSC_EN",
        "Revserved BIt",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "1 - PLL is currently locked;  0 - PLL is not currently locked.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_VIDEO_NUM.
static const field_t hw_ccm_analog_pll_video_num[] =
{
    {
        "A",
        "30 bit numerator of fractional loop divider(Signed number), absolute v"
        "alue should be less than denominator",
        0, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_VIDEO_DENOM.
static const field_t hw_ccm_analog_pll_video_denom[] =
{
    {
        "B",
        "30 bit Denominator of fractional loop divider.",
        0, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PLL_ENET.
static const field_t hw_ccm_analog_pll_enet[] =
{
    {
        "DIV_SELECT",
        "Controls the frequency of the ethernet reference clock.00 - 25MHz; 01 "
        "- 50MHz; 10 - 100MHz (not 50% duty cycle); 11 - 125MHz;",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_LF",
        "Reserved by Freescale",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_LF",
        "Reserved by Freescale",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_CP",
        "Reserved by Freescale",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOUBLE_CP",
        "Reserved by Freescale",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HOLD_RING_OFF",
        "Analog debug bit",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWERDOWN",
        "Powers down the PLL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE",
        "Enable the ethernet clock output.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_CLK_SRC",
        "Determines the bypass source.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the pll.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD_OFFSET_EN",
        "Enables an offset in the phase frequency detector.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_125M",
        "Enables an offset in the phase frequency detector.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_100M",
        "Enables an offset in the phase frequency detector.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "1 - PLL is currently locked; 0 - PLL is not currently locked.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PFD_480.
static const field_t hw_ccm_analog_pfd_480[] =
{
    {
        "PFD0_FRAC",
        "This field controls the fractional divide value.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD0_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD0_CLKGATE",
        "If set to 1, the IO fractional divider clock (reference ref_pfd0) is o"
        "ff (power savings).",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD1_FRAC",
        "This field controls the fractional divide value.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD1_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD1_CLKGATE",
        "IO Clock Gate.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD2_FRAC",
        "This field controls the fractional divide value.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD2_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD2_CLKGATE",
        "IO Clock Gate.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD3_FRAC",
        "This field controls the fractional divide value.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD3_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD3_CLKGATE",
        "IO Clock Gate.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_PFD_528.
static const field_t hw_ccm_analog_pfd_528[] =
{
    {
        "PFD0_FRAC",
        "This field controls the fractional divide value.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD0_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD0_CLKGATE",
        "If set to 1, the IO fractional divider clock (reference ref_pfd0) is o"
        "ff (power savings).",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD1_FRAC",
        "This field controls the fractional divide value.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD1_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD1_CLKGATE",
        "IO Clock Gate.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD2_FRAC",
        "This field controls the fractional divide value.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD2_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD2_CLKGATE",
        "IO Clock Gate.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD3_FRAC",
        "This field controls the fractional divide value.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFD3_STABLE",
        "This read-only bitfield is for DIAGNOSTIC PURPOSES ONLY since the frac"
        "tional divider should become stable quickly enough that this field wil"
        "l never need to be used by either device driver or application code.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFD3_CLKGATE",
        "IO Clock Gate.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_MISC0.
static const field_t hw_ccm_analog_misc0[] =
{
    {
        "STOP_MODE_CONFIG",
        "Configure the analog behavior in stop mode.",
        11, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CCM_ANALOG_MISC2.
static const field_t hw_ccm_analog_misc2[] =
{
    {
        "PLL3_DISABLE",
        "Default value of \"0\".",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUDIO_DIV_LSB",
        "LSB of Post-divider for Audio PLL: 0x0=div-by-1(default), 0x1=div-by-2"
        ", 0x2=div-by-1, 0x3=div-by-4.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUDIO_DIV_MSB",
        "MSB of Post-divider for Audio PLL: 0x0=div-by-1(default), 0x1=div-by-2"
        ", 0x2=div-by-1, 0x3=div-by-4.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VIDEO_DIV",
        "Post-divider for video: 0x0=div-by-1(default), 0x1=div-by-2, 0x2=div-b"
        "y-1, 0x3=div-by-4.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a CCM_ANALOG module.
static const reg_t hw_ccm_analog[] =
{
    {
        "PLL_ARM",
        "The control register provides control for the system PLL.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_ccm_analog_pll_arm
    },
    {
        "PLL_USB1",
        "The control register provides control for USBPHY0 480MHz PLL.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_ccm_analog_pll_usb1
    },
    {
        "PLL_USB2",
        "The control register provides control for USBPHY1 480MHz PLL.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_ccm_analog_pll_usb2
    },
    {
        "PLL_SYS",
        "The control register provides control for the 528MHz PLL.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_ccm_analog_pll_sys
    },
    {
        "PLL_AUDIO",
        "The control register provides control for the audio PLL.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_ccm_analog_pll_audio
    },
    {
        "PLL_AUDIO_NUM",
        "This register contains the numerator (A) of Audio PLL fractional loop "
        "divider.(Signed number), absolute value should be less than denominato"
        "r  Absolute value should be less than denominator",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ccm_analog_pll_audio_num
    },
    {
        "PLL_AUDIO_DENOM",
        "This register contains the Denominator (B) of Audio PLL fractional loo"
        "p divider.(unsigned number)",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ccm_analog_pll_audio_denom
    },
    {
        "PLL_VIDEO",
        "The control register provides control for the Video PLL.",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_ccm_analog_pll_video
    },
    {
        "PLL_VIDEO_NUM",
        "This register contains the numerator (A) of Video PLL fractional loop "
        "divider.(Signed number)  Absolute value should be less than denominato"
        "r",
        4, // Width in bytes
        0x000000b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ccm_analog_pll_video_num
    },
    {
        "PLL_VIDEO_DENOM",
        "This register contains the Denominator (B) of Video PLL fractional loo"
        "p divider.(Unsigned number)",
        4, // Width in bytes
        0x000000c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ccm_analog_pll_video_denom
    },
    {
        "PLL_ENET",
        "The control register provides control for the ENET PLL.",
        4, // Width in bytes
        0x000000e0, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_ccm_analog_pll_enet
    },
    {
        "PFD_480",
        "The PFD_480 control register provides control for PFD clock generation"
        ".",
        4, // Width in bytes
        0x000000f0, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_ccm_analog_pfd_480
    },
    {
        "PFD_528",
        "The PFD_528 control register provides control for PFD clock generation"
        ".",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_ccm_analog_pfd_528
    },
    {
        "MISC0",
        "This register defines the control for miscellaneous CCM Analog blocks.",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ccm_analog_misc0
    },
    {
        "MISC2",
        "This register defines the control for miscellaneous CCM Analog blocks.",
        4, // Width in bytes
        0x00000170, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_ccm_analog_misc2
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark CSI
#endif

// Bitfields in register CSI_CSICR1.
static const field_t hw_csi_csicr1[] =
{
    {
        "PIXEL_BIT",
        "Pixel Bit.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REDGE",
        "Valid Pixel Clock Edge Select.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INV_PCLK",
        "Invert Pixel Clock Input.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INV_DATA",
        "Invert Data Input.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GCLK_MODE",
        "Gated Clock Mode Enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_RXFIFO",
        "Asynchronous RXFIFO Clear.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_STATFIFO",
        "Asynchronous STATFIFO Clear.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PACK_DIR",
        "Data Packing Direction.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FCC",
        "FIFO Clear Control.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CCIR_EN",
        "CCIR656 Interface Enable.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSYNC_POL",
        "HSYNC Polarity Select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SOF_INTEN",
        "Start Of Frame (SOF) Interrupt Enable.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SOF_POL",
        "SOF Interrupt Polarity.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFF_INTEN",
        "RxFIFO Full Interrupt Enable.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FB1_DMA_DONE_INTEN",
        "Frame Buffer1 DMA Transfer Done Interrupt Enable.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FB2_DMA_DONE_INTEN",
        "Frame Buffer2 DMA Transfer Done Interrupt Enable.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATFF_INTEN",
        "STATFIFO Full Interrupt Enable.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFF_DMA_DONE_INTEN",
        "STATFIFO DMA Transfer Done Interrupt Enable.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RF_OR_INTEN",
        "RxFIFO Overrun Interrupt Enable.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SF_OR_INTEN",
        "STAT FIFO Overrun Interrupt Enable.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COF_INT_EN",
        "Change Of Image Field (COF) Interrupt Enable.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CCIR_MODE",
        "CCIR Mode Select.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRP_IF_EN",
        "CSI-PrP Interface Enable.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EOF_INT_EN",
        "End-of-Frame Interrupt Enable.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EXT_VSYNC",
        "External VSYNC Enable.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWAP16_EN",
        "SWAP 16-Bit Enable.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSICR2.
static const field_t hw_csi_csicr2[] =
{
    {
        "HSC",
        "Horizontal Skip Count.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSC",
        "Vertical Skip Count.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVRM",
        "Live View Resolution Mode.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BTS",
        "Bayer Tile Start.",
        19, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SCE",
        "Skip Count Enable.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AFS",
        "Auto Focus Spread.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DRM",
        "Double Resolution Mode.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_BURST_TYPE_SFF",
        "Burst Type of DMA Transfer from STATFIFO.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_BURST_TYPE_RFF",
        "Burst Type of DMA Transfer from RxFIFO.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSICR3.
static const field_t hw_csi_csicr3[] =
{
    {
        "ECC_AUTO_EN",
        "Automatic Error Correction Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ECC_INT_EN",
        "Error Detection Interrupt Enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZERO_PACK_EN",
        "Dummy Zero Packing Enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TWO_8BIT_SENSOR",
        "Two 8-bit Sensor Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFF_LEVEL",
        "RxFIFO Full Level .",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HRESP_ERR_EN",
        "Hresponse Error Enable.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATFF_LEVEL",
        "STATFIFO Full Level.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_REQ_EN_SFF",
        "DMA Request Enable for STATFIFO.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_REQ_EN_RFF",
        "DMA Request Enable for RxFIFO.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_REFLASH_SFF",
        "Reflash DMA Controller for STATFIFO.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_REFLASH_RFF",
        "Reflash DMA Controller for RxFIFO.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRMCNT_RST",
        "Frame Count Reset.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRMCNT",
        "Frame Counter.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSISTATFIFO.
static const field_t hw_csi_csistatfifo[] =
{
    {
        "STAT",
        "Static data from sensor",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIRFIFO.
static const field_t hw_csi_csirfifo[] =
{
    {
        "IMAGE",
        "Received image data",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIRXCNT.
static const field_t hw_csi_csirxcnt[] =
{
    {
        "RXCNT",
        "RxFIFO Count.",
        0, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSISR.
static const field_t hw_csi_csisr[] =
{
    {
        "DRDY",
        "RXFIFO Data Ready.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ECC_INT",
        "CCIR Error Interrupt.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HRESP_ERR_INT",
        "Hresponse Error Interrupt Status.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COF_INT",
        "Change Of Field Interrupt Status.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "F1_INT",
        "CCIR Field 1 Interrupt Status.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "F2_INT",
        "CCIR Field 2 Interrupt Status.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SOF_INT",
        "Start of Frame Interrupt Status.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EOF_INT",
        "End of Frame (EOF) Interrupt Status.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFF_INT",
        "RXFIFO Full Interrupt Status.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_TSF_DONE_FB1",
        "DMA Transfer Done in Frame Buffer1.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_TSF_DONE_FB2",
        "DMA Transfer Done in Frame Buffer2.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATFF_INT",
        "STATFIFO Full Interrupt Status.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_TSF_DONE_SFF",
        "DMA Transfer Done from StatFIFO.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RF_OR_INT",
        "RxFIFO Overrun Interrupt Status.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SF_OR_INT",
        "STATFIFO Overrun Interrupt Status.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIDMASA_STATFIFO.
static const field_t hw_csi_csidmasa_statfifo[] =
{
    {
        "DMA_START_ADDR_SFF",
        "DMA Start Address for STATFIFO.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIDMATS_STATFIFO.
static const field_t hw_csi_csidmats_statfifo[] =
{
    {
        "DMA_TSF_SIZE_SFF",
        "DMA Transfer Size for STATFIFO.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIDMASA_FB1.
static const field_t hw_csi_csidmasa_fb1[] =
{
    {
        "DMA_START_ADDR_FB1",
        "DMA Start Address in Frame Buffer1.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIDMASA_FB2.
static const field_t hw_csi_csidmasa_fb2[] =
{
    {
        "DMA_START_ADDR_FB2",
        "DMA Start Address in Frame Buffer2.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIFBUF_PARA.
static const field_t hw_csi_csifbuf_para[] =
{
    {
        "FBUF_STRIDE",
        "Frame Buffer Parameter.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSI_CSIIMAG_PARA.
static const field_t hw_csi_csiimag_para[] =
{
    {
        "IMAGE_HEIGHT",
        "Image Height.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IMAGE_WIDTH",
        "Image Width.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a CSI module.
static const reg_t hw_csi[] =
{
    {
        "CSICR1",
        "This register controls the sensor interface timing and interrupt gener"
        "ation.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        26, // Number of bitfields
        hw_csi_csicr1
    },
    {
        "CSICR2",
        "This register provides the statistic block with data about which live "
        "view resolution is being used, and the starting sensor pixel of the Ba"
        "yer pattern.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_csi_csicr2
    },
    {
        "CSICR3",
        "This read/write register acts as an extension of the functionality of "
        "the CSI Control register 1, adding additional control and features.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        13, // Number of bitfields
        hw_csi_csicr3
    },
    {
        "CSISTATFIFO",
        "The StatFIFO is a read-only register containing statistic data from th"
        "e sensor.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_csi_csistatfifo
    },
    {
        "CSIRFIFO",
        "This read-only register contains received image data.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_csi_csirfifo
    },
    {
        "CSIRXCNT",
        "This register works for EOF interrupt generation.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_csi_csirxcnt
    },
    {
        "CSISR",
        "This read/write register shows sensor interface status, and which kind"
        " of interrupt is being generated.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_csi_csisr
    },
    {
        "CSIDMASA_STATFIFO",
        "This register provides the start address for the embedded DMA controll"
        "er of STATFIFO.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_csi_csidmasa_statfifo
    },
    {
        "CSIDMATS_STATFIFO",
        "This register provides the total transfer size for the embedded DMA co"
        "ntroller of STATFIFO.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_csi_csidmats_statfifo
    },
    {
        "CSIDMASA_FB1",
        "This register provides the start address in the frame buffer1 for the "
        "embedded DMA controller of RxFIFO.",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_csi_csidmasa_fb1
    },
    {
        "CSIDMASA_FB2",
        "This register provides the start address in the frame buffer2 for the "
        "embedded DMA controller of RxFIFO.",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_csi_csidmasa_fb2
    },
    {
        "CSIFBUF_PARA",
        "This register provides the stride of the frame buffer to show how many"
        " words to skip before starting to write the next row of the image.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_csi_csifbuf_para
    },
    {
        "CSIIMAG_PARA",
        "This register provides the width and the height of the image from the "
        "sensor.",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_csi_csiimag_para
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark CSU
#endif

// Bitfields in register CSU_CSL0.
static const field_t hw_csu_csl0[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL1.
static const field_t hw_csu_csl1[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL2.
static const field_t hw_csu_csl2[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL3.
static const field_t hw_csu_csl3[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL4.
static const field_t hw_csu_csl4[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL5.
static const field_t hw_csu_csl5[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL6.
static const field_t hw_csu_csl6[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL7.
static const field_t hw_csu_csl7[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL8.
static const field_t hw_csu_csl8[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL9.
static const field_t hw_csu_csl9[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL10.
static const field_t hw_csu_csl10[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL11.
static const field_t hw_csu_csl11[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL12.
static const field_t hw_csu_csl12[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL13.
static const field_t hw_csu_csl13[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL14.
static const field_t hw_csu_csl14[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL15.
static const field_t hw_csu_csl15[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL16.
static const field_t hw_csu_csl16[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL17.
static const field_t hw_csu_csl17[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL18.
static const field_t hw_csu_csl18[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL19.
static const field_t hw_csu_csl19[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL20.
static const field_t hw_csu_csl20[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL21.
static const field_t hw_csu_csl21[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL22.
static const field_t hw_csu_csl22[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL23.
static const field_t hw_csu_csl23[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL24.
static const field_t hw_csu_csl24[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL25.
static const field_t hw_csu_csl25[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL26.
static const field_t hw_csu_csl26[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL27.
static const field_t hw_csu_csl27[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL28.
static const field_t hw_csu_csl28[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL29.
static const field_t hw_csu_csl29[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL30.
static const field_t hw_csu_csl30[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL31.
static const field_t hw_csu_csl31[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL32.
static const field_t hw_csu_csl32[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL33.
static const field_t hw_csu_csl33[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL34.
static const field_t hw_csu_csl34[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL35.
static const field_t hw_csu_csl35[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL36.
static const field_t hw_csu_csl36[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL37.
static const field_t hw_csu_csl37[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL38.
static const field_t hw_csu_csl38[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_CSL39.
static const field_t hw_csu_csl39[] =
{
    {
        "SUR_S2",
        "Secure user read access control for the second slave",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S2",
        "Secure supervisor read access control for the second slave",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S2",
        "Non-secure user read access control for the second slave",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S2",
        "Non-secure supervisor read access control for the second slave",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S2",
        "Secure user write access control for the second slave",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S2",
        "Secure supervisor write access control for the second slave",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S2",
        "Non-secure user write access control for the second slave",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S2",
        "Non-secure supervisor write access control for the second slave",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S2",
        "Lock bit corresponding to the second slave.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUR_S1",
        "Secure user read access control for the first slave",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSR_S1",
        "Secure supervisor read access control for the first slave",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUR_S1",
        "Non-secure user read access control for the first slave",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSR_S1",
        "Non-secure supervisor read access control for the first slave",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUW_S1",
        "Secure user write access control for the first slave",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SSW_S1",
        "Secure supervisor write access control for the first slave",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUW_S1",
        "Non-secure user write access control for the first slave",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSW_S1",
        "Non-secure supervisor write access control for the first slave",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_S1",
        "Lock bit corresponding to the first slave.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_HP0.
static const field_t hw_csu_hp0[] =
{
    {
        "HP",
        "Privilege indicator bits",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L",
        "Lock bit set by secure software",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP1",
        "Privilege indicator bits",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L1",
        "Lock bit set by secure software",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP2",
        "Privilege indicator bits",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2",
        "Lock bit set by secure software",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP3",
        "Privilege indicator bits",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L3",
        "Lock bit set by secure software",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP4",
        "Privilege indicator bits",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L4",
        "Lock bit set by secure software",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP5",
        "Privilege indicator bits",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L5",
        "Lock bit set by secure software",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP6",
        "Privilege indicator bits",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L6",
        "Lock bit set by secure software",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP7",
        "Privilege indicator bits",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L7",
        "Lock bit set by secure software",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP8",
        "Privilege indicator bits",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L8",
        "Lock bit set by secure software",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP9",
        "Privilege indicator bits",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L9",
        "Lock bit set by secure software",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP10",
        "Privilege indicator bits",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L10",
        "Lock bit set by secure software",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP11",
        "Privilege indicator bits",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L11",
        "Lock bit set by secure software",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP12",
        "Privilege indicator bits",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L12",
        "Lock bit set by secure software",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP13",
        "Privilege indicator bits",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L13",
        "Lock bit set by secure software",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP14",
        "Privilege indicator bits",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L14",
        "Lock bit set by secure software",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP15",
        "Privilege indicator bits",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L15",
        "Lock bit set by secure software",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_HP1.
static const field_t hw_csu_hp1[] =
{
    {
        "HP",
        "Privilege indicator bits",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L",
        "Lock bit set by secure software",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP1",
        "Privilege indicator bits",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L1",
        "Lock bit set by secure software",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP2",
        "Privilege indicator bits",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2",
        "Lock bit set by secure software",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP3",
        "Privilege indicator bits",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L3",
        "Lock bit set by secure software",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP4",
        "Privilege indicator bits",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L4",
        "Lock bit set by secure software",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP5",
        "Privilege indicator bits",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L5",
        "Lock bit set by secure software",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP6",
        "Privilege indicator bits",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L6",
        "Lock bit set by secure software",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP7",
        "Privilege indicator bits",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L7",
        "Lock bit set by secure software",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP8",
        "Privilege indicator bits",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L8",
        "Lock bit set by secure software",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP9",
        "Privilege indicator bits",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L9",
        "Lock bit set by secure software",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP10",
        "Privilege indicator bits",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L10",
        "Lock bit set by secure software",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP11",
        "Privilege indicator bits",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L11",
        "Lock bit set by secure software",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP12",
        "Privilege indicator bits",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L12",
        "Lock bit set by secure software",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP13",
        "Privilege indicator bits",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L13",
        "Lock bit set by secure software",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP14",
        "Privilege indicator bits",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L14",
        "Lock bit set by secure software",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HP15",
        "Privilege indicator bits",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L15",
        "Lock bit set by secure software",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register CSU_SA.
static const field_t hw_csu_sa[] =
{
    {
        "NSA",
        "Non-Secure Access Policy indicator bits",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L",
        "Lock bit set by secure software",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA1",
        "Non-Secure Access Policy indicator bits",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L1",
        "Lock bit set by secure software",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA2",
        "Non-Secure Access Policy indicator bits",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2",
        "Lock bit set by secure software",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA3",
        "Non-Secure Access Policy indicator bits",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L3",
        "Lock bit set by secure software",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA4",
        "Non-Secure Access Policy indicator bits",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L4",
        "Lock bit set by secure software",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA5",
        "Non-Secure Access Policy indicator bits",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L5",
        "Lock bit set by secure software",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA6",
        "Non-Secure Access Policy indicator bits",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L6",
        "Lock bit set by secure software",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA7",
        "Non-Secure Access Policy indicator bits",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L7",
        "Lock bit set by secure software",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA8",
        "Non-Secure Access Policy indicator bits",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L8",
        "Lock bit set by secure software",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA9",
        "Non-Secure Access Policy indicator bits",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L9",
        "Lock bit set by secure software",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA10",
        "Non-Secure Access Policy indicator bits",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L10",
        "Lock bit set by secure software",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA11",
        "Non-Secure Access Policy indicator bits",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L11",
        "Lock bit set by secure software",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA12",
        "Non-Secure Access Policy indicator bits",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L12",
        "Lock bit set by secure software",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA13",
        "Non-Secure Access Policy indicator bits",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L13",
        "Lock bit set by secure software",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA14",
        "Non-Secure Access Policy indicator bits",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L14",
        "Lock bit set by secure software",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NSA15",
        "Non-Secure Access Policy indicator bits",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L15",
        "Lock bit set by secure software",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a CSU module.
static const reg_t hw_csu[] =
{
    {
        "CSL0",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl0
    },
    {
        "CSL1",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl1
    },
    {
        "CSL2",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl2
    },
    {
        "CSL3",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl3
    },
    {
        "CSL4",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl4
    },
    {
        "CSL5",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl5
    },
    {
        "CSL6",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl6
    },
    {
        "CSL7",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl7
    },
    {
        "CSL8",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl8
    },
    {
        "CSL9",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl9
    },
    {
        "CSL10",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl10
    },
    {
        "CSL11",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl11
    },
    {
        "CSL12",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl12
    },
    {
        "CSL13",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl13
    },
    {
        "CSL14",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl14
    },
    {
        "CSL15",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl15
    },
    {
        "CSL16",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl16
    },
    {
        "CSL17",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl17
    },
    {
        "CSL18",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl18
    },
    {
        "CSL19",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl19
    },
    {
        "CSL20",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl20
    },
    {
        "CSL21",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000054, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl21
    },
    {
        "CSL22",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl22
    },
    {
        "CSL23",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000005c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl23
    },
    {
        "CSL24",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl24
    },
    {
        "CSL25",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl25
    },
    {
        "CSL26",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000068, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl26
    },
    {
        "CSL27",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000006c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl27
    },
    {
        "CSL28",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl28
    },
    {
        "CSL29",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000074, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl29
    },
    {
        "CSL30",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000078, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl30
    },
    {
        "CSL31",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000007c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl31
    },
    {
        "CSL32",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl32
    },
    {
        "CSL33",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000084, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl33
    },
    {
        "CSL34",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000088, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl34
    },
    {
        "CSL35",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000008c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl35
    },
    {
        "CSL36",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl36
    },
    {
        "CSL37",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000094, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl37
    },
    {
        "CSL38",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x00000098, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl38
    },
    {
        "CSL39",
        "There are 40 Config Security Level (CSU_CSL0-CSU_CSL39) registers.",
        4, // Width in bytes
        0x0000009c, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_csu_csl39
    },
    {
        "HP0",
        "The HP register may be programmed to determine the privilege (either U"
        "ser Mode or Supervisor Mode) for up to sixteen different masters.",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        32, // Number of bitfields
        hw_csu_hp0
    },
    {
        "HP1",
        "The HP register may be programmed to determine the privilege (either U"
        "ser Mode or Supervisor Mode) for up to sixteen different masters.",
        4, // Width in bytes
        0x00000204, // Base address offset
        true, // Readable
        true, // Writable
        32, // Number of bitfields
        hw_csu_hp1
    },
    {
        "SA",
        "The Secure Access register may be programmed to specify the access pol"
        "icy (either Secure or Non-secure) for up to sixteen different masters.",
        4, // Width in bytes
        0x00000218, // Base address offset
        true, // Readable
        true, // Writable
        32, // Number of bitfields
        hw_csu_sa
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark DBGMON
#endif

// Bitfields in register DBGMON_HW_DBGMON_CTRL.
static const field_t hw_dbgmon_hw_dbgmon_ctrl[] =
{
    {
        "RUN",
        "Set this bit to one to enable the DBGMON operation",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SNAP",
        "Set this bit to snapshot the registers selected by REQSEL to SNVS doma"
        "in registers",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR",
        "Set this bit to clear the registers in SOC domain.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_SNVS",
        "Set this bit to clear the registers in SNVS domain",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDR_TRAPMODE",
        "The bit defines the address trap function.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ID_TRAPMODE",
        "The bit defines the ID trap function.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REQSEL",
        "This field defines which sets of AXI transaction will be snaped to SNV"
        "S domain registers.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WORKMODE",
        "This field defines whether ignore the transaction in IRQ",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDR_MASKEN",
        "This field control the address mask function,",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDOG_IRQ_SEL",
        "Select the source of WDOG IRQ.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE",
        "This bit must be set to zero for normal oepration.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFTRST",
        "Set to zero for normal operation.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_MASTER_EN.
static const field_t hw_dbgmon_hw_dbgmon_master_en[] =
{
    {
        "MID0",
        "Set to 1 to enable monitoring on MasterID 0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID1",
        "Set to 1 to enable monitoring on MasterID 1.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID2",
        "Set to 1 to enable monitoring on MasterID 2.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID3",
        "Set to 1 to enable monitoring on MasterID 3.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID4",
        "Set to 1 to enable monitoring on MasterID 4.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID5",
        "Set to 1 to enable monitoring on MasterID 5",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID6",
        "Set to 1 to enable monitoring on MasterID 6.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID7",
        "Set to 1 to enable monitoring on MasterID 7.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID8",
        "Set to 1 to enable monitoring on MasterID 8.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID9",
        "Set to 1 to enable monitoring on MasterID 9.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID10",
        "Set to 1 to enable monitoring on MasterID 10.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID11",
        "Set to 1 to enable monitoring on MasterID 11.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID12",
        "Set to 1 to enable monitoring on MasterID 12.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID13",
        "Set to 1 to enable monitoring on MasterID 13.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID14",
        "Set to 1 to enable monitoring on MasterID 14.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MID15",
        "Set to 1 to enable monitoring on MasterID 15.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_IRQ.
static const field_t hw_dbgmon_hw_dbgmon_irq[] =
{
    {
        "ADDR_TRAP_IRQEN",
        "Address trap interrupt control.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ID_TRAP_IRQEN",
        "ID trap interrupt control.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDR_TRAP_IRQ",
        "This bit indicates the Address trap interrupt is happening.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ID_TRAP_IRQ",
        "This bit indicates the ID trap interrupt is happening.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRQ_MID",
        "This field indicate which master sends the interrupt, will not update "
        "until all interrupts are cleared.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_TRAP_ADDR_LOW.
static const field_t hw_dbgmon_hw_dbgmon_trap_addr_low[] =
{
    {
        "ADDRESS",
        "This field contains 32-bit low address for the address trap range",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_TRAP_ADDR_HIGH.
static const field_t hw_dbgmon_hw_dbgmon_trap_addr_high[] =
{
    {
        "ADDRESS",
        "This field contains 32-bit high address for the address trap range",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_TRAP_ID.
static const field_t hw_dbgmon_hw_dbgmon_trap_id[] =
{
    {
        "TRAP_ID_LOW",
        "This field contains 16-bit low ID for ID trap range",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRAP_ID_HIGH",
        "This field contains 16-bit high ID for ID trap range",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_SNVS_ADDR.
static const field_t hw_dbgmon_hw_dbgmon_snvs_addr[] =
{
    {
        "ADDR",
        "This field contains 32-bit Address in SNVS domain register",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_SNVS_DATA.
static const field_t hw_dbgmon_hw_dbgmon_snvs_data[] =
{
    {
        "DATA",
        "This field contains 32-bit Data of AXI transaction in SNVS domain regi"
        "ster",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_SNVS_INFO.
static const field_t hw_dbgmon_hw_dbgmon_snvs_info[] =
{
    {
        "COMPLETE",
        "The field indicates whether the AXI transaction in SNVS domain complet"
        "e.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RDWR",
        "The field indicates the read/write attribute of AXI transaction in SNV"
        "S domain.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ID",
        "The field contain the ID of the AXI transaction in SNVS domain",
        16, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DBGMON_HW_DBGMON_VERSION.
static const field_t hw_dbgmon_hw_dbgmon_version[] =
{
    {
        "STEP",
        "Fixed read-only value reflecting the stepping of the RTL version",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MINOR",
        "Fixed read-only value reflecting the MINOR field of the RTL version",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-only value reflecting the MAJOR field of the RTL version",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a DBGMON module.
static const reg_t hw_dbgmon[] =
{
    {
        "HW_DBGMON_CTRL",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_dbgmon_hw_dbgmon_ctrl
    },
    {
        "HW_DBGMON_MASTER_EN",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_dbgmon_hw_dbgmon_master_en
    },
    {
        "HW_DBGMON_IRQ",
        "",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_dbgmon_hw_dbgmon_irq
    },
    {
        "HW_DBGMON_TRAP_ADDR_LOW",
        "",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dbgmon_hw_dbgmon_trap_addr_low
    },
    {
        "HW_DBGMON_TRAP_ADDR_HIGH",
        "",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dbgmon_hw_dbgmon_trap_addr_high
    },
    {
        "HW_DBGMON_TRAP_ID",
        "",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dbgmon_hw_dbgmon_trap_id
    },
    {
        "HW_DBGMON_SNVS_ADDR",
        "",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dbgmon_hw_dbgmon_snvs_addr
    },
    {
        "HW_DBGMON_SNVS_DATA",
        "",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dbgmon_hw_dbgmon_snvs_data
    },
    {
        "HW_DBGMON_SNVS_INFO",
        "",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_dbgmon_hw_dbgmon_snvs_info
    },
    {
        "HW_DBGMON_VERSION",
        "",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_dbgmon_hw_dbgmon_version
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark DCP
#endif

// Bitfields in register DCP_CTRL.
static const field_t hw_dcp_ctrl[] =
{
    {
        "CHANNEL_INTERRUPT_ENABLE",
        "Per-channel interrupt enable bit.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_CONTEXT_SWITCHING",
        "Enable automatic context switching for the channels.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_CONTEXT_CACHING",
        "Software should set this bit to enable caching of contexts between ope"
        "rations.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GATHER_RESIDUAL_WRITES",
        "Software should set this bit to enable ragged writes to unaligned buff"
        "ers to be gathered between multiple write operations.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRESENT_SHA",
        "Indicates whether the SHA1/SHA2 functions are present.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PRESENT_CRYPTO",
        "Indicates whether the crypto (Cipher/Hash) functions are present.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CLKGATE",
        "This bit must be set to zero for normal operation.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFTRST",
        "Set this bit to zero to enable normal DCP operation.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_STAT.
static const field_t hw_dcp_stat[] =
{
    {
        "IRQ",
        "Indicates which channels have pending interrupt requests.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "READY_CHANNELS",
        "Indicates which channels are ready to proceed with a transfer (active "
        "channel also included).",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CUR_CHANNEL",
        "Current (active) channel (encoded).",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OTP_KEY_READY",
        "When set, indicates that the OTP key has been shifted from the fuse bl"
        "ock and is ready for use.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CHANNELCTRL.
static const field_t hw_dcp_channelctrl[] =
{
    {
        "ENABLE_CHANNEL",
        "Setting a bit in this field will enabled the DMA channel associated wi"
        "th it.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HIGH_PRIORITY_CHANNEL",
        "Setting a bit in this field causes the corresponding channel to have h"
        "igh-priority arbitration.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CH0_IRQ_MERGED",
        "Indicates that the interrupt for channel 0 should be merged with the o"
        "ther interrupts on the shared dcp_irq interrupt.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CAPABILITY0.
static const field_t hw_dcp_capability0[] =
{
    {
        "NUM_KEYS",
        "Encoded value indicating the number of key storage locations implement"
        "ed in the design.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NUM_CHANNELS",
        "Encoded value indicating the number of channels implemented in the des"
        "ign.",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DISABLE_UNIQUE_KEY",
        "Write to a 1 disable the per-device unique key.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_DECRYPT",
        "Write to a 1 to disable decryption.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CAPABILITY1.
static const field_t hw_dcp_capability1[] =
{
    {
        "CIPHER_ALGORITHMS",
        "One-hot field indicating which cipher algorithms are available.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HASH_ALGORITHMS",
        "One-hot field indicating which hashing features are implemented in HW.",
        16, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CONTEXT.
static const field_t hw_dcp_context[] =
{
    {
        "ADDR",
        "Context pointer address.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_KEY.
static const field_t hw_dcp_key[] =
{
    {
        "SUBWORD",
        "Key subword pointer.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INDEX",
        "Key index pointer.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_KEYDATA.
static const field_t hw_dcp_keydata[] =
{
    {
        "DATA",
        "Word 0 data for key.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PACKET0.
static const field_t hw_dcp_packet0[] =
{
    {
        "ADDR",
        "Next Pointer Register,",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PACKET1.
static const field_t hw_dcp_packet1[] =
{
    {
        "INTERRUPT",
        "Reflects whether the channel should issue an interrupt upon completion"
        " of the packet.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DECR_SEMAPHORE",
        "Reflects whether the channel's semaphore should be decremented at the "
        "end of the current operation.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CHAIN",
        "Reflects whether the next command pointer register should be loaded in"
        "to the channel's current descriptor pointer.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CHAIN_CONTIGUOUS",
        "Reflects whether the next packet's address is located following this p"
        "acket's payload.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENABLE_MEMCOPY",
        "Reflects whether the selected hashing function should be enabled for t"
        "his operation.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENABLE_CIPHER",
        "Reflects whether the selected cipher function should be enabled for th"
        "is operation.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENABLE_HASH",
        "Reflects whether the selected hashing function should be enabled for t"
        "his operation.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENABLE_BLIT",
        "Reflects whether the DCP should perform a blit operation.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CIPHER_ENCRYPT",
        "When the cipher block is enabled, this bit indicates whether the opera"
        "tion is encryption or decryption.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CIPHER_INIT",
        "Reflects whether the cipher block should load the initialization vecto"
        "r from the payload for this operation.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OTP_KEY",
        "Reflects whether a hardware-based key should be used.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PAYLOAD_KEY",
        "When set, indicates the payload contains the key.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HASH_INIT",
        "Reflects whether the current hashing block is the initial block in the"
        " hashing operation, so the hash registers should be initialized before"
        " the operation.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HASH_TERM",
        "Reflects whether the current hashing block is the final block in the h"
        "ashing operation, so the hash padding should be applied by hardware.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CHECK_HASH",
        "Reflects whether the calculated hash value should be compared against "
        "the hash provided in the payload.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HASH_OUTPUT",
        "When hashing is enabled, this bit controls whether the input or output"
        " data is hashed.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CONSTANT_FILL",
        "When this bit is set (MEMCOPY and BLIT modes only), the DCP will simpl"
        "y fill the destination buffer with the value found in the  Source Addr"
        "ess field.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TEST_SEMA_IRQ",
        "This bit is used to test the channel semaphore transition to 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "KEY_BYTESWAP",
        "Reflects whether the DCP engine will swap key bytes (big-endian key).",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "KEY_WORDSWAP",
        "Reflects whether the DCP engine will swap key words (big-endian key).",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "INPUT_BYTESWAP",
        "Reflects whether the DCP engine will byteswap input data (big-endian d"
        "ata).",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "INPUT_WORDSWAP",
        "Reflects whether the DCP engine will wordswap input data (big-endian d"
        "ata).",
        21, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OUTPUT_BYTESWAP",
        "Reflects whether the DCP engine will byteswap output data (big-endian "
        "data).",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OUTPUT_WORDSWAP",
        "Reflects whether the DCP engine will wordswap output data (big-endian "
        "data).",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TAG",
        "Packet Tag",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PACKET2.
static const field_t hw_dcp_packet2[] =
{
    {
        "CIPHER_SELECT",
        "Cipher Selection Field",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CIPHER_MODE",
        "Cipher Mode Selection Field.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "KEY_SELECT",
        "Key Selection Field.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HASH_SELECT",
        "Hash Selection Field",
        16, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CIPHER_CFG",
        "Cipher configuration bits.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PACKET3.
static const field_t hw_dcp_packet3[] =
{
    {
        "ADDR",
        "Source Buffer Address Pointer.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PACKET4.
static const field_t hw_dcp_packet4[] =
{
    {
        "ADDR",
        "Destination Buffer Address Pointer.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PACKET5.
static const field_t hw_dcp_packet5[] =
{
    {
        "COUNT",
        "Byte Count register.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PACKET6.
static const field_t hw_dcp_packet6[] =
{
    {
        "ADDR",
        "This regiser reflects the payload pointer for the current control pack"
        "et.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH0CMDPTR.
static const field_t hw_dcp_ch0cmdptr[] =
{
    {
        "ADDR",
        "Pointer to descriptor structure to be processed for channel 0.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH0SEMA.
static const field_t hw_dcp_ch0sema[] =
{
    {
        "INCREMENT",
        "The value written to this field is added to the semaphore count in an "
        "atomic way such that simultaneous software adds and DCP hardware subst"
        "racts happening on the same clock are protected.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE",
        "This read-only field shows the current (instantaneous) value of the se"
        "maphore counter.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH0STAT.
static const field_t hw_dcp_ch0stat[] =
{
    {
        "HASH_MISMATCH",
        "The bit indicates that a hashing check operation mismatched for contro"
        "l packets that enable the HASH_CHECK bit.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SETUP",
        "This bit indicates that the hardware has detected an invalid programmi"
        "ng configuration such as a buffer length that is not a multiple of the"
        " natural data size for the operation.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PACKET",
        "This bit indicates that a a bus error occurred when reading the packet"
        " or payload or when writing status back to the packet payload.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SRC",
        "This bit indicates a bus error occurred when reading from the source b"
        "uffer.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_DST",
        "This bit indicates a bus error occurred when storing to the destinatio"
        "n buffer.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PAGEFAULT",
        "This bit indicates a page fault occurred while converting a virtual ad"
        "dress to a physical address..",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_CODE",
        "Indicates additional error codes for some error conditions.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TAG",
        "Indicates the tag from the last completed packet in the command struct"
        "ure",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH0OPTS.
static const field_t hw_dcp_ch0opts[] =
{
    {
        "RECOVERY_TIMER",
        "This field indicates the recovery time for the channel.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH1CMDPTR.
static const field_t hw_dcp_ch1cmdptr[] =
{
    {
        "ADDR",
        "Pointer to descriptor structure to be processed for channel 1.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH1SEMA.
static const field_t hw_dcp_ch1sema[] =
{
    {
        "INCREMENT",
        "The value written to this field is added to the semaphore count in an "
        "atomic way such that simultaneous software adds and DCP hardware subst"
        "racts happening on the same clock are protected.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE",
        "This read-only field shows the current (instantaneous) value of the se"
        "maphore counter.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH1STAT.
static const field_t hw_dcp_ch1stat[] =
{
    {
        "HASH_MISMATCH",
        "The bit indicates that a hashing check operation mismatched for contro"
        "l packets that enable the HASH_CHECK bit.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SETUP",
        "This bit indicates that the hardware detected an invalid programming c"
        "onfiguration such as a buffer length that is not a multiple of the nat"
        "ural data size for the operation.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PACKET",
        "This bit indicates that a bus error occurs when reading the packet or "
        "payload or when writing status back to the packet paylaod.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SRC",
        "This bit indicates a bus error occurred when reading from the source b"
        "uffer.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_DST",
        "This bit indicates a bus error occurred when storing to the destinatio"
        "n buffer.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PAGEFAULT",
        "This bit indicates a page fault occurred while converting a virtual ad"
        "dress to a physical address..",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_CODE",
        "Indicates additional error codes for some error conditions.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TAG",
        "Indicates the tag from the last completed packet in the command struct"
        "ure",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH1OPTS.
static const field_t hw_dcp_ch1opts[] =
{
    {
        "RECOVERY_TIMER",
        "This field indicates the recovery time for the channel.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH2CMDPTR.
static const field_t hw_dcp_ch2cmdptr[] =
{
    {
        "ADDR",
        "Pointer to descriptor structure to be processed for channel 2.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH2SEMA.
static const field_t hw_dcp_ch2sema[] =
{
    {
        "INCREMENT",
        "The value written to this field is added to the semaphore count in an "
        "atomic way such that simultaneous software adds and DCP hardware subst"
        "racts happening on the same clock are protected.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE",
        "This read-only field shows the current (instantaneous) value of the se"
        "maphore counter.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH2STAT.
static const field_t hw_dcp_ch2stat[] =
{
    {
        "HASH_MISMATCH",
        "The bit indicates that a hashing check operation mismatched for contro"
        "l packets that enable the HASH_CHECK bit.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SETUP",
        "This bit indicates that the hardware detected an invalid programming c"
        "onfiguration such as a buffer length that is not a multiple of the nat"
        "ural data size for the operation.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PACKET",
        "This bit indicates that a bus error occurred when reading the packet o"
        "r payload or when writing status back to the packet paylaod.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SRC",
        "This bit indicates a bus error occurred when reading from the source b"
        "uffer.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_DST",
        "This bit indicates a bus error occurred when storing to the destinatio"
        "n buffer.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PAGEFAULT",
        "This bit indicates a page fault occurred while converting a virtual ad"
        "dress to a physical address..",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_CODE",
        "Indicates additional error codes for some error conditions.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TAG",
        "Indicates the tag from the last completed packet in the command struct"
        "ure",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH2OPTS.
static const field_t hw_dcp_ch2opts[] =
{
    {
        "RECOVERY_TIMER",
        "This field indicates the recovery time for the channel.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH3CMDPTR.
static const field_t hw_dcp_ch3cmdptr[] =
{
    {
        "ADDR",
        "Pointer to descriptor structure to be processed for channel 3.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH3SEMA.
static const field_t hw_dcp_ch3sema[] =
{
    {
        "INCREMENT",
        "The value written to this field is added to the semaphore count in an "
        "atomic way such that simultaneous software adds and DCP hardware subst"
        "racts happening on the same clock are protected.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE",
        "This read-only field shows the current (instantaneous) value of the se"
        "maphore counter.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH3STAT.
static const field_t hw_dcp_ch3stat[] =
{
    {
        "HASH_MISMATCH",
        "The bit indicates that a hashing check operation mismatched for contro"
        "l packets that enable the HASH_CHECK bit.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SETUP",
        "This bit indicates that the hardware detected an invalid programming c"
        "onfiguration such as a buffer length that is not a multiple of the nat"
        "ural data size for the operation.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PACKET",
        "This bit indicates that a bus error occurred when reading the packet o"
        "r payload or when writing status back to the packet paylaod.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_SRC",
        "This bit indicates a bus error occurred when reading from the source b"
        "uffer.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_DST",
        "This bit indicates a bus error occurred when storing to the destinatio"
        "n buffer.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_PAGEFAULT",
        "This bit indicates a page fault occurred while converting a virtual ad"
        "dress to a physical address..",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERROR_CODE",
        "Indicates additional error codes for some error conditions.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TAG",
        "Indicates the tag from the last completed packet in the command struct"
        "ure",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_CH3OPTS.
static const field_t hw_dcp_ch3opts[] =
{
    {
        "RECOVERY_TIMER",
        "This field indicates the recovery time for the channel.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_DBGSELECT.
static const field_t hw_dcp_dbgselect[] =
{
    {
        "INDEX",
        "Selects a value to read via the debug data register.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_DBGDATA.
static const field_t hw_dcp_dbgdata[] =
{
    {
        "DATA",
        "Debug Data",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_PAGETABLE.
static const field_t hw_dcp_pagetable[] =
{
    {
        "ENABLE",
        "Page Table Enable control.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FLUSH",
        "Page Table Flush control.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BASE",
        "Page Table Base Address.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DCP_VERSION.
static const field_t hw_dcp_version[] =
{
    {
        "STEP",
        "Fixed read-onlyl value reflecting the stepping of version of the desig"
        "n implementation.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MINOR",
        "Fixed read-onlyl value reflecting the MINOR version of the design impl"
        "ementation.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-onlyl value reflecting the MAJOR version of the design impl"
        "ementation.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a DCP module.
static const reg_t hw_dcp[] =
{
    {
        "CTRL",
        "The CTRL register contains controls for the DCP module.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_dcp_ctrl
    },
    {
        "STAT",
        "The DCP Interrupt Status register provides channel interrupt status in"
        "formation.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_dcp_stat
    },
    {
        "CHANNELCTRL",
        "The DCP Channel Control register provides controls for channel arbitra"
        "tion and channel enables.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_dcp_channelctrl
    },
    {
        "CAPABILITY0",
        "This register contains additional information about the DCP module imp"
        "lementation parameters.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_dcp_capability0
    },
    {
        "CAPABILITY1",
        "This register contains information about the algorithms available on t"
        "he implementation.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_dcp_capability1
    },
    {
        "CONTEXT",
        "This register contains a pointer to the memory region to be used for D"
        "CP context swap operations.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_context
    },
    {
        "KEY",
        "This register contains a pointer to the key location to be written.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dcp_key
    },
    {
        "KEYDATA",
        "This register provides write access to the key/key subword specified b"
        "y the Key Index Register.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_keydata
    },
    {
        "PACKET0",
        "This register displays the values for the current work packet offset 0"
        "x00 (Next Command) field.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dcp_packet0
    },
    {
        "PACKET1",
        "This register displays the values for the current work packet offset 0"
        "x04 (control) field.",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        false, // Writable
        25, // Number of bitfields
        hw_dcp_packet1
    },
    {
        "PACKET2",
        "This register displays the values for the current work packet offset 0"
        "x08 (Control1) field.",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        false, // Writable
        5, // Number of bitfields
        hw_dcp_packet2
    },
    {
        "PACKET3",
        "This register displays the values for the current work packet offset 0"
        "x0C (Source Address) field.",
        4, // Width in bytes
        0x000000b0, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dcp_packet3
    },
    {
        "PACKET4",
        "This register displays the values for the current work packet offset 0"
        "x10 (Destination Address) field.",
        4, // Width in bytes
        0x000000c0, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dcp_packet4
    },
    {
        "PACKET5",
        "This register displays the values for the current work packet offset 0"
        "x14 (Buffer Size) field.",
        4, // Width in bytes
        0x000000d0, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dcp_packet5
    },
    {
        "PACKET6",
        "This register displays the values for the current work packet offset 0"
        "x1C (Payload Pointer) field.",
        4, // Width in bytes
        0x000000e0, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dcp_packet6
    },
    {
        "CH0CMDPTR",
        "The DCP channel 0 current command address register points to the multi"
        "word descriptor that is to be executed (or currently being executed).",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch0cmdptr
    },
    {
        "CH0SEMA",
        "The DCP Channel 0 semaphore register is used to synchronize the ARM pl"
        "atform instruction stream and the DMA chain processing state.",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dcp_ch0sema
    },
    {
        "CH0STAT",
        "The DCP Channel 0 Interrupt Status register contains the interrupt sta"
        "tus bit and the tag of the last completed operation from the command c"
        "hain.",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_dcp_ch0stat
    },
    {
        "CH0OPTS",
        "The DCP Channel 0 Options Status register contains optional control in"
        "formation that may be used to further tune the behavior of the channel"
        ".",
        4, // Width in bytes
        0x00000130, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch0opts
    },
    {
        "CH1CMDPTR",
        "The DCP channel 1 current command address register points to the multi"
        "word descriptor that is to be executed (or currently being executed).",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch1cmdptr
    },
    {
        "CH1SEMA",
        "The DCP Channel 1 semaphore register is used to synchronize the ARM pl"
        "atform instruction stream and the DMA chain processing state.",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dcp_ch1sema
    },
    {
        "CH1STAT",
        "The DCP Channel 1 Interrupt Status register contains the interrupt sta"
        "tus bit and the tag of the last completed operation from the command c"
        "hain.",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_dcp_ch1stat
    },
    {
        "CH1OPTS",
        "The DCP Channel 1 Options Status register contains optional control in"
        "formation that may be used to further tune the behavior of the channel"
        ".",
        4, // Width in bytes
        0x00000170, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch1opts
    },
    {
        "CH2CMDPTR",
        "The DCP channel 2 current command address register points to the multi"
        "word descriptor that is to be executed (or currently being executed).",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch2cmdptr
    },
    {
        "CH2SEMA",
        "The DCP Channel 2 semaphore register is used to synchronize the ARM pl"
        "atform instruction stream and the DMA chain processing state.",
        4, // Width in bytes
        0x00000190, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dcp_ch2sema
    },
    {
        "CH2STAT",
        "The DCP Channel 2 Interrupt Status register contains the interrupt sta"
        "tus bit and the tag of the last completed operation from the command c"
        "hain.",
        4, // Width in bytes
        0x000001a0, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_dcp_ch2stat
    },
    {
        "CH2OPTS",
        "The DCP Channel 2 Options Status register contains optional control in"
        "formation that may be used to further tune the behavior of the channel"
        ".",
        4, // Width in bytes
        0x000001b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch2opts
    },
    {
        "CH3CMDPTR",
        "The DCP channel 3 current command address register points to the multi"
        "word descriptor that is to be executed (or currently being executed).",
        4, // Width in bytes
        0x000001c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch3cmdptr
    },
    {
        "CH3SEMA",
        "The DCP Channel 3 semaphore register is used to synchronize the ARM pl"
        "atform instruction stream and the DMA chain processing state.",
        4, // Width in bytes
        0x000001d0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dcp_ch3sema
    },
    {
        "CH3STAT",
        "The DCP Channel 3 Interrupt Status register contains the interrupt sta"
        "tus bit and the tag of the last completed operation from the command c"
        "hain.",
        4, // Width in bytes
        0x000001e0, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_dcp_ch3stat
    },
    {
        "CH3OPTS",
        "The DCP Channel 3 Options Status register contains optional control in"
        "formation that may be used to further tune the behavior of the channel"
        ".",
        4, // Width in bytes
        0x000001f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_ch3opts
    },
    {
        "DBGSELECT",
        "This register selects a debug register to view.",
        4, // Width in bytes
        0x00000400, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_dcp_dbgselect
    },
    {
        "DBGDATA",
        "Reading this register returns the debug data value from the selected i"
        "ndex.",
        4, // Width in bytes
        0x00000410, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_dcp_dbgdata
    },
    {
        "PAGETABLE",
        "The DCP Page Table register controls the virtual memory functionality "
        "of the DCP.",
        4, // Width in bytes
        0x00000420, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_dcp_pagetable
    },
    {
        "VERSION",
        "Read-only register indicating implemented version of the DCP.",
        4, // Width in bytes
        0x00000430, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_dcp_version
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark DVFSC
#endif

// Bitfields in register DVFSC_THRS.
static const field_t hw_dvfsc_thrs[] =
{
    {
        "PNCTHR",
        "Panic threshold for load tracking",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DWTHR",
        "Down threshold for load tracking",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPTHR",
        "Upper threshold for load tracking",
        22, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_COUN.
static const field_t hw_dvfsc_coun[] =
{
    {
        "UPCNT",
        "UP counter threshold value",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DN_CNT",
        "Down counter threshold value",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_SIG1.
static const field_t hw_dvfsc_sig1[] =
{
    {
        "WSW6",
        "General purpose load tracking signal weight dvfs_w_sig[6]",
        2, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW7",
        "General purpose load tracking signal weight dvfs_w_sig[7]",
        5, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW8",
        "General purpose load tracking signal weight dvfs_w_sig[8]",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW9",
        "General purpose load tracking signal weight dvfs_w_sig[9]",
        11, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW10",
        "General purpose load tracking signal weight dvfs_w_sig[10]",
        14, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW11",
        "General purpose load tracking signal weight dvfs_w_sig[11]",
        17, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW12",
        "General purpose load tracking signal weight dvfs_w_sig[12]",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW13",
        "General purpose load tracking signal weight dvfs_w_sig[13]",
        23, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW14",
        "General purpose load tracking signal weight dvfs_w_sig[14]",
        26, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW15",
        "General purpose load tracking signal weight dvfs_w_sig[15]",
        29, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSSIG0.
static const field_t hw_dvfsc_dvfssig0[] =
{
    {
        "WSW0",
        "General purpose load tracking signal weight dvfs_w_sig[0].",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW1",
        "General purpose load tracking signal weight dvfs_w_sig[1].",
        6, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW2",
        "General purpose load tracking signal weight dvfs_w_sig[2]",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW3",
        "General purpose load tracking signal weight dvfs_w_sig[3]",
        23, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW4",
        "General purpose load tracking signal weight dvfs_w_sig[4]",
        26, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WSW5",
        "General purpose load tracking signal weight dvfs_w_sig[5]",
        29, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSGPC0.
static const field_t hw_dvfsc_dvfsgpc0[] =
{
    {
        "GPBC0",
        "GPBC0 - General Purpose bits Counter 0  During period of this counter "
        "the GeP bit 0 will be set and WSW0 will be added to the calculations.",
        0, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C0ACT",
        "C0ACT - Counter 0 active indicator",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "C0STRT",
        "C0STRT - Counter 0 start  Setting of this bit will initialize down cou"
        "nting of the GPC0 value.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSGPC1.
static const field_t hw_dvfsc_dvfsgpc1[] =
{
    {
        "GPBC1",
        "GPBC1 - General Purpose bits Counter 1  During period of this counter "
        "the GeP bit 1 will be set and WSW1 will be added to the calculations.",
        0, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C1ACT",
        "C1ACT - Counter 1 active indicator",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "C1STRT",
        "C1STRT - Counter 1start  Setting of this bit will initialize down coun"
        "ting of the GPC1 value.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSGPBT.
static const field_t hw_dvfsc_dvfsgpbt[] =
{
    {
        "GPB0",
        "General purpose bit 0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB1",
        "General purpose bit 1.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB2",
        "General purpose bit 2.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB3",
        "General purpose bit 3.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB4",
        "General purpose bit 4.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB5",
        "General purpose bit 5.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB6",
        "General purpose bit 6.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB7",
        "General purpose bit 7.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB8",
        "General purpose bit 8.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB9",
        "General purpose bit 9.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB10",
        "General purpose bit 10.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB11",
        "General purpose bit 11.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB12",
        "General purpose bit 12.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB13",
        "General purpose bit 13.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB14",
        "General purpose bit 14.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPB15",
        "General purpose bit 15.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSEMAC.
static const field_t hw_dvfsc_dvfsemac[] =
{
    {
        "EMAC",
        "EMAC - EMA control value",
        0, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DVFEN0",
        "DVFS tracking for core0 enable.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FSVAI0",
        "DVFS Frequency adjustment status of core 0.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WFIM0",
        "DVFS Wait for Interrupt of core 0 mask bit",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_CNTR.
static const field_t hw_dvfsc_cntr[] =
{
    {
        "LTBRSR",
        "LTBRSR - Load Tracking Buffer Register Source:",
        3, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LTBRSH",
        "LTBRSH - Load Tracking Buffer Register Shift:",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFUS",
        "PFUS - Periodic Frequency Update Status",
        6, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFUE",
        "PFUE - Period Frequency Update Enable",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIV_RATIO",
        "DIV_RATIO - Divider value.",
        11, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MINF",
        "Minimum frequency reached.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MAXF",
        "Maximum frequency reached.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FSVAI",
        "FSVAI  DVFS Frequency adjustment interrupt.",
        20, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FSVAIM",
        "DVFS Frequency adjustment interrupt mask.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PIRQS",
        "PIRQS - Pattern IRQ Source  * write '1' to clear.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DVFIS",
        "DVFS Interrupt select.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LBFL0",
        "Load buffer 0 - full status bit.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LBFL1",
        "Load buffer 1 - full status bit.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LBMI",
        "Load buffer full mask interrupt.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DVFEV",
        "Always give a DVFS event.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIV3CK",
        "DIV3CK - div_3_clk division ratio inside the DVFS module.",
        29, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSLTR0_0.
static const field_t hw_dvfsc_dvfsltr0_0[] =
{
    {
        "LTS0_0",
        "core 0 Load Tracking Sample 0",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_1",
        "core 0 Load Tracking Sample 1",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_2",
        "core 0 Load Tracking Sample 2",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_3",
        "core 0 Load Tracking Sample 3",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_4",
        "core 0 Load Tracking Sample 4",
        16, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_5",
        "core 0 Load Tracking Sample 5",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_6",
        "core 0 Load Tracking Sample 6",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_7",
        "core 0 Load Tracking Sample 7",
        28, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSLTR0_1.
static const field_t hw_dvfsc_dvfsltr0_1[] =
{
    {
        "LTS0_8",
        "core 0 Load Tracking Sample 8",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_9",
        "core 0 Load Tracking Sample 9",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_10",
        "core 0 Load Tracking Sample 10",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_11",
        "core 0 Load Tracking Sample 11",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_12",
        "core 0 Load Tracking Sample 12",
        16, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_13",
        "core 0 Load Tracking Sample 13",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_14",
        "core 0 Load Tracking Sample 14",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS0_15",
        "core 0 Load Tracking Sample 15",
        28, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSLTR1_0.
static const field_t hw_dvfsc_dvfsltr1_0[] =
{
    {
        "LTS1_0",
        "core 0 Load Tracking Sample 0",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_1",
        "core 0 Load Tracking Sample 1",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_2",
        "core 0 Load Tracking Sample 2",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_3",
        "core 0 Load Tracking Sample 3",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_4",
        "core 0 Load Tracking Sample 4",
        16, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_5",
        "core 0 Load Tracking Sample 5",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_6",
        "core 0 Load Tracking Sample 6",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_7",
        "core 0 Load Tracking Sample 7",
        28, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSLTR1_1.
static const field_t hw_dvfsc_dvfsltr1_1[] =
{
    {
        "LTS1_8",
        "core 0 Load Tracking Sample 8",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_9",
        "core 0 Load Tracking Sample 9",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_10",
        "core 0 Load Tracking Sample 10",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_11",
        "core 0 Load Tracking Sample 11",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_12",
        "core 0 Load Tracking Sample 12",
        16, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_13",
        "core 0 Load Tracking Sample 13",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_14",
        "core 0 Load Tracking Sample 14",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LTS1_15",
        "core 0 Load Tracking Sample 15",
        28, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSPT0.
static const field_t hw_dvfsc_dvfspt0[] =
{
    {
        "FPTN0",
        "FPTN0 - Frequency pattern 0 counter  During period of this counter the"
        " frequency will be reduced from the EMA-detected level.",
        0, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PT0A",
        "PT0A - Pattern 0 currently active (read-only)",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSPT1.
static const field_t hw_dvfsc_dvfspt1[] =
{
    {
        "FPTN1",
        "FPTN1 - Frequency pattern 1 counter  During period of this counter the"
        " frequency will be set to the EMA-detected level.",
        0, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PT1A",
        "PT1A - Pattern 1 currently active (read-only)",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSPT2.
static const field_t hw_dvfsc_dvfspt2[] =
{
    {
        "FPTN2",
        "FPTN2 - Frequency pattern 2 counter  During period of this counter the"
        " frequency will be increased to higher, than detected by the EMA-detec"
        "ted level.",
        0, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PT2A",
        "PT2A - Pattern 2 currently active (read-only)",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "P2THR",
        "P2THR - Pattern 2 Threshold  Threshold of current DVFS load (after EMA"
        "), for generating interrupts with PFUS indicators 110, 111.",
        26, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register DVFSC_DVFSPT3.
static const field_t hw_dvfsc_dvfspt3[] =
{
    {
        "FPTN3",
        "FPTN3 - Frequency pattern 3 counter  During period of this counter the"
        " frequency will be set to the EMA-detected level.",
        0, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PT3A",
        "PT3A - Pattern 3 currently active (read-only)",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a DVFSC module.
static const reg_t hw_dvfsc[] =
{
    {
        "THRS",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_dvfsc_thrs
    },
    {
        "COUN",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dvfsc_coun
    },
    {
        "SIG1",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_dvfsc_sig1
    },
    {
        "DVFSSIG0",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_dvfsc_dvfssig0
    },
    {
        "DVFSGPC0",
        "DVFS general purpose bits weight counter.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_dvfsc_dvfsgpc0
    },
    {
        "DVFSGPC1",
        "DVFS general purpose bits weight counter1.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_dvfsc_dvfsgpc1
    },
    {
        "DVFSGPBT",
        "",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_dvfsc_dvfsgpbt
    },
    {
        "DVFSEMAC",
        "",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_dvfsc_dvfsemac
    },
    {
        "CNTR",
        "DIV3CK division         DIV3CK setting  dividing ratio  sum_3 passing "
        "bits  div_1_clk cumulative divider      00  1  4-0  1*512=512    001  "
        "4  6-2  4*512=2048    010  16  8-4  16*512=8192    011  64  10-6  64*5"
        "12=32768    100  256  12-8  256*512=131072    101  1024  16-10  1024*5"
        "12=524288       Preliminary Divider definition       DIV_RATIO value  "
        "ARM clk division ratio      000000  1    000001  2    000010  3    ...",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_dvfsc_cntr
    },
    {
        "DVFSLTR0_0",
        "",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        false, // Writable
        8, // Number of bitfields
        hw_dvfsc_dvfsltr0_0
    },
    {
        "DVFSLTR0_1",
        "",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        false, // Writable
        8, // Number of bitfields
        hw_dvfsc_dvfsltr0_1
    },
    {
        "DVFSLTR1_0",
        "",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        false, // Writable
        8, // Number of bitfields
        hw_dvfsc_dvfsltr1_0
    },
    {
        "DVFSLTR1_1",
        "",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        false, // Writable
        8, // Number of bitfields
        hw_dvfsc_dvfsltr1_1
    },
    {
        "DVFSPT0",
        "",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dvfsc_dvfspt0
    },
    {
        "DVFSPT1",
        "",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dvfsc_dvfspt1
    },
    {
        "DVFSPT2",
        "",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_dvfsc_dvfspt2
    },
    {
        "DVFSPT3",
        "",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_dvfsc_dvfspt3
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark ECSPI
#endif

// Bitfields in register ECSPI_RXDATA.
static const field_t hw_ecspi_rxdata[] =
{
    {
        "ECSPI_RXDATA",
        "Receive Data.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_TXDATA.
static const field_t hw_ecspi_txdata[] =
{
    {
        "ECSPI_TXDATA",
        "Transmit Data.",
        0, // LSB
        31, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_CONREG.
static const field_t hw_ecspi_conreg[] =
{
    {
        "EN",
        "SPI Block Enable Control.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HT",
        "Hardware Trigger Enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "XCH",
        "SPI Exchange Bit.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SMC",
        "Start Mode Control.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CHANNEL_MODE",
        "SPI CHANNEL MODE selects the mode for each SPI channel.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POST_DIVIDER",
        "SPI Post Divider.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRE_DIVIDER",
        "SPI Pre Divider.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DRCTL",
        "SPI Data Ready Control.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CHANNEL_SELECT",
        "SPI CHANNEL SELECT bits.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BURST_LENGTH",
        "Burst Length.",
        20, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_CONFIGREG.
static const field_t hw_ecspi_configreg[] =
{
    {
        "SCLK_PHA",
        "SPI Clock/Data Phase Control.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SCLK_POL",
        "SPI Clock Polarity Control.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SS_CTL",
        "SPI SS Wave Form Select.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SS_POL",
        "SPI SS Polarity Select.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_CTL",
        "DATA CTL.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SCLK_CTL",
        "SCLK CTL.",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HT_LENGTH",
        "HT LENGTH.",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_INTREG.
static const field_t hw_ecspi_intreg[] =
{
    {
        "TEEN",
        "TXFIFO Empty Interrupt enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TDREN",
        "TXFIFO Data Request Interrupt enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFEN",
        "TXFIFO Full Interrupt enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RREN",
        "RXFIFO Ready Interrupt enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RDREN",
        "RXFIFO Data Request Interrupt enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFEN",
        "RXFIFO Full Interrupt enable.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROEN",
        "RXFIFO Overflow Interrupt enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCEN",
        "Transfer Completed Interrupt enable.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_DMAREG.
static const field_t hw_ecspi_dmareg[] =
{
    {
        "TX_THRESHOLD",
        "TX THRESHOLD.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TEDEN",
        "TXFIFO Empty DMA Request Enable.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RX_THRESHOLD",
        "RX THRESHOLD.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDEN",
        "RXFIFO DMA Request Enable.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RX_DMA_LENGTH",
        "RX DMA LENGTH.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXTDEN",
        "RXFIFO TAIL DMA Request Enable.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_STATREG.
static const field_t hw_ecspi_statreg[] =
{
    {
        "TE",
        "TXFIFO Empty.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TDR",
        "TXFIFO Data Request.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TF",
        "TXFIFO Full.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RR",
        "RXFIFO Ready.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RDR",
        "RXFIFO Data Request.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RF",
        "RXFIFO Full.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RO",
        "RXFIFO Overflow.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TC",
        "Transfer Completed Status bit.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_PERIODREG.
static const field_t hw_ecspi_periodreg[] =
{
    {
        "SAMPLE_PERIOD",
        "Sample Period Control.",
        0, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSRC",
        "Clock Source Control.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSD_CTL",
        "Chip Select Delay Control bits.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_TESTREG.
static const field_t hw_ecspi_testreg[] =
{
    {
        "TXCNT",
        "TXFIFO Counter.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXCNT",
        "RXFIFO Counter.",
        8, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LBC",
        "Loop Back Control.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ECSPI_MSGDATA.
static const field_t hw_ecspi_msgdata[] =
{
    {
        "ECSPI_MSGDATA",
        "ECSPI_MSGDATA holds the top word of MSG Data FIFO.",
        0, // LSB
        31, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a ECSPI module.
static const reg_t hw_ecspi[] =
{
    {
        "RXDATA",
        "The Receive Data register (ECSPI_RXDATA) is a read-only register that "
        "forms the top word of the 64 x 32 receive FIFO.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_ecspi_rxdata
    },
    {
        "TXDATA",
        "The Transmit Data (ECSPI_TXDATA) register is a write-only data registe"
        "r that forms the bottom word of the 64 x 32 TXFIFO.",
        4, // Width in bytes
        0x00000004, // Base address offset
        false, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ecspi_txdata
    },
    {
        "CONREG",
        "The Control Register (ECSPI_CONREG) allows software to enable the ECSP"
        "I , configure its operating modes, specify the divider value, and SPI_"
        "RDY control signal, and define the transfer length.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_ecspi_conreg
    },
    {
        "CONFIGREG",
        "The Config Register (ECSPI_CONFIGREG) allows software to configure eac"
        "h SPI channel, configure its operating modes, specify the phase and po"
        "larity of the clock, configure the Chip Select (SS), and define the HT"
        " transfer length.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_ecspi_configreg
    },
    {
        "INTREG",
        "The Interrupt Control Register (ECSPI_INTREG) enables the generation o"
        "f interrupts to the host processor.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_ecspi_intreg
    },
    {
        "DMAREG",
        "The Direct Memory Access Control Register (ECSPI_DMAREG) provides soft"
        "ware a way to use an on-chip DMA controller for ECSPI data.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_ecspi_dmareg
    },
    {
        "STATREG",
        "The ECSPI Status Register (ECSPI_STATREG) reflects the status of the E"
        "CSPI's operating condition.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_ecspi_statreg
    },
    {
        "PERIODREG",
        "The Sample Period Control Register (ECSPI_PERIODREG) provides software"
        " a way to insert delays (wait states) between consecutive SPI transfer"
        "s.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_ecspi_periodreg
    },
    {
        "TESTREG",
        "The Test Control Register (ECSPI_TESTREG) provides software a mechanis"
        "m to internally connect the receive and transmit devices of the ECSPI "
        " , and monitor the contents of the receive and transmit FIFOs.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_ecspi_testreg
    },
    {
        "MSGDATA",
        "The Message Data Register (ECSPI_MSGDATA) forms the top word of the 16"
        " x 32 MSG Data FIFO.",
        4, // Width in bytes
        0x00000040, // Base address offset
        false, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ecspi_msgdata
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark EIM
#endif

// Bitfields in register EIM_CS0GCR1.
static const field_t hw_eim_cs0gcr1[] =
{
    {
        "CSEN",
        "CS Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Synchronous Write Data.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRD",
        "Synchronous Read Data.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUM",
        "Multiplexed Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WFL",
        "Write Fix Latency.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFL",
        "Read Fix Latency.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRE",
        "Configuration Register Enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CREP",
        "Configuration Register Enable Polarity.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BL",
        "Burst Length.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WC",
        "Write Continuous.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCD",
        "Burst Clock Divisor.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCS",
        "Burst Clock Start.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSZ",
        "Data Port Size.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SP",
        "Supervisor Protect.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSREC",
        "CS Recovery.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUS",
        "Address UnShifted.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GBC",
        "Gap Between Chip Selects.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WP",
        "Write Protect.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSZ",
        "Page Size.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS0GCR2.
static const field_t hw_eim_cs0gcr2[] =
{
    {
        "ADH",
        "Address hold time - This bit field determine the address hold time aft"
        "er ADV negation when mum = 1 (muxed mode).",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAPS",
        "Data Acknowledge Poling Start.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAE",
        "Data Acknowledge Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAP",
        "Data Acknowledge Polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUX16_BYP_GRANT",
        "Muxed 16 bypass grant.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS0RCR1.
static const field_t hw_eim_cs0rcr1[] =
{
    {
        "RCSN",
        "Read CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSA",
        "Read CS Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEN",
        "OE Negation.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEA",
        "OE Assertion.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVN",
        "ADV Negation.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RAL",
        "Read ADV Low.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVA",
        "ADV Assertion.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWSC",
        "Read Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS0RCR2.
static const field_t hw_eim_cs0rcr2[] =
{
    {
        "RBEN",
        "Read BE Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBE",
        "Read BE enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBEA",
        "Read BE Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Read Latency.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PAT",
        "Page Access Time.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "APR",
        "Asynchronous Page Read.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS0WCR1.
static const field_t hw_eim_cs0wcr1[] =
{
    {
        "WCSN",
        "Write CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WCSA",
        "Write CS Assertion.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEN",
        "WE Negation.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEA",
        "WE Assertion.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEN",
        "BE[3:0] Negation.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEA",
        "BE Assertion.",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVN",
        "ADV Negation.",
        18, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVA",
        "ADV Assertion.",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WWSC",
        "Write Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBED",
        "Write Byte Enable Disable.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAL",
        "Write ADV Low.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS0WCR2.
static const field_t hw_eim_cs0wcr2[] =
{
    {
        "WBCDD",
        "Write Burst Clock Divisor Decrement.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS1GCR1.
static const field_t hw_eim_cs1gcr1[] =
{
    {
        "CSEN",
        "CS Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Synchronous Write Data.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRD",
        "Synchronous Read Data.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUM",
        "Multiplexed Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WFL",
        "Write Fix Latency.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFL",
        "Read Fix Latency.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRE",
        "Configuration Register Enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CREP",
        "Configuration Register Enable Polarity.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BL",
        "Burst Length.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WC",
        "Write Continuous.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCD",
        "Burst Clock Divisor.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCS",
        "Burst Clock Start.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSZ",
        "Data Port Size.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SP",
        "Supervisor Protect.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSREC",
        "CS Recovery.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUS",
        "Address UnShifted.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GBC",
        "Gap Between Chip Selects.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WP",
        "Write Protect.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSZ",
        "Page Size.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS1GCR2.
static const field_t hw_eim_cs1gcr2[] =
{
    {
        "ADH",
        "Address hold time - This bit field determine the address hold time aft"
        "er ADV negation when mum = 1 (muxed mode).",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAPS",
        "Data Acknowledge Poling Start.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAE",
        "Data Acknowledge Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAP",
        "Data Acknowledge Polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUX16_BYP_GRANT",
        "Muxed 16 bypass grant.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS1RCR1.
static const field_t hw_eim_cs1rcr1[] =
{
    {
        "RCSN",
        "Read CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSA",
        "Read CS Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEN",
        "OE Negation.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEA",
        "OE Assertion.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVN",
        "ADV Negation.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RAL",
        "Read ADV Low.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVA",
        "ADV Assertion.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWSC",
        "Read Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS1RCR2.
static const field_t hw_eim_cs1rcr2[] =
{
    {
        "RBEN",
        "Read BE Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBE",
        "Read BE enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBEA",
        "Read BE Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Read Latency.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PAT",
        "Page Access Time.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "APR",
        "Asynchronous Page Read.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS1WCR1.
static const field_t hw_eim_cs1wcr1[] =
{
    {
        "WCSN",
        "Write CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WCSA",
        "Write CS Assertion.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEN",
        "WE Negation.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEA",
        "WE Assertion.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEN",
        "BE[3:0] Negation.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEA",
        "BE Assertion.",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVN",
        "ADV Negation.",
        18, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVA",
        "ADV Assertion.",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WWSC",
        "Write Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBED",
        "Write Byte Enable Disable.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAL",
        "Write ADV Low.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS1WCR2.
static const field_t hw_eim_cs1wcr2[] =
{
    {
        "WBCDD",
        "Write Burst Clock Divisor Decrement.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS2GCR1.
static const field_t hw_eim_cs2gcr1[] =
{
    {
        "CSEN",
        "CS Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Synchronous Write Data.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRD",
        "Synchronous Read Data.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUM",
        "Multiplexed Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WFL",
        "Write Fix Latency.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFL",
        "Read Fix Latency.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRE",
        "Configuration Register Enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CREP",
        "Configuration Register Enable Polarity.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BL",
        "Burst Length.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WC",
        "Write Continuous.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCD",
        "Burst Clock Divisor.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCS",
        "Burst Clock Start.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSZ",
        "Data Port Size.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SP",
        "Supervisor Protect.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSREC",
        "CS Recovery.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUS",
        "Address UnShifted.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GBC",
        "Gap Between Chip Selects.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WP",
        "Write Protect.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSZ",
        "Page Size.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS2GCR2.
static const field_t hw_eim_cs2gcr2[] =
{
    {
        "ADH",
        "Address hold time - This bit field determine the address hold time aft"
        "er ADV negation when mum = 1 (muxed mode).",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAPS",
        "Data Acknowledge Poling Start.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAE",
        "Data Acknowledge Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAP",
        "Data Acknowledge Polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUX16_BYP_GRANT",
        "Muxed 16 bypass grant.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS2RCR1.
static const field_t hw_eim_cs2rcr1[] =
{
    {
        "RCSN",
        "Read CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSA",
        "Read CS Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEN",
        "OE Negation.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEA",
        "OE Assertion.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVN",
        "ADV Negation.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RAL",
        "Read ADV Low.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVA",
        "ADV Assertion.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWSC",
        "Read Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS2RCR2.
static const field_t hw_eim_cs2rcr2[] =
{
    {
        "RBEN",
        "Read BE Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBE",
        "Read BE enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBEA",
        "Read BE Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Read Latency.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PAT",
        "Page Access Time.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "APR",
        "Asynchronous Page Read.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS2WCR1.
static const field_t hw_eim_cs2wcr1[] =
{
    {
        "WCSN",
        "Write CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WCSA",
        "Write CS Assertion.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEN",
        "WE Negation.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEA",
        "WE Assertion.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEN",
        "BE[3:0] Negation.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEA",
        "BE Assertion.",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVN",
        "ADV Negation.",
        18, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVA",
        "ADV Assertion.",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WWSC",
        "Write Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBED",
        "Write Byte Enable Disable.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAL",
        "Write ADV Low.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS2WCR2.
static const field_t hw_eim_cs2wcr2[] =
{
    {
        "WBCDD",
        "Write Burst Clock Divisor Decrement.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS3GCR1.
static const field_t hw_eim_cs3gcr1[] =
{
    {
        "CSEN",
        "CS Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Synchronous Write Data.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRD",
        "Synchronous Read Data.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUM",
        "Multiplexed Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WFL",
        "Write Fix Latency.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFL",
        "Read Fix Latency.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRE",
        "Configuration Register Enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CREP",
        "Configuration Register Enable Polarity.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BL",
        "Burst Length.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WC",
        "Write Continuous.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCD",
        "Burst Clock Divisor.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCS",
        "Burst Clock Start.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSZ",
        "Data Port Size.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SP",
        "Supervisor Protect.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSREC",
        "CS Recovery.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUS",
        "Address UnShifted.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GBC",
        "Gap Between Chip Selects.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WP",
        "Write Protect.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSZ",
        "Page Size.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS3GCR2.
static const field_t hw_eim_cs3gcr2[] =
{
    {
        "ADH",
        "Address hold time - This bit field determine the address hold time aft"
        "er ADV negation when mum = 1 (muxed mode).",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAPS",
        "Data Acknowledge Poling Start.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAE",
        "Data Acknowledge Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAP",
        "Data Acknowledge Polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUX16_BYP_GRANT",
        "Muxed 16 bypass grant.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS3RCR1.
static const field_t hw_eim_cs3rcr1[] =
{
    {
        "RCSN",
        "Read CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSA",
        "Read CS Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEN",
        "OE Negation.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEA",
        "OE Assertion.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVN",
        "ADV Negation.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RAL",
        "Read ADV Low.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVA",
        "ADV Assertion.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWSC",
        "Read Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS3RCR2.
static const field_t hw_eim_cs3rcr2[] =
{
    {
        "RBEN",
        "Read BE Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBE",
        "Read BE enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBEA",
        "Read BE Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Read Latency.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PAT",
        "Page Access Time.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "APR",
        "Asynchronous Page Read.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS3WCR1.
static const field_t hw_eim_cs3wcr1[] =
{
    {
        "WCSN",
        "Write CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WCSA",
        "Write CS Assertion.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEN",
        "WE Negation.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEA",
        "WE Assertion.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEN",
        "BE[3:0] Negation.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEA",
        "BE Assertion.",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVN",
        "ADV Negation.",
        18, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVA",
        "ADV Assertion.",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WWSC",
        "Write Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBED",
        "Write Byte Enable Disable.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAL",
        "Write ADV Low.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS3WCR2.
static const field_t hw_eim_cs3wcr2[] =
{
    {
        "WBCDD",
        "Write Burst Clock Divisor Decrement.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS4GCR1.
static const field_t hw_eim_cs4gcr1[] =
{
    {
        "CSEN",
        "CS Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Synchronous Write Data.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRD",
        "Synchronous Read Data.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUM",
        "Multiplexed Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WFL",
        "Write Fix Latency.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFL",
        "Read Fix Latency.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRE",
        "Configuration Register Enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CREP",
        "Configuration Register Enable Polarity.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BL",
        "Burst Length.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WC",
        "Write Continuous.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCD",
        "Burst Clock Divisor.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCS",
        "Burst Clock Start.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSZ",
        "Data Port Size.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SP",
        "Supervisor Protect.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSREC",
        "CS Recovery.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUS",
        "Address UnShifted.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GBC",
        "Gap Between Chip Selects.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WP",
        "Write Protect.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSZ",
        "Page Size.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS4GCR2.
static const field_t hw_eim_cs4gcr2[] =
{
    {
        "ADH",
        "Address hold time - This bit field determine the address hold time aft"
        "er ADV negation when mum = 1 (muxed mode).",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAPS",
        "Data Acknowledge Poling Start.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAE",
        "Data Acknowledge Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAP",
        "Data Acknowledge Polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUX16_BYP_GRANT",
        "Muxed 16 bypass grant.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS4RCR1.
static const field_t hw_eim_cs4rcr1[] =
{
    {
        "RCSN",
        "Read CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSA",
        "Read CS Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEN",
        "OE Negation.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEA",
        "OE Assertion.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVN",
        "ADV Negation.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RAL",
        "Read ADV Low.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVA",
        "ADV Assertion.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWSC",
        "Read Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS4RCR2.
static const field_t hw_eim_cs4rcr2[] =
{
    {
        "RBEN",
        "Read BE Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBE",
        "Read BE enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBEA",
        "Read BE Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Read Latency.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PAT",
        "Page Access Time.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "APR",
        "Asynchronous Page Read.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS4WCR1.
static const field_t hw_eim_cs4wcr1[] =
{
    {
        "WCSN",
        "Write CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WCSA",
        "Write CS Assertion.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEN",
        "WE Negation.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEA",
        "WE Assertion.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEN",
        "BE[3:0] Negation.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEA",
        "BE Assertion.",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVN",
        "ADV Negation.",
        18, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVA",
        "ADV Assertion.",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WWSC",
        "Write Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBED",
        "Write Byte Enable Disable.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAL",
        "Write ADV Low.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS4WCR2.
static const field_t hw_eim_cs4wcr2[] =
{
    {
        "WBCDD",
        "Write Burst Clock Divisor Decrement.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS5GCR1.
static const field_t hw_eim_cs5gcr1[] =
{
    {
        "CSEN",
        "CS Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Synchronous Write Data.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRD",
        "Synchronous Read Data.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUM",
        "Multiplexed Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WFL",
        "Write Fix Latency.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFL",
        "Read Fix Latency.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRE",
        "Configuration Register Enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CREP",
        "Configuration Register Enable Polarity.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BL",
        "Burst Length.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WC",
        "Write Continuous.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCD",
        "Burst Clock Divisor.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCS",
        "Burst Clock Start.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSZ",
        "Data Port Size.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SP",
        "Supervisor Protect.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSREC",
        "CS Recovery.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUS",
        "Address UnShifted.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GBC",
        "Gap Between Chip Selects.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WP",
        "Write Protect.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSZ",
        "Page Size.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS5GCR2.
static const field_t hw_eim_cs5gcr2[] =
{
    {
        "ADH",
        "Address hold time - This bit field determine the address hold time aft"
        "er ADV negation when mum = 1 (muxed mode).",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAPS",
        "Data Acknowledge Poling Start.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAE",
        "Data Acknowledge Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAP",
        "Data Acknowledge Polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUX16_BYP_GRANT",
        "Muxed 16 bypass grant.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS5RCR1.
static const field_t hw_eim_cs5rcr1[] =
{
    {
        "RCSN",
        "Read CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCSA",
        "Read CS Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEN",
        "OE Negation.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OEA",
        "OE Assertion.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVN",
        "ADV Negation.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RAL",
        "Read ADV Low.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RADVA",
        "ADV Assertion.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWSC",
        "Read Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS5RCR2.
static const field_t hw_eim_cs5rcr2[] =
{
    {
        "RBEN",
        "Read BE Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBE",
        "Read BE enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RBEA",
        "Read BE Assertion.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Read Latency.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PAT",
        "Page Access Time.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "APR",
        "Asynchronous Page Read.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS5WCR1.
static const field_t hw_eim_cs5wcr1[] =
{
    {
        "WCSN",
        "Write CS Negation.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WCSA",
        "Write CS Assertion.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEN",
        "WE Negation.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WEA",
        "WE Assertion.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEN",
        "BE[3:0] Negation.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBEA",
        "BE Assertion.",
        15, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVN",
        "ADV Negation.",
        18, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WADVA",
        "ADV Assertion.",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WWSC",
        "Write Wait State Control.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBED",
        "Write Byte Enable Disable.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAL",
        "Write ADV Low.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_CS5WCR2.
static const field_t hw_eim_cs5wcr2[] =
{
    {
        "WBCDD",
        "Write Burst Clock Divisor Decrement.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_WCR.
static const field_t hw_eim_wcr[] =
{
    {
        "BCM",
        "Burst Clock Mode.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GBCD",
        "General Burst Clock Divisor.",
        1, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CONT_BCLK_SEL",
        "When this bit is set BCLK pin output continuous clock.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INTEN",
        "Interrupt Enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INTPOL",
        "Interrupt Polarity.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDOG_EN",
        "Memory WDog enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDOG_LIMIT",
        "Memory Watch Dog (WDog) cycle limit.",
        9, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRUN_ACLK_EN",
        "",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_DCR.
static const field_t hw_eim_dcr[] =
{
    {
        "DLL_CTRL_ENABLE",
        "",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_RESET",
        "",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_FORCE_UPD",
        "",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_OFFSET_DEC",
        "",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_OFFSET",
        "",
        4, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DLL_CTRL_GATE_UPDATE",
        "",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_OVERRIDE",
        "Use override value in DLL_CTRL_SLV_OVERRIDE_VAL bit field instead of D"
        "LL locked value as delay.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_OVERRIDE_VAL",
        "",
        9, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_REF_INITIAL_VAL",
        "",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_UPDATE_INT",
        "Update interval of slave DLL.",
        23, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_REF_UPDATE_INT",
        "Update interval of reference DLL.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_DSR.
static const field_t hw_eim_dsr[] =
{
    {
        "DLL_STS_SLV_LOCK",
        "",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_STS_REF_LOCK",
        "",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_STS_SLV_SEL",
        "",
        2, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_STS_REF_SEL",
        "",
        9, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_WIAR.
static const field_t hw_eim_wiar[] =
{
    {
        "IPS_REQ",
        "IPS request.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IPS_ACK",
        "IPS ACK.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INT",
        "Interrupt.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ERRST",
        "READY After Reset.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACLK_EN",
        "ACLK enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EIM_EAR.
static const field_t hw_eim_ear[] =
{
    {
        "ERROR_ADDR",
        "Error Address.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a EIM module.
static const reg_t hw_eim[] =
{
    {
        "CS0GCR1",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        19, // Number of bitfields
        hw_eim_cs0gcr1
    },
    {
        "CS0GCR2",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_eim_cs0gcr2
    },
    {
        "CS0RCR1",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_eim_cs0rcr1
    },
    {
        "CS0RCR2",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_eim_cs0rcr2
    },
    {
        "CS0WCR1",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_eim_cs0wcr1
    },
    {
        "CS0WCR2",
        "",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_eim_cs0wcr2
    },
    {
        "CS1GCR1",
        "",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        19, // Number of bitfields
        hw_eim_cs1gcr1
    },
    {
        "CS1GCR2",
        "",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_eim_cs1gcr2
    },
    {
        "CS1RCR1",
        "",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_eim_cs1rcr1
    },
    {
        "CS1RCR2",
        "",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_eim_cs1rcr2
    },
    {
        "CS1WCR1",
        "",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_eim_cs1wcr1
    },
    {
        "CS1WCR2",
        "",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_eim_cs1wcr2
    },
    {
        "CS2GCR1",
        "",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        19, // Number of bitfields
        hw_eim_cs2gcr1
    },
    {
        "CS2GCR2",
        "",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_eim_cs2gcr2
    },
    {
        "CS2RCR1",
        "",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_eim_cs2rcr1
    },
    {
        "CS2RCR2",
        "",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_eim_cs2rcr2
    },
    {
        "CS2WCR1",
        "",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_eim_cs2wcr1
    },
    {
        "CS2WCR2",
        "",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_eim_cs2wcr2
    },
    {
        "CS3GCR1",
        "",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        19, // Number of bitfields
        hw_eim_cs3gcr1
    },
    {
        "CS3GCR2",
        "",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_eim_cs3gcr2
    },
    {
        "CS3RCR1",
        "",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_eim_cs3rcr1
    },
    {
        "CS3RCR2",
        "",
        4, // Width in bytes
        0x00000054, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_eim_cs3rcr2
    },
    {
        "CS3WCR1",
        "",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_eim_cs3wcr1
    },
    {
        "CS3WCR2",
        "",
        4, // Width in bytes
        0x0000005c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_eim_cs3wcr2
    },
    {
        "CS4GCR1",
        "",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        19, // Number of bitfields
        hw_eim_cs4gcr1
    },
    {
        "CS4GCR2",
        "",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_eim_cs4gcr2
    },
    {
        "CS4RCR1",
        "",
        4, // Width in bytes
        0x00000068, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_eim_cs4rcr1
    },
    {
        "CS4RCR2",
        "",
        4, // Width in bytes
        0x0000006c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_eim_cs4rcr2
    },
    {
        "CS4WCR1",
        "",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_eim_cs4wcr1
    },
    {
        "CS4WCR2",
        "",
        4, // Width in bytes
        0x00000074, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_eim_cs4wcr2
    },
    {
        "CS5GCR1",
        "",
        4, // Width in bytes
        0x00000078, // Base address offset
        true, // Readable
        true, // Writable
        19, // Number of bitfields
        hw_eim_cs5gcr1
    },
    {
        "CS5GCR2",
        "",
        4, // Width in bytes
        0x0000007c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_eim_cs5gcr2
    },
    {
        "CS5RCR1",
        "",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_eim_cs5rcr1
    },
    {
        "CS5RCR2",
        "",
        4, // Width in bytes
        0x00000084, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_eim_cs5rcr2
    },
    {
        "CS5WCR1",
        "",
        4, // Width in bytes
        0x00000088, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_eim_cs5wcr1
    },
    {
        "CS5WCR2",
        "",
        4, // Width in bytes
        0x0000008c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_eim_cs5wcr2
    },
    {
        "WCR",
        "",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_eim_wcr
    },
    {
        "DCR",
        "",
        4, // Width in bytes
        0x00000094, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_eim_dcr
    },
    {
        "DSR",
        "",
        4, // Width in bytes
        0x00000098, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_eim_dsr
    },
    {
        "WIAR",
        "",
        4, // Width in bytes
        0x0000009c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_eim_wiar
    },
    {
        "EAR",
        "",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_eim_ear
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark LCDIF
#endif

// Bitfields in register LCDIF_CTRL.
static const field_t hw_lcdif_ctrl[] =
{
    {
        "RUN",
        "When this bit is set by software, the eLCDIF will begin transferring d"
        "ata between the SoC and the display.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_FORMAT_24_BIT",
        "Used only when WORD_LENGTH = 3, i.e.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_FORMAT_18_BIT",
        "Used only when WORD_LENGTH = 2, i.e.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_FORMAT_16_BIT",
        "When this bit is 1 and WORD_LENGTH = 0, it implies that the the 16-bit"
        " data is in ARGB555 format.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASTER",
        "Set this bit to make the eLCDIF act as a bus master.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_PXP_HANDSHAKE",
        "If this bit is set and LCDIF_MASTER bit is set, the eLCDIF will act as"
        " bus master and the handshake mechanism between eLCDIF and ePXP will b"
        "e turned on.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RGB_TO_YCBCR422_CSC",
        "Set this bit to 1 to enable conversion from RGB to YCbCr colorspace.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WORD_LENGTH",
        "Input data format.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCD_DATABUS_WIDTH",
        "LCD Data bus transfer width.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSC_DATA_SWIZZLE",
        "This field specifies how to swap the bytes after the data has been con"
        "verted into an internal representation of 24 bits per pixel and before"
        " it is transmitted over the LCD interface bus.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INPUT_DATA_SWIZZLE",
        "This field specifies how to swap the bytes fetched by the bus master i"
        "nterface.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_SELECT",
        "Command Mode polarity bit.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOTCLK_MODE",
        "Set this bit to 1 to make the hardware go into the DOTCLK mode, i.e.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_MODE",
        "Setting this bit to 1 will make the eLCDIF hardware go into VSYNC mode"
        ".",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS_COUNT",
        "When this bit is 0, it means that eLCDIF will stop the block operation"
        " and turn off the RUN bit after the amount of data indicated by the LC"
        "DIF_TRANSFER_COUNT register has been transferred out.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DVI_MODE",
        "Set this bit to 1 to get into theITU-R BT.656 digital video interface "
        "mode.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SHIFT_NUM_BITS",
        "The data to be transmitted is shifted left or right by this number of "
        "bits.",
        21, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_SHIFT_DIR",
        "Use this bit to determine the direction of shift of transmit data.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAIT_FOR_VSYNC_EDGE",
        "Setting this bit to 1 will make the hardware wait for the triggering V"
        "SYNC edge before starting write transfers to the LCD.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "READ_WRITEB",
        "By default, eLCDIF is in the write mode.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "YCBCR422_INPUT",
        "Zero implies input data is in RGB color space.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE",
        "This bit must be set to zero for normal operation.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFTRST",
        "This bit must be set to zero to enable normal operation of the eLCDIF.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CTRL1.
static const field_t hw_lcdif_ctrl1[] =
{
    {
        "RESET",
        "Reset bit for the external LCD controller.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MODE86",
        "This bit is used to select between the 8080 and 6800 series of micropr"
        "ocessor modes.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BUSY_ENABLE",
        "This bit enables the use of the interface's busy signal input.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_EDGE_IRQ",
        "This bit is set to indicate that an interrupt is requested by the eLCD"
        "IF block.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CUR_FRAME_DONE_IRQ",
        "This bit is set to indicate that an interrupt is requested by the eLCD"
        "IF block.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNDERFLOW_IRQ",
        "This bit is set to indicate that an interrupt is requested by the eLCD"
        "IF block.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OVERFLOW_IRQ",
        "This bit is set to indicate that an interrupt is requested by the eLCD"
        "IF block.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_EDGE_IRQ_EN",
        "This bit is set to enable an interrupt every time the hardware encount"
        "ers the leading VSYNC edge in the VSYNC and DOTCLK modes, or the begin"
        "ning of every field in DVI mode.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CUR_FRAME_DONE_IRQ_EN",
        "This bit is set to 1 enable an interrupt every time the hardware enter"
        "s in the vertical blanking state.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNDERFLOW_IRQ_EN",
        "This bit is set to enable an underflow interrupt in the TXFIFO in the "
        "write mode.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OVERFLOW_IRQ_EN",
        "This bit is set to enable an overflow interrupt in the TXFIFO in the w"
        "rite mode.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYTE_PACKING_FORMAT",
        "This bitfield is used to show which data bytes in a 32-bit word are va"
        "lid.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRQ_ON_ALTERNATE_FIELDS",
        "If this bit is set, the eLCDIF block will assert the cur_frame_done in"
        "terrupt only on alternate fields, otherwise it will issue the interrup"
        "t on both odd and even field.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FIFO_CLEAR",
        "Set this bit to clear all the data in the latency FIFO (LFIFO), TXFIFO"
        " and the RXFIFO.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "START_INTERLACE_FROM_SECOND_FIELD",
        "The default is to grab the odd lines first and then the even lines.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INTERLACE_FIELDS",
        "Set this bit if it is required that the eLCDIF block fetches odd lines"
        " in one field and even lines in the other field.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RECOVER_ON_UNDERFLOW",
        "Set this bit to enable the eLCDIF block to recover in the next field/f"
        "rame if there was an underflow in the current field/frame.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BM_ERROR_IRQ",
        "This bit is set to indicate that an interrupt is requested by the eLCD"
        "IF block.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BM_ERROR_IRQ_EN",
        "This bit is set to enable bus master error interrupt in the eLCDIF mas"
        "ter mode.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COMBINE_MPU_WR_STRB",
        "If this bit is not set, the write strobe will be driven on LCD_WR_RWn "
        "pin in the 8080 mode and on the LCD_RD_E pin in the 6800 mode.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CTRL2.
static const field_t hw_lcdif_ctrl2[] =
{
    {
        "INITIAL_DUMMY_READ",
        "The value in this field determines the number of dummy 8/16/18/24-bit "
        "subwords that have to be read back from the LCD panel/controller.",
        1, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "READ_MODE_NUM_PACKED_SUBWORDS",
        "Indicates the number of valid 8/16/18/24-bit subwords that will be pac"
        "ked into the 32-bit word in read mode.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "READ_MODE_6_BIT_INPUT",
        "Setting this bit to 1 indicates to eLCDIF that even though LCD_DATABUS"
        "_WIDTH is set to 8 bits, the input data is actually only 6 bits wide a"
        "nd exists on D5-D0.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "READ_MODE_OUTPUT_IN_RGB_FORMAT",
        "Setting this bit will enable the eLCDIF to convert the incoming data t"
        "o the RGB format given by WORD_LENGTH bitfield.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "READ_PACK_DIR",
        "The default value of 0 indicates data is stored in the little endian f"
        "ormat.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EVEN_LINE_PATTERN",
        "This field determines the order of the RGB components of each pixel in"
        " EVEN lines (line numbers 2,4,6,..).",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODD_LINE_PATTERN",
        "This field determines the order of the RGB components of each pixel in"
        " ODD lines (line numbers 1,3,5,..).",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BURST_LEN_8",
        "By default, when the eLCDIF is in the bus master mode, it will issue A"
        "XI bursts of length 16 (except when in packed 24 bpp mode, it will iss"
        "ue bursts of length 15).",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OUTSTANDING_REQS",
        "This bitfield indicates the maximum number of outstanding transactions"
        " that eLCDIF should request when it is acting as a bus master.",
        21, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_TRANSFER_COUNT.
static const field_t hw_lcdif_transfer_count[] =
{
    {
        "H_COUNT",
        "Total valid data (pixels) in each horizontal line.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "V_COUNT",
        "Number of horizontal lines per frame which contain valid data.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CUR_BUF.
static const field_t hw_lcdif_cur_buf[] =
{
    {
        "ADDR",
        "-",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_NEXT_BUF.
static const field_t hw_lcdif_next_buf[] =
{
    {
        "ADDR",
        "-",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_TIMING.
static const field_t hw_lcdif_timing[] =
{
    {
        "DATA_SETUP",
        "Data bus setup time in CLK_DIS_LCDIFn cycles.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_HOLD",
        "Data bus hold time in CLK_DIS_LCDIFn cycles.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMD_SETUP",
        "Number of CLK_DIS_LCDIFn cycles that the the DCn signal is active befo"
        "re CEn is asserted.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMD_HOLD",
        "Number of CLK_DIS_LCDIFn cycles that the DCn signal is active after CE"
        "n is deasserted.",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_VDCTRL0.
static const field_t hw_lcdif_vdctrl0[] =
{
    {
        "VSYNC_PULSE_WIDTH",
        "Number of units for which VSYNC signal is active.",
        0, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_LINE_MODE",
        "When this bit is 0, the first field (VSYNC period) will end in half a "
        "horizontal line and the second field will begin with half a horizontal"
        " line.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HALF_LINE",
        "Setting this bit to 1 will make the total VSYNC period equal to the VS"
        "YNC_PERIOD field plus half the HORIZONTAL_PERIOD field (i.e.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_PULSE_WIDTH_UNIT",
        "Default 0 for counting VSYNC_PULSE_WIDTH in terms of CLK_DIS_LCDIFn cy"
        "cles.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_PERIOD_UNIT",
        "Default 0 for counting VSYNC_PERIOD in terms of CLK_DIS_LCDIFn cycles.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_POL",
        "Default 0 active low during valid data transfer on each horizontal lin"
        "e.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOTCLK_POL",
        "Default is data launched at negative edge of DOTCLK and captured at po"
        "sitive edge.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSYNC_POL",
        "Default 0 active low during HSYNC_PULSE_WIDTH time and will be high du"
        "ring the rest of the HSYNC period.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_POL",
        "Default 0 active low during VSYNC_PULSE_WIDTH time and will be high du"
        "ring the rest of the VSYNC period.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_PRESENT",
        "Setting this bit to 1 will make the hardware generate the ENABLE signa"
        "l in the DOTCLK mode, thereby making it the true RGB interface along w"
        "ith the remaining three signals VSYNC, HSYNC and DOTCLK.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_OEB",
        "0 means the VSYNC signal is an output, 1 means it is an input.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_VDCTRL1.
static const field_t hw_lcdif_vdctrl1[] =
{
    {
        "VSYNC_PERIOD",
        "Total number of units between two positive or two negative edges of th"
        "e VSYNC signal.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_VDCTRL2.
static const field_t hw_lcdif_vdctrl2[] =
{
    {
        "HSYNC_PERIOD",
        "Total number of CLK_DIS_LCDIFn cycles between two positive or two nega"
        "tive edges of the HSYNC signal.",
        0, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSYNC_PULSE_WIDTH",
        "Number of CLK_DIS_LCDIFn cycles for which HSYNC signal is active.",
        18, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_VDCTRL3.
static const field_t hw_lcdif_vdctrl3[] =
{
    {
        "VERTICAL_WAIT_CNT",
        "In the VSYNC interface mode, wait for this number of CLK_DIS_LCDIFn cy"
        "cles from the falling VSYNC edge (or rising if VSYNC_POL is 1) before "
        "starting LCD transactions and is applicable only if WAIT_FOR_VSYNC_EDG"
        "E is set.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HORIZONTAL_WAIT_CNT",
        "In the DOTCLK mode, wait for this number of clocks from falling edge ("
        "or rising if HSYNC_POL is 1) of HSYNC signal to account for horizontal"
        " back porch plus the number of DOTCLKs before the moving picture infor"
        "mation begins.",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSYNC_ONLY",
        "This bit must be set to 1 in the VSYNC mode of operation, and 0 in the"
        " DOTCLK mode of operation.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MUX_SYNC_SIGNALS",
        "When this bit is set, the eLCDIF block will internally mux HSYNC with "
        "LCD_D14, DOTCLK with LCD_D13 and ENABLE with LCD_D12, otherwise these "
        "signals will go out on separate pins.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_VDCTRL4.
static const field_t hw_lcdif_vdctrl4[] =
{
    {
        "DOTCLK_H_VALID_DATA_CNT",
        "Total number of CLK_DIS_LCDIFn cycles on each horizontal line that car"
        "ry valid data in DOTCLK mode.",
        0, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SYNC_SIGNALS_ON",
        "Set this field to 1 if the LCD controller requires that the VSYNC or V"
        "SYNC/HSYNC/DOTCLK control signals should be active atleast one frame b"
        "efore the data transfers actually start and remain active atleast one "
        "frame after the data transfers end.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOTCLK_DLY_SEL",
        "This bitfield selects the amount of time by which the DOTCLK signal sh"
        "ould be delayed before coming out of the LCD_DOTCK pin.",
        29, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DVICTRL0.
static const field_t hw_lcdif_dvictrl0[] =
{
    {
        "H_BLANKING_CNT",
        "Number of blanking samples to be inserted between EAV and SAV during h"
        "orizontal blanking interval.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "H_ACTIVE_CNT",
        "Number of active video samples to be transmitted.",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DVICTRL1.
static const field_t hw_lcdif_dvictrl1[] =
{
    {
        "F2_START_LINE",
        "Vertical line number from which Field 2 begins.",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "F1_END_LINE",
        "Vertical line number at which Field1 ends.",
        10, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "F1_START_LINE",
        "Vertical line number from which Field 1 begins.",
        20, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DVICTRL2.
static const field_t hw_lcdif_dvictrl2[] =
{
    {
        "V1_BLANK_END_LINE",
        "Vertical line number in the beginning part of Field2 where first Verti"
        "cal Blanking interval ends.",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "V1_BLANK_START_LINE",
        "Vertical line number towards the end of Field1 where first Vertical Bl"
        "anking interval starts.",
        10, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "F2_END_LINE",
        "Vertical line number at which Field 2 ends.",
        20, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DVICTRL3.
static const field_t hw_lcdif_dvictrl3[] =
{
    {
        "V_LINES_CNT",
        "Total number of vertical lines per frame (generally 525 or 625)",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "V2_BLANK_END_LINE",
        "Vertical line number in the beginning part of Field1 where second Vert"
        "ical Blanking interval ends.",
        10, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "V2_BLANK_START_LINE",
        "Vertical line number towards the end of Field2 where second Vertical B"
        "lanking interval starts.",
        20, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DVICTRL4.
static const field_t hw_lcdif_dvictrl4[] =
{
    {
        "H_FILL_CNT",
        "Number of active video samples that have to be filled with the filler "
        "data in the front and back portions of the active horizontal interval.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CR_FILL_VALUE",
        "Value of CR component of filler data.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CB_FILL_VALUE",
        "Value of CB component of filler data",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "Y_FILL_VALUE",
        "Value of Y component of filler data",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CSC_COEFF0.
static const field_t hw_lcdif_csc_coeff0[] =
{
    {
        "CSC_SUBSAMPLE_FILTER",
        "This register describes the filtering and subsampling scheme to be per"
        "formed on the chroma components in order to convert from YCbCr 4:4:4 t"
        "o YCbCr 4:2:2 space.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C0",
        "Two's complement red multiplier coefficient for Y",
        16, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CSC_COEFF1.
static const field_t hw_lcdif_csc_coeff1[] =
{
    {
        "C1",
        "Two's complement green multiplier coefficient for Y",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C2",
        "Two's complement blue multiplier coefficient for Y",
        16, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CSC_COEFF2.
static const field_t hw_lcdif_csc_coeff2[] =
{
    {
        "C3",
        "Two's complement red multiplier coefficient for Cb",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C4",
        "Two's complement green multiplier coefficient for Cb",
        16, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CSC_COEFF3.
static const field_t hw_lcdif_csc_coeff3[] =
{
    {
        "C5",
        "Two's complement blue multiplier coefficient for Cb",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C6",
        "Two's complement red multiplier coefficient for Cr",
        16, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CSC_COEFF4.
static const field_t hw_lcdif_csc_coeff4[] =
{
    {
        "C7",
        "Two's complement green multiplier coefficient for Cr",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C8",
        "Two's complement blue multiplier coefficient for Cr",
        16, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CSC_OFFSET.
static const field_t hw_lcdif_csc_offset[] =
{
    {
        "Y_OFFSET",
        "Two's complement offset for the Y component",
        0, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CBCR_OFFSET",
        "Two's complement offset for the Cb and Cr components",
        16, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CSC_LIMIT.
static const field_t hw_lcdif_csc_limit[] =
{
    {
        "Y_MAX",
        "Upper limit of Y after RGB to 4:2:2 YCbCr conversion",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "Y_MIN",
        "Lower limit of Y after RGB to 4:2:2 YCbCr conversion",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CBCR_MAX",
        "Upper limit of Cb and Cr after RGB to 4:2:2 YCbCr conversion",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CBCR_MIN",
        "Lower limit of Cb and Cr after RGB to 4:2:2 YCbCr conversion",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DATA.
static const field_t hw_lcdif_data[] =
{
    {
        "DATA_ZERO",
        "Byte 0 (least significant byte) of data written to eLCDIF.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_ONE",
        "Byte 1 of data written to eLCDIF.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_TWO",
        "Byte 2 of data written to eLCDIF.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_THREE",
        "Byte 3 (most significant byte) of data written to LCDIF.",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_BM_ERROR_STAT.
static const field_t hw_lcdif_bm_error_stat[] =
{
    {
        "ADDR",
        "Virtual address at which bus master error occurred.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_CRC_STAT.
static const field_t hw_lcdif_crc_stat[] =
{
    {
        "CRC_VALUE",
        "Calculated CRC value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_STAT.
static const field_t hw_lcdif_stat[] =
{
    {
        "LFIFO_COUNT",
        "Read only view of the current count in Latency buffer (LFIFO).",
        0, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DVI_CURRENT_FIELD",
        "Read only view of the current field being transmitted.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BUSY",
        "Read only view of the input busy signal from the external LCD controll"
        "er.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXFIFO_EMPTY",
        "Read only view of the signal that indicates that LCD write dapatath FI"
        "FO is empty, will be generally used in the read mode of the LCD interf"
        "ace.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXFIFO_FULL",
        "Read only view of the signal that indicates that LCD write datapath FI"
        "FO is full, will be generally used in the write mode of the LCD interf"
        "ace.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LFIFO_EMPTY",
        "Read only view of the signal that indicates that LCD read dapatath FIF"
        "O is empty, will be generally used in the read mode of the LCD interfa"
        "ce.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LFIFO_FULL",
        "Read only view of the signal that indicates that LCD read datapath FIF"
        "O is full, will be generally used in the write mode of the LCD interfa"
        "ce.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PRESENT",
        "0: eLCDIF not present on this product  1: eLCDIF is present.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_VERSION.
static const field_t hw_lcdif_version[] =
{
    {
        "STEP",
        "Fixed read-only value reflecting the stepping of RTL version.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MINOR",
        "Fixed read-only value reflecting the MINOR field of RTL version.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-only value reflecting the MAJOR field of RTL version.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DEBUG0.
static const field_t hw_lcdif_debug0[] =
{
    {
        "MST_WORDS",
        "Read only view of the current bursts issued by the AXI bus master.",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MST_OUTSTANDING_REQS",
        "Read only view of the current outstanding requests issued by the AXI b"
        "us master.",
        4, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MST_AVALID",
        "Read only view of the mst_avalid signal issued by the AXI bus master.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CUR_REQ_STATE",
        "Read only view of the request state machine.",
        10, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PXP_B1_DONE",
        "Buffer1 done signal issued by eLCDIF.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PXP_LCDIF_B1_READY",
        "Buffer1 ready signal issued by ePXP.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PXP_B0_DONE",
        "Buffer0 done signal issued by eLCDIF.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PXP_LCDIF_B0_READY",
        "Buffer0 ready signal issued by ePXP.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CUR_STATE",
        "Read only view of the current state machine state in the current mode "
        "of operation.",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EMPTY_WORD",
        "Indicates that the current word is empty.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CUR_FRAME_TX",
        "This bit is 1 for the time the current frame is being transmitted in t"
        "he VSYNC mode.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VSYNC",
        "Read only view of VSYNC signal.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HSYNC",
        "Read only view of HSYNC signal.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENABLE",
        "Read only view of ENABLE signal.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SYNC_SIGNALS_ON_REG",
        "Read only view of internal sync_signals_on_reg signal.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WAIT_FOR_VSYNC_EDGE_OUT",
        "Read only view of WAIT_FOR_VSYNC_EDGE bit in the VSYNC mode after it c"
        "omes out of the TXFIFO.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "STREAMING_END_DETECTED",
        "Read only view of the DOTCLK_MODE or DVI_MODE bit going from 1 to 0.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DEBUG1.
static const field_t hw_lcdif_debug1[] =
{
    {
        "V_DATA_COUNT",
        "Read only view of the current state of the vertical data counter.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "H_DATA_COUNT",
        "Read only view of the current state of the horizontal data counter.",
        16, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_DEBUG2.
static const field_t hw_lcdif_debug2[] =
{
    {
        "MST_ADDRESS",
        "Read only view of the current address issued by the AXI bus master.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register LCDIF_THRES.
static const field_t hw_lcdif_thres[] =
{
    {
        "PANIC",
        "This value should be set to a value of pixels, from 0 to 511.",
        0, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FASTCLOCK",
        "This value should be set to a value of pixels, from 0 to 511.",
        16, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a LCDIF module.
static const reg_t hw_lcdif[] =
{
    {
        "CTRL",
        "The LCD Interface Control Register provides overall control of the eLC"
        "DIF block.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        23, // Number of bitfields
        hw_lcdif_ctrl
    },
    {
        "CTRL1",
        "The eLCDIF Control Register provides overall control of the eLCDIF blo"
        "ck.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        20, // Number of bitfields
        hw_lcdif_ctrl1
    },
    {
        "CTRL2",
        "The eLCDIF Control Register provides overall control of the eLCDIF blo"
        "ck.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_lcdif_ctrl2
    },
    {
        "TRANSFER_COUNT",
        "This register tells the eLCDIF how much data will be sent for this fra"
        "me, or transaction.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_transfer_count
    },
    {
        "CUR_BUF",
        "This register indicates the address of the current frame being transmi"
        "tted by eLCDIF.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_lcdif_cur_buf
    },
    {
        "NEXT_BUF",
        "This register indicates the address of next frame that will be transmi"
        "tted by eLCDIF.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_lcdif_next_buf
    },
    {
        "TIMING",
        "The LCD interface timing register controls the various setup and hold "
        "times enforced by the LCD interface in the 6800/8080 MPU and VSYNC mod"
        "es of operation.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_lcdif_timing
    },
    {
        "VDCTRL0",
        "This register is used to control the VSYNC and DOTCLK modes of the LCD"
        "IF so as to work with different types of LCDs like moving picture disp"
        "lays and delta pixel displays.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_lcdif_vdctrl0
    },
    {
        "VDCTRL1",
        "This register is used to control the VSYNC signal in the VSYNC and DOT"
        "CLK modes of the block.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_lcdif_vdctrl1
    },
    {
        "VDCTRL2",
        "This register is used to control the HSYNC signal in the DOTCLK mode o"
        "f the block.",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_vdctrl2
    },
    {
        "VDCTRL3",
        "This register is used to determine the vertical and horizontal wait co"
        "unts.",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_lcdif_vdctrl3
    },
    {
        "VDCTRL4",
        "This register is used to control the DOTCLK mode of the block.",
        4, // Width in bytes
        0x000000b0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_lcdif_vdctrl4
    },
    {
        "DVICTRL0",
        "The Digital Video interface Control0 register provides the overall con"
        "trol of the Digital Video interface.",
        4, // Width in bytes
        0x000000c0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_dvictrl0
    },
    {
        "DVICTRL1",
        "The Digital Video interface Control1 register provides the overall con"
        "trol of the Digital Video interface.",
        4, // Width in bytes
        0x000000d0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_lcdif_dvictrl1
    },
    {
        "DVICTRL2",
        "The Digital Video interface Control2 register provides the overall con"
        "trol of the Digital Video interface.",
        4, // Width in bytes
        0x000000e0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_lcdif_dvictrl2
    },
    {
        "DVICTRL3",
        "The Digital Video interface Control3 register provides the overall con"
        "trol of the Digital Video interface.",
        4, // Width in bytes
        0x000000f0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_lcdif_dvictrl3
    },
    {
        "DVICTRL4",
        "The Digital Video interface Control4 register provides the overall con"
        "trol of the Digital Video interface.",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_lcdif_dvictrl4
    },
    {
        "CSC_COEFF0",
        "LCDIF_CSC_COEFF0 register provides overall control over color space co"
        "nversion from RGB to 4:2:2 YCbCr.",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_csc_coeff0
    },
    {
        "CSC_COEFF1",
        "LCDIF_CSC_COEFF1 register provides overall control over color space co"
        "nversion from RGB to 4:2:2 YCbCr.",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_csc_coeff1
    },
    {
        "CSC_COEFF2",
        "LCDIF_CSC_COEFF2 register provides overall control over color space co"
        "nversion from RGB to 4:2:2 YCbCr.",
        4, // Width in bytes
        0x00000130, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_csc_coeff2
    },
    {
        "CSC_COEFF3",
        "LCDIF_CSC_COEFF3 register provides overall control over color space co"
        "nversion from RGB to 4:2:2 YCbCr.",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_csc_coeff3
    },
    {
        "CSC_COEFF4",
        "LCDIF_CSC_COEFF4 register provides overall control over color space co"
        "nversion from RGB to 4:2:2 YCbCr.",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_csc_coeff4
    },
    {
        "CSC_OFFSET",
        "LCDIF_CSC_ register provides overall control over color space conversi"
        "on from RGB to 4:2:2 YCbCr.",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_csc_offset
    },
    {
        "CSC_LIMIT",
        "LCDIF_CSC_CTRL0 register provides overall control over color space con"
        "version from RGB to 4:2:2 YCbCr.",
        4, // Width in bytes
        0x00000170, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_lcdif_csc_limit
    },
    {
        "DATA",
        "This register is used to transfer data using the PIO interface mode of"
        " operation.",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_lcdif_data
    },
    {
        "BM_ERROR_STAT",
        "This register reflects the virtual address at which the AXI master rec"
        "eived an error response from the slave.",
        4, // Width in bytes
        0x00000190, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_lcdif_bm_error_stat
    },
    {
        "CRC_STAT",
        "This register reflects the CRC value of each frame sent out by eLCDIF.",
        4, // Width in bytes
        0x000001a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_lcdif_crc_stat
    },
    {
        "STAT",
        "The LCD interface status register can be used to check the current sta"
        "tus of the eLCDIF block.",
        4, // Width in bytes
        0x000001b0, // Base address offset
        true, // Readable
        false, // Writable
        8, // Number of bitfields
        hw_lcdif_stat
    },
    {
        "VERSION",
        "The LCD interface version register can be used to read the version of "
        "the eLCDIF IP being used in this SoC.",
        4, // Width in bytes
        0x000001c0, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_lcdif_version
    },
    {
        "DEBUG0",
        "The LCD interface debug0 register provides a diagnostic view of the st"
        "ate machine and other useful internal signals.",
        4, // Width in bytes
        0x000001d0, // Base address offset
        true, // Readable
        false, // Writable
        17, // Number of bitfields
        hw_lcdif_debug0
    },
    {
        "DEBUG1",
        "The LCD interface debug1 register provides a diagnostic view of the st"
        "ate machine and other useful internal signals.",
        4, // Width in bytes
        0x000001e0, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_lcdif_debug1
    },
    {
        "DEBUG2",
        "The LCD interface debug2 register provides a diagnostic view of the st"
        "ate machine and other useful internal signals.",
        4, // Width in bytes
        0x000001f0, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_lcdif_debug2
    },
    {
        "THRES",
        "This register is used to activate control signals when the number of p"
        "ixels reaches the programmed threshold.",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_lcdif_thres
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark EPDC
#endif

// Bitfields in register EPDC_CTRL.
static const field_t hw_epdc_ctrl[] =
{
    {
        "BURST_LEN_8",
        "0- EPDC display fifo logic will issue AXI bursts of length 16.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_DATA_SWIZZLE",
        "Specifies how to swap the bytes for the LUT data before store to LUTRA"
        "M.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPD_DATA_SWIZZLE",
        "Specifies how to swap the bytes for the UPD data before the WB constru"
        "ction.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE",
        "This bit must be set to zero for normal operation.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFTRST",
        "Set this bit to zero to enable normal EPDC operation.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_WVADDR.
static const field_t hw_epdc_wvaddr[] =
{
    {
        "ADDR",
        "Start address of waveform tables.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_WB_ADDR.
static const field_t hw_epdc_wb_addr[] =
{
    {
        "ADDR",
        "Address for EPDC working buffer.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_RES.
static const field_t hw_epdc_res[] =
{
    {
        "HORIZONTAL",
        "Horizontal Resolution (in pixels)",
        0, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VERTICAL",
        "Vertical Resoltion (in pixels)",
        16, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_FORMAT.
static const field_t hw_epdc_format[] =
{
    {
        "TFT_PIXEL_FORMAT",
        "EPDC TFT Pixel Format.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BUF_PIXEL_FORMAT",
        "EPDC Input Buffer Pixel format.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEFAULT_TFT_PIXEL",
        "Default TFT pixel value.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BUF_PIXEL_SCALE",
        "Selects method of conversion from 8-bit input",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_FIFOCTRL.
static const field_t hw_epdc_fifoctrl[] =
{
    {
        "FIFO_L_LEVEL",
        "Lower level value of FIFO watermark.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FIFO_H_LEVEL",
        "Upper level value of FIFO watermark.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FIFO_INIT_LEVEL",
        "This register sets the watermark for the pixel-fifo.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_PRIORITY",
        "Enable watermark-based priority elevation mechanism.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_ADDR.
static const field_t hw_epdc_upd_addr[] =
{
    {
        "ADDR",
        "Address for incoming region update.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_STRIDE.
static const field_t hw_epdc_upd_stride[] =
{
    {
        "STRIDE",
        "line stride for incoming region update",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_CORD.
static const field_t hw_epdc_upd_cord[] =
{
    {
        "XCORD",
        "X co-ordinate for incoming region update",
        0, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "YCORD",
        "Y co-ordinate for incoming region update",
        16, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_SIZE.
static const field_t hw_epdc_upd_size[] =
{
    {
        "WIDTH",
        "Width (in pixels)",
        0, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HEIGHT",
        "Height (in pixels)",
        16, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_CTRL.
static const field_t hw_epdc_upd_ctrl[] =
{
    {
        "UPDATE_MODE",
        "Update Mode",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DRY_RUN",
        "Enable Dry Run mode(set to 1).",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUTOWV",
        "enable automatical waveform mode selection",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUTOWV_PAUSE",
        "",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NO_LUT_CANCEL",
        "EPDC will cancel LUT loading for void update (no real update needed be"
        "cause of partial or collision), set this bit to 1 to disable this feat"
        "ure",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAVEFORM_MODE",
        "Waveform Mode 0-255",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_SEL",
        "LUT select 0-63",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USE_FIXED",
        "Use fixed pixel values (requires programming of EPDC_UPD_FIXED)",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_FIXED.
static const field_t hw_epdc_upd_fixed[] =
{
    {
        "FIXCP",
        "CP value if fixecp_en is set to 1.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FIXNP",
        "NP value if fixenp_en is set to 1.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FIXCP_EN",
        "If set to 1, current updated region has the CP value defined by FIXCP",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FIXNP_EN",
        "If set to 1, current updated region has the NP value defined by FIXNP",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TEMP.
static const field_t hw_epdc_temp[] =
{
    {
        "TEMPERATURE",
        "Temperature Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_AUTOWV_LUT.
static const field_t hw_epdc_autowv_lut[] =
{
    {
        "ADDR",
        "ADDR",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA",
        "DATA",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_CTRL.
static const field_t hw_epdc_tce_ctrl[] =
{
    {
        "PIXELS_PER_SDCLK",
        "Number of TFT pixels per SDCLK period.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDDO_WIDTH",
        "Selects either 8 or 16 bit SDDO bus format",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DUAL_SCAN",
        "Enables dual scan-mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SCAN_DIR_0",
        "Determines scan direction for each half of the TFT panel",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SCAN_DIR_1",
        "Determines scan direction for each half of the TFT panel",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVDS_MODE",
        "If set, the upper 8-bit of the SDDO bus are used for LVDS differential"
        " signalling.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVDS_MODE_CE",
        "If set (together with LVDS_MODE=1), SDCE[9:5] shall be driven as the d"
        "ifferential inverse of SDCE[4:0].",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_MODE",
        "If set, SDDO data is driven on both positive and negative edges of SDC"
        "LK.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VCOM_MODE",
        "This field determines the method used to drive the VCOM signal.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VCOM_VAL",
        "When VCOM_MODE = MANUAL, this value is used to manually set the VCOM v"
        "alue for the VCOM[1:0] pins",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSCAN_HOLDOFF",
        "This period (expressed in vertical lines), sets the portion of the ver"
        "tical blanking available for new LUTs to be activated.",
        16, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_SDCFG.
static const field_t hw_epdc_tce_sdcfg[] =
{
    {
        "PIXELS_PER_CE",
        "Number of pixels (outputs) per source-driver IC.",
        0, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDDO_INVERT",
        "Setting this bit to 1 reverses the polarity of each SDDO bit so 0xAAAA"
        " in 16-bit mode for example becomes 0x5555.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDDO_REFORMAT",
        "This register defines the various re-formatting options to enable more"
        " flexibility in the source-driver interface:",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM_CE",
        "Number of source driver IC chip-enables.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDSHR",
        "Value for source-driver shift direction output port",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDCLK_HOLD",
        "Setting this bit to 1 holds the SDCLK low during LINE_BEGIN",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_GDCFG.
static const field_t hw_epdc_tce_gdcfg[] =
{
    {
        "GDSP_MODE",
        "Selects method for driving GDSP pulse.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GDOE_MODE",
        "Selects method for driving GDOE signal.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GDRL",
        "Value for gate-driver right/left shift output port",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PERIOD_VSCAN",
        "when vscan state is splited, this reg defines the counter period",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_HSCAN1.
static const field_t hw_epdc_tce_hscan1[] =
{
    {
        "LINE_SYNC",
        "Number of PIXCLK cycles for line sync duration.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LINE_SYNC_WIDTH",
        "Number of PIXCLK cycles for the SDLE active time.",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_HSCAN2.
static const field_t hw_epdc_tce_hscan2[] =
{
    {
        "LINE_BEGIN",
        "Number of PIXCLK cycles for line begin duration.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LINE_END",
        "Number of PIXCLK cycles for line end duration.",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_VSCAN.
static const field_t hw_epdc_tce_vscan[] =
{
    {
        "FRAME_SYNC",
        "Number of lines for frame sync duration.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRAME_BEGIN",
        "Number of lines for frame begin duration.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRAME_END",
        "Number of lines for frame end duration.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_OE.
static const field_t hw_epdc_tce_oe[] =
{
    {
        "SDOEZ_DLY",
        "Number of PIXCLK cycles from SDLE falling edge to SDOEZ rising (Must b"
        "e greater than 0)",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDOEZ_WIDTH",
        "Number of PIXCLK cycles from SDOEZ high to SDOEZ falling (Must be grea"
        "ter than 0)",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDOED_DLY",
        "Number of PIXCLK cycles from SDOEZ low to SDOED rising (Must be greate"
        "r than 0)",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDOED_WIDTH",
        "Number of PIXCLK cycles from SDOED high to SDOED falling (Must be grea"
        "ter than 0)",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_POLARITY.
static const field_t hw_epdc_tce_polarity[] =
{
    {
        "SDCE_POL",
        "0 = Active Low, 1 = Active High.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDLE_POL",
        "0 = Active Low, 1 = Active High.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDOE_POL",
        "0 = Active Low, 1 = Active High.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GDOE_POL",
        "0 = Active Low, 1 = Active High.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GDSP_POL",
        "0 = Active Low, 1 = Active High.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_TIMING1.
static const field_t hw_epdc_tce_timing1[] =
{
    {
        "SDCLK_SHIFT",
        "This register can be used to implement additional timing setup/hold ad"
        "justment of source driver signals by adjusting the SDCLK up to 4 cycle"
        "s",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDCLK_INVERT",
        "Invert phase of SDCLK",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDLE_SHIFT",
        "This register can be used to implement additional timing setup/hold ad"
        "justment of source driver signals by adjusting the SDCLK up to 3 PIXCL"
        "K cycles",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_TIMING2.
static const field_t hw_epdc_tce_timing2[] =
{
    {
        "GDSP_OFFSET",
        "This register allows the user to shift the GDSP pulse by N PIXCLKs whe"
        "re N=1 to 65535.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GDCLK_HP",
        "This register controls the GDCLK high-pulse width.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_TCE_TIMING3.
static const field_t hw_epdc_tce_timing3[] =
{
    {
        "GDCLK_OFFSET",
        "This register allows the user to shift the GDCLK from the line time by"
        " N PIXCLK cycles.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GDOE_OFFSET",
        "When using GDOE_MODE=1, this register sets the delay from GDCLK to the"
        " GDOE in terms of N PIXCLK cycles",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_CTRL0.
static const field_t hw_epdc_pigeon_ctrl0[] =
{
    {
        "FD_PERIOD",
        "period of line counter during FD phase",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LD_PERIOD",
        "period of pclk counter during LD phase",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_CTRL1.
static const field_t hw_epdc_pigeon_ctrl1[] =
{
    {
        "FRAME_CNT_PERIOD",
        "period of frame counter",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRAME_CNT_CYCLES",
        "max cycles of frame counter",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_IRQ_MASK1.
static const field_t hw_epdc_irq_mask1[] =
{
    {
        "LUTN_CMPLT_IRQ_EN",
        "LUT0~31 Complete Interrupt Enable",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_IRQ_MASK2.
static const field_t hw_epdc_irq_mask2[] =
{
    {
        "LUTN_CMPLT_IRQ_EN",
        "LUT32~64 Complete Interrupt Enable",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_IRQ1.
static const field_t hw_epdc_irq1[] =
{
    {
        "LUTN_CMPLT_IRQ",
        "LUT 0~31 Complete Interrupt",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_IRQ2.
static const field_t hw_epdc_irq2[] =
{
    {
        "LUTN_CMPLT_IRQ",
        "LUT 32~64 Complete Interrupt",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_IRQ_MASK.
static const field_t hw_epdc_irq_mask[] =
{
    {
        "WB_CMPLT_IRQ_EN",
        "Enable WB complete interrupt",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COL_IRQ_EN",
        "Enable collision detection interrupts for all LUTs",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCE_UNDERRUN_IRQ_EN",
        "Enable pixel FIFO under-run condition detection.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRAME_END_IRQ_EN",
        "If this bit is set, EPDC will assert the current frame end interrupt.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BUS_ERROR_IRQ_EN",
        "Enable AXI BUS ERROR interrupt detection.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCE_IDLE_IRQ_EN",
        "Enable TCE Idle interrupt detection.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPD_DONE_IRQ_EN",
        "Enable UPD complete interrupt",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWR_IRQ_EN",
        "Enable power interrupt",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_IRQ.
static const field_t hw_epdc_irq[] =
{
    {
        "WB_CMPLT_IRQ",
        "Working buffer process complete Interrupt",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_COL_IRQ",
        "Collision detection interrupt.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCE_UNDERRUN_IRQ",
        "Interrupt to indicate that a pixel FIFO under-run has occured.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRAME_END_IRQ",
        "Interrupt to indicate EPDC has completed the current frame and is in t"
        "he vertical blanking period.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BUS_ERROR_IRQ",
        "Interrupt to indicate AXI BUS error occurs.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCE_IDLE_IRQ",
        "Interrupt to indicate that the TCE has completed TFT frame scans and i"
        "s in an idle state.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPD_DONE_IRQ",
        "Working buffer process complete Interrupt",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWR_IRQ",
        "Power Interrupt",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_STATUS_LUTS1.
static const field_t hw_epdc_status_luts1[] =
{
    {
        "LUTN_STS",
        "LUT 0~31 Status : 1=ACTIVE, 0=IDLE",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_STATUS_LUTS2.
static const field_t hw_epdc_status_luts2[] =
{
    {
        "LUTN_STS",
        "LUT 32~63 Status : 1=ACTIVE, 0=IDLE",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_STATUS_NEXTLUT.
static const field_t hw_epdc_status_nextlut[] =
{
    {
        "NEXT_LUT",
        "Next available LUT value",
        0, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NEXT_LUT_VALID",
        "This bitfield can be used to check against a LUTs full condition",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_STATUS_COL1.
static const field_t hw_epdc_status_col1[] =
{
    {
        "LUTN_COL_STS",
        "LUTn Collision Status",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_STATUS_COL2.
static const field_t hw_epdc_status_col2[] =
{
    {
        "LUTN_COL_STS",
        "LUTn Collision Status",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_STATUS.
static const field_t hw_epdc_status[] =
{
    {
        "WB_BUSY",
        "Working buffer process is busy cannot accept new update requests.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LUTS_BUSY",
        "Provides a summary status of LUTs.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LUTS_UNDERRUN",
        "Provides a summary status of LUT fill.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UPD_VOID",
        "shows the update buffer is void (don't need any real pixel update)",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HISTOGRAM_NP",
        "Indicates which histogram matched the processed bitmap(NP).",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HISTOGRAM_CP",
        "Indicates which histogram matched the existing bitmap(CP).",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_COL_CORD.
static const field_t hw_epdc_upd_col_cord[] =
{
    {
        "XCORD",
        "X co-ordinate for collision region of the latest completed update",
        0, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "YCORD",
        "Y co-ordinate for collision region of the latest completed update",
        16, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_UPD_COL_SIZE.
static const field_t hw_epdc_upd_col_size[] =
{
    {
        "WIDTH",
        "Width (in pixels)",
        0, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HEIGHT",
        "Height (in pixels)",
        16, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST1_PARAM.
static const field_t hw_epdc_hist1_param[] =
{
    {
        "VALUE0",
        "value for 1-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST2_PARAM.
static const field_t hw_epdc_hist2_param[] =
{
    {
        "VALUE0",
        "Black value for 2-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "White value for 2-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST4_PARAM.
static const field_t hw_epdc_hist4_param[] =
{
    {
        "VALUE0",
        "GRAY0 (Black) value for 4-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "GRAY1 value for 4-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE2",
        "GRAY2 value for 4-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE3",
        "GRAY3 (White) value for 4-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST8_PARAM0.
static const field_t hw_epdc_hist8_param0[] =
{
    {
        "VALUE0",
        "GRAY0 (Black) value for 8-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "GRAY1 value for 8-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE2",
        "GRAY2 value for 8-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE3",
        "GRAY3 value for 8-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST8_PARAM1.
static const field_t hw_epdc_hist8_param1[] =
{
    {
        "VALUE4",
        "GRAY4 value for 8-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE5",
        "GRAY5 value for 8-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE6",
        "GRAY6 value for 8-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE7",
        "GRAY7 (White) value for 8-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST16_PARAM0.
static const field_t hw_epdc_hist16_param0[] =
{
    {
        "VALUE0",
        "GRAY0 (Black) value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "GRAY1 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE2",
        "GRAY2 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE3",
        "GRAY3 value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST16_PARAM1.
static const field_t hw_epdc_hist16_param1[] =
{
    {
        "VALUE4",
        "GRAY4 value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE5",
        "GRAY5 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE6",
        "GRAY6 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE7",
        "GRAY7 value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST16_PARAM2.
static const field_t hw_epdc_hist16_param2[] =
{
    {
        "VALUE8",
        "GRAY8 value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE9",
        "GRAY9 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE10",
        "GRAY10 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE11",
        "GRAY11 value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_HIST16_PARAM3.
static const field_t hw_epdc_hist16_param3[] =
{
    {
        "VALUE12",
        "GRAY12 value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE13",
        "GRAY13 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE14",
        "GRAY14 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE15",
        "GRAY15 (White) value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_GPIO.
static const field_t hw_epdc_gpio[] =
{
    {
        "BDR",
        "Controls ipp_epdc_bdr[1:0] output",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWRCTRL",
        "Controls ipp_epdc_pwrctrl[3:0] output",
        2, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWRCOM",
        "Controls ipp_epdc_pwrcom output",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWRWAKE",
        "Controls ipp_epdc_pwrwake output",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWRSTAT",
        "reflect ipp_epdc_pwrstat input",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_VERSION.
static const field_t hw_epdc_version[] =
{
    {
        "STEP",
        "Fixed read-only value reflecting the stepping of the RTL version.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MINOR",
        "Fixed read-only value reflecting the MINOR field of the RTL version.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-only value reflecting the MAJOR field of the RTL version.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_0_0.
static const field_t hw_epdc_pigeon_0_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_0_1.
static const field_t hw_epdc_pigeon_0_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_0_2.
static const field_t hw_epdc_pigeon_0_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_1_0.
static const field_t hw_epdc_pigeon_1_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_1_1.
static const field_t hw_epdc_pigeon_1_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_1_2.
static const field_t hw_epdc_pigeon_1_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_2_0.
static const field_t hw_epdc_pigeon_2_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_2_1.
static const field_t hw_epdc_pigeon_2_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_2_2.
static const field_t hw_epdc_pigeon_2_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_3_0.
static const field_t hw_epdc_pigeon_3_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_3_1.
static const field_t hw_epdc_pigeon_3_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_3_2.
static const field_t hw_epdc_pigeon_3_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_4_0.
static const field_t hw_epdc_pigeon_4_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_4_1.
static const field_t hw_epdc_pigeon_4_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_4_2.
static const field_t hw_epdc_pigeon_4_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_5_0.
static const field_t hw_epdc_pigeon_5_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_5_1.
static const field_t hw_epdc_pigeon_5_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_5_2.
static const field_t hw_epdc_pigeon_5_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_6_0.
static const field_t hw_epdc_pigeon_6_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_6_1.
static const field_t hw_epdc_pigeon_6_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_6_2.
static const field_t hw_epdc_pigeon_6_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_7_0.
static const field_t hw_epdc_pigeon_7_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_7_1.
static const field_t hw_epdc_pigeon_7_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_7_2.
static const field_t hw_epdc_pigeon_7_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_8_0.
static const field_t hw_epdc_pigeon_8_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_8_1.
static const field_t hw_epdc_pigeon_8_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_8_2.
static const field_t hw_epdc_pigeon_8_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_9_0.
static const field_t hw_epdc_pigeon_9_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_9_1.
static const field_t hw_epdc_pigeon_9_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_9_2.
static const field_t hw_epdc_pigeon_9_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_10_0.
static const field_t hw_epdc_pigeon_10_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_10_1.
static const field_t hw_epdc_pigeon_10_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_10_2.
static const field_t hw_epdc_pigeon_10_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_11_0.
static const field_t hw_epdc_pigeon_11_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_11_1.
static const field_t hw_epdc_pigeon_11_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_11_2.
static const field_t hw_epdc_pigeon_11_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_12_0.
static const field_t hw_epdc_pigeon_12_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_12_1.
static const field_t hw_epdc_pigeon_12_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_12_2.
static const field_t hw_epdc_pigeon_12_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_13_0.
static const field_t hw_epdc_pigeon_13_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_13_1.
static const field_t hw_epdc_pigeon_13_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_13_2.
static const field_t hw_epdc_pigeon_13_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_14_0.
static const field_t hw_epdc_pigeon_14_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_14_1.
static const field_t hw_epdc_pigeon_14_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_14_2.
static const field_t hw_epdc_pigeon_14_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_15_0.
static const field_t hw_epdc_pigeon_15_0[] =
{
    {
        "EN",
        "enable pigeon mode on this signal",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POL",
        "polarity of signal output",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INC_SEL",
        "event to incrment local counter",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OFFSET",
        "offset on pclk unit.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT_SEL",
        "select global counters as mask condition, use together with MASK_CNT",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_CNT",
        "When the global counter selected through MASK_CNT_SEL matches value in"
        " this reg, pigeon local counter start ticking.",
        12, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STATE_MASK",
        "state_mask = (FS|FB|FD|FE) and (LS|LB|LD|LE) , select any combination "
        "of scan states as reference point for local counter to start ticking",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_15_1.
static const field_t hw_epdc_pigeon_15_1[] =
{
    {
        "SET_CNT",
        "assert signal output when counter match this value",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLR_CNT",
        "deassert signal output when counter match this value",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_PIGEON_15_2.
static const field_t hw_epdc_pigeon_15_2[] =
{
    {
        "SIG_LOGIC",
        "logic operation with another signal  sigout : final output signal of t"
        "his generator  mask : final mask of this generator  this_sig : interme"
        "diate signal of this generator before logic operation  other_masks : i"
        "ntermediate mask result of this generator before logic operation  sig_"
        "another : signal selected other generators",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SIG_ANOTHER",
        "select another signal for logic operation or as mask or counter tick e"
        "vent",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPDC_WB_ADDR_TCE.
static const field_t hw_epdc_wb_addr_tce[] =
{
    {
        "ADDR",
        "Address for EPDC working buffer (only for TCE).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a EPDC module.
static const reg_t hw_epdc[] =
{
    {
        "CTRL",
        "EPDC Main control register  This register controls various high-level "
        "functions of the EPDC",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_epdc_ctrl
    },
    {
        "WVADDR",
        "EPDC Waveform Address Pointer  N/A   EXAMPLE   N/A",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_wvaddr
    },
    {
        "WB_ADDR",
        "EPDC Working Buffer Address  This register controls various functions "
        "throughout the digital portion of the chip.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_wb_addr
    },
    {
        "RES",
        "EPDC Screen Resolution.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_res
    },
    {
        "FORMAT",
        "EPDC Pixel format control register.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_format
    },
    {
        "FIFOCTRL",
        "Allows for programmability of pixel FIFO watermarks used in conjunctio"
        "n with system arbitration hardware  This register houses FIFO control "
        "bits",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_fifoctrl
    },
    {
        "UPD_ADDR",
        "EPDC Update Region Address  When stride feature not enabled(EPDC_UPD_S"
        "TRIDE==0), it must be aligned to 64-bit boundary, this is default beha"
        "vior to keep compatible like before; When stride feature is enabled(EP"
        "DC_UPD_STRIDE!=0), it can start from any byte address, no alignment re"
        "quirement, but still recomended be aligned get best bus performance.",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_upd_addr
    },
    {
        "UPD_STRIDE",
        "EPDC Update Region Stride  When UPD_STRIDE==0 (stride feature disabled"
        "), UPD buffer line must start from 64-bit boundary and end on 64-bit b"
        "oundary(padding if not).",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_upd_stride
    },
    {
        "UPD_CORD",
        "EPDC Update Command Co-ordinate  N/A   EXAMPLE   N/A",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_upd_cord
    },
    {
        "UPD_SIZE",
        "EPDC Update Command Size  This register controls various functions thr"
        "oughout the digital portion of the chip.",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_upd_size
    },
    {
        "UPD_CTRL",
        "EPDC Update Command Control.",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_epdc_upd_ctrl
    },
    {
        "UPD_FIXED",
        "EPDC Update Control register for fixed-pixel updates (enabled via EPDC"
        "_UPD_CTRL[USE_FIXED])",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_upd_fixed
    },
    {
        "TEMP",
        "EPDC Temperature Compensation Register  N/A   EXAMPLE   N/A",
        4, // Width in bytes
        0x000001a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_temp
    },
    {
        "AUTOWV_LUT",
        "This register is used to access the waveform mode lookup table.",
        4, // Width in bytes
        0x000001c0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_autowv_lut
    },
    {
        "TCE_CTRL",
        "TCE general control register  This register houses Horizontal scan tim"
        "ing.",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_epdc_tce_ctrl
    },
    {
        "TCE_SDCFG",
        "Source-driver configuration register",
        4, // Width in bytes
        0x00000220, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_epdc_tce_sdcfg
    },
    {
        "TCE_GDCFG",
        "This register houses gate-driver configuration.",
        4, // Width in bytes
        0x00000240, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_tce_gdcfg
    },
    {
        "TCE_HSCAN1",
        "Horizontal scan timing registers.",
        4, // Width in bytes
        0x00000260, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_tce_hscan1
    },
    {
        "TCE_HSCAN2",
        "Horizontal scan timing registers.",
        4, // Width in bytes
        0x00000280, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_tce_hscan2
    },
    {
        "TCE_VSCAN",
        "Vertical scan timing registers  This register houses vertical scan tim"
        "ing.",
        4, // Width in bytes
        0x000002a0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_epdc_tce_vscan
    },
    {
        "TCE_OE",
        "This register contain delay programming values for the SDOEZ and SDOED"
        " source driver control signals  This register contain delay programmin"
        "g values for the SDOZ and SDOE source driver control signals",
        4, // Width in bytes
        0x000002c0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_tce_oe
    },
    {
        "TCE_POLARITY",
        "This registers allows for programming the polarity of source/gate driv"
        "er control signals  This register houses FIFO control bits",
        4, // Width in bytes
        0x000002e0, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_epdc_tce_polarity
    },
    {
        "TCE_TIMING1",
        "This register contains various timing adjustment controls  This regist"
        "er houses general purpose timing adjustment registers",
        4, // Width in bytes
        0x00000300, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_epdc_tce_timing1
    },
    {
        "TCE_TIMING2",
        "This register contains various timing adjustment controls  This regist"
        "er houses general purpose timing adjustment registers",
        4, // Width in bytes
        0x00000310, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_tce_timing2
    },
    {
        "TCE_TIMING3",
        "This register contains various timing adjustment controls  This regist"
        "er houses general purpose timing adjustment registers",
        4, // Width in bytes
        0x00000320, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_tce_timing3
    },
    {
        "PIGEON_CTRL0",
        "This register contains global counter settings for Pigeon Mode  This r"
        "egister houses general purpose timing adjustment registers",
        4, // Width in bytes
        0x00000380, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_ctrl0
    },
    {
        "PIGEON_CTRL1",
        "This register contains global counter setting for pigeon mode  This re"
        "gister houses general purpose timing adjustment registers",
        4, // Width in bytes
        0x00000390, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_ctrl1
    },
    {
        "IRQ_MASK1",
        "Controls masking EPDC LUT complete interrupts  This register controls "
        "LUT0~31 IRQ masks for EPDC interrupts",
        4, // Width in bytes
        0x000003c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_irq_mask1
    },
    {
        "IRQ_MASK2",
        "Controls masking EPDC LUT complete interrupts  This register controls "
        "LUT0~31 IRQ masks for EPDC interrupts",
        4, // Width in bytes
        0x000003d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_irq_mask2
    },
    {
        "IRQ1",
        "EPDC LUT Completion IRQs.",
        4, // Width in bytes
        0x000003e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_irq1
    },
    {
        "IRQ2",
        "EPDC LUT Completion IRQs.",
        4, // Width in bytes
        0x000003f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_irq2
    },
    {
        "IRQ_MASK",
        "Controls masking for all EPDC interrupts  This register controls IRQ m"
        "asks for all EPDC interrupts",
        4, // Width in bytes
        0x00000400, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_epdc_irq_mask
    },
    {
        "IRQ",
        "EPDC LUT Completion IRQs.",
        4, // Width in bytes
        0x00000420, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_epdc_irq
    },
    {
        "STATUS_LUTS1",
        "EPDC Status Register - LUTS 0~31",
        4, // Width in bytes
        0x00000440, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_status_luts1
    },
    {
        "STATUS_LUTS2",
        "EPDC Status Register - LUTS 0~31",
        4, // Width in bytes
        0x00000450, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_status_luts2
    },
    {
        "STATUS_NEXTLUT",
        "Holds value of next available LUT.",
        4, // Width in bytes
        0x00000460, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_status_nextlut
    },
    {
        "STATUS_COL1",
        "EPDC LUT Collision Status Register and works in conjuction with EPDC_I"
        "RQ[LUT_COL_IRQ].",
        4, // Width in bytes
        0x00000480, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_status_col1
    },
    {
        "STATUS_COL2",
        "EPDC LUT Collision Status Register and works in conjuction with EPDC_I"
        "RQ[LUT_COL_IRQ].",
        4, // Width in bytes
        0x00000490, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_status_col2
    },
    {
        "STATUS",
        "Register to house non LUT specific status bits  This register houses g"
        "eneral status bits",
        4, // Width in bytes
        0x000004a0, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_epdc_status
    },
    {
        "UPD_COL_CORD",
        "EPDC Collision Region Co-ordinate, cleared when new update issued  Thi"
        "s register only valid after WB completion and collision happens.",
        4, // Width in bytes
        0x000004c0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_upd_col_cord
    },
    {
        "UPD_COL_SIZE",
        "EPDC Collision Region Size of the latest completed update cleared when"
        " new update issued  This register only valid after WB completion and c"
        "ollision happens.",
        4, // Width in bytes
        0x000004e0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_upd_col_size
    },
    {
        "HIST1_PARAM",
        "This register specifies the valid values for a 1-level(single color) h"
        "istogram.",
        4, // Width in bytes
        0x00000600, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_hist1_param
    },
    {
        "HIST2_PARAM",
        "This register specifies the valid values for a 2-level histogram.",
        4, // Width in bytes
        0x00000610, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_hist2_param
    },
    {
        "HIST4_PARAM",
        "This register specifies the valid values for a 4-level histogram.",
        4, // Width in bytes
        0x00000620, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_hist4_param
    },
    {
        "HIST8_PARAM0",
        "This register specifies four of the valid values for an 8-level histog"
        "ram.",
        4, // Width in bytes
        0x00000630, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_hist8_param0
    },
    {
        "HIST8_PARAM1",
        "This register specifies four of the valid values for an 8-level histog"
        "ram.",
        4, // Width in bytes
        0x00000640, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_hist8_param1
    },
    {
        "HIST16_PARAM0",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x00000650, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_hist16_param0
    },
    {
        "HIST16_PARAM1",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x00000660, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_hist16_param1
    },
    {
        "HIST16_PARAM2",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x00000670, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_hist16_param2
    },
    {
        "HIST16_PARAM3",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x00000680, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_epdc_hist16_param3
    },
    {
        "GPIO",
        "GPIO register to control ipp_epdc_bdr[1:0], ipp_epdc_pwr[3:0] and ipp_"
        "epdc_pwrcom output signals  Houses software control signal reisters",
        4, // Width in bytes
        0x00000700, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_epdc_gpio
    },
    {
        "VERSION",
        "This register reflects the version number for the EPDC.",
        4, // Width in bytes
        0x000007f0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_epdc_version
    },
    {
        "PIGEON_0_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000800, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_0_0
    },
    {
        "PIGEON_0_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000810, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_0_1
    },
    {
        "PIGEON_0_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000820, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_0_2
    },
    {
        "PIGEON_1_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000840, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_1_0
    },
    {
        "PIGEON_1_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000850, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_1_1
    },
    {
        "PIGEON_1_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000860, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_1_2
    },
    {
        "PIGEON_2_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000880, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_2_0
    },
    {
        "PIGEON_2_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000890, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_2_1
    },
    {
        "PIGEON_2_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000008a0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_2_2
    },
    {
        "PIGEON_3_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000008c0, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_3_0
    },
    {
        "PIGEON_3_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000008d0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_3_1
    },
    {
        "PIGEON_3_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000008e0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_3_2
    },
    {
        "PIGEON_4_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000900, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_4_0
    },
    {
        "PIGEON_4_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000910, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_4_1
    },
    {
        "PIGEON_4_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000920, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_4_2
    },
    {
        "PIGEON_5_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000940, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_5_0
    },
    {
        "PIGEON_5_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000950, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_5_1
    },
    {
        "PIGEON_5_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000960, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_5_2
    },
    {
        "PIGEON_6_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000980, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_6_0
    },
    {
        "PIGEON_6_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000990, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_6_1
    },
    {
        "PIGEON_6_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000009a0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_6_2
    },
    {
        "PIGEON_7_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000009c0, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_7_0
    },
    {
        "PIGEON_7_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000009d0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_7_1
    },
    {
        "PIGEON_7_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x000009e0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_7_2
    },
    {
        "PIGEON_8_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a00, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_8_0
    },
    {
        "PIGEON_8_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a10, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_8_1
    },
    {
        "PIGEON_8_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a20, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_8_2
    },
    {
        "PIGEON_9_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a40, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_9_0
    },
    {
        "PIGEON_9_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a50, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_9_1
    },
    {
        "PIGEON_9_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a60, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_9_2
    },
    {
        "PIGEON_10_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a80, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_10_0
    },
    {
        "PIGEON_10_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000a90, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_10_1
    },
    {
        "PIGEON_10_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000aa0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_10_2
    },
    {
        "PIGEON_11_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000ac0, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_11_0
    },
    {
        "PIGEON_11_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000ad0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_11_1
    },
    {
        "PIGEON_11_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000ae0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_11_2
    },
    {
        "PIGEON_12_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b00, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_12_0
    },
    {
        "PIGEON_12_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b10, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_12_1
    },
    {
        "PIGEON_12_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b20, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_12_2
    },
    {
        "PIGEON_13_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b40, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_13_0
    },
    {
        "PIGEON_13_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b50, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_13_1
    },
    {
        "PIGEON_13_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b60, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_13_2
    },
    {
        "PIGEON_14_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b80, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_14_0
    },
    {
        "PIGEON_14_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000b90, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_14_1
    },
    {
        "PIGEON_14_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000ba0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_14_2
    },
    {
        "PIGEON_15_0",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000bc0, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_epdc_pigeon_15_0
    },
    {
        "PIGEON_15_1",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000bd0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_15_1
    },
    {
        "PIGEON_15_2",
        "parameters for timing signal generation",
        4, // Width in bytes
        0x00000be0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_epdc_pigeon_15_2
    },
    {
        "WB_ADDR_TCE",
        "EPDC Working Buffer Address used by TCE only",
        4, // Width in bytes
        0x00000c10, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epdc_wb_addr_tce
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark EPIT
#endif

// Bitfields in register EPIT_CR.
static const field_t hw_epit_cr[] =
{
    {
        "EN",
        "This bit enables the EPIT.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENMOD",
        "EPIT enable mode.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCIEN",
        "Output compare interrupt enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RLD",
        "Counter reload control.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRESCALAR",
        "Counter clock prescaler value.",
        4, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Software reset.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IOVW",
        "EPIT counter overwrite enable.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DBGEN",
        "This bit is used to keep the EPIT functional in debug mode.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAITEN",
        "This read/write control bit enables the operation of the EPIT during w"
        "ait mode.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STOPEN",
        "EPIT stop mode enable.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OM",
        "EPIT output mode.This bit field determines the mode of EPIT output on "
        "the output pin.",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKSRC",
        "Select clock source  These bits determine which clock input is to be s"
        "elected for running the counter.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPIT_SR.
static const field_t hw_epit_sr[] =
{
    {
        "OCIF",
        "Output compare interrupt flag.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPIT_LR.
static const field_t hw_epit_lr[] =
{
    {
        "LOAD",
        "Load value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPIT_CMPR.
static const field_t hw_epit_cmpr[] =
{
    {
        "COMPARE",
        "Compare Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register EPIT_CNR.
static const field_t hw_epit_cnr[] =
{
    {
        "COUNT",
        "Counter value.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a EPIT module.
static const reg_t hw_epit[] =
{
    {
        "CR",
        "The EPIT control register (EPIT_CR) is used to configure the operating"
        " settings of the EPIT.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_epit_cr
    },
    {
        "SR",
        "The EPIT status register (EPIT_SR) has a single status bit for the out"
        "put compare event.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epit_sr
    },
    {
        "LR",
        "The EPIT load register (EPIT_LR) contains the value that is to be load"
        "ed into the counter when EPIT counter reaches zero if the RLD bit in E"
        "PIT_CR is set.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epit_lr
    },
    {
        "CMPR",
        "The EPIT compare register (EPIT_CMPR) holds the value that determines "
        "when a compare event is generated.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_epit_cmpr
    },
    {
        "CNR",
        "The EPIT counter register (EPIT_CNR) contains the current count value "
        "and can be read at any time without disturbing the counter.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_epit_cnr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark FEC
#endif

// Bitfields in register FEC_EIR.
static const field_t hw_fec_eir[] =
{
    {
        "UN",
        "Transmit FIFO underrun.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Collision retry limit.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LC",
        "Late collision.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EBERR",
        "Ethernet bus error.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MII",
        "MII interrupt.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXB",
        "Receive buffer interrupt.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXF",
        "Receive frame interrupt.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXB",
        "Transmit buffer interrupt.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXF",
        "Transmit frame interrupt.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GRA",
        "Graceful stop complete.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BABT",
        "Babbling transmit error.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BABR",
        "Babbling receive error.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HBERR",
        "Heartbeat error.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_EIMR.
static const field_t hw_fec_eimr[] =
{
    {
        "UN",
        "Interrupt mask.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Interrupt mask.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LC",
        "Interrupt mask.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EBERR",
        "Interrupt mask.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MII",
        "Interrupt mask.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXB",
        "Interrupt mask.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXF",
        "Interrupt mask.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXB",
        "Interrupt mask.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXF",
        "Interrupt mask.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GRA",
        "Interrupt mask.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BABT",
        "Interrupt mask.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BABR",
        "Interrupt mask.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HBERR",
        "Interrupt mask.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_RDAR.
static const field_t hw_fec_rdar[] =
{
    {
        "R_DES_ACTIVE",
        "Set to one when this register is written, regardless of the value writ"
        "ten.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_TDAR.
static const field_t hw_fec_tdar[] =
{
    {
        "X_DES_ACTIVE",
        "Set to one when this register is written, regardless of the value writ"
        "ten.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_ECR.
static const field_t hw_fec_ecr[] =
{
    {
        "RESET",
        "When this bit is set, the equivalent of a hardware reset is performed "
        "but it is local to the FEC, ETHER_EN is cleared and all other FEC regi"
        "sters take their reset values.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ETHER_EN",
        "When this bit is set, the FEC is enabled, and reception and transmissi"
        "on are possible.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_MMFR.
static const field_t hw_fec_mmfr[] =
{
    {
        "DATA",
        "Management frame data.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TA",
        "Turn around.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RA",
        "Register address.",
        18, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PA",
        "PHY address.",
        23, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OP",
        "Operation code.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ST",
        "Start of frame delimiter.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_MSCR.
static const field_t hw_fec_mscr[] =
{
    {
        "MII_SPEED",
        "MII_SPEED controls the frequency of the MII management interface clock"
        " (FEC_MDC) relative to the system clock.",
        1, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIS_PREAMBLE",
        "Asserting this bit causes preamble (0xFFFF_FFFF) not to be prepended t"
        "o the MII management frame.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_MIBC.
static const field_t hw_fec_mibc[] =
{
    {
        "MB_IDLE",
        "A read-only status bit.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MIB_DISABLE",
        "A read/write control bit.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_RCR.
static const field_t hw_fec_rcr[] =
{
    {
        "LOOP",
        "Internal loopback.When LOOP is set to 1, transmitted frames are looped"
        " back internal to the device and the transmit output signals are not a"
        "sserted.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DRT",
        "Disable receive on transmit.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MII_MODE",
        "Media independent interface mode.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PROM",
        "Promiscuous mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BC_REJ",
        "Broadcast frame reject.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FCE",
        "Flow control enable.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MAX_FL",
        "Maximum frame length.",
        16, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_TCR.
static const field_t hw_fec_tcr[] =
{
    {
        "GTS",
        "Graceful transmit stop.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HBC",
        "Heartbeat control.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FDEN",
        "Full duplex enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFC_PAUSE",
        "Transmit frame control pause.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFC_PAUSE",
        "Receive frame control pause.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_PALR.
static const field_t hw_fec_palr[] =
{
    {
        "PADDR1",
        "Bytes 0 (bits 31:24), 1 (bits 23:16), 2 (bits 15:8) and 3 (bits 7:0) o"
        "f the 6-byte individual address to be used for exact match, and the so"
        "urce address field in pause frames.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_PAUR.
static const field_t hw_fec_paur[] =
{
    {
        "TYPE",
        "Type field in pause frames.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PADDR2",
        "Bytes 4 (bits 31:24) and 5 (bits 23:16) of the 6-byte individual addre"
        "ss to be used for exact match, and the source address field in pause f"
        "rames.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_OPDR.
static const field_t hw_fec_opdr[] =
{
    {
        "PAUSE_DUR",
        "Pause duration field used in pause frames.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OPCODE",
        "Opcode field used in pause frames.",
        16, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_IAUR.
static const field_t hw_fec_iaur[] =
{
    {
        "IADDR1",
        "The upper 32 bits of the 64-bit hash table used in the address recogni"
        "tion process for receive frames with a unicast address.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_IALR.
static const field_t hw_fec_ialr[] =
{
    {
        "IADDR2",
        "The lower 32 bits of the 64-bit hash table used in the address recogni"
        "tion process for receive frames with a unicast address.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_GAUR.
static const field_t hw_fec_gaur[] =
{
    {
        "GADDR1",
        "The GADDR1 register contains the upper 32 bits of the 64-bit hash tabl"
        "e used in the address recognition process for receive frames with a mu"
        "lticast address.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_GALR.
static const field_t hw_fec_galr[] =
{
    {
        "GADDR2",
        "The GADDR2 register contains the lower 32 bits of the 64-bit hash tabl"
        "e used in the address recognition process for receive frames with a mu"
        "lticast address.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_TFWR.
static const field_t hw_fec_tfwr[] =
{
    {
        "X_WMRK",
        "Number of bytes written to transmit FIFO before transmission of a fram"
        "e begins",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_FRBR.
static const field_t hw_fec_frbr[] =
{
    {
        "R_BOUND",
        "Read-only.",
        2, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_FRSR.
static const field_t hw_fec_frsr[] =
{
    {
        "R_FSTART",
        "Address of first receive FIFO location.",
        2, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_ERDSR.
static const field_t hw_fec_erdsr[] =
{
    {
        "R_DES_START",
        "Pointer to start of receive buffer descriptor queue.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_ETDSR.
static const field_t hw_fec_etdsr[] =
{
    {
        "X_DES_START",
        "Pointer to start of transmit buffer descriptor queue.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register FEC_EMRBR.
static const field_t hw_fec_emrbr[] =
{
    {
        "R_BUF_SIZE",
        "Receive buffer size.",
        4, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a FEC module.
static const reg_t hw_fec[] =
{
    {
        "EIR",
        "The FEC_EIR bit assignments are shown below.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        13, // Number of bitfields
        hw_fec_eir
    },
    {
        "EIMR",
        "The FEC_EIMR controls which of the interrupt events flagged in the FEC"
        "_EIR are allowed to generate actual interrupts.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        13, // Number of bitfields
        hw_fec_eimr
    },
    {
        "RDAR",
        "FEC_RDAR is a user-writeable command register which indicates that the"
        " receive descriptor ring has been updated, and that empty receive buff"
        "ers have been produced by the driver with the empty bit in RxBD set.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_rdar
    },
    {
        "TDAR",
        "The FEC_TDAR is a command register,which is written by the user, to in"
        "dicate that the transmit descriptor ring has been updated (transmit bu"
        "ffers have been produced by the driver with the ready bit set in the b"
        "uffer descriptor).",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_tdar
    },
    {
        "ECR",
        "The ECR is used to enable/disable the FEC_ ECR which is a read/write u"
        "ser register, though both fields in this register can also be altered "
        "by hardware.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_fec_ecr
    },
    {
        "MMFR",
        "The FEC_MMFR is used to communicate with the attached MII compatible P"
        "HY device(s) by providing read/write access to their MII registers.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_fec_mmfr
    },
    {
        "MSCR",
        "The FEC_MSCR provides control of the frequency of the MII clock (FEC_M"
        "DC signal), and allows a preamble drop on the MII management frame .",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_fec_mscr
    },
    {
        "MIBC",
        "The MIB control register is a read/write register which is used to pro"
        "vide control of and to observe the state of the Message Information Bl"
        "ock (MIB).",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_fec_mibc
    },
    {
        "RCR",
        "The FEC_RCR is programmed by the user, and controls the operational mo"
        "de of the receive block.",
        4, // Width in bytes
        0x00000084, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_fec_rcr
    },
    {
        "TCR",
        "This register is read/write register which is written by the user to c"
        "onfigure the transmit block.",
        4, // Width in bytes
        0x000000c4, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_fec_tcr
    },
    {
        "PALR",
        "The FEC_PALR, which is written by the user, contains the lower 32 bits"
        " (bytes 0,1,2,3) of the 48-bit address used in the address recognition"
        " process to check for possible match between the DA field of receive f"
        "rames and an individual DA.",
        4, // Width in bytes
        0x000000e4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_palr
    },
    {
        "PAUR",
        "The FEC_PAUR, which is written by the user, and contains the upper 16 "
        "bits (bytes 4 and 5) of the 48-bit address used in the address recogni"
        "tion process to check for possible match between the DA field of recei"
        "ve frames and an individual DA.",
        4, // Width in bytes
        0x000000e8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_fec_paur
    },
    {
        "OPDR",
        "FEC_OPDR contains the 16-bit Opcode, and 16-bit pause duration fields "
        "used in transmission of a pause frame.",
        4, // Width in bytes
        0x000000ec, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_fec_opdr
    },
    {
        "IAUR",
        "The FEC_IAUR, which is written by the user, contains the upper 32 bits"
        " of the 64-bit individual address hash table used in the address recog"
        "nition process to check for possible match between the DA field of rec"
        "eive frames and an individual DA.",
        4, // Width in bytes
        0x00000118, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_iaur
    },
    {
        "IALR",
        "The FEC_IALR, which is written by the user, contains the lower 32 bits"
        " of the 64-bit individual address hash table used in the address recog"
        "nition process to check for possible match with the DA field of receiv"
        "e frames with an individual DA.",
        4, // Width in bytes
        0x0000011c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_ialr
    },
    {
        "GAUR",
        "The FEC_GAUR, which is written by the user, contains the upper 32 bits"
        " of the 64-bit hash table used in the address recognition process for "
        "receive frames with a multicast address.",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_gaur
    },
    {
        "GALR",
        "The FEC_GALR, which is written by the user, contains the lower 32 bits"
        " of the 64-bit hash table used in the address recognition process for "
        "receive frames with a multicast address.",
        4, // Width in bytes
        0x00000124, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_galr
    },
    {
        "TFWR",
        "The FEC_TFWR is programmed by the user to control the amount of data r"
        "equired in the transmit FIFO before transmission of a frame can begin.",
        4, // Width in bytes
        0x00000144, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_tfwr
    },
    {
        "FRBR",
        "The FEC_FRBR register can be read to determine the upper address bound"
        " of the FIFO RAM.",
        4, // Width in bytes
        0x0000014c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_fec_frbr
    },
    {
        "FRSR",
        "FEC_FRSR is an 8-bit register programmed by the user to indicate the s"
        "tarting address of the receive FIFO.",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_frsr
    },
    {
        "ERDSR",
        "The register, which is written by the user, provides a pointer to the "
        "start of the circular receive buffer descriptor queue in external memo"
        "ry.",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_erdsr
    },
    {
        "ETDSR",
        "The register, which is written by the user, provides a pointer to the "
        "start of the circular transmit buffer descriptor queue in external mem"
        "ory.",
        4, // Width in bytes
        0x00000184, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_etdsr
    },
    {
        "EMRBR",
        "The FEC_EMRBR is a user-programmable register which dictates the maxim"
        "um size of all receive buffers.",
        4, // Width in bytes
        0x00000188, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_fec_emrbr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark GPC
#endif

// Bitfields in register GPC_CNTR.
static const field_t hw_gpc_cntr[] =
{
    {
        "GPU_VPU_PDN_REQ",
        "GPU Power Down request.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU_VPU_PUP_REQ",
        "GPU Power Up request.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DVFS0CR",
        "DVFS0 (ARM) Change request (bit is read-only)",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "GPCIRQM",
        "GPC interrupt/event masking",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_PGR.
static const field_t hw_gpc_pgr[] =
{
    {
        "DRCIC",
        "Debug ref cir in mux control",
        29, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_IMR1.
static const field_t hw_gpc_imr1[] =
{
    {
        "IMR1",
        "IRQ[63:32] masking bits: 1-irq masked, 0-irq is not masked",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_IMR2.
static const field_t hw_gpc_imr2[] =
{
    {
        "IMR2",
        "IRQ[95:64] masking bits: 1-irq masked, 0-irq is not masked",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_IMR3.
static const field_t hw_gpc_imr3[] =
{
    {
        "IMR3",
        "IRQ[127:96] masking bits: 1-irq masked, 0-irq is not masked",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_IMR4.
static const field_t hw_gpc_imr4[] =
{
    {
        "IMR4",
        "IRQ[159:128] masking bits: 1-irq masked, 0-irq is not masked",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_ISR1.
static const field_t hw_gpc_isr1[] =
{
    {
        "ISR1",
        "IRQ[63:32] status, read only",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_ISR2.
static const field_t hw_gpc_isr2[] =
{
    {
        "ISR2",
        "IRQ[95:64] status, read only",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_ISR3.
static const field_t hw_gpc_isr3[] =
{
    {
        "ISR3",
        "IRQ[127:96] status, read only",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPC_ISR4.
static const field_t hw_gpc_isr4[] =
{
    {
        "ISR4",
        "IRQ[159:128] status, read only",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a GPC module.
static const reg_t hw_gpc[] =
{
    {
        "CNTR",
        "CNTR - Interface control register",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_gpc_cntr
    },
    {
        "PGR",
        "PGR - Power Gating Register",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpc_pgr
    },
    {
        "IMR1",
        "IMR1 Register - masking of irq[63:32].",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpc_imr1
    },
    {
        "IMR2",
        "IMR2 Register - masking of irq[95:64].",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpc_imr2
    },
    {
        "IMR3",
        "IMR3 Register - masking of irq[127:96].",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpc_imr3
    },
    {
        "IMR4",
        "IMR4 Register - masking of irq[159:128].",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpc_imr4
    },
    {
        "ISR1",
        "ISR1 Register - status of irq [63:32].",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpc_isr1
    },
    {
        "ISR2",
        "ISR2 Register - status of irq [95:64].",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpc_isr2
    },
    {
        "ISR3",
        "ISR3 Register - status of irq [127:96].",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpc_isr3
    },
    {
        "ISR4",
        "ISR4 Register - status of irq [159:128].",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpc_isr4
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark GPIO
#endif

// Bitfields in register GPIO_DR.
static const field_t hw_gpio_dr[] =
{
    {
        "DR",
        "Data bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPIO_GDIR.
static const field_t hw_gpio_gdir[] =
{
    {
        "GDIR",
        "GPIO direction bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPIO_PSR.
static const field_t hw_gpio_psr[] =
{
    {
        "PSR",
        "GPIO pad status bits (status bits).",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPIO_ICR1.
static const field_t hw_gpio_icr1[] =
{
    {
        "ICR0",
        "Interrupt configuration 1 fields.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR1",
        "Interrupt configuration 1 fields.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR2",
        "Interrupt configuration 1 fields.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR3",
        "Interrupt configuration 1 fields.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR4",
        "Interrupt configuration 1 fields.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR5",
        "Interrupt configuration 1 fields.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR6",
        "Interrupt configuration 1 fields.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR7",
        "Interrupt configuration 1 fields.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR8",
        "Interrupt configuration 1 fields.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR9",
        "Interrupt configuration 1 fields.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR10",
        "Interrupt configuration 1 fields.",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR11",
        "Interrupt configuration 1 fields.",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR12",
        "Interrupt configuration 1 fields.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR13",
        "Interrupt configuration 1 fields.",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR14",
        "Interrupt configuration 1 fields.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR15",
        "Interrupt configuration 1 fields.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPIO_ICR2.
static const field_t hw_gpio_icr2[] =
{
    {
        "ICR16",
        "Interrupt configuration 2 fields.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR17",
        "Interrupt configuration 2 fields.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR18",
        "Interrupt configuration 2 fields.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR19",
        "Interrupt configuration 2 fields.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR20",
        "Interrupt configuration 2 fields.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR21",
        "Interrupt configuration 2 fields.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR22",
        "Interrupt configuration 2 fields.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR23",
        "Interrupt configuration 2 fields.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR24",
        "Interrupt configuration 2 fields.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR25",
        "Interrupt configuration 2 fields.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR26",
        "Interrupt configuration 2 fields.",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR27",
        "Interrupt configuration 2 fields.",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR28",
        "Interrupt configuration 2 fields.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR29",
        "Interrupt configuration 2 fields.",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR30",
        "Interrupt configuration 2 fields.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICR31",
        "Interrupt configuration 2 fields.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPIO_IMR.
static const field_t hw_gpio_imr[] =
{
    {
        "IMR",
        "Interrupt Mask bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPIO_ISR.
static const field_t hw_gpio_isr[] =
{
    {
        "ISR",
        "Interrupt status bits - Bit n of this register is asserted (active hig"
        "h) when the active condition (as determined by the corresponding ICR b"
        "it) is detected on the GPIO input and is waiting for service.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPIO_EDGE_SEL.
static const field_t hw_gpio_edge_sel[] =
{
    {
        "GPIO_EDGE_SEL",
        "Edge select.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a GPIO module.
static const reg_t hw_gpio[] =
{
    {
        "DR",
        "The 32-bit GPIO_DR register stores data that is ready to be driven to "
        "the output lines.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpio_dr
    },
    {
        "GDIR",
        "GPIO_GDIR functions as direction control when the IOMUXC is in GPIO mo"
        "de.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpio_gdir
    },
    {
        "PSR",
        "GPIO_PSR is a read-only register.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpio_psr
    },
    {
        "ICR1",
        "GPIO_ICR1 contains 16 two-bit fields, where each field specifies the i"
        "nterrupt configuration for a different input signal.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_gpio_icr1
    },
    {
        "ICR2",
        "GPIO_ICR2 contains 16 two-bit fields, where each field specifies the i"
        "nterrupt configuration for a different input signal.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_gpio_icr2
    },
    {
        "IMR",
        "GPIO_IMR contains masking bits for each interrupt line.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpio_imr
    },
    {
        "ISR",
        "The GPIO_ISR functions as an interrupt status indicator.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpio_isr
    },
    {
        "EDGE_SEL",
        "GPIO_EDGE_SEL may be used to override the ICR registers' configuration"
        ".",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpio_edge_sel
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark GPT
#endif

// Bitfields in register GPT_CR.
static const field_t hw_gpt_cr[] =
{
    {
        "EN",
        "GPT Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENMOD",
        "GPT Enable mode.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DBGEN",
        "GPT debug mode enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAITEN",
        "GPT Wait Mode enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOZEEN",
        "GPT Doze Mode Enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STOPEN",
        "GPT Stop Mode enable.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKSRC",
        "Clock Source select.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRR",
        "Free-Run or Restart mode.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "_24MEN",
        "Enable 24MHz clock input from crystal.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Software reset.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IM1",
        "See IM2",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IM2",
        "IM2 (bits 19-18, Input Capture Channel 2 operating mode)  IM1 (bits 17"
        "-16, Input Capture Channel 1 operating mode)  The IM n bit field deter"
        "mines the transition on the input pin (for Input capture channel n ), "
        "which will trigger a capture event.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OM1",
        "See OM3",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OM2",
        "See OM3",
        23, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OM3",
        "OM3 (bits 28-26) controls the Output Compare Channel 3 operating mode.",
        26, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FO1",
        "See F03",
        29, // LSB
        29, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FO2",
        "See F03",
        30, // LSB
        30, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FO3",
        "FO3 Force Output Compare Channel 3  FO2 Force Output Compare Channel 2"
        "  FO1 Force Output Compare Channel 1  The FO n bit causes the pin acti"
        "on programmed for the timer Output Compare n pin (according to the OM "
        "n bits in this register).",
        31, // LSB
        31, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_PR.
static const field_t hw_gpt_pr[] =
{
    {
        "PRESCALER",
        "Prescaler bits.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRESCALER24M",
        "Prescaler bits.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_SR.
static const field_t hw_gpt_sr[] =
{
    {
        "OF1",
        "See OF3",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OF2",
        "See OF3",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OF3",
        "OF3 Output Compare 3 Flag  OF2 Output Compare 2 Flag  OF1 Output Compa"
        "re 1 Flag  The OF n bit indicates that a compare event has occurred on"
        " Output Compare channel n .",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IF1",
        "See IF2",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IF2",
        "IF2 Input capture 2 Flag  IF1 Input capture 1 Flag  The IF n bit indic"
        "ates that a capture event has occurred on Input Capture channel n .",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROV",
        "Rollover Flag.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_IR.
static const field_t hw_gpt_ir[] =
{
    {
        "OF1IE",
        "See OF3IE",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OF2IE",
        "See OF3IE",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OF3IE",
        "OF3IE Output Compare 3 Interrupt Enable  OF2IE Output Compare 2 Interr"
        "upt Enable  OF1IE Output Compare 1 Interrupt Enable  The OF n IE bit c"
        "ontrols the Output Compare Channel n interrupt.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IF1IE",
        "See IF2IE",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IF2IE",
        "IF2IE Input capture 2 Interrupt Enable  IF1IE Input capture 1 Interrup"
        "t Enable  The IF n IE bit controls the IF n IE Input Capture n Interru"
        "pt Enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROVIE",
        "Rollover Interrupt Enable.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_OCR1.
static const field_t hw_gpt_ocr1[] =
{
    {
        "COMP",
        "Compare Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_OCR2.
static const field_t hw_gpt_ocr2[] =
{
    {
        "COMP",
        "Compare Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_OCR3.
static const field_t hw_gpt_ocr3[] =
{
    {
        "COMP",
        "Compare Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_ICR1.
static const field_t hw_gpt_icr1[] =
{
    {
        "CAPT",
        "Capture Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_ICR2.
static const field_t hw_gpt_icr2[] =
{
    {
        "CAPT",
        "Capture Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPT_CNT.
static const field_t hw_gpt_cnt[] =
{
    {
        "COUNT",
        "Counter Value.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a GPT module.
static const reg_t hw_gpt[] =
{
    {
        "CR",
        "The GPT Control Register (GPT_CR) is used to program and configure GPT"
        " operations.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        18, // Number of bitfields
        hw_gpt_cr
    },
    {
        "PR",
        "The GPT Prescaler Register (GPT_PR) contains bits that determine the d"
        "ivide value of the clock that runs the counter.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_gpt_pr
    },
    {
        "SR",
        "The GPT Status Register (GPT_SR) contains bits that indicate that a co"
        "unter has rolled over, and if any event has occurred on the Input Capt"
        "ure and Output Compare channels.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_gpt_sr
    },
    {
        "IR",
        "The GPT Interrupt Register (GPT_IR) contains bits that control whether"
        " interrupts are generated after rollover, input capture and output com"
        "pare events.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_gpt_ir
    },
    {
        "OCR1",
        "The GPT Compare Register 1 (GPT_OCR1) holds the value that determines "
        "when a compare event will be generated on Output Compare Channel 1.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpt_ocr1
    },
    {
        "OCR2",
        "The GPT Compare Register 2 (GPT_OCR2) holds the value that determines "
        "when a compare event will be generated on Output Compare Channel 2.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpt_ocr2
    },
    {
        "OCR3",
        "The GPT Compare Register 3 (GPT_OCR3) holds the value that determines "
        "when a compare event will be generated on Output Compare Channel 3.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpt_ocr3
    },
    {
        "ICR1",
        "The GPT Input Capture Register 1 (GPT_ICR1) is a read-only register th"
        "at holds the value that was in the counter during the last capture eve"
        "nt on Input Capture Channel 1.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpt_icr1
    },
    {
        "ICR2",
        "The GPT Input capture Register 2 (GPT_ICR2) is a read-only register wh"
        "ich holds the value that was in the counter during the last capture ev"
        "ent on input capture channel 2.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpt_icr2
    },
    {
        "CNT",
        "The GPT Counter Register (GPT_CNT) is the main counter's register.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpt_cnt
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark GPU2D
#endif

// Bitfields in register GPU2D_AQHICLOCKCONTROL.
static const field_t hw_gpu2d_aqhiclockcontrol[] =
{
    {
        "MULTI_PIPE_USE_SINGLE_AXI",
        "Force all the transactions to go to one AXI.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MULTI_PIPE_REG_SELECT",
        "Determines which HI/MC to use while reading registers.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ISOLATE_GPU",
        "Isolate GPU bit",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDLE_VG",
        "VG pipe is idle.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDLE2_D",
        "2D pipe is idle.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDLE3_D",
        "3D pipe is idle.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SOFT_RESET",
        "Soft resets the IP.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_DEBUG_REGISTERS",
        "Disable debug registers.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_RAM_CLOCK_GATING",
        "Disables clock gating for rams.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FSCALE_CMD_LOAD",
        "",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FSCALE_VAL",
        "",
        23, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLK2D_DIS",
        "Disable 2D clock.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLK3D_DIS",
        "Disable 3D clock.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_AQHIIDLE.
static const field_t hw_gpu2d_aqhiidle[] =
{
    {
        "AXI_LP",
        "AXI is in low power mode.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_TS",
        "TS is idle.",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_FP",
        "FP is idle.",
        21, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_IM",
        "IM is idle.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_VG",
        "VG is idle.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_TX",
        "TX is idle.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_RA",
        "RA is idle.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_SE",
        "SE is idle.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_PA",
        "PA is idle.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_SH",
        "SH is idle.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_PE",
        "PE is idle.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_DE",
        "DE is idle.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDLE_FE",
        "FE is idle.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_AQAXICONFIG.
static const field_t hw_gpu2d_aqaxiconfig[] =
{
    {
        "ARCACHE",
        "",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AWCACHE",
        "",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARID",
        "",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AWID",
        "",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_AQAXISTATUS.
static const field_t hw_gpu2d_aqaxistatus[] =
{
    {
        "WR_ERR_ID",
        "",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_ERR_ID",
        "",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DET_WR_ERR",
        "",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DET_RD_ERR",
        "",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_AQINTRACKNOWLEDGE.
static const field_t hw_gpu2d_aqintracknowledge[] =
{
    {
        "INTR_VEC",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_AQINTRENBL.
static const field_t hw_gpu2d_aqintrenbl[] =
{
    {
        "INTR_ENBL_VEC",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_AQIDENT.
static const field_t hw_gpu2d_aqident[] =
{
    {
        "CUSTOMER",
        "Customer value.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TECHNOLOGY",
        "Technology value.",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REVISION",
        "Revision value.",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PRODUCT",
        "Product value.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FAMILY",
        "Family value.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_FEATURES.
static const field_t hw_gpu2d_features[] =
{
    {
        "FE20_BIT_INDEX",
        "Supports 20 bit index.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RS_YUV_TARGET",
        "Supports resolving into YUV target.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BYTE_WRITE_3D",
        "3D PE has byte write capability.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FE20",
        "FE 2.0 is present.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VGTS",
        "VG tessellator is present.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PIPE_VG",
        "VG pipe is present.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MEM32_BIT_SUPPORT",
        "32 bit memory address support.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "YUY2_RENDER_TARGET",
        "YUY2 support in PE and YUY2 to RGB conversion in resolve.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HALF_TX_CACHE",
        "TX cache is half.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HALF_PE_CACHE",
        "PE cache is half.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "YUY2_AVERAGING",
        "YUY2 averaging support in resolve.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NO_SCALER",
        "IP does not have 2D scaler.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BYTE_WRITE_2D",
        "Supports byte write in 2D.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BUFFER_INTERLEAVING",
        "IP supports interleaving depth and color buffers.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NO422_TEXTURE",
        "IP does not have 422 texture input format.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NO_EZ",
        "IP does not have early-Z.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MIN_AREA",
        "IP is configured to have minimum area.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CG",
        "Second level clock gating is available.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "YUV420_TILER",
        "YUV 4:2:0 tiler is available.",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HIGH_DYNAMIC_RANGE",
        "Shows if the IP has HDR support.",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FAST_SCALER",
        "Shows if the IP has HD scaler.",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ETC1_TEXTURE_COMPRESSION",
        "ETC1 texture compression.",
        21, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PIPE_2D",
        "Shows if there is 2D engine.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DC",
        "Shows if there is a display controller in the IP.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MSAA",
        "MSAA support.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "YUV420_FILTER",
        "YUV 4:2:0 support in filter blit.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ZCOMPRESSION",
        "Depth and color compression.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DEBUG_MODE",
        "Debug registers.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DXT_TEXTURE_COMPRESSION",
        "DXT texture compression.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PIPE_3D",
        "3D pipe.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SPECIAL_ANTI_ALIASING",
        "Full-screen anti-aliasing.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FAST_CLEAR",
        "Fast clear.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CHIPID.
static const field_t hw_gpu2d_chipid[] =
{
    {
        "ID",
        "Id.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CHIPREV.
static const field_t hw_gpu2d_chiprev[] =
{
    {
        "REV",
        "Revision.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CHIPDATE.
static const field_t hw_gpu2d_chipdate[] =
{
    {
        "DATE",
        "Date.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CHIPTIME.
static const field_t hw_gpu2d_chiptime[] =
{
    {
        "TIME",
        "Time.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CHIPCUSTOMER.
static const field_t hw_gpu2d_chipcustomer[] =
{
    {
        "GROUP",
        "Group.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "COMPANY",
        "Company.",
        16, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_MINORFEATURES0.
static const field_t hw_gpu2d_minorfeatures0[] =
{
    {
        "FLIP_Y",
        "Y flipping capability is added to resolve.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DUAL_RETURN_BUS",
        "Dual Return Bus from HI to clients.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENDIANNESS_CONFIG",
        "Configurable endianness support.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TEXTURE8_K",
        "Supports 8Kx8K textures.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CORRECT_TEXTURE_CONVERTER",
        "Driver hack is not needed.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SPECIAL_MSAA_LOD",
        "Special LOD calculation when MSAA is on.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FAST_CLEAR_FLUSH",
        "Proper flush is done in fast clear cache.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "_2DPE20",
        "2D PE 2.0 is present.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CORRECT_AUTO_DISABLE",
        "Auto disable in FC is correct.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RENDER_8K",
        "Supports 8K render target.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TILE_STATUS_2BITS",
        "2 bits are used instead of 4 bits for tile status.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SEPARATE_TILE_STATUS_WHEN_INTERLEAVED",
        "Use 2 separate tile status buffers in interleaved mode.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SUPER_TILED_32X32",
        "32x32 super tile is available.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VG_20",
        "Major updates to VG pipe (TS buffer tiling.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TS_EXTENDED_COMMANDS",
        "New commands added to the tessellator.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "COMPRESSION_FIFO_FIXED",
        "If this bit is not set, the FIFO counter should be set to 50.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EXTRA_SHADER_INSTRUCTIONS0",
        "Floor, ceil, and sign instructions are available.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VG_FILTER",
        "VG filter is available.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VG_21",
        "Minor updates to VG pipe (Event generation from VG, TS, PE).",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SHADER_GETS_W",
        "W is sent to SH from RA.",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EXTRA_SHADER_INSTRUCTIONS1",
        "Sqrt, sin, cos instructions are available.",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DEFAULT_REG0",
        "Unavailable registers will return 0.",
        21, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MC_20",
        "New style MC with separate paths for color and depth.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SHADER_MSAA_SIDEBAND",
        "Put the MSAA data into sideband fifo.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BUG_FIXES0",
        "",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VAA",
        "VAA is available or not.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BYPASS_IN_MSAA",
        "Shader supports bypass mode when MSAA is enabled.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HIERARCHICAL_Z",
        "Hierarchiccal Z is supported.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NEW_TEXTURE",
        "New texture unit is available.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    {
        "A8_TARGET_SUPPORT",
        "2D engine supports A8 target.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CORRECT_STENCIL",
        "Correct stencil behavior in depth only.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENHANCE_VR",
        "Enhance VR and add a mode to walk 16 pixels in 16-bit mode in Vertical"
        " pass to improve $ hit rate when rotating 90/270.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CACHECONTROL.
static const field_t hw_gpu2d_cachecontrol[] =
{
    {
        "NOT_USED",
        "Reserved",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_RESETMEMCOUNTERS.
static const field_t hw_gpu2d_resetmemcounters[] =
{
    {
        "RESET",
        "",
        0, // LSB
        31, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALREADS.
static const field_t hw_gpu2d_totalreads[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALWRITES.
static const field_t hw_gpu2d_totalwrites[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CHIPSPECS.
static const field_t hw_gpu2d_chipspecs[] =
{
    {
        "VERTEX_OUTPUT_BUFFER_SIZE",
        "Log2 of vertex output buffer size.",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NUM_PIXEL_PIPES",
        "Number of pixel pipes.",
        4, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NUM_SHADER_CORES",
        "Number of shader cores.",
        7, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VERTEX_CACHE_SIZE",
        "Number of entries in the vertex shader cache.",
        15, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "THREAD_COUNT",
        "Log2 of thread count.",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TEMP_REGISTERS",
        "Log2 of temporary registers.",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "STREAMS",
        "Number of vertex streams.",
        28, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALWRITEBURSTS.
static const field_t hw_gpu2d_totalwritebursts[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALWRITEREQS.
static const field_t hw_gpu2d_totalwritereqs[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALWRITELASTS.
static const field_t hw_gpu2d_totalwritelasts[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALREADBURSTS.
static const field_t hw_gpu2d_totalreadbursts[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALREADREQS.
static const field_t hw_gpu2d_totalreadreqs[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALREADLASTS.
static const field_t hw_gpu2d_totalreadlasts[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_GPOUT0.
static const field_t hw_gpu2d_gpout0[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_GPOUT1.
static const field_t hw_gpu2d_gpout1[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_GPOUT2.
static const field_t hw_gpu2d_gpout2[] =
{
    {
        "COUNT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_AXICONTROL.
static const field_t hw_gpu2d_axicontrol[] =
{
    {
        "WR_FULL_BURST_MODE",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_MINORFEATURES1.
static const field_t hw_gpu2d_minorfeatures1[] =
{
    {
        "TEXTURE_STRIDE",
        "Texture has stride and memory addressing.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BUG_FIXES2",
        "",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BUG_FIXES1",
        "",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VG_DOUBLE_BUFFER",
        "Double buffering support for VG (second TS-->VG semaphore is present).",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "V2_COMPRESSION",
        "V2 compression.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RSUV_SWIZZLE",
        "Resolve UV swizzle.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALCYCLES.
static const field_t hw_gpu2d_totalcycles[] =
{
    {
        "CYCLES",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_TOTALIDLECYLES.
static const field_t hw_gpu2d_totalidlecyles[] =
{
    {
        "CYCLES",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_CHIPSPECS2.
static const field_t hw_gpu2d_chipspecs2[] =
{
    {
        "CYCLES",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_MODULEPOWERCONTROLS.
static const field_t hw_gpu2d_modulepowercontrols[] =
{
    {
        "TURN_OFF_COUNTER",
        "Counter value for clock gating the module if the module is idle for th"
        "is amount of clock cycles.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TURN_ON_COUNTER",
        "Number of clock cycles to wait after turning on the clock.",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_STARVE_MODULE_CLOCK_GATING",
        "Disables module level clock gating for starve/idle condition.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_STALL_MODULE_CLOCK_GATING",
        "Disables module level clock gating for stall condition.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_MODULE_CLOCK_GATING",
        "Enables module level clock gating.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_MODULEPOWERMODULECONTROL.
static const field_t hw_gpu2d_modulepowermodulecontrol[] =
{
    {
        "DISABLE_MODULE_CLOCK_GATING_TS",
        "Disables module level clock gating for TS.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_FP",
        "Disables module level clock gating for FP.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_IM",
        "Disables module level clock gating for IM.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_VG",
        "Disables module level clock gating for VG.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_TX",
        "Disables module level clock gating for TX.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_RA",
        "Disables module level clock gating for RA.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_SE",
        "Disables module level clock gating for SE.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_PA",
        "Disables module level clock gating for PA.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_SH",
        "Disables module level clock gating for SH.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_PE",
        "Disables module level clock gating for PE.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_DE",
        "Disables module level clock gating for DE.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_MODULE_CLOCK_GATING_FE",
        "Disables module level clock gating for FE.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register GPU2D_MODULEPOWERMODULESTATUS.
static const field_t hw_gpu2d_modulepowermodulestatus[] =
{
    {
        "MODULE_CLOCK_GATED_TS",
        "Module level clock gating is ON for TS.",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_FP",
        "Module level clock gating is ON for FP.",
        21, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_IM",
        "Module level clock gating is ON for IM.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_VG",
        "Module level clock gating is ON for VG.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_TX",
        "Module level clock gating is ON for TX.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_RA",
        "Module level clock gating is ON for RA.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_SE",
        "Module level clock gating is ON for SE.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_PA",
        "Module level clock gating is ON for PA.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_SH",
        "Module level clock gating is ON for SH.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_PE",
        "Module level clock gating is ON for PE.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_DE",
        "Module level clock gating is ON for DE.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MODULE_CLOCK_GATED_FE",
        "Module level clock gating is ON for FE.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a GPU2D module.
static const reg_t hw_gpu2d[] =
{
    {
        "AQHICLOCKCONTROL",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        13, // Number of bitfields
        hw_gpu2d_aqhiclockcontrol
    },
    {
        "AQHIIDLE",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        false, // Writable
        13, // Number of bitfields
        hw_gpu2d_aqhiidle
    },
    {
        "AQAXICONFIG",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_gpu2d_aqaxiconfig
    },
    {
        "AQAXISTATUS",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_gpu2d_aqaxistatus
    },
    {
        "AQINTRACKNOWLEDGE",
        "Interrupt acknowledge register.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_aqintracknowledge
    },
    {
        "AQINTRENBL",
        "Interrupt enable register.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_aqintrenbl
    },
    {
        "AQIDENT",
        "Identification register.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        false, // Writable
        5, // Number of bitfields
        hw_gpu2d_aqident
    },
    {
        "FEATURES",
        "Shows which features are enabled in this chip.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        false, // Writable
        32, // Number of bitfields
        hw_gpu2d_features
    },
    {
        "CHIPID",
        "Shows the ID for the chip in BCD.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_chipid
    },
    {
        "CHIPREV",
        "Shows the revision for the chip in BCD.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_chiprev
    },
    {
        "CHIPDATE",
        "Shows the release date for the IP.",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_chipdate
    },
    {
        "CHIPTIME",
        "Shows the release time for the IP.",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_chiptime
    },
    {
        "CHIPCUSTOMER",
        "Shows the customer and group for the IP.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_gpu2d_chipcustomer
    },
    {
        "MINORFEATURES0",
        "Shows which minor features are enabled in this chip.",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        false, // Writable
        32, // Number of bitfields
        hw_gpu2d_minorfeatures0
    },
    {
        "CACHECONTROL",
        "",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_cachecontrol
    },
    {
        "RESETMEMCOUNTERS",
        "Writing 1 will reset the counters and stop counting.",
        4, // Width in bytes
        0x0000003c, // Base address offset
        false, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_resetmemcounters
    },
    {
        "TOTALREADS",
        "Total reads in terms of 64bits.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalreads
    },
    {
        "TOTALWRITES",
        "Total writes in terms of 64bits.",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalwrites
    },
    {
        "CHIPSPECS",
        "Specs for the chip.",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_gpu2d_chipspecs
    },
    {
        "TOTALWRITEBURSTS",
        "Total write Data Count in terms of 64bits.",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalwritebursts
    },
    {
        "TOTALWRITEREQS",
        "Total write Request Count.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalwritereqs
    },
    {
        "TOTALWRITELASTS",
        "Total WLAST Count.",
        4, // Width in bytes
        0x00000054, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalwritelasts
    },
    {
        "TOTALREADBURSTS",
        "Total Read Data Count in terms of 64bits.",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalreadbursts
    },
    {
        "TOTALREADREQS",
        "Total Read Request Count.",
        4, // Width in bytes
        0x0000005c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalreadreqs
    },
    {
        "TOTALREADLASTS",
        "Total RLAST Count.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalreadlasts
    },
    {
        "GPOUT0",
        "General Purpose output register0.",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_gpout0
    },
    {
        "GPOUT1",
        "General Purpose output register1.",
        4, // Width in bytes
        0x00000068, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_gpout1
    },
    {
        "GPOUT2",
        "General Purpose output register2.",
        4, // Width in bytes
        0x0000006c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_gpout2
    },
    {
        "AXICONTROL",
        "Special Handling on AXI Bus",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_axicontrol
    },
    {
        "MINORFEATURES1",
        "Shows which features are enabled in this chip.",
        4, // Width in bytes
        0x00000074, // Base address offset
        true, // Readable
        false, // Writable
        6, // Number of bitfields
        hw_gpu2d_minorfeatures1
    },
    {
        "TOTALCYCLES",
        "Total cycles.",
        4, // Width in bytes
        0x00000078, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalcycles
    },
    {
        "TOTALIDLECYLES",
        "Total cycles where the GPU is idle.",
        4, // Width in bytes
        0x0000007c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_totalidlecyles
    },
    {
        "CHIPSPECS2",
        "Specs for the chip.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_gpu2d_chipspecs2
    },
    {
        "MODULEPOWERCONTROLS",
        "The Power Management register set has just a few registers for control"
        "ling clock gating within the core.",
        4, // Width in bytes
        0x00000084, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_gpu2d_modulepowercontrols
    },
    {
        "MODULEPOWERMODULECONTROL",
        "Module level control registers.",
        4, // Width in bytes
        0x00000088, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_gpu2d_modulepowermodulecontrol
    },
    {
        "MODULEPOWERMODULESTATUS",
        "Module level control status.",
        4, // Width in bytes
        0x0000008c, // Base address offset
        true, // Readable
        false, // Writable
        12, // Number of bitfields
        hw_gpu2d_modulepowermodulestatus
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark I2C
#endif

// Bitfields in register I2C_IADR.
static const field_t hw_i2c_iadr[] =
{
    {
        "ADR",
        "Slave address.",
        1, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register I2C_IFDR.
static const field_t hw_i2c_ifdr[] =
{
    {
        "IC",
        "I2C clock rate.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register I2C_I2CR.
static const field_t hw_i2c_i2cr[] =
{
    {
        "RSTA",
        "Repeat start.",
        2, // LSB
        2, // MSB
        false, // Readable
        true // Writable
    },
    {
        "TXAK",
        "Transmit acknowledge enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MTX",
        "Transmit/receive mode select bit.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MSTA",
        "Master/slave mode select bit.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IIEN",
        "I2C interrupt enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IEN",
        "I2C enable.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register I2C_I2SR.
static const field_t hw_i2c_i2sr[] =
{
    {
        "RXAK",
        "Received acknowledge.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IIF",
        "I2C interrupt.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRW",
        "Slave read/write.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IAL",
        "Arbitration lost.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IBB",
        "I2C bus busy bit.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IAAS",
        "I2C addressed as a slave bit.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ICF",
        "Data transferring bit.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register I2C_I2DR.
static const field_t hw_i2c_i2dr[] =
{
    {
        "DATA",
        "Data Byte.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a I2C module.
static const reg_t hw_i2c[] =
{
    {
        "IADR",
        "",
        2, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_i2c_iadr
    },
    {
        "IFDR",
        "The I2C_IFDR provides a programmable prescaler to configure the clock "
        "for bit-rate selection.",
        2, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_i2c_ifdr
    },
    {
        "I2CR",
        "The I2C_I2CR is used to enable the I2C and the I2C interrupt.",
        2, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_i2c_i2cr
    },
    {
        "I2SR",
        "The I2C_I2SR contains bits that indicate transaction direction and sta"
        "tus.",
        2, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_i2c_i2sr
    },
    {
        "I2DR",
        "In master-receive mode, reading the data register allows a read to occ"
        "ur and initiates the next byte to be received.",
        2, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_i2c_i2dr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark IOMUXC
#endif

// Bitfields in register IOMUXC_GPR0.
static const field_t hw_iomuxc_gpr0[] =
{
    {
        "DMAREQ_MUX_SEL0",
        "Selects between two possible sources for SDMA_EVENT[2]:",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAREQ_MUX_SEL1",
        "Selects between two possible sources for SDMA_EVENT[3]:",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAREQ_MUX_SEL2",
        "Selects between two possible sources for SDMA_EVENT[4]:",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAREQ_MUX_SEL3",
        "Selects between two possible sources for SDMA_EVENT[5]:",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAREQ_MUX_SEL4",
        "Selects between two possible sources for SDMA_EVENT[10]:",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAREQ_MUX_SEL5",
        "Selects between two possible sources for SDMA_EVENT[9]:",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAREQ_MUX_SEL6",
        "Selects between two possible sources for SDMA_EVENT[23]:",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAREQ_MUX_SEL7",
        "Selects between two possible sources for SDMA_EVENT[14]:",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR1.
static const field_t hw_iomuxc_gpr1[] =
{
    {
        "ACT_CS0",
        "See description for ADDRS3[10]",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRS0",
        "See description for ADDRS3[10]",
        1, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACT_CS1",
        "See description for ADDRS3[10]",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRS1",
        "See description for ADDRS3[10]",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACT_CS2",
        "See description for ADDRS3[10]",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRS2",
        "See description for ADDRS3[10]",
        7, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACT_CS3",
        "See description for ADDRS3[10]",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRS3",
        "Active Chip Select and Address Space.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GINT",
        "Global interrupt \"0\" bit (connected to ARM IRQ#0 and GPC)",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENET_CLK_SEL",
        "choose enet reference clk mode",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USB_EXP_MODE",
        "USB Exposure mode",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADD_DS",
        "set to 0 will make output driver ~10% stronger at highest strength (DS"
        "E=111) for use in case if IO buffer operation at WCS and 200 MHz is ma"
        "rginal",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENET_CLK_SEL_FROM_ANALOG_LOOPBACK",
        "When ENET_CLK_SEL(GPR1[14]) is program to 0(means get enet txclk from "
        "internal anatop), use these 2 bits to select from 4 options of loopbac"
        "k path.",
        17, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EXC_MON",
        "Exclusive monitor response select of illegal command (of lal gaskets, "
        "except MMDC)",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR2.
static const field_t hw_iomuxc_gpr2[] =
{
    {
        "EPDC_MEM_EN_POWERSAVING",
        "enable power saving features on epdc memory",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_MEM_SHUTDOWN",
        "set to bring memory to shutdown state (most power saving state, Shut D"
        "own periphery and core, no memory retention)",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_MEM_DEEPSLEEP",
        "control how memory enter Deep Sleep mode (shutdown periphery power, bu"
        "t maintain memory contents, outputs of memory are pulled low)",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_MEM_LIGHTSLEEP",
        "set to bring memory to light sleep state (Low leakage mode, maintain m"
        "emory contents, no change to memory output)",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDC_MEM_EN_POWERSAVING",
        "enable power saving features on SPDC memory",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDC_MEM_SHUTDOWN",
        "set to bring memory to shutdown state (most power saving state, Shut D"
        "own periphery and core, no memory retention)",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDC_MEM_DEEPSLEEP",
        "control how memory enter Deep Sleep mode (shutdown periphery power, bu"
        "t maintain memory contents, outputs of memory are pulled low)",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPDC_MEM_LIGHTSLEEP",
        "set to bring memory to light sleep state (Low leakage mode, maintain m"
        "emory contents, no change to memory output)",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_MEM_EN_POWERSAVING",
        "enable power saving features on PXP memory",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_MEM_SHUTDOWN",
        "set to bring memory to shutdown state (most power saving state, Shut D"
        "own periphery and core, no memory retention)",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_MEM_DEEPSLEEP",
        "control how memory enter Deep Sleep mode (shutdown periphery power, bu"
        "t maintain memory contents, outputs of memory are pulled low)",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_MEM_LIGHTSLEEP",
        "set to bring memory to light sleep state (Low leakage mode, maintain m"
        "emory contents, no change to memory output)",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_MEM_EN_POWERSAVING",
        "enable power saving features on LCDIF memory",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_MEM_SHUTDOWN",
        "set to bring memory to shutdown state (most power saving state, Shut D"
        "own periphery and core, no memory retention)",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_MEM_DEEPSLEEP",
        "control how memory enter Deep Sleep mode (shutdown periphery power, bu"
        "t maintain memory contents, outputs of memory are pulled low)",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_MEM_LIGHTSLEEP",
        "set to bring memory to light sleep state (Low leakage mode, maintain m"
        "emory contents, no change to memory output)",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCP_MEM_EN_POWERSAVING",
        "enable power saving features on DCP memory",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCP_MEM_SHUTDOWN",
        "set to bring memory to shutdown state (most power saving state, Shut D"
        "own periphery and core, no memory retention)",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCP_MEM_DEEPSLEEP",
        "control how memory enter Deep Sleep mode (shutdown periphery power, bu"
        "t maintain memory contents, outputs of memory are pulled low)",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCP_MEM_LIGHTSLEEP",
        "set to bring memory to light sleep state (Low leakage mode, maintain m"
        "emory contents, no change to memory output)",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2_MEM_EN_POWERSAVING",
        "enable power saving features on L2 memory",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2_MEM_SHUTDOWN",
        "set to bring memory to shutdown state (most power saving state, Shut D"
        "own periphery and core, no memory retention)",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2_MEM_DEEPSLEEP",
        "control how memory enter Deep Sleep mode (shutdown periphery power, bu"
        "t maintain memory contents, outputs of memory are pulled low)",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2_MEM_LIGHTSLEEP",
        "set to bring memory to light sleep state (Low leakage mode, maintain m"
        "emory contents, no change to memory output)",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR3.
static const field_t hw_iomuxc_gpr3[] =
{
    {
        "OCRAM_L2_CTL",
        "OCRAM_L2_CTL[3] write address pipeline control bit.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_L2_STATUS",
        "This field shows the OCRAM_L2 pipeline settings status, controlled by "
        "OCRAM_L2_CTL[3:0] bits respectively.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TZASC1_BOOT_LOCK",
        "TZASC-1 secure boot lock",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CORE_DBG_ACK_EN",
        "Mask control of Core debug acknowledge to global debug acknowledge",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_STATUS",
        "This field shows the OCRAM pipeline settings status, controlled by OCR"
        "AM_CTL[24:21] bits respectively.",
        17, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OCRAM_CTL",
        "OCRAM_CTL[24] write address pipeline control bit.",
        21, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHCX_RD_CACHE_CTL",
        "Control uSDHCx [1-4] blocks cacheable attribute of AXI read transactio"
        "ns",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USDHCX_WR_CACHE_CTL",
        "Control uSDHCx [1-4] blocks cacheable attribute of AXI write transacti"
        "ons",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR4.
static const field_t hw_iomuxc_gpr4[] =
{
    {
        "SOC_VERSION",
        "This is status (read only) field.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RNGB_STOP_ACK",
        "RNGB stop acknowledge.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SDMA_STOP_ACK",
        "SDMA stop acknowledge.",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR5.
static const field_t hw_iomuxc_gpr5[] =
{
    {
        "ARM_WFI",
        "ARM WFI event out indicating on WFI state of the cores (these are stat"
        "us, read only bits)",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ARM_WFE",
        "ARM WFE event out indication on WFE state of the cores (these are stat"
        "us, read only bits)",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "L2_CLK_STOP",
        "L2 cache clock stop indication (this is a status, read only bit]",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR6.
static const field_t hw_iomuxc_gpr6[] =
{
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR7.
static const field_t hw_iomuxc_gpr7[] =
{
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR8.
static const field_t hw_iomuxc_gpr8[] =
{
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR9.
static const field_t hw_iomuxc_gpr9[] =
{
    {
        "TZASC1_BYP",
        "TZASC-1 BYPASS MUX control",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR10.
static const field_t hw_iomuxc_gpr10[] =
{
    {
        "DBG_EN",
        "ARM non secure (non-invasive) debug enable",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DBG_CLK_EN",
        "ARM Debug clock enable",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEC_ERR_RESP",
        "Security error response enable for all security gaskets (on both AHB a"
        "nd AXI busses)",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_L2_TZ_EN",
        "OCRAM_L2 TrustZone (TZ) enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_L2_TZ_ADDR",
        "OCRAM_L2 TrustZone (TZ) start address.",
        4, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_TZ_EN",
        "OCRAM TrustZone (TZ) enable.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCRAM_TZ_ADDR",
        "OCRAM TrustZone (TZ) start address.",
        11, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_DBG_EN",
        "Lock DBG_EN field for changes.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_DBG_CLK_EN",
        "Lock DBG_CLK_EN field for changes.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_SEC_ERR_RESP",
        "Lock SEC_ERR_RESP field for changes.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_OCRAM_L2_TZ_EN",
        "Lock OCRAM_L2_TZ_EN field for changes.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_OCRAM_L2_TZ_ADDR",
        "Lock OCRAM_L2_TZ_ADDR field for changes.",
        20, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_OCRAM_TZ_EN",
        "Lock OCRAM_TZ_EN field for changes.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK_OCRAM_TZ_ADDR",
        "Lock OCRAM_TZ_ADDR field for changes.",
        27, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR11.
static const field_t hw_iomuxc_gpr11[] =
{
    {
        "OCRAM_L2_EN",
        "set to use L2 cache as ocram",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LOCK_OCRAM_L2_EN",
        "lock ocram_l2 enable bit",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR12.
static const field_t hw_iomuxc_gpr12[] =
{
    {
        "GPU_AWQOS0",
        "QOS control for write channel of gpu port m_g_0",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU_ARQOS0",
        "QOS control for read channel of gpu port m_g_0",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU_AWQOS1",
        "QOS control for write channel of gpu port m_g_1",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU_ARQOS1",
        "QOS control for read channel of gpu port m_g_1",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU_AWQOS2",
        "QOS control for write channel of gpu port m_g_2",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPU_ARQOS2",
        "QOS control for read channel of gpu port m_g_2",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARMP_APB_CLK_EN",
        "ARM platform APB clock enable",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARMP_ATB_CLK_EN",
        "ARM platform ATB clock enable",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARMP_AHB_CLK_EN",
        "ARM platform AHB clock enable",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARMP_IPG_CLK_EN",
        "ARM platform IPG clock enable",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCP_KEY_SEL",
        "select 128bit dcp key from 256bit's key from snvs/ocotp",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPR13.
static const field_t hw_iomuxc_gpr13[] =
{
    {
        "PXP_RD_CACHE_VAL",
        "PXP block cacheable attribute value of AXI read transactions  The valu"
        "e of PXP_RD_CACHE_VAL is affecting the transactions only if PXP_RD_CAC"
        "HE_SEL is set.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_WR_CACHE_VAL",
        "PXP block cacheable attribute value of AXI write transactions  The val"
        "ue of PXP_WR_CACHE_VAL is affecting the transactions only if PXP_WR_CA"
        "CHE_SEL is set.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_RD_CACHE_SEL",
        "This bit selects the cacheable attribute of PXP AXI read transcations)",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PXP_WR_CACHE_SEL",
        "This bit selects the cacheable attribute of PXP AXI write transcations",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_RD_CACHE_VAL",
        "EPDC block cacheable attribute value of AXI read transactions  The val"
        "ue of EPDC_RD_CACHE_VAL is affecting the transactions only if EPDC_RD_"
        "CACHE_SEL is set.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_WR_CACHE_VAL",
        "EPDC block cacheable attribute value of AXI write transactions  The va"
        "lue of EPDC_WR_CACHE_VAL is affecting the transactions only if EPDC_WR"
        "_CACHE_SEL is set.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_RD_CACHE_SEL",
        "This bit selects the cacheable attribute of EPDC AXI read transcations"
        ")",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_WR_CACHE_SEL",
        "This bit selects the cacheable attribute of EPDC AXI write transcation"
        "s",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_RD_CACHE_VAL",
        "LCDIF block cacheable attribute value of AXI read transactions  The va"
        "lue of LCDIF_RD_CACHE_VAL is affecting the transactions only if LCDIF_"
        "RD_CACHE_SEL is set.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_RD_CACHE_SEL",
        "This bit selects the cacheable attribute of LCDIF AXI read transcation"
        "s)",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDMA_STOP_REQ",
        "SDMA stop request",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_AUD_MCLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_aud_mclk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_AUD_RXC.
static const field_t hw_iomuxc_sw_mux_ctl_pad_aud_rxc[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_AUD_RXD.
static const field_t hw_iomuxc_sw_mux_ctl_pad_aud_rxd[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_AUD_RXFS.
static const field_t hw_iomuxc_sw_mux_ctl_pad_aud_rxfs[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_AUD_TXC.
static const field_t hw_iomuxc_sw_mux_ctl_pad_aud_txc[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_AUD_TXD.
static const field_t hw_iomuxc_sw_mux_ctl_pad_aud_txd[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_AUD_TXFS.
static const field_t hw_iomuxc_sw_mux_ctl_pad_aud_txfs[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI1_MISO.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi1_miso[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI1_MOSI.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi1_mosi[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI1_SCLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi1_sclk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI1_SS0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi1_ss0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI2_MISO.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi2_miso[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI2_MOSI.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi2_mosi[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI2_SCLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi2_sclk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_ECSPI2_SS0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ecspi2_ss0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_BDR0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_bdr0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_BDR1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_bdr1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D10.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d10[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D11.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d11[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D12.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d12[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D13.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d13[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D14.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d14[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D15.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d15[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D4.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d4[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D5.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d5[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D6.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d6[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D7.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d7[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D8.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d8[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_D9.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_d9[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_GDCLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_gdclk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_GDOE.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_gdoe[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_GDRL.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_gdrl[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_GDSP.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_gdsp[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRCOM.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrcom[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRCTRL0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRCTRL1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRCTRL2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRCTRL3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRINT.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrint[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRSTAT.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrstat[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_PWRWAKEUP.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_pwrwakeup[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDCE0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdce0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDCE1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdce1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDCE2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdce2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDCE3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdce3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDCLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdclk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDLE.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdle[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDOE.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdoe[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_SDSHR.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_sdshr[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_VCOM0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_vcom0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_EPDC_VCOM1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_epdc_vcom1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_CRS_DV.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_crs_dv[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_MDC.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_mdc[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_MDIO.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_mdio[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_REF_CLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_ref_clk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_RX_ER.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_rx_er[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_RXD0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_rxd0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_RXD1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_rxd1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_TX_CLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_tx_clk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_TX_EN.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_tx_en[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_TXD0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_txd0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_FEC_TXD1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_fec_txd1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_HSIC_DAT.
static const field_t hw_iomuxc_sw_mux_ctl_pad_hsic_dat[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_HSIC_STROBE.
static const field_t hw_iomuxc_sw_mux_ctl_pad_hsic_strobe[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_I2C1_SCL.
static const field_t hw_iomuxc_sw_mux_ctl_pad_i2c1_scl[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_I2C1_SDA.
static const field_t hw_iomuxc_sw_mux_ctl_pad_i2c1_sda[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_I2C2_SCL.
static const field_t hw_iomuxc_sw_mux_ctl_pad_i2c2_scl[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_I2C2_SDA.
static const field_t hw_iomuxc_sw_mux_ctl_pad_i2c2_sda[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL4.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col4[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL5.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col5[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL6.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col6[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_COL7.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_col7[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW4.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row4[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW5.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row5[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW6.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row6[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_KEY_ROW7.
static const field_t hw_iomuxc_sw_mux_ctl_pad_key_row7[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_CLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_clk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT10.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat10[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT11.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat11[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT12.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat12[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT13.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat13[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT14.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat14[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT15.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat15[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT16.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat16[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT17.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat17[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT18.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat18[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT19.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat19[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT20.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat20[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT21.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat21[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT22.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat22[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT23.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat23[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT4.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat4[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT5.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat5[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT6.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat6[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT7.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat7[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT8.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat8[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_DAT9.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_dat9[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_ENABLE.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_enable[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_HSYNC.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_hsync[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_RESET.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_reset[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_LCD_VSYNC.
static const field_t hw_iomuxc_sw_mux_ctl_pad_lcd_vsync[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_PWM1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_pwm1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_REF_CLK_24M.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ref_clk_24m[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_REF_CLK_32K.
static const field_t hw_iomuxc_sw_mux_ctl_pad_ref_clk_32k[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_CLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_clk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_CMD.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_cmd[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT4.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat4[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT5.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat5[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT6.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat6[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD1_DAT7.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd1_dat7[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_CLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_clk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_CMD.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_cmd[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT4.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat4[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT5.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat5[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT6.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat6[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_DAT7.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_dat7[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD2_RST.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd2_rst[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD3_CLK.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd3_clk[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD3_CMD.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd3_cmd[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD3_DAT0.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd3_dat0[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD3_DAT1.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd3_dat1[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD3_DAT2.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd3_dat2[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_SD3_DAT3.
static const field_t hw_iomuxc_sw_mux_ctl_pad_sd3_dat3[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_UART1_RXD.
static const field_t hw_iomuxc_sw_mux_ctl_pad_uart1_rxd[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_UART1_TXD.
static const field_t hw_iomuxc_sw_mux_ctl_pad_uart1_txd[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_MUX_CTL_PAD_WDOG_B.
static const field_t hw_iomuxc_sw_mux_ctl_pad_wdog_b[] =
{
    {
        "MUX_MODE",
        "MUX Mode Select Field.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SION",
        "Software Input On Field.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_AUD_MCLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_aud_mclk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: AUD_MCLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: AUD_MCLK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: AUD_MCLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: AUD_MC"
        "LK.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: AUD_M"
        "CLK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: AUD_M"
        "CLK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " AUD_MCLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: AUD_MC"
        "LK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: AUD_M"
        "CLK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_AUD_RXC.
static const field_t hw_iomuxc_sw_pad_ctl_pad_aud_rxc[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: AUD_RXC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: AUD_RXC.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: AUD_RXC.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: AUD_RX"
        "C.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: AUD_R"
        "XC.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: AUD_R"
        "XC.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " AUD_RXC.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: AUD_RX"
        "C.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: AUD_R"
        "XC.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_AUD_RXD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_aud_rxd[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: AUD_RXD.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: AUD_RXD.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: AUD_RXD.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: AUD_RX"
        "D.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: AUD_R"
        "XD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: AUD_R"
        "XD.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " AUD_RXD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: AUD_RX"
        "D.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: AUD_R"
        "XD.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_AUD_RXFS.
static const field_t hw_iomuxc_sw_pad_ctl_pad_aud_rxfs[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: AUD_RXFS.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: AUD_RXFS.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: AUD_RXFS.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: AUD_RX"
        "FS.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: AUD_R"
        "XFS.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: AUD_R"
        "XFS.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " AUD_RXFS.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: AUD_RX"
        "FS.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: AUD_R"
        "XFS.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_AUD_TXC.
static const field_t hw_iomuxc_sw_pad_ctl_pad_aud_txc[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: AUD_TXC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: AUD_TXC.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: AUD_TXC.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: AUD_TX"
        "C.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: AUD_T"
        "XC.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: AUD_T"
        "XC.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " AUD_TXC.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: AUD_TX"
        "C.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: AUD_T"
        "XC.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_AUD_TXD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_aud_txd[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: AUD_TXD.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: AUD_TXD.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: AUD_TXD.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: AUD_TX"
        "D.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: AUD_T"
        "XD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: AUD_T"
        "XD.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " AUD_TXD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: AUD_TX"
        "D.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: AUD_T"
        "XD.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_AUD_TXFS.
static const field_t hw_iomuxc_sw_pad_ctl_pad_aud_txfs[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: AUD_TXFS.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: AUD_TXFS.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: AUD_TXFS.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: AUD_TX"
        "FS.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: AUD_T"
        "XFS.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: AUD_T"
        "XFS.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " AUD_TXFS.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: AUD_TX"
        "FS.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: AUD_T"
        "XFS.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a0[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A0.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a1[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A1.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A10.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a10[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A10.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "10.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A10.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A11.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a11[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A11.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "11.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A11.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A12.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a12[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A12.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "12.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A12.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A13.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a13[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A13.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "13.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A13.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A14.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a14[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A14.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "14.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A14.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A15.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a15[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A15.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "15.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A15.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a2[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A2.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A2.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a3[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A3.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A3.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A4.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a4[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A4.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "4.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A4.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A5.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a5[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A5.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "5.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A5.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A6.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a6[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A6.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "6.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A6.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A7.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a7[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A7.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "7.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A7.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A8.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a8[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A8.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "8.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A8.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_A9.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_a9[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "A9.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_A"
        "9.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_A9.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_CAS.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_cas[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_CAS.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "CAS.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_C"
        "AS.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_CAS.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_CS0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_cs0[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_CTLDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "CS0.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_C"
        "S0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_CS0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_CS1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_cs1[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_CTLDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "CS1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_C"
        "S1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_CS1.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_DQM0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_dqm0[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_DQM0"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "DQM0.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_D"
        "QM0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_DQM0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_DQM1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_dqm1[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_DQM1"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "DQM1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_D"
        "QM1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_DQM1.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_DQM2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_dqm2[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_DQM2"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "DQM2.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_D"
        "QM2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_DQM2.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_DQM3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_dqm3[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_DQM3"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "DQM3.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_D"
        "QM3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_DQM3.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_RAS.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_ras[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_RAS.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "RAS.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_R"
        "AS.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_RAS.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_RESET.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_reset[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_RESE"
        "T.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "RESET.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "RESET.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "RESET.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_RESET.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_R"
        "ESET.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_RESET.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Select one out of next values for pad: DRAM_RESET.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDBA0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdba0[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDBA0.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DBA0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDBA0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDBA1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdba1[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_ADDDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDBA1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DBA1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDBA1.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDBA2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdba2[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_CTLDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDBA2.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDBA2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDBA2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DBA2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDBA2.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDCKE0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdcke0[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_CTLDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDCKE0.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDCKE0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDCKE0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDCKE0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DCKE0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDCKE0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDCKE1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdcke1[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_CTLDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDCKE1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDCKE1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDCKE1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDCKE1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DCKE1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDCKE1.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDCLK_0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdclk_0[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_SDCL"
        "K_0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDCLK_0.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DCLK_0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDCLK_0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDODT0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdodt0[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_SDOD"
        "T0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDODT0.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDODT0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDODT0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDODT0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DODT0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDODT0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDODT1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdodt1[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_SDOD"
        "T1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDODT1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDODT1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDODT1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDODT1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DODT1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDODT1.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDQS0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdqs0[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_SDQS"
        "0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Read Only Field",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDQS0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDQS0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDQS0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Read Only Field  Can be configured using Grou"
        "p Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRHYS",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Read Only Field  Can be configured using "
        "Group Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRMODE_CTL",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDQS1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdqs1[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_SDQS"
        "1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Read Only Field",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDQS1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDQS1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDQS1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Read Only Field  Can be configured using Grou"
        "p Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRHYS",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Read Only Field  Can be configured using "
        "Group Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRMODE_CTL",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDQS2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdqs2[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_SDQS"
        "2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Read Only Field",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDQS2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDQS2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDQS2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Read Only Field  Can be configured using Grou"
        "p Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRHYS",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Read Only Field  Can be configured using "
        "Group Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRMODE_CTL",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDQS3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdqs3[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: DRAM_SDQS"
        "3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Read Only Field",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: DRAM_"
        "SDQS3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: DRAM_"
        "SDQS3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " DRAM_SDQS3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Read Only Field  Can be configured using Grou"
        "p Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRHYS",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Read Only Field  Can be configured using "
        "Group Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRMODE_CTL",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_DRAM_SDWE.
static const field_t hw_iomuxc_sw_pad_ctl_pad_dram_sdwe[] =
{
    {
        "DSE",
        "Drive Strength Field  Read Only Field  Can be configured using Group C"
        "ontrol Register: IOMUXC_SW_PAD_CTL_GRP_CTLDS",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Select one out of next values for pad: DRAM_"
        "SDWE.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPKE",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field  Can be configured using Gro"
        "up Control Register: IOMUXC_SW_PAD_CTL_GRP_DDRPK",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: DRAM_S"
        "DWE.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: DR"
        "AM_SDWE.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Read Only Field  Can be configured using Group Con"
        "trol Register: IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI1_MISO.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi1_miso[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI1_MISO.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI1_MI"
        "SO.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI1_MISO.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI1"
        "_MISO.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "1_MISO.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "1_MISO.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI1_MISO.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI1"
        "_MISO.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "1_MISO.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI1_MOSI.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi1_mosi[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI1_MOSI.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI1_MO"
        "SI.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI1_MOSI.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI1"
        "_MOSI.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "1_MOSI.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "1_MOSI.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI1_MOSI.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI1"
        "_MOSI.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "1_MOSI.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI1_SCLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi1_sclk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI1_SCLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI1_SC"
        "LK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI1_SCLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI1"
        "_SCLK.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "1_SCLK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "1_SCLK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI1_SCLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI1"
        "_SCLK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "1_SCLK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI1_SS0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi1_ss0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI1_SS0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI1_SS"
        "0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI1_SS0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI1"
        "_SS0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "1_SS0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "1_SS0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI1_SS0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI1"
        "_SS0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "1_SS0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI2_MISO.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi2_miso[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI2_MISO.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI2_MI"
        "SO.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI2_MISO.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI2"
        "_MISO.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "2_MISO.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "2_MISO.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI2_MISO.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI2"
        "_MISO.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "2_MISO.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI2_MOSI.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi2_mosi[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI2_MOSI.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI2_MO"
        "SI.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI2_MOSI.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI2"
        "_MOSI.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "2_MOSI.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "2_MOSI.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI2_MOSI.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI2"
        "_MOSI.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "2_MOSI.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI2_SCLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi2_sclk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI2_SCLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI2_SC"
        "LK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI2_SCLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI2"
        "_SCLK.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "2_SCLK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "2_SCLK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI2_SCLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI2"
        "_SCLK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "2_SCLK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_ECSPI2_SS0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ecspi2_ss0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: ECSPI2_SS0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: ECSPI2_SS"
        "0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: ECSPI2_SS0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: ECSPI2"
        "_SS0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: ECSPI"
        "2_SS0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: ECSPI"
        "2_SS0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " ECSPI2_SS0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: ECSPI2"
        "_SS0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: ECSPI"
        "2_SS0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_BDR0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_bdr0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_BDR0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_BDR0"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_BDR0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_B"
        "DR0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "BDR0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "BDR0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_BDR0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_B"
        "DR0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "BDR0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_BDR1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_bdr1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_BDR1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_BDR1"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_BDR1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_B"
        "DR1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "BDR1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "BDR1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_BDR1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_B"
        "DR1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "BDR1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D10.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d10[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D10.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D10.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D10.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "10.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D10.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D10.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D10.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "10.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D10.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D11.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d11[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D11.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D11.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D11.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "11.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D11.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D11.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D11.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "11.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D11.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D12.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d12[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D12.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D12.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D12.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "12.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D12.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D12.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D12.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "12.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D12.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D13.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d13[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D13.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D13.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D13.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "13.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D13.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D13.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D13.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "13.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D13.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D14.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d14[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D14.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D14.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D14.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "14.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D14.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D14.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D14.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "14.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D14.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D15.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d15[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D15.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D15.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D15.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "15.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D15.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D15.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D15.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "15.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D15.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D4.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d4[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D4.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D4.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D4.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "4.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D4.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D4.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D4.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "4.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D4.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D5.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d5[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D5.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D5.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D5.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "5.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D5.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D5.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D5.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "5.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D5.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D6.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d6[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D6.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D6.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D6.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "6.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D6.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D6.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D6.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "6.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D6.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D7.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d7[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D7.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D7.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D7.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "7.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D7.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D7.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D7.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "7.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D7.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D8.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d8[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D8.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D8.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D8.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "8.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D8.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D8.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D8.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "8.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D8.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_D9.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_d9[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_D9.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_D9.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_D9.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_D"
        "9.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "D9.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "D9.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_D9.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_D"
        "9.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "D9.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_GDCLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_gdclk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_GDCLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_GDCL"
        "K.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_GDCLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_G"
        "DCLK.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "GDCLK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "GDCLK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_GDCLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_G"
        "DCLK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "GDCLK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_GDOE.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_gdoe[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_GDOE.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_GDOE"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_GDOE.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_G"
        "DOE.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "GDOE.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "GDOE.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_GDOE.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_G"
        "DOE.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "GDOE.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_GDRL.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_gdrl[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_GDRL.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_GDRL"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_GDRL.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_G"
        "DRL.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "GDRL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "GDRL.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_GDRL.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_G"
        "DRL.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "GDRL.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_GDSP.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_gdsp[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_GDSP.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_GDSP"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_GDSP.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_G"
        "DSP.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "GDSP.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "GDSP.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_GDSP.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_G"
        "DSP.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "GDSP.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRCOM.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrcom[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRCOM.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRC"
        "OM.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRCOM.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCOM.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCOM.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRCOM.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRCOM.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCOM.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCOM.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRCTRL0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRCTRL0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRC"
        "TRL0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRCTRL0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRCTRL0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRCTRL1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRCTRL1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRC"
        "TRL1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRCTRL1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRCTRL1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRCTRL2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRCTRL2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRC"
        "TRL2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRCTRL2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRCTRL2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRCTRL3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRCTRL3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRC"
        "TRL3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRCTRL3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRCTRL3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRCTRL3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRCTRL3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRINT.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrint[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRINT.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRI"
        "NT.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRINT.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRINT.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRINT.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRINT.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRINT.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRINT.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRINT.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRSTAT.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrstat[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRSTAT.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRS"
        "TAT.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRSTAT.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRSTAT.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRSTAT.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRSTAT.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRSTAT.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRSTAT.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRSTAT.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_PWRWAKEUP.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_pwrwakeup[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_PWRWAKEUP"
        ".",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_PWRW"
        "AKEUP.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_PWRWAKEUP.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_P"
        "WRWAKEUP.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "PWRWAKEUP.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "PWRWAKEUP.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_PWRWAKEUP.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_P"
        "WRWAKEUP.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "PWRWAKEUP.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDCE0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdce0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDCE0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDCE"
        "0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDCE0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDCE0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDCE0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDCE1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdce1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDCE1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDCE"
        "1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDCE1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDCE1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDCE1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDCE2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdce2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDCE2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDCE"
        "2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDCE2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDCE2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDCE2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDCE3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdce3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDCE3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDCE"
        "3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDCE3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDCE3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDCE3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DCE3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDCE3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDCLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdclk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDCLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDCL"
        "K.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDCLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DCLK.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDCLK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDCLK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDCLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DCLK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDCLK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDLE.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdle[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDLE.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDLE"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDLE.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DLE.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDLE.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDLE.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDLE.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DLE.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDLE.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDOE.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdoe[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDOE.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDOE"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDOE.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DOE.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDOE.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDOE.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDOE.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DOE.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDOE.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_SDSHR.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_sdshr[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_SDSHR.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_SDSH"
        "R.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_SDSHR.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_S"
        "DSHR.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "SDSHR.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "SDSHR.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_SDSHR.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_S"
        "DSHR.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "SDSHR.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_VCOM0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_vcom0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_VCOM0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_VCOM"
        "0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_VCOM0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_V"
        "COM0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "VCOM0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "VCOM0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_VCOM0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_V"
        "COM0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "VCOM0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_EPDC_VCOM1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_epdc_vcom1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: EPDC_VCOM1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: EPDC_VCOM"
        "1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: EPDC_VCOM1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: EPDC_V"
        "COM1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: EPDC_"
        "VCOM1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: EPDC_"
        "VCOM1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " EPDC_VCOM1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: EPDC_V"
        "COM1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: EPDC_"
        "VCOM1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_CRS_DV.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_crs_dv[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_CRS_DV.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_CRS_D"
        "V.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_CRS_DV.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_CR"
        "S_DV.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_C"
        "RS_DV.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_C"
        "RS_DV.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_CRS_DV.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_CR"
        "S_DV.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_C"
        "RS_DV.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_MDC.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_mdc[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_MDC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_MDC.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_MDC.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_MD"
        "C.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_M"
        "DC.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_M"
        "DC.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_MDC.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_MD"
        "C.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_M"
        "DC.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_MDIO.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_mdio[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_MDIO.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_MDIO.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_MDIO.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_MD"
        "IO.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_M"
        "DIO.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_M"
        "DIO.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_MDIO.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_MD"
        "IO.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_M"
        "DIO.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_REF_CLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_ref_clk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_REF_CLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_REF_C"
        "LK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_REF_CLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_RE"
        "F_CLK.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_R"
        "EF_CLK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_R"
        "EF_CLK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_REF_CLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_RE"
        "F_CLK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_R"
        "EF_CLK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_RX_ER.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_rx_er[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_RX_ER.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_RX_ER"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_RX_ER.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_RX"
        "_ER.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_R"
        "X_ER.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_R"
        "X_ER.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_RX_ER.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_RX"
        "_ER.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_R"
        "X_ER.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_RXD0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_rxd0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_RXD0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_RXD0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_RXD0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_RX"
        "D0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_R"
        "XD0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_R"
        "XD0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_RXD0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_RX"
        "D0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_R"
        "XD0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_RXD1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_rxd1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_RXD1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_RXD1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_RXD1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_RX"
        "D1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_R"
        "XD1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_R"
        "XD1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_RXD1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_RX"
        "D1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_R"
        "XD1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_TX_CLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_tx_clk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_TX_CLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_TX_CL"
        "K.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_TX_CLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_TX"
        "_CLK.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_T"
        "X_CLK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_T"
        "X_CLK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_TX_CLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_TX"
        "_CLK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_T"
        "X_CLK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_TX_EN.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_tx_en[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_TX_EN.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_TX_EN"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_TX_EN.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_TX"
        "_EN.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_T"
        "X_EN.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_T"
        "X_EN.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_TX_EN.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_TX"
        "_EN.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_T"
        "X_EN.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_TXD0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_txd0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_TXD0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_TXD0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_TXD0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_TX"
        "D0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_T"
        "XD0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_T"
        "XD0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_TXD0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_TX"
        "D0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_T"
        "XD0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_FEC_TXD1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_fec_txd1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: FEC_TXD1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: FEC_TXD1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: FEC_TXD1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: FEC_TX"
        "D1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: FEC_T"
        "XD1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: FEC_T"
        "XD1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " FEC_TXD1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: FEC_TX"
        "D1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: FEC_T"
        "XD1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_HSIC_DAT.
static const field_t hw_iomuxc_sw_pad_ctl_pad_hsic_dat[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: HSIC_DAT.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Read Only Field",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: HSIC_"
        "DAT.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: HSIC_"
        "DAT.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " HSIC_DAT.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: HSIC_D"
        "AT.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: HS"
        "IC_DAT.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Select one out of next values for pad: HSIC_DAT.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_HSIC_STROBE.
static const field_t hw_iomuxc_sw_pad_ctl_pad_hsic_strobe[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: HSIC_STRO"
        "BE.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT",
        "On Die Termination Field  Read Only Field",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: HSIC_"
        "STROBE.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: HSIC_"
        "STROBE.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " HSIC_STROBE.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: HSIC_S"
        "TROBE.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for pad: HS"
        "IC_STROBE.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_SEL",
        "DDR Standard Field  Select one out of next values for pad: HSIC_STROBE"
        ".",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DO_TRIM",
        "Pad Output Delay Field  Read Only Field",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_I2C1_SCL.
static const field_t hw_iomuxc_sw_pad_ctl_pad_i2c1_scl[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: I2C1_SCL.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: I2C1_SCL.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: I2C1_SCL.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: I2C1_S"
        "CL.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: I2C1_"
        "SCL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: I2C1_"
        "SCL.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " I2C1_SCL.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: I2C1_S"
        "CL.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: I2C1_"
        "SCL.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_I2C1_SDA.
static const field_t hw_iomuxc_sw_pad_ctl_pad_i2c1_sda[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: I2C1_SDA.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: I2C1_SDA.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: I2C1_SDA.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: I2C1_S"
        "DA.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: I2C1_"
        "SDA.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: I2C1_"
        "SDA.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " I2C1_SDA.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: I2C1_S"
        "DA.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: I2C1_"
        "SDA.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_I2C2_SCL.
static const field_t hw_iomuxc_sw_pad_ctl_pad_i2c2_scl[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: I2C2_SCL.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: I2C2_SCL.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: I2C2_SCL.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: I2C2_S"
        "CL.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: I2C2_"
        "SCL.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: I2C2_"
        "SCL.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " I2C2_SCL.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: I2C2_S"
        "CL.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: I2C2_"
        "SCL.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_I2C2_SDA.
static const field_t hw_iomuxc_sw_pad_ctl_pad_i2c2_sda[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: I2C2_SDA.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: I2C2_SDA.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: I2C2_SDA.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: I2C2_S"
        "DA.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: I2C2_"
        "SDA.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: I2C2_"
        "SDA.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " I2C2_SDA.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: I2C2_S"
        "DA.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: I2C2_"
        "SDA.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_JTAG_MOD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_jtag_mod[] =
{
    {
        "SRE",
        "Slew Rate Field  Read Only Field",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Read Only Field",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Read Only Field",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Read Only Field",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: JTAG_"
        "MOD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " JTAG_MOD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: JTAG_M"
        "OD.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Read Only Field",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_JTAG_TCK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_jtag_tck[] =
{
    {
        "SRE",
        "Slew Rate Field  Read Only Field",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Read Only Field",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Read Only Field",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Read Only Field",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: JTAG_"
        "TCK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " JTAG_TCK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: JTAG_T"
        "CK.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Read Only Field",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_JTAG_TDI.
static const field_t hw_iomuxc_sw_pad_ctl_pad_jtag_tdi[] =
{
    {
        "SRE",
        "Slew Rate Field  Read Only Field",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Read Only Field",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Read Only Field",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Read Only Field",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: JTAG_"
        "TDI.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " JTAG_TDI.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: JTAG_T"
        "DI.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Read Only Field",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_JTAG_TDO.
static const field_t hw_iomuxc_sw_pad_ctl_pad_jtag_tdo[] =
{
    {
        "SRE",
        "Slew Rate Field  Read Only Field",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Read Only Field",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Read Only Field",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Read Only Field",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: JTAG_"
        "TDO.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Read Only Field",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Read Only Field",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Read Only Field",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_JTAG_TMS.
static const field_t hw_iomuxc_sw_pad_ctl_pad_jtag_tms[] =
{
    {
        "SRE",
        "Slew Rate Field  Read Only Field",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Read Only Field",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Read Only Field",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Read Only Field",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: JTAG_"
        "TMS.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " JTAG_TMS.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: JTAG_T"
        "MS.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Read Only Field",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_JTAG_TRSTB.
static const field_t hw_iomuxc_sw_pad_ctl_pad_jtag_trstb[] =
{
    {
        "SRE",
        "Slew Rate Field  Read Only Field",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Read Only Field",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Read Only Field",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Read Only Field",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: JTAG_"
        "TRSTB.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Read Only Field",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " JTAG_TRSTB.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: JTAG_T"
        "RSTB.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Read Only Field",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL4.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col4[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL4.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL4.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL4.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L4.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL4.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL4.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL4.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L4.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL4.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL5.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col5[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL5.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL5.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL5.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L5.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL5.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL5.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL5.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L5.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL5.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL6.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col6[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL6.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL6.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL6.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L6.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL6.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL6.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL6.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L6.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL6.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_COL7.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_col7[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_COL7.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_COL7.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_COL7.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_CO"
        "L7.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_C"
        "OL7.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_C"
        "OL7.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_COL7.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_CO"
        "L7.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_C"
        "OL7.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW4.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row4[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW4.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW4.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW4.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W4.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW4.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW4.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW4.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W4.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW4.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW5.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row5[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW5.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW5.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW5.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W5.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW5.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW5.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW5.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W5.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW5.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW6.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row6[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW6.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW6.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW6.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W6.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW6.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW6.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW6.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W6.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW6.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_KEY_ROW7.
static const field_t hw_iomuxc_sw_pad_ctl_pad_key_row7[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: KEY_ROW7.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: KEY_ROW7.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: KEY_ROW7.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: KEY_RO"
        "W7.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: KEY_R"
        "OW7.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: KEY_R"
        "OW7.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " KEY_ROW7.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: KEY_RO"
        "W7.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: KEY_R"
        "OW7.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_CLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_clk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_CLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_CLK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_CLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_CL"
        "K.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_C"
        "LK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_C"
        "LK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_CLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_CL"
        "K.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_C"
        "LK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT10.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat10[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT10.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT10"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT10.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T10.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT10.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT10.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT10.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T10.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT10.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT11.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat11[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT11.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT11"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT11.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T11.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT11.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT11.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT11.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T11.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT11.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT12.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat12[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT12.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT12"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT12.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T12.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT12.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT12.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT12.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T12.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT12.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT13.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat13[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT13.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT13"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT13.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T13.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT13.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT13.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT13.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T13.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT13.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT14.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat14[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT14.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT14"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT14.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T14.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT14.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT14.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT14.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T14.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT14.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT15.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat15[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT15.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT15"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT15.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T15.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT15.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT15.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT15.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T15.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT15.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT16.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat16[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT16.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT16"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT16.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T16.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT16.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT16.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT16.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T16.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT16.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT17.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat17[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT17.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT17"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT17.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T17.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT17.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT17.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT17.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T17.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT17.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT18.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat18[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT18.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT18"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT18.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T18.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT18.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT18.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT18.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T18.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT18.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT19.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat19[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT19.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT19"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT19.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T19.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT19.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT19.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT19.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T19.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT19.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT20.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat20[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT20.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT20"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT20.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T20.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT20.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT20.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT20.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T20.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT20.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT21.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat21[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT21.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT21"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT21.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T21.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT21.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT21.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT21.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T21.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT21.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT22.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat22[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT22.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT22"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT22.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T22.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT22.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT22.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT22.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T22.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT22.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT23.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat23[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT23.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT23"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT23.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T23.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT23.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT23.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT23.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T23.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT23.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT4.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat4[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT4.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT4.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT4.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T4.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT4.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT4.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT4.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T4.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT4.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT5.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat5[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT5.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT5.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT5.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T5.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT5.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT5.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT5.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T5.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT5.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT6.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat6[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT6.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT6.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT6.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T6.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT6.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT6.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT6.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T6.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT6.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT7.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat7[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT7.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT7.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT7.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T7.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT7.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT7.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT7.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T7.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT7.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT8.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat8[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT8.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT8.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT8.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T8.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT8.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT8.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT8.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T8.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT8.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_DAT9.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_dat9[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_DAT9.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_DAT9.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_DAT9.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_DA"
        "T9.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_D"
        "AT9.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_D"
        "AT9.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_DAT9.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_DA"
        "T9.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_D"
        "AT9.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_ENABLE.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_enable[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_ENABLE.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_ENABL"
        "E.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_ENABLE.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_EN"
        "ABLE.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_E"
        "NABLE.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_E"
        "NABLE.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_ENABLE.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_EN"
        "ABLE.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_E"
        "NABLE.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_HSYNC.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_hsync[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_HSYNC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_HSYNC"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_HSYNC.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_HS"
        "YNC.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_H"
        "SYNC.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_H"
        "SYNC.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_HSYNC.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_HS"
        "YNC.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_H"
        "SYNC.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_RESET.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_reset[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_RESET.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_RESET"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_RESET.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_RE"
        "SET.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_R"
        "ESET.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_R"
        "ESET.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_RESET.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_RE"
        "SET.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_R"
        "ESET.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_LCD_VSYNC.
static const field_t hw_iomuxc_sw_pad_ctl_pad_lcd_vsync[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: LCD_VSYNC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: LCD_VSYNC"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: LCD_VSYNC.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: LCD_VS"
        "YNC.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: LCD_V"
        "SYNC.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: LCD_V"
        "SYNC.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " LCD_VSYNC.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: LCD_VS"
        "YNC.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: LCD_V"
        "SYNC.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_PWM1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_pwm1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: PWM1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: PWM1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: PWM1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: PWM1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: PWM1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: PWM1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " PWM1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: PWM1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: PWM1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_REF_CLK_24M.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ref_clk_24m[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: REF_CLK_24M.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: REF_CLK_2"
        "4M.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: REF_CLK_24M.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: REF_CL"
        "K_24M.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: REF_C"
        "LK_24M.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: REF_C"
        "LK_24M.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " REF_CLK_24M.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: REF_CL"
        "K_24M.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: REF_C"
        "LK_24M.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_REF_CLK_32K.
static const field_t hw_iomuxc_sw_pad_ctl_pad_ref_clk_32k[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: REF_CLK_32K.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: REF_CLK_3"
        "2K.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: REF_CLK_32K.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: REF_CL"
        "K_32K.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: REF_C"
        "LK_32K.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: REF_C"
        "LK_32K.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " REF_CLK_32K.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: REF_CL"
        "K_32K.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: REF_C"
        "LK_32K.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_CLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_clk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_CLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_CLK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_CLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_CL"
        "K.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_C"
        "LK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_C"
        "LK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_CLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_CL"
        "K.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_C"
        "LK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_CMD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_cmd[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_CMD.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_CMD.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_CMD.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_CM"
        "D.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_C"
        "MD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_C"
        "MD.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_CMD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_CM"
        "D.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_C"
        "MD.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT4.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat4[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT4.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT4.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT4.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T4.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT4.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT4.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT4.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T4.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT4.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT5.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat5[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT5.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT5.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT5.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T5.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT5.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT5.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT5.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T5.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT5.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT6.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat6[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT6.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT6.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT6.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T6.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT6.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT6.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT6.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T6.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT6.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD1_DAT7.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd1_dat7[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD1_DAT7.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD1_DAT7.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD1_DAT7.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD1_DA"
        "T7.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD1_D"
        "AT7.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD1_D"
        "AT7.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD1_DAT7.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD1_DA"
        "T7.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD1_D"
        "AT7.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_CLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_clk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_CLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_CLK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_CLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_CL"
        "K.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_C"
        "LK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_C"
        "LK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_CLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_CL"
        "K.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_C"
        "LK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_CMD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_cmd[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_CMD.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_CMD.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_CMD.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_CM"
        "D.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_C"
        "MD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_C"
        "MD.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_CMD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_CM"
        "D.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_C"
        "MD.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT4.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat4[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT4.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT4.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT4.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T4.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT4.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT4.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT4.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T4.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT4.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT5.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat5[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT5.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT5.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT5.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T5.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT5.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT5.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT5.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T5.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT5.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT6.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat6[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT6.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT6.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT6.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T6.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT6.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT6.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT6.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T6.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT6.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_DAT7.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_dat7[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_DAT7.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_DAT7.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_DAT7.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_DA"
        "T7.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_D"
        "AT7.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_D"
        "AT7.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_DAT7.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_DA"
        "T7.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_D"
        "AT7.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD2_RST.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd2_rst[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD2_RST.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD2_RST.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD2_RST.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD2_RS"
        "T.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD2_R"
        "ST.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD2_R"
        "ST.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD2_RST.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD2_RS"
        "T.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD2_R"
        "ST.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD3_CLK.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd3_clk[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD3_CLK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD3_CLK.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD3_CLK.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD3_CL"
        "K.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD3_C"
        "LK.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD3_C"
        "LK.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD3_CLK.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD3_CL"
        "K.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD3_C"
        "LK.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD3_CMD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd3_cmd[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD3_CMD.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD3_CMD.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD3_CMD.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD3_CM"
        "D.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD3_C"
        "MD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD3_C"
        "MD.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD3_CMD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD3_CM"
        "D.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD3_C"
        "MD.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD3_DAT0.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd3_dat0[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD3_DAT0.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD3_DAT0.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD3_DAT0.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD3_DA"
        "T0.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD3_D"
        "AT0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD3_D"
        "AT0.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD3_DAT0.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD3_DA"
        "T0.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD3_D"
        "AT0.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD3_DAT1.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd3_dat1[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD3_DAT1.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD3_DAT1.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD3_DAT1.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD3_DA"
        "T1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD3_D"
        "AT1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD3_D"
        "AT1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD3_DAT1.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD3_DA"
        "T1.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD3_D"
        "AT1.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD3_DAT2.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd3_dat2[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD3_DAT2.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD3_DAT2.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD3_DAT2.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD3_DA"
        "T2.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD3_D"
        "AT2.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD3_D"
        "AT2.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD3_DAT2.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD3_DA"
        "T2.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD3_D"
        "AT2.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_SD3_DAT3.
static const field_t hw_iomuxc_sw_pad_ctl_pad_sd3_dat3[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: SD3_DAT3.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: SD3_DAT3.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: SD3_DAT3.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: SD3_DA"
        "T3.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: SD3_D"
        "AT3.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: SD3_D"
        "AT3.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " SD3_DAT3.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: SD3_DA"
        "T3.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: SD3_D"
        "AT3.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_UART1_RXD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_uart1_rxd[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: UART1_RXD.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: UART1_RXD"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: UART1_RXD.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: UART1_"
        "RXD.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: UART1"
        "_RXD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: UART1"
        "_RXD.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " UART1_RXD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: UART1_"
        "RXD.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: UART1"
        "_RXD.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_UART1_TXD.
static const field_t hw_iomuxc_sw_pad_ctl_pad_uart1_txd[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: UART1_TXD.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: UART1_TXD"
        ".",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: UART1_TXD.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: UART1_"
        "TXD.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: UART1"
        "_TXD.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: UART1"
        "_TXD.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " UART1_TXD.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: UART1_"
        "TXD.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: UART1"
        "_TXD.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_PAD_WDOG_B.
static const field_t hw_iomuxc_sw_pad_ctl_pad_wdog_b[] =
{
    {
        "SRE",
        "Slew Rate Field  Select one out of next values for pad: WDOG_B.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for pad: WDOG_B.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPEED",
        "Speed Field  Select one out of next values for pad: WDOG_B.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODE",
        "Open Drain Enable Field  Select one out of next values for pad: WDOG_B"
        ".",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for pad: WDOG_"
        "B.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for pad: WDOG_"
        "B.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PUS",
        "Pull Up / Down Configure Field  Select one out of next values for pad:"
        " WDOG_B.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for pad: WDOG_B"
        ".",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVE",
        "Low Voltage Enable Field  Select one out of next values for pad: WDOG_"
        "B.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_ADDDS.
static const field_t hw_iomuxc_sw_pad_ctl_grp_addds[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for group: ADDDS ("
        "Pads: DRAM_A0 DRAM_A1 DRAM_A10 DRAM_A11 DRAM_A12 DRAM_A13 DRAM_A14 DRA"
        "M_A15 DRAM_A2 DRAM_A3 DRAM_A4 DRAM_A5 DRAM_A6 DRAM_A7 DRAM_A8 DRAM_A9 "
        "DRAM_SDBA0 DRAM_SDBA1).",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_DDRMODE_CTL.
static const field_t hw_iomuxc_sw_pad_ctl_grp_ddrmode_ctl[] =
{
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for group: "
        "DDRMODE_CTL (Pads: DRAM_SDQS0 DRAM_SDQS1 DRAM_SDQS2 DRAM_SDQS3).",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_DDRPKE.
static const field_t hw_iomuxc_sw_pad_ctl_grp_ddrpke[] =
{
    {
        "PKE",
        "Pull / Keep Enable Field  Select one out of next values for group: DDR"
        "PKE (Pads: DRAM_A0 DRAM_A1 DRAM_A10 DRAM_A11 DRAM_A12 DRAM_A13 DRAM_A1"
        "4 DRAM_A15 DRAM_A2 DRAM_A3 DRAM_A4 DRAM_A5 DRAM_A6 DRAM_A7 DRAM_A8 DRA"
        "M_A9 DRAM_CAS DRAM_CS0 DRAM_CS1 DRAM_D0 DRAM_D1 DRAM_D10 DRAM_D11 DRAM"
        "_D12 DRAM_D13 DRAM_D14 DRAM_D15 DRAM_D16 DRAM_D17 DRAM_D18 DRAM_D19 DR"
        "AM_D2 DRAM_D20 DRAM_D21 DRAM_D22 DRAM_D23 DRAM_D24 DRAM_D25 DRAM_D26 D"
        "RAM_D27 DRAM_D28 DRAM_D29 DRAM_D3 DRAM_D30 DRAM_D31 DRAM_D4 DRAM_D5 DR"
        "AM_D6 DRAM_D7 DRAM_D8 DRAM_D9 DRAM_DQM0 DRAM_DQM1 DRAM_DQM2 DRAM_DQM3 "
        "DRAM_RAS DRAM_SDBA0 DRAM_SDBA1 DRAM_SDCLK_0 DRAM_SDWE).",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_DDRPK.
static const field_t hw_iomuxc_sw_pad_ctl_grp_ddrpk[] =
{
    {
        "PUE",
        "Pull / Keep Select Field  Select one out of next values for group: DDR"
        "PK (Pads: DRAM_A0 DRAM_A1 DRAM_A10 DRAM_A11 DRAM_A12 DRAM_A13 DRAM_A14"
        " DRAM_A15 DRAM_A2 DRAM_A3 DRAM_A4 DRAM_A5 DRAM_A6 DRAM_A7 DRAM_A8 DRAM"
        "_A9 DRAM_CAS DRAM_CS0 DRAM_CS1 DRAM_D0 DRAM_D1 DRAM_D10 DRAM_D11 DRAM_"
        "D12 DRAM_D13 DRAM_D14 DRAM_D15 DRAM_D16 DRAM_D17 DRAM_D18 DRAM_D19 DRA"
        "M_D2 DRAM_D20 DRAM_D21 DRAM_D22 DRAM_D23 DRAM_D24 DRAM_D25 DRAM_D26 DR"
        "AM_D27 DRAM_D28 DRAM_D29 DRAM_D3 DRAM_D30 DRAM_D31 DRAM_D4 DRAM_D5 DRA"
        "M_D6 DRAM_D7 DRAM_D8 DRAM_D9 DRAM_DQM0 DRAM_DQM1 DRAM_DQM2 DRAM_DQM3 D"
        "RAM_RAS DRAM_SDBA0 DRAM_SDBA1 DRAM_SDCLK_0 DRAM_SDWE).",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_DDRHYS.
static const field_t hw_iomuxc_sw_pad_ctl_grp_ddrhys[] =
{
    {
        "HYS",
        "Hysteresis Enable Field  Select one out of next values for group: DDRH"
        "YS (Pads: DRAM_D0 DRAM_D1 DRAM_D10 DRAM_D11 DRAM_D12 DRAM_D13 DRAM_D14"
        " DRAM_D15 DRAM_D16 DRAM_D17 DRAM_D18 DRAM_D19 DRAM_D2 DRAM_D20 DRAM_D2"
        "1 DRAM_D22 DRAM_D23 DRAM_D24 DRAM_D25 DRAM_D26 DRAM_D27 DRAM_D28 DRAM_"
        "D29 DRAM_D3 DRAM_D30 DRAM_D31 DRAM_D4 DRAM_D5 DRAM_D6 DRAM_D7 DRAM_D8 "
        "DRAM_D9 DRAM_SDQS0 DRAM_SDQS1 DRAM_SDQS2 DRAM_SDQS3).",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_DDRMODE.
static const field_t hw_iomuxc_sw_pad_ctl_grp_ddrmode[] =
{
    {
        "DDR_INPUT",
        "DDR / CMOS Input Mode Field  Select one out of next values for group: "
        "DDRMODE (Pads: DRAM_D0 DRAM_D1 DRAM_D10 DRAM_D11 DRAM_D12 DRAM_D13 DRA"
        "M_D14 DRAM_D15 DRAM_D16 DRAM_D17 DRAM_D18 DRAM_D19 DRAM_D2 DRAM_D20 DR"
        "AM_D21 DRAM_D22 DRAM_D23 DRAM_D24 DRAM_D25 DRAM_D26 DRAM_D27 DRAM_D28 "
        "DRAM_D29 DRAM_D3 DRAM_D30 DRAM_D31 DRAM_D4 DRAM_D5 DRAM_D6 DRAM_D7 DRA"
        "M_D8 DRAM_D9).",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_B0DS.
static const field_t hw_iomuxc_sw_pad_ctl_grp_b0ds[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for group: B0DS (P"
        "ads: DRAM_D0 DRAM_D1 DRAM_D2 DRAM_D3 DRAM_D4 DRAM_D5 DRAM_D6 DRAM_D7).",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_CTLDS.
static const field_t hw_iomuxc_sw_pad_ctl_grp_ctlds[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for group: CTLDS ("
        "Pads: DRAM_CS0 DRAM_CS1 DRAM_SDBA2 DRAM_SDCKE0 DRAM_SDCKE1 DRAM_SDWE).",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_B1DS.
static const field_t hw_iomuxc_sw_pad_ctl_grp_b1ds[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for group: B1DS (P"
        "ads: DRAM_D10 DRAM_D11 DRAM_D12 DRAM_D13 DRAM_D14 DRAM_D15 DRAM_D8 DRA"
        "M_D9).",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE.
static const field_t hw_iomuxc_sw_pad_ctl_grp_ddr_type[] =
{
    {
        "DDR_SEL",
        "DDR Standard Field  Select one out of next values for group: DDR_TYPE "
        "(Pads: DRAM_A0 DRAM_A1 DRAM_A10 DRAM_A11 DRAM_A12 DRAM_A13 DRAM_A14 DR"
        "AM_A15 DRAM_A2 DRAM_A3 DRAM_A4 DRAM_A5 DRAM_A6 DRAM_A7 DRAM_A8 DRAM_A9"
        " DRAM_CAS DRAM_CS0 DRAM_CS1 DRAM_D0 DRAM_D1 DRAM_D10 DRAM_D11 DRAM_D12"
        " DRAM_D13 DRAM_D14 DRAM_D15 DRAM_D16 DRAM_D17 DRAM_D18 DRAM_D19 DRAM_D"
        "2 DRAM_D20 DRAM_D21 DRAM_D22 DRAM_D23 DRAM_D24 DRAM_D25 DRAM_D26 DRAM_"
        "D27 DRAM_D28 DRAM_D29 DRAM_D3 DRAM_D30 DRAM_D31 DRAM_D4 DRAM_D5 DRAM_D"
        "6 DRAM_D7 DRAM_D8 DRAM_D9 DRAM_DQM0 DRAM_DQM1 DRAM_DQM2 DRAM_DQM3 DRAM"
        "_RAS DRAM_SDBA0 DRAM_SDBA1 DRAM_SDBA2 DRAM_SDCKE0 DRAM_SDCKE1 DRAM_SDC"
        "LK_0 DRAM_SDODT0 DRAM_SDODT1 DRAM_SDQS0 DRAM_SDQS1 DRAM_SDQS2 DRAM_SDQ"
        "S3 DRAM_SDWE).",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_B2DS.
static const field_t hw_iomuxc_sw_pad_ctl_grp_b2ds[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for group: B2DS (P"
        "ads: DRAM_D16 DRAM_D17 DRAM_D18 DRAM_D19 DRAM_D20 DRAM_D21 DRAM_D22 DR"
        "AM_D23).",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SW_PAD_CTL_GRP_B3DS.
static const field_t hw_iomuxc_sw_pad_ctl_grp_b3ds[] =
{
    {
        "DSE",
        "Drive Strength Field  Select one out of next values for group: B3DS (P"
        "ads: DRAM_D24 DRAM_D25 DRAM_D26 DRAM_D27 DRAM_D28 DRAM_D29 DRAM_D30 DR"
        "AM_D31).",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USB_OTG_ID_SELECT_INPUT.
static const field_t hw_iomuxc_usb_otg_id_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USB_IPP_IND_H1_ID_SELECT_INPUT.
static const field_t hw_iomuxc_usb_ipp_ind_h1_id_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P4_INPUT_DA_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p4_input_da_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P4_INPUT_DB_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p4_input_db_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P4_INPUT_RXCLK_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p4_input_rxclk_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P4_INPUT_RXFS_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p4_input_rxfs_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P4_INPUT_TXCLK_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p4_input_txclk_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P4_INPUT_TXFS_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p4_input_txfs_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P5_INPUT_DA_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p5_input_da_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P5_INPUT_DB_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p5_input_db_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P5_INPUT_RXCLK_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p5_input_rxclk_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P5_INPUT_RXFS_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p5_input_rxfs_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P5_INPUT_TXCLK_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p5_input_txclk_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P5_INPUT_TXFS_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p5_input_txfs_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P6_INPUT_DA_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p6_input_da_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P6_INPUT_DB_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p6_input_db_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P6_INPUT_RXCLK_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p6_input_rxclk_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P6_INPUT_RXFS_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p6_input_rxfs_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P6_INPUT_TXCLK_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p6_input_txclk_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_AUDMUX_P6_INPUT_TXFS_AMX_SELECT_INPUT.
static const field_t hw_iomuxc_audmux_p6_input_txfs_amx_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CCM_PMIC_VFUNCIONAL_READY_SELECT_INPUT.
static const field_t hw_iomuxc_ccm_pmic_vfuncional_ready_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_0_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_1_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_2_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_3_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_3_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_4_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_4_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_5_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_5_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_6_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_6_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_7_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_7_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_8_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_8_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_9_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_9_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_10_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_10_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_11_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_11_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_12_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_12_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_13_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_13_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_14_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_14_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_D_15_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_d_15_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_HSYNC_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_hsync_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_PIXCLK_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_pixclk_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_CSI_IPP_CSI_VSYNC_SELECT_INPUT.
static const field_t hw_iomuxc_csi_ipp_csi_vsync_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_CSPI_CLK_IN_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_cspi_clk_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_IND_DATAREADY_B_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_ind_dataready_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_IND_MISO_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_ind_miso_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_IND_MOSI_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_ind_mosi_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_IND_SS_B_0_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_ind_ss_b_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_IND_SS_B_1_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_ind_ss_b_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_IND_SS_B_2_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_ind_ss_b_2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI1_IPP_IND_SS_B_3_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi1_ipp_ind_ss_b_3_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI2_IPP_CSPI_CLK_IN_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi2_ipp_cspi_clk_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI2_IPP_IND_MISO_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi2_ipp_ind_miso_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI2_IPP_IND_MOSI_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi2_ipp_ind_mosi_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI2_IPP_IND_SS_B_0_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi2_ipp_ind_ss_b_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI2_IPP_IND_SS_B_1_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi2_ipp_ind_ss_b_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_CSPI_CLK_IN_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_cspi_clk_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_IND_DATAREADY_B_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_ind_dataready_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_IND_MISO_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_ind_miso_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_IND_MOSI_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_ind_mosi_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_IND_SS_B_0_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_ind_ss_b_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_IND_SS_B_1_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_ind_ss_b_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_IND_SS_B_2_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_ind_ss_b_2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI3_IPP_IND_SS_B_3_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi3_ipp_ind_ss_b_3_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI4_IPP_CSPI_CLK_IN_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi4_ipp_cspi_clk_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI4_IPP_IND_MISO_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi4_ipp_ind_miso_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI4_IPP_IND_MOSI_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi4_ipp_ind_mosi_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI4_IPP_IND_SS_B_0_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi4_ipp_ind_ss_b_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI4_IPP_IND_SS_B_1_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi4_ipp_ind_ss_b_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_ECSPI4_IPP_IND_SS_B_2_SELECT_INPUT.
static const field_t hw_iomuxc_ecspi4_ipp_ind_ss_b_2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_EPDC_IPP_EPDC_PWRIRQ_SELECT_INPUT.
static const field_t hw_iomuxc_epdc_ipp_epdc_pwrirq_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_EPDC_IPP_EPDC_PWRSTAT_SELECT_INPUT.
static const field_t hw_iomuxc_epdc_ipp_epdc_pwrstat_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_COL_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_col_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_MDI_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_mdi_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_RDATA_0_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_rdata_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_RDATA_1_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_rdata_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_RX_CLK_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_rx_clk_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_RX_DV_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_rx_dv_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_RX_ER_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_rx_er_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_FEC_FEC_TX_CLK_SELECT_INPUT.
static const field_t hw_iomuxc_fec_fec_tx_clk_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPT_IPP_IND_CAPIN1_SELECT_INPUT.
static const field_t hw_iomuxc_gpt_ipp_ind_capin1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPT_IPP_IND_CAPIN2_SELECT_INPUT.
static const field_t hw_iomuxc_gpt_ipp_ind_capin2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_GPT_IPP_IND_CLKIN_SELECT_INPUT.
static const field_t hw_iomuxc_gpt_ipp_ind_clkin_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_I2C1_IPP_SCL_IN_SELECT_INPUT.
static const field_t hw_iomuxc_i2c1_ipp_scl_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_I2C1_IPP_SDA_IN_SELECT_INPUT.
static const field_t hw_iomuxc_i2c1_ipp_sda_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_I2C2_IPP_SCL_IN_SELECT_INPUT.
static const field_t hw_iomuxc_i2c2_ipp_scl_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_I2C2_IPP_SDA_IN_SELECT_INPUT.
static const field_t hw_iomuxc_i2c2_ipp_sda_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_I2C3_IPP_SCL_IN_SELECT_INPUT.
static const field_t hw_iomuxc_i2c3_ipp_scl_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_I2C3_IPP_SDA_IN_SELECT_INPUT.
static const field_t hw_iomuxc_i2c3_ipp_sda_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_0_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_1_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_2_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_3_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_3_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_4_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_4_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_5_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_5_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_6_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_6_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_COL_7_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_col_7_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_0_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_1_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_2_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_3_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_3_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_4_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_4_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_5_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_5_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_6_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_6_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_KPP_IPP_IND_ROW_7_SELECT_INPUT.
static const field_t hw_iomuxc_kpp_ipp_ind_row_7_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_BUSY_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_busy_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_0_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_0_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_1_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_2_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_3_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_3_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_4_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_4_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_5_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_5_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_6_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_6_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_7_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_7_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_8_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_8_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_9_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_9_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_10_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_10_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_11_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_11_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_12_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_12_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_13_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_13_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_14_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_14_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_15_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_15_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_16_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_16_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_17_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_17_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_18_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_18_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_19_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_19_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_20_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_20_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_21_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_21_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_22_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_22_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_LCDIF_LCDIF_RXDATA_23_SELECT_INPUT.
static const field_t hw_iomuxc_lcdif_lcdif_rxdata_23_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SPDIF_SPDIF_IN1_SELECT_INPUT.
static const field_t hw_iomuxc_spdif_spdif_in1_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_SPDIF_TX_CLK2_SELECT_INPUT.
static const field_t hw_iomuxc_spdif_tx_clk2_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART1_IPP_UART_RTS_B_SELECT_INPUT.
static const field_t hw_iomuxc_uart1_ipp_uart_rts_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART1_IPP_UART_RXD_MUX_SELECT_INPUT.
static const field_t hw_iomuxc_uart1_ipp_uart_rxd_mux_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART2_IPP_UART_RTS_B_SELECT_INPUT.
static const field_t hw_iomuxc_uart2_ipp_uart_rts_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART2_IPP_UART_RXD_MUX_SELECT_INPUT.
static const field_t hw_iomuxc_uart2_ipp_uart_rxd_mux_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART3_IPP_UART_RTS_B_SELECT_INPUT.
static const field_t hw_iomuxc_uart3_ipp_uart_rts_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART3_IPP_UART_RXD_MUX_SELECT_INPUT.
static const field_t hw_iomuxc_uart3_ipp_uart_rxd_mux_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART4_IPP_UART_RTS_B_SELECT_INPUT.
static const field_t hw_iomuxc_uart4_ipp_uart_rts_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART4_IPP_UART_RXD_MUX_SELECT_INPUT.
static const field_t hw_iomuxc_uart4_ipp_uart_rxd_mux_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART5_IPP_UART_RTS_B_SELECT_INPUT.
static const field_t hw_iomuxc_uart5_ipp_uart_rts_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_UART5_IPP_UART_RXD_MUX_SELECT_INPUT.
static const field_t hw_iomuxc_uart5_ipp_uart_rxd_mux_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USB_IPP_IND_OTG2_OC_SELECT_INPUT.
static const field_t hw_iomuxc_usb_ipp_ind_otg2_oc_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USB_IPP_IND_OTG_OC_SELECT_INPUT.
static const field_t hw_iomuxc_usb_ipp_ind_otg_oc_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC1_IPP_CARD_DET_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc1_ipp_card_det_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC1_IPP_WP_ON_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc1_ipp_wp_on_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC2_IPP_CARD_DET_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc2_ipp_card_det_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC2_IPP_WP_ON_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc2_ipp_wp_on_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC3_IPP_CARD_DET_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc3_ipp_card_det_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC3_IPP_DAT4_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc3_ipp_dat4_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC3_IPP_DAT5_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc3_ipp_dat5_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC3_IPP_DAT6_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc3_ipp_dat6_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC3_IPP_DAT7_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc3_ipp_dat7_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC3_IPP_WP_ON_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc3_ipp_wp_on_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_CARD_CLK_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_card_clk_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_CARD_DET_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_card_det_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_CMD_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_cmd_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT0_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat0_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT1_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat1_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT2_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat2_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT3_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat3_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT4_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat4_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT5_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat5_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT6_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat6_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_DAT7_IN_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_dat7_in_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_USDHC4_IPP_WP_ON_SELECT_INPUT.
static const field_t hw_iomuxc_usdhc4_ipp_wp_on_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_EIM_IPP_IND_DTACK_B_SELECT_INPUT.
static const field_t hw_iomuxc_eim_ipp_ind_dtack_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register IOMUXC_EIM_IPP_IND_WAIT_B_SELECT_INPUT.
static const field_t hw_iomuxc_eim_ipp_ind_wait_b_select_input[] =
{
    {
        "DAISY",
        "Selecting Pads Involved in Daisy Chain.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a IOMUXC module.
static const reg_t hw_iomuxc[] =
{
    {
        "GPR0",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_iomuxc_gpr0
    },
    {
        "GPR1",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_iomuxc_gpr1
    },
    {
        "GPR2",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        24, // Number of bitfields
        hw_iomuxc_gpr2
    },
    {
        "GPR3",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_iomuxc_gpr3
    },
    {
        "GPR4",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_iomuxc_gpr4
    },
    {
        "GPR5",
        "",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_iomuxc_gpr5
    },
    {
        "GPR6",
        "",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        0, // Number of bitfields
        hw_iomuxc_gpr6
    },
    {
        "GPR7",
        "",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        0, // Number of bitfields
        hw_iomuxc_gpr7
    },
    {
        "GPR8",
        "",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        0, // Number of bitfields
        hw_iomuxc_gpr8
    },
    {
        "GPR9",
        "",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_gpr9
    },
    {
        "GPR10",
        "",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_iomuxc_gpr10
    },
    {
        "GPR11",
        "",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_gpr11
    },
    {
        "GPR12",
        "",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_iomuxc_gpr12
    },
    {
        "GPR13",
        "SATA_PHY_6    PHUG    FRUG    fast_startup    Frequency Tolerance (ppm"
        ")      000    1    1    None    780      001    2    2    None    780 "
        "     010    1    4    None    6,250      011    2    4    None    6,25"
        "0      1xx    Reserved",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_iomuxc_gpr13
    },
    {
        "SW_MUX_CTL_PAD_AUD_MCLK",
        "",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_aud_mclk
    },
    {
        "SW_MUX_CTL_PAD_AUD_RXC",
        "",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_aud_rxc
    },
    {
        "SW_MUX_CTL_PAD_AUD_RXD",
        "",
        4, // Width in bytes
        0x00000054, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_aud_rxd
    },
    {
        "SW_MUX_CTL_PAD_AUD_RXFS",
        "",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_aud_rxfs
    },
    {
        "SW_MUX_CTL_PAD_AUD_TXC",
        "",
        4, // Width in bytes
        0x0000005c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_aud_txc
    },
    {
        "SW_MUX_CTL_PAD_AUD_TXD",
        "",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_aud_txd
    },
    {
        "SW_MUX_CTL_PAD_AUD_TXFS",
        "",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_aud_txfs
    },
    {
        "SW_MUX_CTL_PAD_ECSPI1_MISO",
        "",
        4, // Width in bytes
        0x00000068, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi1_miso
    },
    {
        "SW_MUX_CTL_PAD_ECSPI1_MOSI",
        "",
        4, // Width in bytes
        0x0000006c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi1_mosi
    },
    {
        "SW_MUX_CTL_PAD_ECSPI1_SCLK",
        "",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi1_sclk
    },
    {
        "SW_MUX_CTL_PAD_ECSPI1_SS0",
        "",
        4, // Width in bytes
        0x00000074, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi1_ss0
    },
    {
        "SW_MUX_CTL_PAD_ECSPI2_MISO",
        "",
        4, // Width in bytes
        0x00000078, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi2_miso
    },
    {
        "SW_MUX_CTL_PAD_ECSPI2_MOSI",
        "",
        4, // Width in bytes
        0x0000007c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi2_mosi
    },
    {
        "SW_MUX_CTL_PAD_ECSPI2_SCLK",
        "",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi2_sclk
    },
    {
        "SW_MUX_CTL_PAD_ECSPI2_SS0",
        "",
        4, // Width in bytes
        0x00000084, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ecspi2_ss0
    },
    {
        "SW_MUX_CTL_PAD_EPDC_BDR0",
        "",
        4, // Width in bytes
        0x00000088, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_bdr0
    },
    {
        "SW_MUX_CTL_PAD_EPDC_BDR1",
        "",
        4, // Width in bytes
        0x0000008c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_bdr1
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D0",
        "",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d0
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D1",
        "",
        4, // Width in bytes
        0x00000094, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d1
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D10",
        "",
        4, // Width in bytes
        0x00000098, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d10
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D11",
        "",
        4, // Width in bytes
        0x0000009c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d11
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D12",
        "",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d12
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D13",
        "",
        4, // Width in bytes
        0x000000a4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d13
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D14",
        "",
        4, // Width in bytes
        0x000000a8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d14
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D15",
        "",
        4, // Width in bytes
        0x000000ac, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d15
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D2",
        "",
        4, // Width in bytes
        0x000000b0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d2
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D3",
        "",
        4, // Width in bytes
        0x000000b4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d3
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D4",
        "",
        4, // Width in bytes
        0x000000b8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d4
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D5",
        "",
        4, // Width in bytes
        0x000000bc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d5
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D6",
        "",
        4, // Width in bytes
        0x000000c0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d6
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D7",
        "",
        4, // Width in bytes
        0x000000c4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d7
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D8",
        "",
        4, // Width in bytes
        0x000000c8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d8
    },
    {
        "SW_MUX_CTL_PAD_EPDC_D9",
        "",
        4, // Width in bytes
        0x000000cc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_d9
    },
    {
        "SW_MUX_CTL_PAD_EPDC_GDCLK",
        "",
        4, // Width in bytes
        0x000000d0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_gdclk
    },
    {
        "SW_MUX_CTL_PAD_EPDC_GDOE",
        "",
        4, // Width in bytes
        0x000000d4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_gdoe
    },
    {
        "SW_MUX_CTL_PAD_EPDC_GDRL",
        "",
        4, // Width in bytes
        0x000000d8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_gdrl
    },
    {
        "SW_MUX_CTL_PAD_EPDC_GDSP",
        "",
        4, // Width in bytes
        0x000000dc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_gdsp
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRCOM",
        "",
        4, // Width in bytes
        0x000000e0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrcom
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRCTRL0",
        "",
        4, // Width in bytes
        0x000000e4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl0
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRCTRL1",
        "",
        4, // Width in bytes
        0x000000e8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl1
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRCTRL2",
        "",
        4, // Width in bytes
        0x000000ec, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl2
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRCTRL3",
        "",
        4, // Width in bytes
        0x000000f0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrctrl3
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRINT",
        "",
        4, // Width in bytes
        0x000000f4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrint
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRSTAT",
        "",
        4, // Width in bytes
        0x000000f8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrstat
    },
    {
        "SW_MUX_CTL_PAD_EPDC_PWRWAKEUP",
        "",
        4, // Width in bytes
        0x000000fc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_pwrwakeup
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDCE0",
        "",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdce0
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDCE1",
        "",
        4, // Width in bytes
        0x00000104, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdce1
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDCE2",
        "",
        4, // Width in bytes
        0x00000108, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdce2
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDCE3",
        "",
        4, // Width in bytes
        0x0000010c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdce3
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDCLK",
        "",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdclk
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDLE",
        "",
        4, // Width in bytes
        0x00000114, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdle
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDOE",
        "",
        4, // Width in bytes
        0x00000118, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdoe
    },
    {
        "SW_MUX_CTL_PAD_EPDC_SDSHR",
        "",
        4, // Width in bytes
        0x0000011c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_sdshr
    },
    {
        "SW_MUX_CTL_PAD_EPDC_VCOM0",
        "",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_vcom0
    },
    {
        "SW_MUX_CTL_PAD_EPDC_VCOM1",
        "",
        4, // Width in bytes
        0x00000124, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_epdc_vcom1
    },
    {
        "SW_MUX_CTL_PAD_FEC_CRS_DV",
        "",
        4, // Width in bytes
        0x00000128, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_crs_dv
    },
    {
        "SW_MUX_CTL_PAD_FEC_MDC",
        "",
        4, // Width in bytes
        0x0000012c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_mdc
    },
    {
        "SW_MUX_CTL_PAD_FEC_MDIO",
        "",
        4, // Width in bytes
        0x00000130, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_mdio
    },
    {
        "SW_MUX_CTL_PAD_FEC_REF_CLK",
        "",
        4, // Width in bytes
        0x00000134, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_ref_clk
    },
    {
        "SW_MUX_CTL_PAD_FEC_RX_ER",
        "",
        4, // Width in bytes
        0x00000138, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_rx_er
    },
    {
        "SW_MUX_CTL_PAD_FEC_RXD0",
        "",
        4, // Width in bytes
        0x0000013c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_rxd0
    },
    {
        "SW_MUX_CTL_PAD_FEC_RXD1",
        "",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_rxd1
    },
    {
        "SW_MUX_CTL_PAD_FEC_TX_CLK",
        "",
        4, // Width in bytes
        0x00000144, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_tx_clk
    },
    {
        "SW_MUX_CTL_PAD_FEC_TX_EN",
        "",
        4, // Width in bytes
        0x00000148, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_tx_en
    },
    {
        "SW_MUX_CTL_PAD_FEC_TXD0",
        "",
        4, // Width in bytes
        0x0000014c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_txd0
    },
    {
        "SW_MUX_CTL_PAD_FEC_TXD1",
        "",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_fec_txd1
    },
    {
        "SW_MUX_CTL_PAD_HSIC_DAT",
        "",
        4, // Width in bytes
        0x00000154, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_hsic_dat
    },
    {
        "SW_MUX_CTL_PAD_HSIC_STROBE",
        "",
        4, // Width in bytes
        0x00000158, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_hsic_strobe
    },
    {
        "SW_MUX_CTL_PAD_I2C1_SCL",
        "",
        4, // Width in bytes
        0x0000015c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_i2c1_scl
    },
    {
        "SW_MUX_CTL_PAD_I2C1_SDA",
        "",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_i2c1_sda
    },
    {
        "SW_MUX_CTL_PAD_I2C2_SCL",
        "",
        4, // Width in bytes
        0x00000164, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_i2c2_scl
    },
    {
        "SW_MUX_CTL_PAD_I2C2_SDA",
        "",
        4, // Width in bytes
        0x00000168, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_i2c2_sda
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL0",
        "",
        4, // Width in bytes
        0x0000016c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col0
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL1",
        "",
        4, // Width in bytes
        0x00000170, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col1
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL2",
        "",
        4, // Width in bytes
        0x00000174, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col2
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL3",
        "",
        4, // Width in bytes
        0x00000178, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col3
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL4",
        "",
        4, // Width in bytes
        0x0000017c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col4
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL5",
        "",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col5
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL6",
        "",
        4, // Width in bytes
        0x00000184, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col6
    },
    {
        "SW_MUX_CTL_PAD_KEY_COL7",
        "",
        4, // Width in bytes
        0x00000188, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_col7
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW0",
        "",
        4, // Width in bytes
        0x0000018c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row0
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW1",
        "",
        4, // Width in bytes
        0x00000190, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row1
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW2",
        "",
        4, // Width in bytes
        0x00000194, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row2
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW3",
        "",
        4, // Width in bytes
        0x00000198, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row3
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW4",
        "",
        4, // Width in bytes
        0x0000019c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row4
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW5",
        "",
        4, // Width in bytes
        0x000001a0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row5
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW6",
        "",
        4, // Width in bytes
        0x000001a4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row6
    },
    {
        "SW_MUX_CTL_PAD_KEY_ROW7",
        "",
        4, // Width in bytes
        0x000001a8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_key_row7
    },
    {
        "SW_MUX_CTL_PAD_LCD_CLK",
        "",
        4, // Width in bytes
        0x000001ac, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_clk
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT0",
        "",
        4, // Width in bytes
        0x000001b0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat0
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT1",
        "",
        4, // Width in bytes
        0x000001b4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat1
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT10",
        "",
        4, // Width in bytes
        0x000001b8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat10
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT11",
        "",
        4, // Width in bytes
        0x000001bc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat11
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT12",
        "",
        4, // Width in bytes
        0x000001c0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat12
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT13",
        "",
        4, // Width in bytes
        0x000001c4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat13
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT14",
        "",
        4, // Width in bytes
        0x000001c8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat14
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT15",
        "",
        4, // Width in bytes
        0x000001cc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat15
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT16",
        "",
        4, // Width in bytes
        0x000001d0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat16
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT17",
        "",
        4, // Width in bytes
        0x000001d4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat17
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT18",
        "",
        4, // Width in bytes
        0x000001d8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat18
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT19",
        "",
        4, // Width in bytes
        0x000001dc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat19
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT2",
        "",
        4, // Width in bytes
        0x000001e0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat2
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT20",
        "",
        4, // Width in bytes
        0x000001e4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat20
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT21",
        "",
        4, // Width in bytes
        0x000001e8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat21
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT22",
        "",
        4, // Width in bytes
        0x000001ec, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat22
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT23",
        "",
        4, // Width in bytes
        0x000001f0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat23
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT3",
        "",
        4, // Width in bytes
        0x000001f4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat3
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT4",
        "",
        4, // Width in bytes
        0x000001f8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat4
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT5",
        "",
        4, // Width in bytes
        0x000001fc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat5
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT6",
        "",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat6
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT7",
        "",
        4, // Width in bytes
        0x00000204, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat7
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT8",
        "",
        4, // Width in bytes
        0x00000208, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat8
    },
    {
        "SW_MUX_CTL_PAD_LCD_DAT9",
        "",
        4, // Width in bytes
        0x0000020c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_dat9
    },
    {
        "SW_MUX_CTL_PAD_LCD_ENABLE",
        "",
        4, // Width in bytes
        0x00000210, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_enable
    },
    {
        "SW_MUX_CTL_PAD_LCD_HSYNC",
        "",
        4, // Width in bytes
        0x00000214, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_hsync
    },
    {
        "SW_MUX_CTL_PAD_LCD_RESET",
        "",
        4, // Width in bytes
        0x00000218, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_reset
    },
    {
        "SW_MUX_CTL_PAD_LCD_VSYNC",
        "",
        4, // Width in bytes
        0x0000021c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_lcd_vsync
    },
    {
        "SW_MUX_CTL_PAD_PWM1",
        "",
        4, // Width in bytes
        0x00000220, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_pwm1
    },
    {
        "SW_MUX_CTL_PAD_REF_CLK_24M",
        "",
        4, // Width in bytes
        0x00000224, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ref_clk_24m
    },
    {
        "SW_MUX_CTL_PAD_REF_CLK_32K",
        "",
        4, // Width in bytes
        0x00000228, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_ref_clk_32k
    },
    {
        "SW_MUX_CTL_PAD_SD1_CLK",
        "",
        4, // Width in bytes
        0x0000022c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_clk
    },
    {
        "SW_MUX_CTL_PAD_SD1_CMD",
        "",
        4, // Width in bytes
        0x00000230, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_cmd
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT0",
        "",
        4, // Width in bytes
        0x00000234, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat0
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT1",
        "",
        4, // Width in bytes
        0x00000238, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat1
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT2",
        "",
        4, // Width in bytes
        0x0000023c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat2
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT3",
        "",
        4, // Width in bytes
        0x00000240, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat3
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT4",
        "",
        4, // Width in bytes
        0x00000244, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat4
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT5",
        "",
        4, // Width in bytes
        0x00000248, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat5
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT6",
        "",
        4, // Width in bytes
        0x0000024c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat6
    },
    {
        "SW_MUX_CTL_PAD_SD1_DAT7",
        "",
        4, // Width in bytes
        0x00000250, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd1_dat7
    },
    {
        "SW_MUX_CTL_PAD_SD2_CLK",
        "",
        4, // Width in bytes
        0x00000254, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_clk
    },
    {
        "SW_MUX_CTL_PAD_SD2_CMD",
        "",
        4, // Width in bytes
        0x00000258, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_cmd
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT0",
        "",
        4, // Width in bytes
        0x0000025c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat0
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT1",
        "",
        4, // Width in bytes
        0x00000260, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat1
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT2",
        "",
        4, // Width in bytes
        0x00000264, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat2
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT3",
        "",
        4, // Width in bytes
        0x00000268, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat3
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT4",
        "",
        4, // Width in bytes
        0x0000026c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat4
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT5",
        "",
        4, // Width in bytes
        0x00000270, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat5
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT6",
        "",
        4, // Width in bytes
        0x00000274, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat6
    },
    {
        "SW_MUX_CTL_PAD_SD2_DAT7",
        "",
        4, // Width in bytes
        0x00000278, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_dat7
    },
    {
        "SW_MUX_CTL_PAD_SD2_RST",
        "",
        4, // Width in bytes
        0x0000027c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd2_rst
    },
    {
        "SW_MUX_CTL_PAD_SD3_CLK",
        "",
        4, // Width in bytes
        0x00000280, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd3_clk
    },
    {
        "SW_MUX_CTL_PAD_SD3_CMD",
        "",
        4, // Width in bytes
        0x00000284, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd3_cmd
    },
    {
        "SW_MUX_CTL_PAD_SD3_DAT0",
        "",
        4, // Width in bytes
        0x00000288, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd3_dat0
    },
    {
        "SW_MUX_CTL_PAD_SD3_DAT1",
        "",
        4, // Width in bytes
        0x0000028c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd3_dat1
    },
    {
        "SW_MUX_CTL_PAD_SD3_DAT2",
        "",
        4, // Width in bytes
        0x00000290, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd3_dat2
    },
    {
        "SW_MUX_CTL_PAD_SD3_DAT3",
        "",
        4, // Width in bytes
        0x00000294, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_sd3_dat3
    },
    {
        "SW_MUX_CTL_PAD_UART1_RXD",
        "",
        4, // Width in bytes
        0x00000298, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_uart1_rxd
    },
    {
        "SW_MUX_CTL_PAD_UART1_TXD",
        "",
        4, // Width in bytes
        0x0000029c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_uart1_txd
    },
    {
        "SW_MUX_CTL_PAD_WDOG_B",
        "",
        4, // Width in bytes
        0x000002a0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_iomuxc_sw_mux_ctl_pad_wdog_b
    },
    {
        "SW_PAD_CTL_PAD_AUD_MCLK",
        "",
        4, // Width in bytes
        0x000002a4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_aud_mclk
    },
    {
        "SW_PAD_CTL_PAD_AUD_RXC",
        "",
        4, // Width in bytes
        0x000002a8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_aud_rxc
    },
    {
        "SW_PAD_CTL_PAD_AUD_RXD",
        "",
        4, // Width in bytes
        0x000002ac, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_aud_rxd
    },
    {
        "SW_PAD_CTL_PAD_AUD_RXFS",
        "",
        4, // Width in bytes
        0x000002b0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_aud_rxfs
    },
    {
        "SW_PAD_CTL_PAD_AUD_TXC",
        "",
        4, // Width in bytes
        0x000002b4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_aud_txc
    },
    {
        "SW_PAD_CTL_PAD_AUD_TXD",
        "",
        4, // Width in bytes
        0x000002b8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_aud_txd
    },
    {
        "SW_PAD_CTL_PAD_AUD_TXFS",
        "",
        4, // Width in bytes
        0x000002bc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_aud_txfs
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A0",
        "",
        4, // Width in bytes
        0x000002c0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A1",
        "",
        4, // Width in bytes
        0x000002c4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a1
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A10",
        "",
        4, // Width in bytes
        0x000002c8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a10
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A11",
        "",
        4, // Width in bytes
        0x000002cc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a11
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A12",
        "",
        4, // Width in bytes
        0x000002d0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a12
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A13",
        "",
        4, // Width in bytes
        0x000002d4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a13
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A14",
        "",
        4, // Width in bytes
        0x000002d8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a14
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A15",
        "",
        4, // Width in bytes
        0x000002dc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a15
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A2",
        "",
        4, // Width in bytes
        0x000002e0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a2
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A3",
        "",
        4, // Width in bytes
        0x000002e4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a3
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A4",
        "",
        4, // Width in bytes
        0x000002e8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a4
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A5",
        "",
        4, // Width in bytes
        0x000002ec, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a5
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A6",
        "",
        4, // Width in bytes
        0x000002f0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a6
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A7",
        "",
        4, // Width in bytes
        0x000002f4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a7
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A8",
        "",
        4, // Width in bytes
        0x000002f8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a8
    },
    {
        "SW_PAD_CTL_PAD_DRAM_A9",
        "",
        4, // Width in bytes
        0x000002fc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_a9
    },
    {
        "SW_PAD_CTL_PAD_DRAM_CAS",
        "",
        4, // Width in bytes
        0x00000300, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_cas
    },
    {
        "SW_PAD_CTL_PAD_DRAM_CS0",
        "",
        4, // Width in bytes
        0x00000304, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_cs0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_CS1",
        "",
        4, // Width in bytes
        0x00000308, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_cs1
    },
    {
        "SW_PAD_CTL_PAD_DRAM_DQM0",
        "",
        4, // Width in bytes
        0x0000030c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_dqm0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_DQM1",
        "",
        4, // Width in bytes
        0x00000310, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_dqm1
    },
    {
        "SW_PAD_CTL_PAD_DRAM_DQM2",
        "",
        4, // Width in bytes
        0x00000314, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_dqm2
    },
    {
        "SW_PAD_CTL_PAD_DRAM_DQM3",
        "",
        4, // Width in bytes
        0x00000318, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_dqm3
    },
    {
        "SW_PAD_CTL_PAD_DRAM_RAS",
        "",
        4, // Width in bytes
        0x0000031c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_ras
    },
    {
        "SW_PAD_CTL_PAD_DRAM_RESET",
        "",
        4, // Width in bytes
        0x00000320, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_reset
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDBA0",
        "",
        4, // Width in bytes
        0x00000324, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdba0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDBA1",
        "",
        4, // Width in bytes
        0x00000328, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdba1
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDBA2",
        "",
        4, // Width in bytes
        0x0000032c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdba2
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDCKE0",
        "",
        4, // Width in bytes
        0x00000330, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdcke0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDCKE1",
        "",
        4, // Width in bytes
        0x00000334, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdcke1
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDCLK_0",
        "",
        4, // Width in bytes
        0x00000338, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdclk_0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDODT0",
        "",
        4, // Width in bytes
        0x0000033c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdodt0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDODT1",
        "",
        4, // Width in bytes
        0x00000340, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdodt1
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDQS0",
        "",
        4, // Width in bytes
        0x00000344, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdqs0
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDQS1",
        "",
        4, // Width in bytes
        0x00000348, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdqs1
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDQS2",
        "",
        4, // Width in bytes
        0x0000034c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdqs2
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDQS3",
        "",
        4, // Width in bytes
        0x00000350, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdqs3
    },
    {
        "SW_PAD_CTL_PAD_DRAM_SDWE",
        "",
        4, // Width in bytes
        0x00000354, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_dram_sdwe
    },
    {
        "SW_PAD_CTL_PAD_ECSPI1_MISO",
        "",
        4, // Width in bytes
        0x00000358, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi1_miso
    },
    {
        "SW_PAD_CTL_PAD_ECSPI1_MOSI",
        "",
        4, // Width in bytes
        0x0000035c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi1_mosi
    },
    {
        "SW_PAD_CTL_PAD_ECSPI1_SCLK",
        "",
        4, // Width in bytes
        0x00000360, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi1_sclk
    },
    {
        "SW_PAD_CTL_PAD_ECSPI1_SS0",
        "",
        4, // Width in bytes
        0x00000364, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi1_ss0
    },
    {
        "SW_PAD_CTL_PAD_ECSPI2_MISO",
        "",
        4, // Width in bytes
        0x00000368, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi2_miso
    },
    {
        "SW_PAD_CTL_PAD_ECSPI2_MOSI",
        "",
        4, // Width in bytes
        0x0000036c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi2_mosi
    },
    {
        "SW_PAD_CTL_PAD_ECSPI2_SCLK",
        "",
        4, // Width in bytes
        0x00000370, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi2_sclk
    },
    {
        "SW_PAD_CTL_PAD_ECSPI2_SS0",
        "",
        4, // Width in bytes
        0x00000374, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ecspi2_ss0
    },
    {
        "SW_PAD_CTL_PAD_EPDC_BDR0",
        "",
        4, // Width in bytes
        0x00000378, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_bdr0
    },
    {
        "SW_PAD_CTL_PAD_EPDC_BDR1",
        "",
        4, // Width in bytes
        0x0000037c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_bdr1
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D0",
        "",
        4, // Width in bytes
        0x00000380, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d0
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D1",
        "",
        4, // Width in bytes
        0x00000384, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d1
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D10",
        "",
        4, // Width in bytes
        0x00000388, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d10
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D11",
        "",
        4, // Width in bytes
        0x0000038c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d11
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D12",
        "",
        4, // Width in bytes
        0x00000390, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d12
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D13",
        "",
        4, // Width in bytes
        0x00000394, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d13
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D14",
        "",
        4, // Width in bytes
        0x00000398, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d14
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D15",
        "",
        4, // Width in bytes
        0x0000039c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d15
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D2",
        "",
        4, // Width in bytes
        0x000003a0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d2
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D3",
        "",
        4, // Width in bytes
        0x000003a4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d3
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D4",
        "",
        4, // Width in bytes
        0x000003a8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d4
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D5",
        "",
        4, // Width in bytes
        0x000003ac, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d5
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D6",
        "",
        4, // Width in bytes
        0x000003b0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d6
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D7",
        "",
        4, // Width in bytes
        0x000003b4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d7
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D8",
        "",
        4, // Width in bytes
        0x000003b8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d8
    },
    {
        "SW_PAD_CTL_PAD_EPDC_D9",
        "",
        4, // Width in bytes
        0x000003bc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_d9
    },
    {
        "SW_PAD_CTL_PAD_EPDC_GDCLK",
        "",
        4, // Width in bytes
        0x000003c0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_gdclk
    },
    {
        "SW_PAD_CTL_PAD_EPDC_GDOE",
        "",
        4, // Width in bytes
        0x000003c4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_gdoe
    },
    {
        "SW_PAD_CTL_PAD_EPDC_GDRL",
        "",
        4, // Width in bytes
        0x000003c8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_gdrl
    },
    {
        "SW_PAD_CTL_PAD_EPDC_GDSP",
        "",
        4, // Width in bytes
        0x000003cc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_gdsp
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRCOM",
        "",
        4, // Width in bytes
        0x000003d0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrcom
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRCTRL0",
        "",
        4, // Width in bytes
        0x000003d4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl0
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRCTRL1",
        "",
        4, // Width in bytes
        0x000003d8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl1
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRCTRL2",
        "",
        4, // Width in bytes
        0x000003dc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl2
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRCTRL3",
        "",
        4, // Width in bytes
        0x000003e0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrctrl3
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRINT",
        "",
        4, // Width in bytes
        0x000003e4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrint
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRSTAT",
        "",
        4, // Width in bytes
        0x000003e8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrstat
    },
    {
        "SW_PAD_CTL_PAD_EPDC_PWRWAKEUP",
        "",
        4, // Width in bytes
        0x000003ec, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_pwrwakeup
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDCE0",
        "",
        4, // Width in bytes
        0x000003f0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdce0
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDCE1",
        "",
        4, // Width in bytes
        0x000003f4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdce1
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDCE2",
        "",
        4, // Width in bytes
        0x000003f8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdce2
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDCE3",
        "",
        4, // Width in bytes
        0x000003fc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdce3
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDCLK",
        "",
        4, // Width in bytes
        0x00000400, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdclk
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDLE",
        "",
        4, // Width in bytes
        0x00000404, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdle
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDOE",
        "",
        4, // Width in bytes
        0x00000408, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdoe
    },
    {
        "SW_PAD_CTL_PAD_EPDC_SDSHR",
        "",
        4, // Width in bytes
        0x0000040c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_sdshr
    },
    {
        "SW_PAD_CTL_PAD_EPDC_VCOM0",
        "",
        4, // Width in bytes
        0x00000410, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_vcom0
    },
    {
        "SW_PAD_CTL_PAD_EPDC_VCOM1",
        "",
        4, // Width in bytes
        0x00000414, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_epdc_vcom1
    },
    {
        "SW_PAD_CTL_PAD_FEC_CRS_DV",
        "",
        4, // Width in bytes
        0x00000418, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_crs_dv
    },
    {
        "SW_PAD_CTL_PAD_FEC_MDC",
        "",
        4, // Width in bytes
        0x0000041c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_mdc
    },
    {
        "SW_PAD_CTL_PAD_FEC_MDIO",
        "",
        4, // Width in bytes
        0x00000420, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_mdio
    },
    {
        "SW_PAD_CTL_PAD_FEC_REF_CLK",
        "",
        4, // Width in bytes
        0x00000424, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_ref_clk
    },
    {
        "SW_PAD_CTL_PAD_FEC_RX_ER",
        "",
        4, // Width in bytes
        0x00000428, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_rx_er
    },
    {
        "SW_PAD_CTL_PAD_FEC_RXD0",
        "",
        4, // Width in bytes
        0x0000042c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_rxd0
    },
    {
        "SW_PAD_CTL_PAD_FEC_RXD1",
        "",
        4, // Width in bytes
        0x00000430, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_rxd1
    },
    {
        "SW_PAD_CTL_PAD_FEC_TX_CLK",
        "",
        4, // Width in bytes
        0x00000434, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_tx_clk
    },
    {
        "SW_PAD_CTL_PAD_FEC_TX_EN",
        "",
        4, // Width in bytes
        0x00000438, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_tx_en
    },
    {
        "SW_PAD_CTL_PAD_FEC_TXD0",
        "",
        4, // Width in bytes
        0x0000043c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_txd0
    },
    {
        "SW_PAD_CTL_PAD_FEC_TXD1",
        "",
        4, // Width in bytes
        0x00000440, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_fec_txd1
    },
    {
        "SW_PAD_CTL_PAD_HSIC_DAT",
        "",
        4, // Width in bytes
        0x00000444, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_hsic_dat
    },
    {
        "SW_PAD_CTL_PAD_HSIC_STROBE",
        "",
        4, // Width in bytes
        0x00000448, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_hsic_strobe
    },
    {
        "SW_PAD_CTL_PAD_I2C1_SCL",
        "",
        4, // Width in bytes
        0x0000044c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_i2c1_scl
    },
    {
        "SW_PAD_CTL_PAD_I2C1_SDA",
        "",
        4, // Width in bytes
        0x00000450, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_i2c1_sda
    },
    {
        "SW_PAD_CTL_PAD_I2C2_SCL",
        "",
        4, // Width in bytes
        0x00000454, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_i2c2_scl
    },
    {
        "SW_PAD_CTL_PAD_I2C2_SDA",
        "",
        4, // Width in bytes
        0x00000458, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_i2c2_sda
    },
    {
        "SW_PAD_CTL_PAD_JTAG_MOD",
        "",
        4, // Width in bytes
        0x0000045c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_jtag_mod
    },
    {
        "SW_PAD_CTL_PAD_JTAG_TCK",
        "",
        4, // Width in bytes
        0x00000460, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_jtag_tck
    },
    {
        "SW_PAD_CTL_PAD_JTAG_TDI",
        "",
        4, // Width in bytes
        0x00000464, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_jtag_tdi
    },
    {
        "SW_PAD_CTL_PAD_JTAG_TDO",
        "",
        4, // Width in bytes
        0x00000468, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_jtag_tdo
    },
    {
        "SW_PAD_CTL_PAD_JTAG_TMS",
        "",
        4, // Width in bytes
        0x0000046c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_jtag_tms
    },
    {
        "SW_PAD_CTL_PAD_JTAG_TRSTB",
        "",
        4, // Width in bytes
        0x00000470, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_jtag_trstb
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL0",
        "",
        4, // Width in bytes
        0x00000474, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col0
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL1",
        "",
        4, // Width in bytes
        0x00000478, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col1
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL2",
        "",
        4, // Width in bytes
        0x0000047c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col2
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL3",
        "",
        4, // Width in bytes
        0x00000480, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col3
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL4",
        "",
        4, // Width in bytes
        0x00000484, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col4
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL5",
        "",
        4, // Width in bytes
        0x00000488, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col5
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL6",
        "",
        4, // Width in bytes
        0x0000048c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col6
    },
    {
        "SW_PAD_CTL_PAD_KEY_COL7",
        "",
        4, // Width in bytes
        0x00000490, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_col7
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW0",
        "",
        4, // Width in bytes
        0x00000494, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row0
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW1",
        "",
        4, // Width in bytes
        0x00000498, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row1
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW2",
        "",
        4, // Width in bytes
        0x0000049c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row2
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW3",
        "",
        4, // Width in bytes
        0x000004a0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row3
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW4",
        "",
        4, // Width in bytes
        0x000004a4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row4
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW5",
        "",
        4, // Width in bytes
        0x000004a8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row5
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW6",
        "",
        4, // Width in bytes
        0x000004ac, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row6
    },
    {
        "SW_PAD_CTL_PAD_KEY_ROW7",
        "",
        4, // Width in bytes
        0x000004b0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_key_row7
    },
    {
        "SW_PAD_CTL_PAD_LCD_CLK",
        "",
        4, // Width in bytes
        0x000004b4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_clk
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT0",
        "",
        4, // Width in bytes
        0x000004b8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat0
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT1",
        "",
        4, // Width in bytes
        0x000004bc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat1
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT10",
        "",
        4, // Width in bytes
        0x000004c0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat10
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT11",
        "",
        4, // Width in bytes
        0x000004c4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat11
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT12",
        "",
        4, // Width in bytes
        0x000004c8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat12
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT13",
        "",
        4, // Width in bytes
        0x000004cc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat13
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT14",
        "",
        4, // Width in bytes
        0x000004d0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat14
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT15",
        "",
        4, // Width in bytes
        0x000004d4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat15
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT16",
        "",
        4, // Width in bytes
        0x000004d8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat16
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT17",
        "",
        4, // Width in bytes
        0x000004dc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat17
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT18",
        "",
        4, // Width in bytes
        0x000004e0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat18
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT19",
        "",
        4, // Width in bytes
        0x000004e4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat19
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT2",
        "",
        4, // Width in bytes
        0x000004e8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat2
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT20",
        "",
        4, // Width in bytes
        0x000004ec, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat20
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT21",
        "",
        4, // Width in bytes
        0x000004f0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat21
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT22",
        "",
        4, // Width in bytes
        0x000004f4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat22
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT23",
        "",
        4, // Width in bytes
        0x000004f8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat23
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT3",
        "",
        4, // Width in bytes
        0x000004fc, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat3
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT4",
        "",
        4, // Width in bytes
        0x00000500, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat4
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT5",
        "",
        4, // Width in bytes
        0x00000504, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat5
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT6",
        "",
        4, // Width in bytes
        0x00000508, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat6
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT7",
        "",
        4, // Width in bytes
        0x0000050c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat7
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT8",
        "",
        4, // Width in bytes
        0x00000510, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat8
    },
    {
        "SW_PAD_CTL_PAD_LCD_DAT9",
        "",
        4, // Width in bytes
        0x00000514, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_dat9
    },
    {
        "SW_PAD_CTL_PAD_LCD_ENABLE",
        "",
        4, // Width in bytes
        0x00000518, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_enable
    },
    {
        "SW_PAD_CTL_PAD_LCD_HSYNC",
        "",
        4, // Width in bytes
        0x0000051c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_hsync
    },
    {
        "SW_PAD_CTL_PAD_LCD_RESET",
        "",
        4, // Width in bytes
        0x00000520, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_reset
    },
    {
        "SW_PAD_CTL_PAD_LCD_VSYNC",
        "",
        4, // Width in bytes
        0x00000524, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_lcd_vsync
    },
    {
        "SW_PAD_CTL_PAD_PWM1",
        "",
        4, // Width in bytes
        0x00000528, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_pwm1
    },
    {
        "SW_PAD_CTL_PAD_REF_CLK_24M",
        "",
        4, // Width in bytes
        0x0000052c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ref_clk_24m
    },
    {
        "SW_PAD_CTL_PAD_REF_CLK_32K",
        "",
        4, // Width in bytes
        0x00000530, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_ref_clk_32k
    },
    {
        "SW_PAD_CTL_PAD_SD1_CLK",
        "",
        4, // Width in bytes
        0x00000534, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_clk
    },
    {
        "SW_PAD_CTL_PAD_SD1_CMD",
        "",
        4, // Width in bytes
        0x00000538, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_cmd
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT0",
        "",
        4, // Width in bytes
        0x0000053c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat0
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT1",
        "",
        4, // Width in bytes
        0x00000540, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat1
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT2",
        "",
        4, // Width in bytes
        0x00000544, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat2
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT3",
        "",
        4, // Width in bytes
        0x00000548, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat3
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT4",
        "",
        4, // Width in bytes
        0x0000054c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat4
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT5",
        "",
        4, // Width in bytes
        0x00000550, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat5
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT6",
        "",
        4, // Width in bytes
        0x00000554, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat6
    },
    {
        "SW_PAD_CTL_PAD_SD1_DAT7",
        "",
        4, // Width in bytes
        0x00000558, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd1_dat7
    },
    {
        "SW_PAD_CTL_PAD_SD2_CLK",
        "",
        4, // Width in bytes
        0x0000055c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_clk
    },
    {
        "SW_PAD_CTL_PAD_SD2_CMD",
        "",
        4, // Width in bytes
        0x00000560, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_cmd
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT0",
        "",
        4, // Width in bytes
        0x00000564, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat0
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT1",
        "",
        4, // Width in bytes
        0x00000568, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat1
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT2",
        "",
        4, // Width in bytes
        0x0000056c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat2
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT3",
        "",
        4, // Width in bytes
        0x00000570, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat3
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT4",
        "",
        4, // Width in bytes
        0x00000574, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat4
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT5",
        "",
        4, // Width in bytes
        0x00000578, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat5
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT6",
        "",
        4, // Width in bytes
        0x0000057c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat6
    },
    {
        "SW_PAD_CTL_PAD_SD2_DAT7",
        "",
        4, // Width in bytes
        0x00000580, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_dat7
    },
    {
        "SW_PAD_CTL_PAD_SD2_RST",
        "",
        4, // Width in bytes
        0x00000584, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd2_rst
    },
    {
        "SW_PAD_CTL_PAD_SD3_CLK",
        "",
        4, // Width in bytes
        0x00000588, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd3_clk
    },
    {
        "SW_PAD_CTL_PAD_SD3_CMD",
        "",
        4, // Width in bytes
        0x0000058c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd3_cmd
    },
    {
        "SW_PAD_CTL_PAD_SD3_DAT0",
        "",
        4, // Width in bytes
        0x00000590, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd3_dat0
    },
    {
        "SW_PAD_CTL_PAD_SD3_DAT1",
        "",
        4, // Width in bytes
        0x00000594, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd3_dat1
    },
    {
        "SW_PAD_CTL_PAD_SD3_DAT2",
        "",
        4, // Width in bytes
        0x00000598, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd3_dat2
    },
    {
        "SW_PAD_CTL_PAD_SD3_DAT3",
        "",
        4, // Width in bytes
        0x0000059c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_sd3_dat3
    },
    {
        "SW_PAD_CTL_PAD_UART1_RXD",
        "",
        4, // Width in bytes
        0x000005a0, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_uart1_rxd
    },
    {
        "SW_PAD_CTL_PAD_UART1_TXD",
        "",
        4, // Width in bytes
        0x000005a4, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_uart1_txd
    },
    {
        "SW_PAD_CTL_PAD_WDOG_B",
        "",
        4, // Width in bytes
        0x000005a8, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_pad_wdog_b
    },
    {
        "SW_PAD_CTL_GRP_ADDDS",
        "",
        4, // Width in bytes
        0x000005ac, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_addds
    },
    {
        "SW_PAD_CTL_GRP_DDRMODE_CTL",
        "",
        4, // Width in bytes
        0x000005b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_ddrmode_ctl
    },
    {
        "SW_PAD_CTL_GRP_DDRPKE",
        "",
        4, // Width in bytes
        0x000005b4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_ddrpke
    },
    {
        "SW_PAD_CTL_GRP_DDRPK",
        "",
        4, // Width in bytes
        0x000005b8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_ddrpk
    },
    {
        "SW_PAD_CTL_GRP_DDRHYS",
        "",
        4, // Width in bytes
        0x000005bc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_ddrhys
    },
    {
        "SW_PAD_CTL_GRP_DDRMODE",
        "",
        4, // Width in bytes
        0x000005c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_ddrmode
    },
    {
        "SW_PAD_CTL_GRP_B0DS",
        "",
        4, // Width in bytes
        0x000005c4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_b0ds
    },
    {
        "SW_PAD_CTL_GRP_CTLDS",
        "",
        4, // Width in bytes
        0x000005c8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_ctlds
    },
    {
        "SW_PAD_CTL_GRP_B1DS",
        "",
        4, // Width in bytes
        0x000005cc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_b1ds
    },
    {
        "SW_PAD_CTL_GRP_DDR_TYPE",
        "",
        4, // Width in bytes
        0x000005d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_ddr_type
    },
    {
        "SW_PAD_CTL_GRP_B2DS",
        "",
        4, // Width in bytes
        0x000005d4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_b2ds
    },
    {
        "SW_PAD_CTL_GRP_B3DS",
        "",
        4, // Width in bytes
        0x000005d8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_sw_pad_ctl_grp_b3ds
    },
    {
        "USB_OTG_ID_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005dc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usb_otg_id_select_input
    },
    {
        "USB_IPP_IND_H1_ID_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usb_ipp_ind_h1_id_select_input
    },
    {
        "AUDMUX_P4_INPUT_DA_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005e4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p4_input_da_amx_select_input
    },
    {
        "AUDMUX_P4_INPUT_DB_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005e8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p4_input_db_amx_select_input
    },
    {
        "AUDMUX_P4_INPUT_RXCLK_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005ec, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p4_input_rxclk_amx_select_input
    },
    {
        "AUDMUX_P4_INPUT_RXFS_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p4_input_rxfs_amx_select_input
    },
    {
        "AUDMUX_P4_INPUT_TXCLK_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005f4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p4_input_txclk_amx_select_input
    },
    {
        "AUDMUX_P4_INPUT_TXFS_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005f8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p4_input_txfs_amx_select_input
    },
    {
        "AUDMUX_P5_INPUT_DA_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000005fc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p5_input_da_amx_select_input
    },
    {
        "AUDMUX_P5_INPUT_DB_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000600, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p5_input_db_amx_select_input
    },
    {
        "AUDMUX_P5_INPUT_RXCLK_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000604, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p5_input_rxclk_amx_select_input
    },
    {
        "AUDMUX_P5_INPUT_RXFS_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000608, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p5_input_rxfs_amx_select_input
    },
    {
        "AUDMUX_P5_INPUT_TXCLK_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000060c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p5_input_txclk_amx_select_input
    },
    {
        "AUDMUX_P5_INPUT_TXFS_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000610, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p5_input_txfs_amx_select_input
    },
    {
        "AUDMUX_P6_INPUT_DA_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000614, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p6_input_da_amx_select_input
    },
    {
        "AUDMUX_P6_INPUT_DB_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000618, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p6_input_db_amx_select_input
    },
    {
        "AUDMUX_P6_INPUT_RXCLK_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000061c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p6_input_rxclk_amx_select_input
    },
    {
        "AUDMUX_P6_INPUT_RXFS_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000620, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p6_input_rxfs_amx_select_input
    },
    {
        "AUDMUX_P6_INPUT_TXCLK_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000624, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p6_input_txclk_amx_select_input
    },
    {
        "AUDMUX_P6_INPUT_TXFS_AMX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000628, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_audmux_p6_input_txfs_amx_select_input
    },
    {
        "CCM_PMIC_VFUNCIONAL_READY_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000062c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ccm_pmic_vfuncional_ready_select_input
    },
    {
        "CSI_IPP_CSI_D_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000630, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_0_select_input
    },
    {
        "CSI_IPP_CSI_D_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000634, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_1_select_input
    },
    {
        "CSI_IPP_CSI_D_2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000638, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_2_select_input
    },
    {
        "CSI_IPP_CSI_D_3_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000063c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_3_select_input
    },
    {
        "CSI_IPP_CSI_D_4_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000640, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_4_select_input
    },
    {
        "CSI_IPP_CSI_D_5_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000644, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_5_select_input
    },
    {
        "CSI_IPP_CSI_D_6_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000648, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_6_select_input
    },
    {
        "CSI_IPP_CSI_D_7_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000064c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_7_select_input
    },
    {
        "CSI_IPP_CSI_D_8_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000650, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_8_select_input
    },
    {
        "CSI_IPP_CSI_D_9_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000654, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_9_select_input
    },
    {
        "CSI_IPP_CSI_D_10_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000658, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_10_select_input
    },
    {
        "CSI_IPP_CSI_D_11_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000065c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_11_select_input
    },
    {
        "CSI_IPP_CSI_D_12_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000660, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_12_select_input
    },
    {
        "CSI_IPP_CSI_D_13_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000664, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_13_select_input
    },
    {
        "CSI_IPP_CSI_D_14_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000668, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_14_select_input
    },
    {
        "CSI_IPP_CSI_D_15_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000066c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_d_15_select_input
    },
    {
        "CSI_IPP_CSI_HSYNC_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000670, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_hsync_select_input
    },
    {
        "CSI_IPP_CSI_PIXCLK_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000674, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_pixclk_select_input
    },
    {
        "CSI_IPP_CSI_VSYNC_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000678, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_csi_ipp_csi_vsync_select_input
    },
    {
        "ECSPI1_IPP_CSPI_CLK_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000067c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_cspi_clk_in_select_input
    },
    {
        "ECSPI1_IPP_IND_DATAREADY_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000680, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_ind_dataready_b_select_input
    },
    {
        "ECSPI1_IPP_IND_MISO_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000684, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_ind_miso_select_input
    },
    {
        "ECSPI1_IPP_IND_MOSI_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000688, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_ind_mosi_select_input
    },
    {
        "ECSPI1_IPP_IND_SS_B_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000068c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_ind_ss_b_0_select_input
    },
    {
        "ECSPI1_IPP_IND_SS_B_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000690, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_ind_ss_b_1_select_input
    },
    {
        "ECSPI1_IPP_IND_SS_B_2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000694, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_ind_ss_b_2_select_input
    },
    {
        "ECSPI1_IPP_IND_SS_B_3_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000698, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi1_ipp_ind_ss_b_3_select_input
    },
    {
        "ECSPI2_IPP_CSPI_CLK_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000069c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi2_ipp_cspi_clk_in_select_input
    },
    {
        "ECSPI2_IPP_IND_MISO_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi2_ipp_ind_miso_select_input
    },
    {
        "ECSPI2_IPP_IND_MOSI_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006a4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi2_ipp_ind_mosi_select_input
    },
    {
        "ECSPI2_IPP_IND_SS_B_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006a8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi2_ipp_ind_ss_b_0_select_input
    },
    {
        "ECSPI2_IPP_IND_SS_B_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006ac, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi2_ipp_ind_ss_b_1_select_input
    },
    {
        "ECSPI3_IPP_CSPI_CLK_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_cspi_clk_in_select_input
    },
    {
        "ECSPI3_IPP_IND_DATAREADY_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006b4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_ind_dataready_b_select_input
    },
    {
        "ECSPI3_IPP_IND_MISO_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006b8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_ind_miso_select_input
    },
    {
        "ECSPI3_IPP_IND_MOSI_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006bc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_ind_mosi_select_input
    },
    {
        "ECSPI3_IPP_IND_SS_B_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_ind_ss_b_0_select_input
    },
    {
        "ECSPI3_IPP_IND_SS_B_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006c4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_ind_ss_b_1_select_input
    },
    {
        "ECSPI3_IPP_IND_SS_B_2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006c8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_ind_ss_b_2_select_input
    },
    {
        "ECSPI3_IPP_IND_SS_B_3_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006cc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi3_ipp_ind_ss_b_3_select_input
    },
    {
        "ECSPI4_IPP_CSPI_CLK_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi4_ipp_cspi_clk_in_select_input
    },
    {
        "ECSPI4_IPP_IND_MISO_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006d4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi4_ipp_ind_miso_select_input
    },
    {
        "ECSPI4_IPP_IND_MOSI_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006d8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi4_ipp_ind_mosi_select_input
    },
    {
        "ECSPI4_IPP_IND_SS_B_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006dc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi4_ipp_ind_ss_b_0_select_input
    },
    {
        "ECSPI4_IPP_IND_SS_B_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi4_ipp_ind_ss_b_1_select_input
    },
    {
        "ECSPI4_IPP_IND_SS_B_2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006e4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_ecspi4_ipp_ind_ss_b_2_select_input
    },
    {
        "EPDC_IPP_EPDC_PWRIRQ_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006e8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_epdc_ipp_epdc_pwrirq_select_input
    },
    {
        "EPDC_IPP_EPDC_PWRSTAT_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006ec, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_epdc_ipp_epdc_pwrstat_select_input
    },
    {
        "FEC_FEC_COL_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_col_select_input
    },
    {
        "FEC_FEC_MDI_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006f4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_mdi_select_input
    },
    {
        "FEC_FEC_RDATA_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006f8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_rdata_0_select_input
    },
    {
        "FEC_FEC_RDATA_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000006fc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_rdata_1_select_input
    },
    {
        "FEC_FEC_RX_CLK_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000700, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_rx_clk_select_input
    },
    {
        "FEC_FEC_RX_DV_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000704, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_rx_dv_select_input
    },
    {
        "FEC_FEC_RX_ER_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000708, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_rx_er_select_input
    },
    {
        "FEC_FEC_TX_CLK_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000070c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_fec_fec_tx_clk_select_input
    },
    {
        "GPT_IPP_IND_CAPIN1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000710, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_gpt_ipp_ind_capin1_select_input
    },
    {
        "GPT_IPP_IND_CAPIN2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000714, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_gpt_ipp_ind_capin2_select_input
    },
    {
        "GPT_IPP_IND_CLKIN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000718, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_gpt_ipp_ind_clkin_select_input
    },
    {
        "I2C1_IPP_SCL_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000071c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_i2c1_ipp_scl_in_select_input
    },
    {
        "I2C1_IPP_SDA_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000720, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_i2c1_ipp_sda_in_select_input
    },
    {
        "I2C2_IPP_SCL_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000724, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_i2c2_ipp_scl_in_select_input
    },
    {
        "I2C2_IPP_SDA_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000728, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_i2c2_ipp_sda_in_select_input
    },
    {
        "I2C3_IPP_SCL_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000072c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_i2c3_ipp_scl_in_select_input
    },
    {
        "I2C3_IPP_SDA_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000730, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_i2c3_ipp_sda_in_select_input
    },
    {
        "KPP_IPP_IND_COL_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000734, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_0_select_input
    },
    {
        "KPP_IPP_IND_COL_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000738, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_1_select_input
    },
    {
        "KPP_IPP_IND_COL_2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000073c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_2_select_input
    },
    {
        "KPP_IPP_IND_COL_3_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000740, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_3_select_input
    },
    {
        "KPP_IPP_IND_COL_4_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000744, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_4_select_input
    },
    {
        "KPP_IPP_IND_COL_5_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000748, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_5_select_input
    },
    {
        "KPP_IPP_IND_COL_6_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000074c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_6_select_input
    },
    {
        "KPP_IPP_IND_COL_7_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000750, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_col_7_select_input
    },
    {
        "KPP_IPP_IND_ROW_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000754, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_0_select_input
    },
    {
        "KPP_IPP_IND_ROW_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000758, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_1_select_input
    },
    {
        "KPP_IPP_IND_ROW_2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000075c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_2_select_input
    },
    {
        "KPP_IPP_IND_ROW_3_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000760, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_3_select_input
    },
    {
        "KPP_IPP_IND_ROW_4_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000764, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_4_select_input
    },
    {
        "KPP_IPP_IND_ROW_5_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000768, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_5_select_input
    },
    {
        "KPP_IPP_IND_ROW_6_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000076c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_6_select_input
    },
    {
        "KPP_IPP_IND_ROW_7_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000770, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_kpp_ipp_ind_row_7_select_input
    },
    {
        "LCDIF_LCDIF_BUSY_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000774, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_busy_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_0_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000778, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_0_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000077c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_1_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000780, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_2_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_3_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000784, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_3_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_4_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000788, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_4_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_5_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000078c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_5_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_6_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000790, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_6_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_7_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000794, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_7_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_8_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000798, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_8_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_9_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000079c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_9_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_10_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_10_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_11_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007a4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_11_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_12_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007a8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_12_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_13_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007ac, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_13_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_14_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_14_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_15_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007b4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_15_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_16_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007b8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_16_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_17_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007bc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_17_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_18_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_18_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_19_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007c4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_19_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_20_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007c8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_20_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_21_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007cc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_21_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_22_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_22_select_input
    },
    {
        "LCDIF_LCDIF_RXDATA_23_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007d4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_lcdif_lcdif_rxdata_23_select_input
    },
    {
        "SPDIF_SPDIF_IN1_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_spdif_spdif_in1_select_input
    },
    {
        "SPDIF_TX_CLK2_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007f4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_spdif_tx_clk2_select_input
    },
    {
        "UART1_IPP_UART_RTS_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007f8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart1_ipp_uart_rts_b_select_input
    },
    {
        "UART1_IPP_UART_RXD_MUX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x000007fc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart1_ipp_uart_rxd_mux_select_input
    },
    {
        "UART2_IPP_UART_RTS_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000800, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart2_ipp_uart_rts_b_select_input
    },
    {
        "UART2_IPP_UART_RXD_MUX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000804, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart2_ipp_uart_rxd_mux_select_input
    },
    {
        "UART3_IPP_UART_RTS_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000808, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart3_ipp_uart_rts_b_select_input
    },
    {
        "UART3_IPP_UART_RXD_MUX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000080c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart3_ipp_uart_rxd_mux_select_input
    },
    {
        "UART4_IPP_UART_RTS_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000810, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart4_ipp_uart_rts_b_select_input
    },
    {
        "UART4_IPP_UART_RXD_MUX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000814, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart4_ipp_uart_rxd_mux_select_input
    },
    {
        "UART5_IPP_UART_RTS_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000818, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart5_ipp_uart_rts_b_select_input
    },
    {
        "UART5_IPP_UART_RXD_MUX_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000081c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_uart5_ipp_uart_rxd_mux_select_input
    },
    {
        "USB_IPP_IND_OTG2_OC_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000820, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usb_ipp_ind_otg2_oc_select_input
    },
    {
        "USB_IPP_IND_OTG_OC_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000824, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usb_ipp_ind_otg_oc_select_input
    },
    {
        "USDHC1_IPP_CARD_DET_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000828, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc1_ipp_card_det_select_input
    },
    {
        "USDHC1_IPP_WP_ON_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000082c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc1_ipp_wp_on_select_input
    },
    {
        "USDHC2_IPP_CARD_DET_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000830, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc2_ipp_card_det_select_input
    },
    {
        "USDHC2_IPP_WP_ON_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000834, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc2_ipp_wp_on_select_input
    },
    {
        "USDHC3_IPP_CARD_DET_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000838, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc3_ipp_card_det_select_input
    },
    {
        "USDHC3_IPP_DAT4_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000083c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc3_ipp_dat4_in_select_input
    },
    {
        "USDHC3_IPP_DAT5_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000840, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc3_ipp_dat5_in_select_input
    },
    {
        "USDHC3_IPP_DAT6_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000844, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc3_ipp_dat6_in_select_input
    },
    {
        "USDHC3_IPP_DAT7_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000848, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc3_ipp_dat7_in_select_input
    },
    {
        "USDHC3_IPP_WP_ON_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000084c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc3_ipp_wp_on_select_input
    },
    {
        "USDHC4_IPP_CARD_CLK_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000850, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_card_clk_in_select_input
    },
    {
        "USDHC4_IPP_CARD_DET_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000854, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_card_det_select_input
    },
    {
        "USDHC4_IPP_CMD_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000858, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_cmd_in_select_input
    },
    {
        "USDHC4_IPP_DAT0_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000085c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat0_in_select_input
    },
    {
        "USDHC4_IPP_DAT1_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000860, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat1_in_select_input
    },
    {
        "USDHC4_IPP_DAT2_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000864, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat2_in_select_input
    },
    {
        "USDHC4_IPP_DAT3_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000868, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat3_in_select_input
    },
    {
        "USDHC4_IPP_DAT4_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000086c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat4_in_select_input
    },
    {
        "USDHC4_IPP_DAT5_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000870, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat5_in_select_input
    },
    {
        "USDHC4_IPP_DAT6_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000874, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat6_in_select_input
    },
    {
        "USDHC4_IPP_DAT7_IN_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000878, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_dat7_in_select_input
    },
    {
        "USDHC4_IPP_WP_ON_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x0000087c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_usdhc4_ipp_wp_on_select_input
    },
    {
        "EIM_IPP_IND_DTACK_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000880, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_eim_ipp_ind_dtack_b_select_input
    },
    {
        "EIM_IPP_IND_WAIT_B_SELECT_INPUT",
        "",
        4, // Width in bytes
        0x00000884, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_iomuxc_eim_ipp_ind_wait_b_select_input
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark KPP
#endif

// Bitfields in register KPP_KPCR.
static const field_t hw_kpp_kpcr[] =
{
    {
        "KRE",
        "Keypad Row Enable.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "KCO",
        "Keypad Column Strobe Open-Drain Enable.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register KPP_KPSR.
static const field_t hw_kpp_kpsr[] =
{
    {
        "KPKD",
        "Keypad Key Depress.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "KPKR",
        "Keypad Key Release.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "KDSC",
        "Key Depress Synchronizer Clear.",
        2, // LSB
        2, // MSB
        false, // Readable
        true // Writable
    },
    {
        "KPP_KRSS",
        "Key Release Synchronizer Set.",
        3, // LSB
        3, // MSB
        false, // Readable
        true // Writable
    },
    {
        "KDIE",
        "Keypad Key Depress Interrupt Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "KRIE",
        "Keypad Release Interrupt Enable.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register KPP_KDDR.
static const field_t hw_kpp_kddr[] =
{
    {
        "KRDD",
        "Keypad Row Data Direction.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "KCDD",
        "Keypad Column Data Direction Register.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register KPP_KPDR.
static const field_t hw_kpp_kpdr[] =
{
    {
        "KRD",
        "Keypad Row Data.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "KCD",
        "Keypad Column Data.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a KPP module.
static const reg_t hw_kpp[] =
{
    {
        "KPCR",
        "The Keypad Control Register determines which of the eight possible col"
        "umn strobes are to be open drain when configured as outputs, and which"
        " of the eight row sense lines are considered in generating an interrup"
        "t to the core.",
        2, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_kpp_kpcr
    },
    {
        "KPSR",
        "The Keypad Status Register reflects the state of the key press detect "
        "circuit.",
        2, // Width in bytes
        0x00000002, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_kpp_kpsr
    },
    {
        "KDDR",
        "The bits in the KPP_KDDR control the direction of the keypad port pins"
        ".",
        2, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_kpp_kddr
    },
    {
        "KPDR",
        "This 16-bit register is used to access the column and row data.",
        2, // Width in bytes
        0x00000006, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_kpp_kpdr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark MMDC
#endif

// Bitfields in register MMDC_MDCTL.
static const field_t hw_mmdc_mdctl[] =
{
    {
        "DSIZ",
        "DDR data bus size.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BL",
        "Burst Length.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COL",
        "Column Address Width.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROW",
        "Row Address Width.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDE_1",
        "MMDC Enable CS1.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDE_0",
        "MMDC Enable CS0.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDPDC.
static const field_t hw_mmdc_mdpdc[] =
{
    {
        "TCKSRE",
        "Valid clock cycles after self-refresh entry.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCKSRX",
        "Valid clock cycles before self-refresh exit.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BOTH_CS_PD",
        "Parallel power down entry to both chip selects.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLOW_PD",
        "Slow/fast power down.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWDT_0",
        "Power Down Timer - Chip Select 0.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWDT_1",
        "Power Down Timer - Chip Select 1.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCKE",
        "CKE minimum pulse width.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRCT_0",
        "Precharge Timer - Chip Select 0.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRCT_1",
        "Precharge Timer - Chip Select 1.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDOTC.
static const field_t hw_mmdc_mdotc[] =
{
    {
        "TODT_IDLE_OFF",
        "ODT turn off latency.",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TODTLON",
        "ODT turn on latency.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TAXPD",
        "Asynchronous ODT to power down exit delay.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TANPD",
        "Asynchronous ODT to power down entry delay.",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TAONPD",
        "Asynchronous RTT turn-on delay (power down with DLL frozen).",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TAOFPD",
        "Asynchronous RTT turn-off delay (power down with DLL frozen).",
        27, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDCFG0.
static const field_t hw_mmdc_mdcfg0[] =
{
    {
        "TCL",
        "CAS Read Latency.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFAW",
        "Four Active Window (all banks).",
        4, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXPDLL",
        "Exit precharge power down with DLL frozen to commands requiring DLL.",
        9, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXP",
        "Exit power down with DLL-on to any valid command.",
        13, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "Exit self refresh to non READ command.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRFC",
        "Refresh command to Active or Refresh command time.",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDCFG1.
static const field_t hw_mmdc_mdcfg1[] =
{
    {
        "TCWL",
        "CAS Write Latency.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TMRD",
        "Mode Register Set command cycle (all banks).",
        5, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TWR",
        "WRITE recovery time (same bank).",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRPA",
        "Precharge-all command period.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRAS",
        "Active to Precharge command period (same bank).",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRC",
        "Active to Active or Refresh command period (same bank).",
        21, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRP",
        "Precharge command period (same bank).",
        26, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRCD",
        "Active command to internal read or write delay time (same bank).",
        29, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDCFG2.
static const field_t hw_mmdc_mdcfg2[] =
{
    {
        "TRRD",
        "Active to Active command period (all banks).",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TWTR",
        "Internal WRITE to READ command delay (same bank).",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRTP",
        "Internal READ command to Precharge command delay (same bank).",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TDLLK",
        "DLL locking time.",
        16, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDMISC.
static const field_t hw_mmdc_mdmisc[] =
{
    {
        "RST",
        "Software Reset.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_TYPE",
        "DDR TYPE.",
        3, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_4_BANK",
        "Number of banks per DDR device.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RALAT",
        "Read Additional Latency.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MIF3_MODE",
        "Command prediction working mode.",
        9, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPDDR2_S2",
        "LPDDR2 S2 device type indication.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BI_ON",
        "Bank Interleaving On.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WALAT",
        "Write Additional latency.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LHD",
        "Latency hiding disable.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDR_MIRROR",
        "Address mirroring.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CALIB_PER_CS",
        "Number of chip-select for calibration process.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CK1_GATING",
        "Gating the secondary DDR clock.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CS1_RDY",
        "External status device on CS1.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CS0_RDY",
        "External status device on CS0.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDSCR.
static const field_t hw_mmdc_mdscr[] =
{
    {
        "CMD_BA",
        "Bank Address.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMD_CS",
        "Chip Select.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMD",
        "Command.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_EN",
        "DQS pads direction.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MRR_READ_DATA_VALID",
        "MRR read data valid.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CON_ACK",
        "Configuration acknowledge.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CON_REQ",
        "Configuration request.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMD_ADDR_LSB",
        "Command/Address LSB.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMD_ADDR_MSB",
        "Command/Address MSB.",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDREF.
static const field_t hw_mmdc_mdref[] =
{
    {
        "START_REF",
        "Manual start of refresh cycle.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFR",
        "Refresh Rate.",
        11, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REF_SEL",
        "Refresh Selector.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REF_CNT",
        "Refresh Counter at DDR clock period  If REF_SEL equals '2' a refresh c"
        "ycle will begin every amount of DDR cycles configured in this field.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDRWD.
static const field_t hw_mmdc_mdrwd[] =
{
    {
        "RTR_DIFF",
        "Read to read delay for different chip-select.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTW_DIFF",
        "Read to write delay for different chip-select.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WTW_DIFF",
        "Write to write delay for different chip-select.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WTR_DIFF",
        "Write to read delay for different chip-select.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTW_SAME",
        "Read to write delay for the same chip-select.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TDAI",
        "Device auto initialization period.(maximum)  This field is relevant on"
        "ly to LPDDR2 mode",
        16, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDOR.
static const field_t hw_mmdc_mdor[] =
{
    {
        "RST_TO_CKE",
        "DDR3: Time from SDE enable to CKE rise.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDE_TO_RST",
        "DDR3: Time from SDE enable until DDR reset# is high.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXPR",
        "DDR3: CKE HIGH to a valid command.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDMRR.
static const field_t hw_mmdc_mdmrr[] =
{
    {
        "MRR_READ_DATA0",
        "MRR DATA that arrived on DQ[7:0]",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MRR_READ_DATA1",
        "MRR DATA that arrived on DQ[15:8]",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MRR_READ_DATA2",
        "MRR DATA that arrived on DQ[23:16]",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MRR_READ_DATA3",
        "MRR DATA that arrived on DQ[31:24]",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDCFG3LP.
static const field_t hw_mmdc_mdcfg3lp[] =
{
    {
        "TRPAB_LP",
        "Precharge (all banks) command period.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRPPB_LP",
        "Precharge (per bank) command period (same bank).",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRCD_LP",
        "Active command to internal read or write delay time (same bank).",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RC_LP",
        "Active to Active or Refresh command period (same bank).",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDMR4.
static const field_t hw_mmdc_mdmr4[] =
{
    {
        "UPDATE_DE_REQ",
        "Update Derated Values Request.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPDATE_DE_ACK",
        "Update Derated Values Acknowledge.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TRCD_DE",
        "tRCD derating value.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRC_DE",
        "tRC derating value.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRAS_DE",
        "tRAS derating value.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRP_DE",
        "tRP derating value.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRRD_DE",
        "tRRD derating value.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MDASP.
static const field_t hw_mmdc_mdasp[] =
{
    {
        "CS0_END",
        "CS0_END.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MAARCR.
static const field_t hw_mmdc_maarcr[] =
{
    {
        "ARCR_GUARD",
        "ARCR Guard.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_DYN_MAX",
        "ARCR Dynamic Maximum.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_DYN_JMP",
        "ARCR Dynamic Jump.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_ACC_HIT",
        "ARCR Access Hit Rate.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_PAG_HIT",
        "ARCR Page Hit Rate.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_RCH_EN",
        "This bit defines whether Real time channel is activated and bypassed a"
        "ll other pending accesses, So accesses with QoS=='F' will be granted t"
        "he highest prioritiy in the optimization/reordering mechanism  Default"
        " value is 0x1 - encoding 1 (Enabled)",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_EXC_ERR_EN",
        "This bit defines whether exclusive read/write access violation of AXI "
        "6.2.4 rule result in SLV Error response or in OKAY response  Default v"
        "alue is 0x1 - encoding 1(response is SLV Error)",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_SEC_ERR_EN",
        "This bit defines whether security read/write access violation result i"
        "n SLV Error response or in OKAY response  Default value is 0x1 - encod"
        "ing 1(response is SLV Error, rresp/bresp=2'b10)",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ARCR_SEC_ERR_LOCK",
        "Once set, this bit locks ARCR_SEC_ERR_EN and prevents from its updatin"
        "g.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MAPSR.
static const field_t hw_mmdc_mapsr[] =
{
    {
        "PSD",
        "Automatic Power Saving Disable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSS",
        "Power Saving Status.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RIS",
        "Read Idle Status.This read only bit indicates whether read request buf"
        "fer is idle (empty) or not.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WIS",
        "Write Idle Status.This read only bit indicates whether write request b"
        "uffer is idle (empty) or not.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PST",
        "Automatic Power saving timer.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPMD",
        "General LPMD request.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DVFS",
        "General DVFS request.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPACK",
        "General low-power acknowledge.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DVACK",
        "General DVFS acknowledge.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MAEXIDR0.
static const field_t hw_mmdc_maexidr0[] =
{
    {
        "EXC_ID_MONITOR0",
        "This feild defines ID for Exclusive monitor#0.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EXC_ID_MONITOR1",
        "This feild defines ID for Exclusive monitor#1.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MAEXIDR1.
static const field_t hw_mmdc_maexidr1[] =
{
    {
        "EXC_ID_MONITOR2",
        "This feild defines ID for Exclusive monitor#2.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EXC_ID_MONITOR3",
        "This feild defines ID for Exclusive monitor#3.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPCR0.
static const field_t hw_mmdc_madpcr0[] =
{
    {
        "DBG_EN",
        "Debug and Profiling Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DBG_RST",
        "Debug and Profiling Reset.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRF_FRZ",
        "Profiling freeze.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CYC_OVF",
        "Total Profiling Cycles Count Overflow.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SBS_EN",
        "Step By Step debug Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SBS",
        "Step By Step trigger.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPCR1.
static const field_t hw_mmdc_madpcr1[] =
{
    {
        "PRF_AXI_ID",
        "Profiling AXI ID.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRF_AXI_ID_MASK",
        "Profiling AXI ID Mask.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPSR0.
static const field_t hw_mmdc_madpsr0[] =
{
    {
        "CYC_COUNT",
        "Total Profiling cycle Count.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPSR1.
static const field_t hw_mmdc_madpsr1[] =
{
    {
        "BUSY_COUNT",
        "Profiling Busy Cycles Count.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPSR2.
static const field_t hw_mmdc_madpsr2[] =
{
    {
        "RD_ACC_COUNT",
        "Profiling Read Access Count.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPSR3.
static const field_t hw_mmdc_madpsr3[] =
{
    {
        "WR_ACC_COUNT",
        "Profiling Write Access Count.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPSR4.
static const field_t hw_mmdc_madpsr4[] =
{
    {
        "RD_BYTES_COUNT",
        "Profiling Read Bytes Count.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MADPSR5.
static const field_t hw_mmdc_madpsr5[] =
{
    {
        "WR_BYTES_COUNT",
        "Profiling Write Bytes Count.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MASBS0.
static const field_t hw_mmdc_masbs0[] =
{
    {
        "SBS_ADDR",
        "Step By Step Address.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MASBS1.
static const field_t hw_mmdc_masbs1[] =
{
    {
        "SBS_VLD",
        "Step By Step Valid.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_TYPE",
        "Step By Step Request Type.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_LOCK",
        "Step By Step Lock.",
        2, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_PROT",
        "Step By Step Protection.",
        4, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_SIZE",
        "Step By Step Size.",
        7, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_BURST",
        "Step By Step Burst.",
        10, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_BUFF",
        "Step By Step Buffered.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_LEN",
        "Step By Step Length.",
        13, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SBS_AXI_ID",
        "Step By Step AXI ID.",
        16, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MAGENP.
static const field_t hw_mmdc_magenp[] =
{
    {
        "GP31_GP0",
        "General purpose read/write bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPZQHWCTRL.
static const field_t hw_mmdc_mpzqhwctrl[] =
{
    {
        "ZQ_MODE",
        "ZQ calibration mode:",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_HW_PER",
        "ZQ periodic calibration time.",
        2, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_HW_PU_RES",
        "ZQ automatic calibration pull-up result.",
        6, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ZQ_HW_PD_RES",
        "ZQ HW calibration pull-down result.",
        11, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ZQ_HW_FOR",
        "Force ZQ automatic calibration process with the i.MX ZQ calibration pa"
        "d.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TZQ_INIT",
        "Device ZQ long/init time.",
        17, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TZQ_OPER",
        "Device ZQ long/oper time.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TZQ_CS",
        "Device ZQ short time.",
        23, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_EARLY_COMPARATOR_EN_TIMER",
        "ZQ early comparator enable timer.",
        27, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPZQSWCTRL.
static const field_t hw_mmdc_mpzqswctrl[] =
{
    {
        "ZQ_SW_FOR",
        "ZQ SW calibration enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_SW_RES",
        "ZQ software calibration result.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ZQ_SW_PU_VAL",
        "ZQ software pull-up resistence.This field determines the value of the "
        "PU resistor during SW ZQ calibration.",
        2, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_SW_PD_VAL",
        "ZQ software pull-down resistence.This field determines the value of th"
        "e PD resistor during SW ZQ calibration.",
        7, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_SW_PD",
        "ZQ software PU/PD calibration.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USE_ZQ_SW_VAL",
        "Use SW ZQ configured value for I/O pads resistor controls.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_CMP_OUT_SMP",
        "Defines the amount of cycles between driving the ZQ signals to the ZQ "
        "pad and till sampling the comparator enable output while performing ZQ"
        " calibration process with the i.MX ZQ calibration pad",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWLGCR.
static const field_t hw_mmdc_mpwlgcr[] =
{
    {
        "HW_WL_EN",
        "Write-Leveling HW (automatic) enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_WL_EN",
        "Write-Leveling SW enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_WL_CNT_EN",
        "SW write-leveling count down enable.This bit when asserted set a certa"
        "in delay of (25+15) cycles from the setting of SW_WL_EN and before dri"
        "ving the DQS to the DDR device.This bit should be asserted before the "
        "first SW write-leveling request and after issuing the write leveling M"
        "RS command",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_SW_RES0",
        "Byte0 write-leveling software result.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_SW_RES1",
        "Byte1 write-leveling software result.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_SW_RES2",
        "Byte2 write-leveling software result.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_SW_RES3",
        "Byte3 write-leveling software result.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_HW_ERR0",
        "Byte0 write-leveling HW calibration error.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_HW_ERR1",
        "Byte1 write-leveling HW calibration error.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_HW_ERR2",
        "Byte2 write-leveling HW calibration error.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_HW_ERR3",
        "Byte3 write-leveling HW calibration error.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWLDECTRL0.
static const field_t hw_mmdc_mpwldectrl0[] =
{
    {
        "WL_DL_ABS_OFFSET0",
        "Absolute write-leveling delay offset for Byte 0.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_HC_DEL0",
        "Write leveling half cycle delay for Byte 0.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_CYC_DEL0",
        "Write leveling cycle delay for Byte 0.",
        9, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_DL_ABS_OFFSET1",
        "Absolute write-leveling delay offset for Byte 1.",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_HC_DEL1",
        "Write leveling half cycle delay for Byte 1.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_CYC_DEL1",
        "Write leveling cycle delay for Byte 1.",
        25, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWLDECTRL1.
static const field_t hw_mmdc_mpwldectrl1[] =
{
    {
        "WL_DL_ABS_OFFSET2",
        "Absolute write-leveling delay offset for Byte 2.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_HC_DEL2",
        "Write leveling half cycle delay for Byte 2.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_CYC_DEL2",
        "Write leveling cycle delay for Byte 2.",
        9, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_DL_ABS_OFFSET3",
        "Absolute write-leveling delay offset for Byte 3.",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_HC_DEL3",
        "Write leveling half cycle delay for Byte 3.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL_CYC_DEL3",
        "Write leveling cycle delay for Byte 3.",
        25, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWLDLST.
static const field_t hw_mmdc_mpwldlst[] =
{
    {
        "WL_DL_UNIT_NUM0",
        "This field reflects the number of delay units that are actually used b"
        "y write leveling delay-line 0.",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_DL_UNIT_NUM1",
        "This field reflects the number of delay units that are actually used b"
        "y write leveling delay-line 1.",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_DL_UNIT_NUM2",
        "This field reflects the number of delay units that are actually used b"
        "y write leveling delay-line 2.",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WL_DL_UNIT_NUM3",
        "This field reflects the number of delay units that are actually used b"
        "y write leveling delay-line 3.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPODTCTRL.
static const field_t hw_mmdc_mpodtctrl[] =
{
    {
        "ODT_WR_PAS_EN",
        "Inactive write CS ODT enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT_WR_ACT_EN",
        "Active write CS ODT enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT_RD_PAS_EN",
        "Inactive read CS ODT enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT_RD_ACT_EN",
        "Active read CS ODT enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT0_INT_RES",
        "On chip ODT byte0 resistor - This field determines the Rtt_Nom of the "
        "on chip ODT byte0 resistor during read accesses.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT1_INT_RES",
        "On chip ODT byte1 resistor - This field determines the Rtt_Nom of the "
        "on chip ODT byte1 resistor during read accesses.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT2_INT_RES",
        "On chip ODT byte2 resistor - This field determines the Rtt_Nom of the "
        "on chip ODT byte2 resistor during read accesses.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ODT3_INT_RES",
        "On chip ODT byte3 resistor - This field determines the Rtt_Nom of the "
        "on chip ODT byte3 resistor during read accesses.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDQBY0DL.
static const field_t hw_mmdc_mprddqby0dl[] =
{
    {
        "RD_DQ0_DEL",
        "Read dqs0 to dq0 delay fine-tuning.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ1_DEL",
        "Read dqs0 to dq1 delay fine-tuning.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ2_DEL",
        "Read dqs0 to dq2 delay fine-tuning.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ3_DEL",
        "Read dqs0 to dq3 delay fine-tuning.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ4_DEL",
        "Read dqs0 to dq4 delay fine-tuning.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ5_DEL",
        "Read dqs0 to dq5 delay fine-tuning.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ6_DEL",
        "Read dqs0 to dq6 delay fine-tuning.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ7_DEL",
        "Read dqs0 to dq7 delay fine-tuning.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDQBY1DL.
static const field_t hw_mmdc_mprddqby1dl[] =
{
    {
        "RD_DQ8_DEL",
        "Read dqs1 to dq8 delay fine-tuning.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ9_DEL",
        "Read dqs1 to dq9 delay fine-tuning.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ10_DEL",
        "Read dqs1 to dq10 delay fine-tuning.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ11_DEL",
        "Read dqs1 to dq11 delay fine-tuning.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ12_DEL",
        "Read dqs1 to dq12 delay fine-tuning.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ13_DEL",
        "Read dqs1 to dq13 delay fine-tuning.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ14_DEL",
        "Read dqs1 to dq14 delay fine-tuning.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ15_DEL",
        "Read dqs1 to dq15 delay fine-tuning.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDQBY2DL.
static const field_t hw_mmdc_mprddqby2dl[] =
{
    {
        "RD_DQ16_DEL",
        "Read dqs2 to dq16 delay fine-tuning.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ17_DEL",
        "Read dqs2 to dq17 delay fine-tuning.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ18_DEL",
        "Read dqs2 to dq18 delay fine-tuning.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ19_DEL",
        "Read dqs2 to dq19 delay fine-tuning.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ20_DEL",
        "Read dqs2 to dq20 delay fine-tuning.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ21_DEL",
        "Read dqs2 to dq21 delay fine-tuning.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ22_DEL",
        "Read dqs2 to dq22 delay fine-tuning.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ23_DEL",
        "Read dqs2 to dq23 delay fine-tuning.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDQBY3DL.
static const field_t hw_mmdc_mprddqby3dl[] =
{
    {
        "RD_DQ24_DEL",
        "Read dqs3 to dq24 delay fine-tuning.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ25_DEL",
        "Read dqs3 to dq25 delay fine-tuning.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ26_DEL",
        "Read dqs3 to dq26 delay fine-tuning.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ27_DEL",
        "Read dqs3 to dq27 delay fine-tuning.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ28_DEL",
        "Read dqs3 to dq28 delay fine-tuning.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ29_DEL",
        "Read dqs3 to dq29 delay fine-tuning.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ30_DEL",
        "Read dqs3 to dq30 delay fine-tuning.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DQ31_DEL",
        "Read dqs3 to dq31 delay fine-tuning.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDQBY0DL.
static const field_t hw_mmdc_mpwrdqby0dl[] =
{
    {
        "WR_DQ0_DEL",
        "Write dq0 delay fine-tuning.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ1_DEL",
        "Write dq1 delay fine-tuning.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ2_DEL",
        "Write dq2 delay fine-tuning.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ3_DEL",
        "Write dq3 delay fine-tuning.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ4_DEL",
        "Write dq4 delay fine-tuning.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ5_DEL",
        "Write dq5 delay fine-tuning.",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ6_DEL",
        "Write dq6 delay fine-tuning.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ7_DEL",
        "Write dq7 delay fine-tuning.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DM0_DEL",
        "Write dm0 delay fine-tuning.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDQBY1DL.
static const field_t hw_mmdc_mpwrdqby1dl[] =
{
    {
        "WR_DQ8_DEL",
        "Write dq8 delay fine-tuning.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ9_DEL",
        "Write dq9 delay fine-tuning.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ10_DEL",
        "Write dq10 delay fine-tuning.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ11_DEL",
        "Write dq11 delay fine-tuning.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ12_DEL",
        "Write dq12 delay fine-tuning.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ13_DEL",
        "Write dq13 delay fine-tuning.",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ14_DEL",
        "Write dq14 delay fine-tuning.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ15_DEL",
        "Write dq15 delay fine-tuning.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DM1_DEL",
        "Write dm1 delay fine-tuning.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDQBY2DL.
static const field_t hw_mmdc_mpwrdqby2dl[] =
{
    {
        "WR_DQ16_DEL",
        "Write dq16 delay fine tuning.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ17_DEL",
        "Write dq17 delay fine tuning.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ18_DEL",
        "Write dq18 delay fine tuning.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ19_DEL",
        "Write dq19 delay fine tuning.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ20_DEL",
        "Write dq20 delay fine tuning.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ21_DEL",
        "Write dq21 delay fine tuning.",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ22_DEL",
        "Write dq22 delay fine tuning.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ23_DEL",
        "Write dq23 delay fine tuning.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DM2_DEL",
        "Write dm2 delay fine-tuning.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDQBY3DL.
static const field_t hw_mmdc_mpwrdqby3dl[] =
{
    {
        "WR_DQ24_DEL",
        "Write dq24 delay fine tuning.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ25_DEL",
        "Write dq25 delay fine tuning.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ26_DEL",
        "Write dq26 delay fine tuning.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ27_DEL",
        "Write dq27 delay fine tuning.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ28_DEL",
        "Write dq28 delay fine tuning.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ29_DEL",
        "Write dq29 delay fine tuning.",
        20, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ30_DEL",
        "Write dq30 delay fine tuning.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DQ31_DEL",
        "Write dq31 delay fine tuning.",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DM3_DEL",
        "Write dm3 delay fine tuning.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDGCTRL0.
static const field_t hw_mmdc_mpdgctrl0[] =
{
    {
        "DG_DL_ABS_OFFSET0",
        "Absolute read DQS gating delay offset for Byte0.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DG_HC_DEL0",
        "Read DQS gating half cycles delay for Byte0  .",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HW_DG_ERR",
        "HW DQS gating error.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DG_DL_ABS_OFFSET1",
        "Absolute read DQS gating delay offset for Byte1.",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DG_EXT_UP",
        "DG extend upper boundary.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DG_HC_DEL1",
        "Read DQS gating half cycles delay for Byte1 .",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HW_DG_EN",
        "Enable automatic read DQS gating calibration.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DG_DIS",
        "Read DQS gating disable.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DG_CMP_CYC",
        "Read DQS gating sample cycle.",
        30, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RST_RD_FIFO",
        "Reset Read Data FIFO and associated pointers.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDGCTRL1.
static const field_t hw_mmdc_mpdgctrl1[] =
{
    {
        "DG_DL_ABS_OFFSET2",
        "Absolute read DQS gating delay offset for Byte2.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DG_HC_DEL2",
        "Read DQS gating half cycles delay for Byte2 .",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DG_DL_ABS_OFFSET3",
        "Absolute read DQS gating delay offset for Byte3.",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DG_HC_DEL3",
        "Read DQS gating half cycles delay for Byte3 .",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDGDLST0.
static const field_t hw_mmdc_mpdgdlst0[] =
{
    {
        "DG_DL_UNIT_NUM0",
        "This field reflects the number of delay units that are actually used b"
        "y read DQS gating delay-line 0.",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DG_DL_UNIT_NUM1",
        "This field reflects the number of delay units that are actually used b"
        "y read DQS gating delay-line 1.",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DG_DL_UNIT_NUM2",
        "This field reflects the number of delay units that are actually used b"
        "y read DQS gating delay-line 2.",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DG_DL_UNIT_NUM3",
        "This field reflects the number of delay units that are actually used b"
        "y read DQS gating delay-line 3.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDLCTL.
static const field_t hw_mmdc_mprddlctl[] =
{
    {
        "RD_DL_ABS_OFFSET0",
        "Absolute read delay offset for Byte0.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DL_ABS_OFFSET1",
        "Absolute read delay offset for Byte1.",
        8, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DL_ABS_OFFSET2",
        "Absolute read delay offset for Byte2.",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DL_ABS_OFFSET3",
        "Absolute read delay offset for Byte3.",
        24, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDLST.
static const field_t hw_mmdc_mprddlst[] =
{
    {
        "RD_DL_UNIT_NUM0",
        "This field reflects the number of delay units that are actually used b"
        "y read delay-line 0.",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RD_DL_UNIT_NUM1",
        "This field reflects the number of delay units that are actually used b"
        "y read delay-line 1.",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RD_DL_UNIT_NUM2",
        "This field reflects the number of delay units that are actually used b"
        "y read delay-line 2.",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RD_DL_UNIT_NUM3",
        "This field reflects the number of delay units that are actually used b"
        "y read delay-line 3.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDLCTL.
static const field_t hw_mmdc_mpwrdlctl[] =
{
    {
        "WR_DL_ABS_OFFSET0",
        "Absolute write delay offset for Byte0.",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DL_ABS_OFFSET1",
        "Absolute write delay offset for Byte1.",
        8, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DL_ABS_OFFSET2",
        "Absolute write delay offset for Byte2.",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_DL_ABS_OFFSET3",
        "Absolute write delay offset for Byte3.",
        24, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDLST.
static const field_t hw_mmdc_mpwrdlst[] =
{
    {
        "WR_DL_UNIT_NUM0",
        "This field reflects the number of delay units that are actually used b"
        "y write delay-line 0.",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WR_DL_UNIT_NUM1",
        "This field reflects the number of delay units that are actually used b"
        "y write delay-line 1.",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WR_DL_UNIT_NUM2",
        "This field reflects the number of delay units that are actually used b"
        "y write delay-line 2.",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WR_DL_UNIT_NUM3",
        "This field reflects the number of delay units that are actually used b"
        "y write delay-line 3.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSDCTRL.
static const field_t hw_mmdc_mpsdctrl[] =
{
    {
        "SDCLK0_DEL",
        "DDR clock0 delay fine tuning.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDCLK1_DEL",
        "DDR clock1 delay fine tuning.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPZQLP2CTL.
static const field_t hw_mmdc_mpzqlp2ctl[] =
{
    {
        "ZQ_LP2_HW_ZQINIT",
        "This register defines the period in cycles that it takes the memory de"
        "vice to perform a Init ZQ calibration.",
        0, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_LP2_HW_ZQCL",
        "This register defines the period in cycles that it takes the memory de"
        "vice to perform a long ZQ calibration.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ZQ_LP2_HW_ZQCS",
        "This register defines the period in cycles that it takes the memory de"
        "vice to perform a Short ZQ calibration.",
        24, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDLHWCTL.
static const field_t hw_mmdc_mprddlhwctl[] =
{
    {
        "HW_RD_DL_ERR0",
        "Automatic (HW) read calibration error of Byte0.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_ERR1",
        "Automatic (HW) read calibration error of Byte1.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_ERR2",
        "Automatic (HW) read calibration error of Byte2.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_ERR3",
        "Automatic (HW) read calibration error of Byte3.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_EN",
        "Enable automatic (HW) read calibration.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HW_RD_DL_CMP_CYC",
        "Automatic (HW) read sample cycle.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDLHWCTL.
static const field_t hw_mmdc_mpwrdlhwctl[] =
{
    {
        "HW_WR_DL_ERR0",
        "Automatic (HW) write calibration error of Byte0.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_ERR1",
        "Automatic (HW) write calibration error of Byte1.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_ERR2",
        "Automatic (HW) write calibration error of Byte2.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_ERR3",
        "Automatic (HW) write calibration error of Byte3.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_EN",
        "Enable automatic (HW) write calibration.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HW_WR_DL_CMP_CYC",
        "Write sample cycle.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDLHWST0.
static const field_t hw_mmdc_mprddlhwst0[] =
{
    {
        "HW_RD_DL_LOW0",
        "Automatic (HW) read calibration result of the lower boundary of Byte0.",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_UP0",
        "Automatic (HW) read calibration result of the upper boundary of Byte0.",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_LOW1",
        "Automatic (HW) read calibration result of the lower boundary of Byte1.",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_UP1",
        "Automatic (HW) read calibration result of the upper boundary of Byte1.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPRDDLHWST1.
static const field_t hw_mmdc_mprddlhwst1[] =
{
    {
        "HW_RD_DL_LOW2",
        "Automatic (HW) read calibration result of the lower boundary of Byte2.",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_UP2",
        "Automatic (HW) read calibration result of the upper boundary of Byte2.",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_LOW3",
        "Automatic (HW) read calibration result of the lower boundary of Byte3.",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_RD_DL_UP3",
        "Automatic (HW) read calibration result of the upper boundary of Byte3.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDLHWST0.
static const field_t hw_mmdc_mpwrdlhwst0[] =
{
    {
        "HW_WR_DL_LOW0",
        "Automatic (HW) write calibration result of the lower boundary of Byte0"
        ".",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_UP0",
        "Automatic (HW) write calibration result of the upper boundary of Byte0"
        ".",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_LOW1",
        "Automatic (HW) write calibration result of the lower boundary of Byte1"
        ".",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_UP1",
        "Aautomatic (HW) write utomatic (HW) write calibration result of the up"
        "per boundary of Byte1.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRDLHWST1.
static const field_t hw_mmdc_mpwrdlhwst1[] =
{
    {
        "HW_WR_DL_LOW2",
        "Automatic (HW) write calibration result of the lower boundary of Byte2"
        ".",
        0, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_UP2",
        "Automatic (HW) write calibration result of the upper boundary of Byte2"
        ".",
        8, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_LOW3",
        "Automatic (HW) write calibration result of the lower boundary of Byte3"
        ".",
        16, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WR_DL_UP3",
        "Automatic (HW) write calibration result of the upper boundary of Byte3"
        ".",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWLHWERR.
static const field_t hw_mmdc_mpwlhwerr[] =
{
    {
        "HW_WL0_DQ",
        "HW write-leveling calibration result of Byte0.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WL1_DQ",
        "HW write-leveling calibration result of Byte1.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WL2_DQ",
        "HW write-leveling calibration result of Byte2.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_WL3_DQ",
        "HW write-leveling calibration result of Byte3.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDGHWST0.
static const field_t hw_mmdc_mpdghwst0[] =
{
    {
        "HW_DG_LOW0",
        "HW DQS gating calibration result of the lower boundary of Byte0.",
        0, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_DG_UP0",
        "HW DQS gating calibration result of the upper boundary of Byte0.",
        16, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDGHWST1.
static const field_t hw_mmdc_mpdghwst1[] =
{
    {
        "HW_DG_LOW1",
        "HW DQS gating calibration result of the lower boundary of Byte1.",
        0, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_DG_UP1",
        "HW DQS gating calibration result of the upper boundary of Byte1.",
        16, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDGHWST2.
static const field_t hw_mmdc_mpdghwst2[] =
{
    {
        "HW_DG_LOW2",
        "HW DQS gating calibration result of the lower boundary of Byte2.",
        0, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_DG_UP2",
        "HW DQS gating calibration result of the upper boundary of Byte2.",
        16, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDGHWST3.
static const field_t hw_mmdc_mpdghwst3[] =
{
    {
        "HW_DG_LOW3",
        "HW DQS gating calibration result of the lower boundary of Byte3.",
        0, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HW_DG_UP3",
        "HW DQS gating calibration result of the upper boundary of Byte3.",
        16, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPPDCMPR1.
static const field_t hw_mmdc_mppdcmpr1[] =
{
    {
        "PDV1",
        "MMDC Pre defined comapre value2.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PDV2",
        "MMDC Pre defined comapre value2.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPPDCMPR2.
static const field_t hw_mmdc_mppdcmpr2[] =
{
    {
        "MPR_CMP",
        "MPR(DDR3)/DQ calibration (LPDDR2) comapre enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MPR_FULL_CMP",
        "MPR(DDR3)/DQ calibration (LPDDR2) full compare enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "READ_LEVEL_PATTERN",
        "MPR(DDR3)/DQ calibration(LPDDR2) read compare pattern.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CA_DL_ABS_OFFSET",
        "Absolute CA (Command/Address of LPDDRR2) offset.",
        16, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PHY_CA_DL_UNIT",
        "This field reflects the number of delay units that are actually used b"
        "y CA (Command/Address of LPDDR2) delay-line",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDAR0.
static const field_t hw_mmdc_mpswdar0[] =
{
    {
        "SW_DUMMY_WR",
        "SW dummy write.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_DUMMY_RD",
        "SW dummy read.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_DUM_CMP0",
        "SW dummy read byte0 compare results.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SW_DUM_CMP1",
        "SW dummy read byte1 compare results.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SW_DUM_CMP2",
        "SW dummy read byte2 compare results.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SW_DUM_CMP3",
        "SW dummy read byte3 compare results.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR0.
static const field_t hw_mmdc_mpswdrdr0[] =
{
    {
        "DUM_RD0",
        "Dummy read data0.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR1.
static const field_t hw_mmdc_mpswdrdr1[] =
{
    {
        "DUM_RD1",
        "Dummy read data1.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR2.
static const field_t hw_mmdc_mpswdrdr2[] =
{
    {
        "DUM_RD2",
        "Dummy read data2.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR3.
static const field_t hw_mmdc_mpswdrdr3[] =
{
    {
        "DUM_RD3",
        "Dummy read data3.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR4.
static const field_t hw_mmdc_mpswdrdr4[] =
{
    {
        "DUM_RD4",
        "Dummy read data4.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR5.
static const field_t hw_mmdc_mpswdrdr5[] =
{
    {
        "DUM_RD5",
        "Dummy read data5.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR6.
static const field_t hw_mmdc_mpswdrdr6[] =
{
    {
        "DUM_RD6",
        "Dummy read data6.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPSWDRDR7.
static const field_t hw_mmdc_mpswdrdr7[] =
{
    {
        "DUM_RD7",
        "Dummy read data7.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPMUR0.
static const field_t hw_mmdc_mpmur0[] =
{
    {
        "MU_BYP_VAL",
        "Number of delay units for measurement bypass.",
        0, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MU_BYP_EN",
        "Measure unit bypass enable.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRC_MSR",
        "Force measuement on delay-lines.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MU_UNIT_DEL_NUM",
        "Number of delay units measured per cycle.",
        16, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPWRCADL.
static const field_t hw_mmdc_mpwrcadl[] =
{
    {
        "WR_CA0_DEL",
        "CA (Command/Address LPDDR2 bus) bit 0 delay fine tuning.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA1_DEL",
        "CA (Command/Address LPDDR2 bus) bit 1 delay fine tuning.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA2_DEL",
        "CA (Command/Address LPDDR2 bus) bit 2 delay fine tuning.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA3_DEL",
        "CA (Command/Address LPDDR2 bus) bit 3 delay fine tuning.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA4_DEL",
        "CA (Command/Address LPDDR2 bus) bit 4 delay fine tuning.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA5_DEL",
        "CA (Command/Address LPDDR2 bus) bit 5 delay fine tuning.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA6_DEL",
        "CA (Command/Address LPDDR2 bus) bit 6 delay fine tuning.",
        12, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA7_DEL",
        "CA (Command/Address LPDDR2 bus) bit 7 delay fine tuning.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA8_DEL",
        "CA (Command/Address LPDDR2 bus) bit 8 delay fine tuning.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_CA9_DEL",
        "CA (Command/Address LPDDR2 bus) bit 9 delay fine tuning.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register MMDC_MPDCCR.
static const field_t hw_mmdc_mpdccr[] =
{
    {
        "WR_DQS0_FT_DCC",
        "Write DQS duty cycle fine tuning control of Byte0.",
        0, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WR_DQS1_FT_DCC",
        "Write DQS duty cycle fine tuning control of Byte1.",
        3, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WR_DQS2_FT_DCC",
        "Write DQS duty cycle fine tuning control of Byte1.",
        6, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WR_DQS3_FT_DCC",
        "Write DQS duty cycle fine tuning control of Byte0.",
        9, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CK_FT0_DCC",
        "Primary duty cycle fine tuning control of DDR clock.",
        12, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CK_FT1_DCC",
        "Secondary duty cycle fine tuning control of DDR clock.",
        16, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RD_DQS0_FT_DCC",
        "Read DQS duty cycle fine tuning control of Byte0.",
        19, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RD_DQS1_FT_DCC",
        "Read DQS duty cycle fine tuning control of Byte1.",
        22, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RD_DQS2_FT_DCC",
        "Read DQS duty cycle fine tuning control of Byte2.",
        25, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RD_DQS3_FT_DCC",
        "Read DQS duty cycle fine tuning control of Byte3.",
        28, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a MMDC module.
static const reg_t hw_mmdc[] =
{
    {
        "MDCTL",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mdctl
    },
    {
        "MDPDC",
        "PRCT field encoding        PRCT[2:0]    Precharge Timer        000    "
        "Disabled (Bit field reset value)      001    2 clocks      010    4 cl"
        "ocks      011    8 clocks      100    16 clocks      101    32 clocks "
        "     110    64 clocks      111    128 clocks         PWDT field encodi"
        "ng        PWDT[3:0]    Power Down Time-out        0000    Disabled (bi"
        "t field reset value)      0001    16 cycles      0010    32 cycles    "
        "  0011    64 cycles      0100    128 cycles      0101    256 cycles   "
        "   0110    512 cycles      0111    1024 cycles      1000    2048 cycle"
        "s      1001    4096 cycles      1010    8196 cycles      1011    16384"
        " cycles      1100    32768 cycles      1101-1111    Reserved",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mdpdc
    },
    {
        "MDOTC",
        "For further information see .",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mdotc
    },
    {
        "MDCFG0",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mdcfg0
    },
    {
        "MDCFG1",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_mmdc_mdcfg1
    },
    {
        "MDCFG2",
        "",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mdcfg2
    },
    {
        "MDMISC",
        "",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_mmdc_mdmisc
    },
    {
        "MDSCR",
        "This register is used to issue special commands manually toward the ex"
        "ternal DDR device (such as load mode register, manual self refresh, ma"
        "nual precharge and so on).",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mdscr
    },
    {
        "MDREF",
        "This register determines the refresh scheme that will be executed towa"
        "rd the DDR device.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mdref
    },
    {
        "MDRWD",
        "This register determines the delay between back to back read and write"
        " accesses.",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mdrwd
    },
    {
        "MDOR",
        "This register defines delays that must be kept when MMDC exits reset.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_mmdc_mdor
    },
    {
        "MDMRR",
        "This register contains data that was collected after issuing MRR comma"
        "nd.",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mdmrr
    },
    {
        "MDCFG3LP",
        "This register is relevant only for LPDDR2 mode.",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mdcfg3lp
    },
    {
        "MDMR4",
        "This register is relevant only for LPDDR2 mode.",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_mmdc_mdmr4
    },
    {
        "MDASP",
        "This register defines the partitioning between chip select 0 and chip "
        "select 1.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_mmdc_mdasp
    },
    {
        "MAARCR",
        "This register determines the values of the weights used for the re-ord"
        "ering arbitration engine.",
        4, // Width in bytes
        0x00000400, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_maarcr
    },
    {
        "MAPSR",
        "The MAPSR determines the power saving features of MMDC.",
        4, // Width in bytes
        0x00000404, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mapsr
    },
    {
        "MAEXIDR0",
        "This register defines the ID to be monitored for exclusive accesses of"
        " monitor0 and monitor1.",
        4, // Width in bytes
        0x00000408, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_mmdc_maexidr0
    },
    {
        "MAEXIDR1",
        "This register defines the ID to be monitored for exclusive accesses of"
        " monitor2 and monitor3.",
        4, // Width in bytes
        0x0000040c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_mmdc_maexidr1
    },
    {
        "MADPCR0",
        "",
        4, // Width in bytes
        0x00000410, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_madpcr0
    },
    {
        "MADPCR1",
        "",
        4, // Width in bytes
        0x00000414, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_mmdc_madpcr1
    },
    {
        "MADPSR0",
        "",
        4, // Width in bytes
        0x00000418, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_madpsr0
    },
    {
        "MADPSR1",
        "The register reflects the total cycles during which the MMDC state mac"
        "hines were busy (both writes and reads).",
        4, // Width in bytes
        0x0000041c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_madpsr1
    },
    {
        "MADPSR2",
        "This register reflects the total number of read accesses (per AXI ID) "
        "toward MMDC.",
        4, // Width in bytes
        0x00000420, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_madpsr2
    },
    {
        "MADPSR3",
        "This register reflects the total number of write accesses (per AXI ID)"
        " toward MMDC.",
        4, // Width in bytes
        0x00000424, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_madpsr3
    },
    {
        "MADPSR4",
        "This register reflects the total number of bytes that were transferred"
        " during read access (per AXI ID) toward MMDC.",
        4, // Width in bytes
        0x00000428, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_madpsr4
    },
    {
        "MADPSR5",
        "This register reflects the total number of bytes that were transferred"
        " during write access (per AXI ID) toward MMDC.",
        4, // Width in bytes
        0x0000042c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_madpsr5
    },
    {
        "MASBS0",
        "",
        4, // Width in bytes
        0x00000430, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_masbs0
    },
    {
        "MASBS1",
        "",
        4, // Width in bytes
        0x00000434, // Base address offset
        true, // Readable
        false, // Writable
        9, // Number of bitfields
        hw_mmdc_masbs1
    },
    {
        "MAGENP",
        "This register is a general 32 bit read/write register.",
        4, // Width in bytes
        0x00000440, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_mmdc_magenp
    },
    {
        "MPZQHWCTRL",
        "",
        4, // Width in bytes
        0x00000800, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mpzqhwctrl
    },
    {
        "MPZQSWCTRL",
        "",
        4, // Width in bytes
        0x00000804, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_mmdc_mpzqswctrl
    },
    {
        "MPWLGCR",
        "",
        4, // Width in bytes
        0x00000808, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_mmdc_mpwlgcr
    },
    {
        "MPWLDECTRL0",
        "",
        4, // Width in bytes
        0x0000080c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mpwldectrl0
    },
    {
        "MPWLDECTRL1",
        "",
        4, // Width in bytes
        0x00000810, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mpwldectrl1
    },
    {
        "MPWLDLST",
        "This register holds the status of the four write leveling delay-lines.",
        4, // Width in bytes
        0x00000814, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mpwldlst
    },
    {
        "MPODTCTRL",
        "In LPDDR2 mode this register should be cleared, so no termination will"
        " be activated",
        4, // Width in bytes
        0x00000818, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_mmdc_mpodtctrl
    },
    {
        "MPRDDQBY0DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e read DQ byte0 relative to the read DQS.",
        4, // Width in bytes
        0x0000081c, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_mmdc_mprddqby0dl
    },
    {
        "MPRDDQBY1DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e read DQ byte1 relative to the read DQS",
        4, // Width in bytes
        0x00000820, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_mmdc_mprddqby1dl
    },
    {
        "MPRDDQBY2DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e read DQ byte2 relative to the read DQS",
        4, // Width in bytes
        0x00000824, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_mmdc_mprddqby2dl
    },
    {
        "MPRDDQBY3DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e read DQ byte3 relative to the read DQS.",
        4, // Width in bytes
        0x00000828, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_mmdc_mprddqby3dl
    },
    {
        "MPWRDQBY0DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e write DQ byte0 relative to the write DQS",
        4, // Width in bytes
        0x0000082c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mpwrdqby0dl
    },
    {
        "MPWRDQBY1DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e write DQ byte1 relative to the write DQS",
        4, // Width in bytes
        0x00000830, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mpwrdqby1dl
    },
    {
        "MPWRDQBY2DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e write DQ byte2 relative to the write DQS",
        4, // Width in bytes
        0x00000834, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mpwrdqby2dl
    },
    {
        "MPWRDQBY3DL",
        "This register is used to add fine-tuning adjustment to every bit in th"
        "e write DQ byte3 relative to the write DQS",
        4, // Width in bytes
        0x00000838, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_mmdc_mpwrdqby3dl
    },
    {
        "MPDGCTRL0",
        "",
        4, // Width in bytes
        0x0000083c, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_mmdc_mpdgctrl0
    },
    {
        "MPDGCTRL1",
        "",
        4, // Width in bytes
        0x00000840, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mpdgctrl1
    },
    {
        "MPDGDLST0",
        "This register holds the status of the 4 dqs gating delay-lines.",
        4, // Width in bytes
        0x00000844, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mpdgdlst0
    },
    {
        "MPRDDLCTL",
        "This register controls read delay-lines functionality; it determines D"
        "QS delay relative to the associated DQ read access.",
        4, // Width in bytes
        0x00000848, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mprddlctl
    },
    {
        "MPRDDLST",
        "This register holds the status of the 4 read delay-lines.",
        4, // Width in bytes
        0x0000084c, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mprddlst
    },
    {
        "MPWRDLCTL",
        "This register controls write delay-lines functionality, it determines "
        "DQ/DM delay relative to the associated DQS in write access.",
        4, // Width in bytes
        0x00000850, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mpwrdlctl
    },
    {
        "MPWRDLST",
        "This register holds the status of the 4 write delay-line.",
        4, // Width in bytes
        0x00000854, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mpwrdlst
    },
    {
        "MPSDCTRL",
        "This register controls the fine tuning of the primary clock (CK0).",
        4, // Width in bytes
        0x00000858, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_mmdc_mpsdctrl
    },
    {
        "MPZQLP2CTL",
        "This register controls the idle time that takes the LPDDR2 device to p"
        "erform ZQ calibration",
        4, // Width in bytes
        0x0000085c, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_mmdc_mpzqlp2ctl
    },
    {
        "MPRDDLHWCTL",
        "",
        4, // Width in bytes
        0x00000860, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mprddlhwctl
    },
    {
        "MPWRDLHWCTL",
        "",
        4, // Width in bytes
        0x00000864, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mpwrdlhwctl
    },
    {
        "MPRDDLHWST0",
        "",
        4, // Width in bytes
        0x00000868, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mprddlhwst0
    },
    {
        "MPRDDLHWST1",
        "",
        4, // Width in bytes
        0x0000086c, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mprddlhwst1
    },
    {
        "MPWRDLHWST0",
        "",
        4, // Width in bytes
        0x00000870, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mpwrdlhwst0
    },
    {
        "MPWRDLHWST1",
        "",
        4, // Width in bytes
        0x00000874, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_mmdc_mpwrdlhwst1
    },
    {
        "MPWLHWERR",
        "",
        4, // Width in bytes
        0x00000878, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mpwlhwerr
    },
    {
        "MPDGHWST0",
        "",
        4, // Width in bytes
        0x0000087c, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_mmdc_mpdghwst0
    },
    {
        "MPDGHWST1",
        "",
        4, // Width in bytes
        0x00000880, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_mmdc_mpdghwst1
    },
    {
        "MPDGHWST2",
        "",
        4, // Width in bytes
        0x00000884, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_mmdc_mpdghwst2
    },
    {
        "MPDGHWST3",
        "",
        4, // Width in bytes
        0x00000888, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_mmdc_mpdghwst3
    },
    {
        "MPPDCMPR1",
        "This register holds the MMDC pre-defined compare value that will be us"
        "ed during automatic read, read DQS gating and write calibration proces"
        "s.",
        4, // Width in bytes
        0x0000088c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_mmdc_mppdcmpr1
    },
    {
        "MPPDCMPR2",
        "",
        4, // Width in bytes
        0x00000890, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_mmdc_mppdcmpr2
    },
    {
        "MPSWDAR0",
        "",
        4, // Width in bytes
        0x00000894, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_mmdc_mpswdar0
    },
    {
        "MPSWDRDR0",
        "",
        4, // Width in bytes
        0x00000898, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr0
    },
    {
        "MPSWDRDR1",
        "",
        4, // Width in bytes
        0x0000089c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr1
    },
    {
        "MPSWDRDR2",
        "",
        4, // Width in bytes
        0x000008a0, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr2
    },
    {
        "MPSWDRDR3",
        "",
        4, // Width in bytes
        0x000008a4, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr3
    },
    {
        "MPSWDRDR4",
        "",
        4, // Width in bytes
        0x000008a8, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr4
    },
    {
        "MPSWDRDR5",
        "",
        4, // Width in bytes
        0x000008ac, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr5
    },
    {
        "MPSWDRDR6",
        "",
        4, // Width in bytes
        0x000008b0, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr6
    },
    {
        "MPSWDRDR7",
        "",
        4, // Width in bytes
        0x000008b4, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_mmdc_mpswdrdr7
    },
    {
        "MPMUR0",
        "",
        4, // Width in bytes
        0x000008b8, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_mmdc_mpmur0
    },
    {
        "MPWRCADL",
        "This register is used to add fine-tuning adjustment to the CA (command"
        "/Address of LPDDR2 bus) relative to the DDR clock",
        4, // Width in bytes
        0x000008bc, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_mmdc_mpwrcadl
    },
    {
        "MPDCCR",
        "This register is used to control the duty cycle of the DQS and the pri"
        "mary clock (CK0) .",
        4, // Width in bytes
        0x000008c0, // Base address offset
        true, // Readable
        false, // Writable
        10, // Number of bitfields
        hw_mmdc_mpdccr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark OCOTP
#endif

// Bitfields in register OCOTP_CTRL.
static const field_t hw_ocotp_ctrl[] =
{
    {
        "ADDR",
        "OTP write and read access address register.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BUSY",
        "OTP controller status bit.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ERROR",
        "Set by the controller when an access to a locked region(OTP or shadow "
        "register) is requested.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RELOAD_SHADOWS",
        "Set to force re-loading the shadow registers (HW/SW capability and LOC"
        "K).",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRC_TEST",
        "Set to calculate CRC according to start address and end address in CRC"
        "_ADDR register.And compare with CRC fuse word according CRC address in"
        " CRC_ADDR register to generate CRC_FAIL flag",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRC_FAIL",
        "Set by controller when calculated CRC value is not equal to appointed "
        "CRC fuse word",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_UNLOCK",
        "Write 0x3E77 to enable OTP write accesses.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_TIMING.
static const field_t hw_ocotp_timing[] =
{
    {
        "STROBE_PROG",
        "This count value specifies the strobe period in one time write OTP.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RELAX",
        "This count value specifies the time to add to all default timing param"
        "eters other than the Tpgm and Trd.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STROBE_READ",
        "This count value specifies the strobe period in one time read OTP.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAIT",
        "This count value specifies time interval between auto read and write a"
        "ccess in one time program.",
        22, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DATA.
static const field_t hw_ocotp_data[] =
{
    {
        "DATA",
        "Used to initiate a write to OTP.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_READ_CTRL.
static const field_t hw_ocotp_read_ctrl[] =
{
    {
        "READ_FUSE",
        "Used to initiate a read to OTP.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_READ_FUSE_DATA.
static const field_t hw_ocotp_read_fuse_data[] =
{
    {
        "DATA",
        "The data read from OTP",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SW_STICKY.
static const field_t hw_ocotp_sw_sticky[] =
{
    {
        "SRK_REVOKE_LOCK",
        "Shadow register write and OTP write lock for SRK_REVOKE region.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FIELD_RETURN_LOCK",
        "Shadow register write and OTP write lock for FIELD_RETURN region.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SCS.
static const field_t hw_ocotp_scs[] =
{
    {
        "HAB_JDE",
        "HAB JTAG Debug Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SPARE",
        "Unallocated read/write bits for implementation specific software use.",
        1, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "When set, all of the bits in this register are locked and can not be c"
        "hanged through SW programming.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CRC_ADDR.
static const field_t hw_ocotp_crc_addr[] =
{
    {
        "DATA_START_ADDR",
        "End address of fuse location for CRC calculation",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_END_ADDR",
        "Start address of fuse location for CRC calculation",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRC_ADDR",
        "Address of 32-bit CRC result for comparing",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CRC_VALUE.
static const field_t hw_ocotp_crc_value[] =
{
    {
        "DATA",
        "The crc32 value based on CRC_ADDR",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_UMC_TIMING.
static const field_t hw_ocotp_umc_timing[] =
{
    {
        "STROBE_PROG_INT",
        "This count value specifies the strobe pulse interval in one time write"
        " OTP.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_VERSION.
static const field_t hw_ocotp_version[] =
{
    {
        "STEP",
        "Fixed read-only value reflecting the stepping of the RTL version.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MINOR",
        "Fixed read-only value reflecting the MINOR field of the RTL version.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-only value reflecting the MAJOR field of the RTL version.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_LOCK.
static const field_t hw_ocotp_lock[] =
{
    {
        "TESTER",
        "Status of shadow register and OTP write lock for tester region.",
        0, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BOOT_CFG",
        "Status of shadow register and OTP write lock for boot_cfg region.",
        2, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MEM_TRIM",
        "Status of shadow register and OTP write lock for mem_trim region.",
        4, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SJC_RESP",
        "Status of shadow register read and write, OTP read and write lock for "
        "sjc_resp region.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAC_ADDR",
        "Status of shadow register and OTP write lock for mac_addr region.",
        8, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "GP1",
        "Status of shadow register and OTP write lock for gp2 region.",
        10, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "GP2",
        "Status of shadow register and OTP write lock for gp2 region.",
        12, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SRK",
        "Status of shadow register and OTP write lock for srk region.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SW_GP",
        "Status of shadow register lock for the region contained in the SW_GP r"
        "egisters.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DCP",
        "Status of shadow register read and write, OTP read and write lock for "
        "otpmk region.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ANALOG",
        "Status of shadow register and OTP write lock for analog region.",
        18, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CRC_GP_LOCK",
        "Status of shadow register write and read, OTP program and read lock fo"
        "r CRC region.",
        20, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MISC_CONF",
        "Status of shadow register and OTP write lock for misc_conf region.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PIN",
        "Status of Pin access lock bit.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "GP_LO_LOCK",
        "Status of shadow register and OTP write lock for GP region.",
        26, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "GP_HI_LOCK",
        "Status of shadow register and OTP write lock for GP region.",
        28, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNALLOCATED",
        "Value of un-used portion of LOCK word",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CFG0.
static const field_t hw_ocotp_cfg0[] =
{
    {
        "BITS",
        "This register contains 32 bits of the Unique ID and SJC_CHALLENGE fiel"
        "d.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CFG1.
static const field_t hw_ocotp_cfg1[] =
{
    {
        "BITS",
        "This register contains 32 bits of the Unique ID and SJC_CHALLENGE fiel"
        "d.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CFG2.
static const field_t hw_ocotp_cfg2[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 0, word 3 (ADDR = 0x03).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CFG3.
static const field_t hw_ocotp_cfg3[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 0, word 4 (ADDR = 0x04).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CFG4.
static const field_t hw_ocotp_cfg4[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 0, word 5 (ADDR = 0x05).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CFG5.
static const field_t hw_ocotp_cfg5[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 0, word 6 (ADDR = 0x06).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CFG6.
static const field_t hw_ocotp_cfg6[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 0, word 7 (ADDR = 0x07).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MEM0.
static const field_t hw_ocotp_mem0[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 0 (ADDR = 0x08).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MEM1.
static const field_t hw_ocotp_mem1[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 1 (ADDR = 0x09).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MEM2.
static const field_t hw_ocotp_mem2[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 2 (ADDR = 0x0A).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MEM3.
static const field_t hw_ocotp_mem3[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 3 (ADDR = 0x0B).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MEM4.
static const field_t hw_ocotp_mem4[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 4 (ADDR = 0x0C).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_ANA0.
static const field_t hw_ocotp_ana0[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 5 (ADDR = 0x0D).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_ANA1.
static const field_t hw_ocotp_ana1[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 6 (ADDR = 0x0E).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_ANA2.
static const field_t hw_ocotp_ana2[] =
{
    {
        "BITS",
        "Reflects value of OTP bank 1, word 7 (ADDR = 0x0F).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP0.
static const field_t hw_ocotp_dcp0[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word0 (Copy of OTP Bank 2, word 0 (ADD"
        "R = 0x10)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP1.
static const field_t hw_ocotp_dcp1[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word1 (Copy of OTP Bank 2, word 1 (ADD"
        "R = 0x11)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP2.
static const field_t hw_ocotp_dcp2[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word2 (Copy of OTP Bank 2, word 2 (ADD"
        "R = 0x12)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP3.
static const field_t hw_ocotp_dcp3[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word3 (Copy of OTP Bank 2, word 3 (ADD"
        "R = 0x13)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP4.
static const field_t hw_ocotp_dcp4[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word4 (Copy of OTP Bank 2, word 4 (ADD"
        "R = 0x14)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP5.
static const field_t hw_ocotp_dcp5[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word5 (Copy of OTP Bank 2, word 5 (ADD"
        "R = 0x15)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP6.
static const field_t hw_ocotp_dcp6[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word6 (Copy of OTP Bank 2, word 6 (ADD"
        "R = 0x16)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_DCP7.
static const field_t hw_ocotp_dcp7[] =
{
    {
        "BITS",
        "Shadow register for the DCP Key word7 (Copy of OTP Bank 2, word 7 (ADD"
        "R = 0x17)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK0.
static const field_t hw_ocotp_srk0[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word0 (Copy of OTP "
        "Bank 3, word 0 (ADDR = 0x1C)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK1.
static const field_t hw_ocotp_srk1[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word1 (Copy of OTP "
        "Bank 3, word 1 (ADDR = 0x1D)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK2.
static const field_t hw_ocotp_srk2[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word2 (Copy of OTP "
        "Bank 3, word 2 (ADDR = 0x1E)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK3.
static const field_t hw_ocotp_srk3[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word3 (Copy of OTP "
        "Bank 3, word 3 (ADDR = 0x1F)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK4.
static const field_t hw_ocotp_srk4[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word4 (Copy of OTP "
        "Bank 3, word 4 (ADDR = 0x20)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK5.
static const field_t hw_ocotp_srk5[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word5 (Copy of OTP "
        "Bank 3, word 5 (ADDR = 0x21)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK6.
static const field_t hw_ocotp_srk6[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word6 (Copy of OTP "
        "Bank 3, word 6 (ADDR = 0x22)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK7.
static const field_t hw_ocotp_srk7[] =
{
    {
        "BITS",
        "Shadow register for the hash of the Super Root Key word7 (Copy of OTP "
        "Bank 3, word 7 (ADDR = 0x23)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SJC_RESP0.
static const field_t hw_ocotp_sjc_resp0[] =
{
    {
        "BITS",
        "Shadow register for the SJC_RESP Key word0 (Copy of OTP Bank 4, word 0"
        " (ADDR = 0x20)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SJC_RESP1.
static const field_t hw_ocotp_sjc_resp1[] =
{
    {
        "BITS",
        "Shadow register for the SJC_RESP Key word1 (Copy of OTP Bank 4, word 1"
        " (ADDR = 0x21)).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MAC0.
static const field_t hw_ocotp_mac0[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 4, word 2 (ADDR = 0x22).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MAC1.
static const field_t hw_ocotp_mac1[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 4, word 3 (ADDR = 0x23).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CRC0.
static const field_t hw_ocotp_crc0[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 4, word 4 (ADDR = 0x24).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_CRC1.
static const field_t hw_ocotp_crc1[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 4, word 5 (ADDR = 0x25).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP1.
static const field_t hw_ocotp_gp1[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 4, word 6 (ADDR = 0x26).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP2.
static const field_t hw_ocotp_gp2[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 4, word 7 (ADDR = 0x27).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SW_GP0.
static const field_t hw_ocotp_sw_gp0[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 0 (ADDR = 0x28).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SW_GP1.
static const field_t hw_ocotp_sw_gp1[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 1 (ADDR = 0x29).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SW_GP2.
static const field_t hw_ocotp_sw_gp2[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 2 (ADDR = 0x2a).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SW_GP3.
static const field_t hw_ocotp_sw_gp3[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 3 (ADDR = 0x2b).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SW_GP4.
static const field_t hw_ocotp_sw_gp4[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 4 (ADDR = 0x2c).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_MISC_CONF.
static const field_t hw_ocotp_misc_conf[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 5 (ADDR = 0x2d).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_FIELD_RETURN.
static const field_t hw_ocotp_field_return[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 6 (ADDR = 0x2e).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_SRK_REVOKE.
static const field_t hw_ocotp_srk_revoke[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 5, word 7 (ADDR = 0x2f).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO0.
static const field_t hw_ocotp_gp_lo0[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 0 (ADDR = 0x30).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO1.
static const field_t hw_ocotp_gp_lo1[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 1 (ADDR = 0x31).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO2.
static const field_t hw_ocotp_gp_lo2[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 2 (ADDR = 0x32).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO3.
static const field_t hw_ocotp_gp_lo3[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 3 (ADDR = 0x33).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO4.
static const field_t hw_ocotp_gp_lo4[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 4 (ADDR = 0x34).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO5.
static const field_t hw_ocotp_gp_lo5[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 5 (ADDR = 0x35).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO6.
static const field_t hw_ocotp_gp_lo6[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 6 (ADDR = 0x36).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_LO7.
static const field_t hw_ocotp_gp_lo7[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 6, word 7 (ADDR = 0x37).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI0.
static const field_t hw_ocotp_gp_hi0[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 0 (ADDR = 0x38).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI1.
static const field_t hw_ocotp_gp_hi1[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 1 (ADDR = 0x39).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI2.
static const field_t hw_ocotp_gp_hi2[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 2 (ADDR = 0x3a).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI3.
static const field_t hw_ocotp_gp_hi3[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 3 (ADDR = 0x3b).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI4.
static const field_t hw_ocotp_gp_hi4[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 4 (ADDR = 0x3c).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI5.
static const field_t hw_ocotp_gp_hi5[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 5 (ADDR = 0x3d).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI6.
static const field_t hw_ocotp_gp_hi6[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 6 (ADDR = 0x3e).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register OCOTP_GP_HI7.
static const field_t hw_ocotp_gp_hi7[] =
{
    {
        "BITS",
        "Reflects value of OTP Bank 7, word 7 (ADDR = 0x3f).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a OCOTP module.
static const reg_t hw_ocotp[] =
{
    {
        "CTRL",
        "The OCOTP Control and Status Register specifies the copy state, as wel"
        "l as the control required for random access of the OTP memory OCOTP_CT"
        "RL: 0x000 OCOTP_CTRL_SET: 0x004 OCOTP_CTRL_CLR: 0x008 OCOTP_CTRL_TOG: "
        "0x00C The OCOTP Control and Status Register provides the necessary sof"
        "tware interface for performing read and write operations to the On-Chi"
        "p OTP (One-Time Programmable ROM).",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_ocotp_ctrl
    },
    {
        "TIMING",
        "The OCOTP Data Register is used for OTP Programming This register spec"
        "ifies timing parameters for programming and reading the OCOTP fuse arr"
        "ay.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_ocotp_timing
    },
    {
        "DATA",
        "The OCOTP Data Register is used for OTP Programming This register is u"
        "sed in conjuction with OCOTP_CTRL to perform one-time writes to the OT"
        "P.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_data
    },
    {
        "READ_CTRL",
        "The OCOTP Register is used for OTP Read This register is used in conju"
        "ction with OCOTP_CTRL to perform one time read to the OTP.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_read_ctrl
    },
    {
        "READ_FUSE_DATA",
        "The OCOTP Data Register is used for OTP Read The data read from OTP EX"
        "AMPLE Empty Example.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_read_fuse_data
    },
    {
        "SW_STICKY",
        "Some SW sticky bits .",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_ocotp_sw_sticky
    },
    {
        "SCS",
        "OCOTP_SCS: 0x060 OCOTP_SCS_SET: 0x064 OCOTP_SCS_CLR: 0x068 OCOTP_SCS_T"
        "OG: 0x06C This register holds volatile configuration values that can b"
        "e set and locked by trusted software.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_ocotp_scs
    },
    {
        "CRC_ADDR",
        "The OCOTP Data Register is used for OTP Read The address for CRC calcu"
        "lation EXAMPLE Empty Example.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_ocotp_crc_addr
    },
    {
        "CRC_VALUE",
        "The OCOTP Data Register is used for OTP Read The crc32 value based on "
        "CRC_ADDR EXAMPLE Empty Example.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_crc_value
    },
    {
        "UMC_TIMING",
        "The OCOTP Data Register is used for OTP Programming This register spec"
        "ifies timing parameters for programming UMC OCOTP fuse array.",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_umc_timing
    },
    {
        "VERSION",
        "This register always returns a known read value for debug purposes it "
        "indicates the version of the block.",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_ocotp_version
    },
    {
        "LOCK",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000400, // Base address offset
        true, // Readable
        true, // Writable
        17, // Number of bitfields
        hw_ocotp_lock
    },
    {
        "CFG0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000410, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_cfg0
    },
    {
        "CFG1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000420, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_cfg1
    },
    {
        "CFG2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000430, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_cfg2
    },
    {
        "CFG3",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000440, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_cfg3
    },
    {
        "CFG4",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000450, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_cfg4
    },
    {
        "CFG5",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000460, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_cfg5
    },
    {
        "CFG6",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000470, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_cfg6
    },
    {
        "MEM0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000480, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_mem0
    },
    {
        "MEM1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000490, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_mem1
    },
    {
        "MEM2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000004a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_mem2
    },
    {
        "MEM3",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000004b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_mem3
    },
    {
        "MEM4",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000004c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_mem4
    },
    {
        "ANA0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000004d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_ana0
    },
    {
        "ANA1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000004e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_ana1
    },
    {
        "ANA2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000004f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_ana2
    },
    {
        "DCP0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000500, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp0
    },
    {
        "DCP1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000510, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp1
    },
    {
        "DCP2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000520, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp2
    },
    {
        "DCP3",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000530, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp3
    },
    {
        "DCP4",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000540, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp4
    },
    {
        "DCP5",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000550, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp5
    },
    {
        "DCP6",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000560, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp6
    },
    {
        "DCP7",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000570, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_dcp7
    },
    {
        "SRK0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000580, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk0
    },
    {
        "SRK1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000590, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk1
    },
    {
        "SRK2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000005a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk2
    },
    {
        "SRK3",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000005b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk3
    },
    {
        "SRK4",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000005c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk4
    },
    {
        "SRK5",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000005d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk5
    },
    {
        "SRK6",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000005e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk6
    },
    {
        "SRK7",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000005f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk7
    },
    {
        "SJC_RESP0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000600, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_sjc_resp0
    },
    {
        "SJC_RESP1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000610, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_sjc_resp1
    },
    {
        "MAC0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000620, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_mac0
    },
    {
        "MAC1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000630, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_mac1
    },
    {
        "CRC0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000640, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_crc0
    },
    {
        "CRC1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000650, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_crc1
    },
    {
        "GP1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000660, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp1
    },
    {
        "GP2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000670, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp2
    },
    {
        "SW_GP0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000680, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_sw_gp0
    },
    {
        "SW_GP1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000690, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_sw_gp1
    },
    {
        "SW_GP2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000006a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_sw_gp2
    },
    {
        "SW_GP3",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000006b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_sw_gp3
    },
    {
        "SW_GP4",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000006c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_sw_gp4
    },
    {
        "MISC_CONF",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000006d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_misc_conf
    },
    {
        "FIELD_RETURN",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000006e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_field_return
    },
    {
        "SRK_REVOKE",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000006f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_srk_revoke
    },
    {
        "GP_LO0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000700, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo0
    },
    {
        "GP_LO1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000710, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo1
    },
    {
        "GP_LO2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000720, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo2
    },
    {
        "GP_LO3",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000730, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo3
    },
    {
        "GP_LO4",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000740, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo4
    },
    {
        "GP_LO5",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000750, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo5
    },
    {
        "GP_LO6",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000760, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo6
    },
    {
        "GP_LO7",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000770, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_lo7
    },
    {
        "GP_HI0",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000780, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi0
    },
    {
        "GP_HI1",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x00000790, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi1
    },
    {
        "GP_HI2",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000007a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi2
    },
    {
        "GP_HI3",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000007b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi3
    },
    {
        "GP_HI4",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000007c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi4
    },
    {
        "GP_HI5",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000007d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi5
    },
    {
        "GP_HI6",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000007e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi6
    },
    {
        "GP_HI7",
        "Copied from the OTP automatically after reset.",
        4, // Width in bytes
        0x000007f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ocotp_gp_hi7
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark PGC
#endif

// Bitfields in register PGC_DISPLAY_CTRL.
static const field_t hw_pgc_display_ctrl[] =
{
    {
        "PCR",
        "Power Control  PCR must not change from power-down request (pdn_req) a"
        "ssertion until the target subsystem is completely powered up.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_DISPLAY_PUPSCR.
static const field_t hw_pgc_display_pupscr[] =
{
    {
        "SW",
        "After a power-up request (pup_req assertion), the PGC waits a number o"
        "f clocks equal to the value of SW before asserting switch_b.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW2ISO",
        "After asserting switch_b, the PGC waits a number of clocks equal to th"
        "e value of SW2ISO before negating isolation.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_DISPLAY_PDNSCR.
static const field_t hw_pgc_display_pdnscr[] =
{
    {
        "ISO",
        "After a power-down request (pdn_req assertion), the PGC waits a number"
        " of clocks equal to the value of ISO before asserting isolation.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ISO2SW",
        "After asserting isolation, the PGC waits a number of clocks equal to t"
        "he value of ISO2SW before negating switch_b.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_DISPLAY_SR.
static const field_t hw_pgc_display_sr[] =
{
    {
        "PSR",
        "Power status.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_GPU_CTRL.
static const field_t hw_pgc_gpu_ctrl[] =
{
    {
        "PCR",
        "Power Control  PCR must not change from power-down request (pdn_req) a"
        "ssertion until the target subsystem is completely powered up.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_GPU_PUPSCR.
static const field_t hw_pgc_gpu_pupscr[] =
{
    {
        "SW",
        "After a power-up request (pup_req assertion), the PGC waits a number o"
        "f clocks equal to the value of SW before asserting switch_b.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW2ISO",
        "After asserting switch_b, the PGC waits a number of clocks equal to th"
        "e value of SW2ISO before negating isolation.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_GPU_PDNSCR.
static const field_t hw_pgc_gpu_pdnscr[] =
{
    {
        "ISO",
        "After a power-down request (pdn_req assertion), the PGC waits a number"
        " of clocks equal to the value of ISO before asserting isolation.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ISO2SW",
        "After asserting isolation, the PGC waits a number of clocks equal to t"
        "he value of ISO2SW before negating switch_b.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_GPU_SR.
static const field_t hw_pgc_gpu_sr[] =
{
    {
        "PSR",
        "Power status.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_CPU_CTRL.
static const field_t hw_pgc_cpu_ctrl[] =
{
    {
        "PCR",
        "Power Control  PCR must not change from power-down request (pdn_req) a"
        "ssertion until the target subsystem is completely powered up.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_CPU_PUPSCR.
static const field_t hw_pgc_cpu_pupscr[] =
{
    {
        "SW",
        "After a power-up request (pup_req assertion), the PGC waits a number o"
        "f clocks equal to the value of SW before asserting switch_b.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW2ISO",
        "After asserting switch_b, the PGC waits a number of clocks equal to th"
        "e value of SW2ISO before negating isolation.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_CPU_PDNSCR.
static const field_t hw_pgc_cpu_pdnscr[] =
{
    {
        "ISO",
        "After a power-down request (pdn_req assertion), the PGC waits a number"
        " of clocks equal to the value of ISO before asserting isolation.",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ISO2SW",
        "After asserting isolation, the PGC waits a number of clocks equal to t"
        "he value of ISO2SW before negating switch_b.",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PGC_CPU_SR.
static const field_t hw_pgc_cpu_sr[] =
{
    {
        "PSR",
        "Power status.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a PGC module.
static const reg_t hw_pgc[] =
{
    {
        "DISPLAY_CTRL",
        "The PGCR enables the response to a power-down request.",
        4, // Width in bytes
        0x00000240, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pgc_display_ctrl
    },
    {
        "DISPLAY_PUPSCR",
        "The PUPSCR contains the power-up timing parameters.",
        4, // Width in bytes
        0x00000244, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pgc_display_pupscr
    },
    {
        "DISPLAY_PDNSCR",
        "The PDNSCR contains the power-down timing parameters.",
        4, // Width in bytes
        0x00000248, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pgc_display_pdnscr
    },
    {
        "DISPLAY_SR",
        "The PDNSCR contains the power-down timing parameters.",
        4, // Width in bytes
        0x0000024c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pgc_display_sr
    },
    {
        "GPU_CTRL",
        "The PGCR enables the response to a power-down request.",
        4, // Width in bytes
        0x00000260, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pgc_gpu_ctrl
    },
    {
        "GPU_PUPSCR",
        "The PUPSCR contains the power-up timing parameters.",
        4, // Width in bytes
        0x00000264, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pgc_gpu_pupscr
    },
    {
        "GPU_PDNSCR",
        "The PDNSCR contains the power-down timing parameters.",
        4, // Width in bytes
        0x00000268, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pgc_gpu_pdnscr
    },
    {
        "GPU_SR",
        "The PDNSCR contains the power-down timing parameters.",
        4, // Width in bytes
        0x0000026c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pgc_gpu_sr
    },
    {
        "CPU_CTRL",
        "The PGCR enables the response to a power-down request.",
        4, // Width in bytes
        0x000002a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pgc_cpu_ctrl
    },
    {
        "CPU_PUPSCR",
        "The PUPSCR contains the power-up timing parameters.",
        4, // Width in bytes
        0x000002a4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pgc_cpu_pupscr
    },
    {
        "CPU_PDNSCR",
        "The PDNSCR contains the power-down timing parameters.",
        4, // Width in bytes
        0x000002a8, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pgc_cpu_pdnscr
    },
    {
        "CPU_SR",
        "The PDNSCR contains the power-down timing parameters.",
        4, // Width in bytes
        0x000002ac, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pgc_cpu_sr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark PMU
#endif

// Bitfields in register PMU_REG_1P1.
static const field_t hw_pmu_reg_1p1[] =
{
    {
        "ENABLE_LINREG",
        "Control bit to enable the regulator output.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_BO",
        "Control bit to enable the brown-out circuitry in the regulator.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_ILIMIT",
        "Control bit to enable the current-limit circuitry in the regulator.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_PULLDOWN",
        "Control bit to enable the pull-down circuitry in the regulator",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BO_OFFSET",
        "Control bits to adjust the regulator brown-out offset voltage in 25mV "
        "steps.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OUTPUT_TRG",
        "Control bits to adjust the regulator output voltage.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BO_VDD1P1",
        "Status bit that signals when a brown-out is detected on the regulator "
        "output.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OK_VDD1P1",
        "Status bit that signals when the regulator output is ok.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PMU_REG_3P0.
static const field_t hw_pmu_reg_3p0[] =
{
    {
        "ENABLE_LINREG",
        "Control bit to enable the regulator output.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_BO",
        "Control bit to enable the brown-out circuitry in the regulator.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_ILIMIT",
        "Control bit to enable the current-limit circuitry in the regulator.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BO_OFFSET",
        "Control bits to adjust the regulator brown-out offset voltage in 25mV "
        "steps.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VBUS_SEL",
        "Select input voltage source for LDO_3P0 from either USB OTG1 or USB OT"
        "G2.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OUTPUT_TRG",
        "Control bits to adjust the regulator output voltage.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BO_VDD3P0",
        "Status bit that signals when a brown-out is detected on the regulator "
        "output.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OK_VDD3P0",
        "Status bit that signals when the regulator output is ok.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PMU_REG_2P5.
static const field_t hw_pmu_reg_2p5[] =
{
    {
        "ENABLE_LINREG",
        "Control bit to enable the regulator output.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_BO",
        "Control bit to enable the brown-out circuitry in the regulator.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_ILIMIT",
        "Control bit to enable the current-limit circuitry in the regulator.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_PULLDOWN",
        "Control bit to enable the pull-down circuitry in the regulator",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BO_OFFSET",
        "Control bits to adjust the regulator brown-out offset voltage in 25mV "
        "steps.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OUTPUT_TRG",
        "Control bits to adjust the regulator output voltage.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BO_VDD2P5",
        "Status bit that signals when a brown-out is detected on the regulator "
        "output.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OK_VDD2P5",
        "Status bit that signals when the regulator output is ok.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ENABLE_WEAK_LINREG",
        "Enables the weak 2p5 regulator.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PMU_REG_CORE.
static const field_t hw_pmu_reg_core[] =
{
    {
        "REG0_TARG",
        "This bitfield defines the target voltage for the arm core power domain"
        ".",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG0_ADJ",
        "This bitfield defines the adjustment bits to calibrate the target valu"
        "e of REG1 (ARM_CORE).",
        5, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG1_TARG",
        "This bitfield defines the target voltage for the VPU/GPU power domain.",
        9, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG1_ADJ",
        "This bitfield defines the adjustment bits to calibrate the target valu"
        "e of REG1 (REG_PU).",
        14, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG2_TARG",
        "This bitfield defines the target voltage for the SOC power domain.",
        18, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG2_ADJ",
        "This bitfield defines the adjustment bits to calibrate the target valu"
        "e of REG2 (REG_SOC).",
        23, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FET_ODRIVE",
        "If set, increases the gate drive on power gating fets to reduce leakag"
        "e in the off state.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PMU_MISC0.
static const field_t hw_pmu_misc0[] =
{
    {
        "REFTOP_PWD",
        "Control bit to power-down the analog bandgap reference circuitry.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_SELFBIASOFF",
        "Control bit to disable the self-bias circuit in the analog bandgap.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_VBGADJ",
        "",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_VBGUP",
        "Status bit which signals that the analog bandgap voltage is up and sta"
        "ble.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STOP_MODE_CONFIG",
        "Configure the analog behavior in stop mode.",
        11, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISCON_HIGH_SNVS",
        "This bit forces the short between VDDHIGH_IN and VSNVS_IN to open when"
        " asserted.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OSC_I",
        "This bit field determines the bias current in the 24MHz oscillator.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OSC_XTALOK",
        "Status bit which signals that the output of the 24MHz crystal oscillat"
        "or is stable.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OSC_XTALOK_EN",
        "This bit is asserted when the 24MHz oscillator is close to its operati"
        "ng frequency.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBCP_VPW_THRESH",
        "This signal alters the voltage that the pwell is charged pumped to.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE_CTRL",
        "This bit allows disabling the clock gate (always un-gated) for the xta"
        "l 24MHz clock that clocks the digital logic in the analog block.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE_DELAY",
        "This field specifies the delay between powering up the XTAL 24MHz cloc"
        "k and release the clock to the digital logic inside the analog block.",
        26, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PMU_MISC1.
static const field_t hw_pmu_misc1[] =
{
    {
        "LVDS1_CLK_SEL",
        "This field selects the clk to be routed to anaclk2/2b.",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVDSCLK1_OBEN",
        "This enables the lvds output buffer for anaclk1/1b.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LVDSCLK1_IBEN",
        "This enables the lvds input buffer for anaclk1/1b.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRQ_TEMPSENSE",
        "This status bit is set to one when when the temperature sensor interru"
        "pt asserts.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRQ_ANA_BO",
        "This status bit is set to one when when any of the analog regulator br"
        "ownout interrupts assert.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRQ_DIG_BO",
        "This status bit is set to one when when any of the digital regulator b"
        "rownout interrupts assert.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PMU_MISC2.
static const field_t hw_pmu_misc2[] =
{
    {
        "REG0_BO_OFFSET",
        "This bit field defines the brown out voltage offset for the CORE power"
        " domain.",
        0, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REG0_BO_STATUS",
        "Reg0 brownout status bit.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REG0_ENABLE_BO",
        "Enables the brownout detection.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PLL3_DISABLE",
        "Default value of \"0\".",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG1_BO_OFFSET",
        "This bit field defines the brown out voltage offset for the xPU power "
        "domain.",
        8, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REG1_BO_STATUS",
        "Reg1 brownout status bit.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REG1_ENABLE_BO",
        "Enables the brownout detection.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUDIO_DIV_LSB",
        "LSB of Audio Pll post divider two bit control: 0x0=div-by-1(default), "
        "0x1=div-by-2, 0x2-div-by-1, 0x3=div-by-4.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG2_BO_OFFSET",
        "This bit field defines the brown out voltage offset for the xPU power "
        "domain.",
        16, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REG2_BO_STATUS",
        "Reg2 brownout status bit.",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REG2_ENABLE_BO",
        "Enables the brownout detection.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG2_OK",
        "Signals that the voltage is above the brownout level for the SOC suppl"
        "y.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AUDIO_DIV_MSB",
        "MSB of 2 bit post-divider field for the Audio PLL: 0x0=div-by-1(defaul"
        "t), 0x1=div-by-2, 0x2=div-by-1, 0x3=div-by-4.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG0_STEP_TIME",
        "Number of clock periods (24MHz clock).",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG1_STEP_TIME",
        "Number of clock periods (24MHz clock).",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REG2_STEP_TIME",
        "Number of clock periods (24MHz clock).",
        28, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VIDEO_DIV",
        "Post-divider for video PLL.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PMU_LOWPWR_CTRL.
static const field_t hw_pmu_lowpwr_ctrl[] =
{
    {
        "RC_OSC_EN",
        "RC Osc.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RC_OSC_PROG",
        "RC osc.",
        1, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OSC_SEL",
        "Select the source for the 24MHz clock.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPBG_SEL",
        "Bandgap select.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPBG_TEST",
        "Low power bandgap test bit.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_IBIAS_OFF",
        "Low power reftop ibias disable.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L1_PWRGATE",
        "L1 power gate control.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2_PWRGATE",
        "L2 power gate control.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CPU_PWRGATE",
        "CPU power gate control.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISPLAY_PWRGATE",
        "Display logic power gate control.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWB_EN",
        "Reverse well bias enable control.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCOSC_CG_OVERRIDE",
        "For debug purposes only.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "XTALOSC_PWRUP_DELAY",
        "Specifies the time delay between when the 24MHz xtal is powered up unt"
        "il it is stable and ready to use.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "XTALOSC_PWRUP_STAT",
        "Status of the 24MHz xtal oscillator.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a PMU module.
static const reg_t hw_pmu[] =
{
    {
        "REG_1P1",
        "This register defines the control and status bits for the 1.1V regulat"
        "or.",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_pmu_reg_1p1
    },
    {
        "REG_3P0",
        "This register defines the control and status bits for the 3.0V regulat"
        "or powered by the host USB VBUS pin.",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_pmu_reg_3p0
    },
    {
        "REG_2P5",
        "This register defines the control and status bits for the 2.5V regulat"
        "or.",
        4, // Width in bytes
        0x00000130, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_pmu_reg_2p5
    },
    {
        "REG_CORE",
        "This register defines the function of the digital regulators",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_pmu_reg_core
    },
    {
        "MISC0",
        "This register defines the control and status bits for miscellaneous an"
        "alog blocks.",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_pmu_misc0
    },
    {
        "MISC1",
        "This register defines the control and status bits for miscellaneous an"
        "alog blocks.",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_pmu_misc1
    },
    {
        "MISC2",
        "This register defines the control and status bits for the digital regu"
        "lators, as well as some miscellaneous analog controls.",
        4, // Width in bytes
        0x00000170, // Base address offset
        true, // Readable
        true, // Writable
        17, // Number of bitfields
        hw_pmu_misc2
    },
    {
        "LOWPWR_CTRL",
        "This register defines the low power configuration bits.",
        4, // Width in bytes
        0x00000260, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_pmu_lowpwr_ctrl
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark PWM
#endif

// Bitfields in register PWM_PWMCR.
static const field_t hw_pwm_pwmcr[] =
{
    {
        "EN",
        "PWM Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REPEAT",
        "Sample Repeat.",
        1, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SWR",
        "Software Reset.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PRESCALER",
        "Counter Clock Prescaler Value.",
        4, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKSRC",
        "Select Clock Source.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POUTC",
        "PWM Output Configuration.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HCTR",
        "Half-word Data Swap Control.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCTR",
        "Byte Data Swap Control.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DBGEN",
        "Debug Mode Enable.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAITEN",
        "Wait Mode Enable.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOZEN",
        "Doze Mode Enable.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STOPEN",
        "Stop Mode Enable.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FWM",
        "FIFO Water Mark.",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PWM_PWMSR.
static const field_t hw_pwm_pwmsr[] =
{
    {
        "FIFOAV",
        "FIFO Available.",
        0, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FE",
        "FIFO Empty Status Bit.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROV",
        "Roll-over Status.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMP",
        "Compare Status.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FWE",
        "FIFO Write Error Status.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PWM_PWMIR.
static const field_t hw_pwm_pwmir[] =
{
    {
        "FIE",
        "FIFO Empty Interrupt Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RIE",
        "Roll-over Interrupt Enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CIE",
        "Compare Interrupt Enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PWM_PWMSAR.
static const field_t hw_pwm_pwmsar[] =
{
    {
        "SAMPLE",
        "Sample Value.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PWM_PWMPR.
static const field_t hw_pwm_pwmpr[] =
{
    {
        "PERIOD",
        "Period Value.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PWM_PWMCNR.
static const field_t hw_pwm_pwmcnr[] =
{
    {
        "COUNT",
        "Counter Value.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a PWM module.
static const reg_t hw_pwm[] =
{
    {
        "PWMCR",
        "The PWM control register (PWM_PWMCR) is used to configure the operatin"
        "g settings of the PWM.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        13, // Number of bitfields
        hw_pwm_pwmcr
    },
    {
        "PWMSR",
        "The PWM status register (PWM_PWMSR) contains seven bits which display "
        "the state of the FIFO and the occurrence of rollover and compare event"
        "s.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_pwm_pwmsr
    },
    {
        "PWMIR",
        "The PWM Interrupt register (PWM_PWMIR) contains three bits which contr"
        "ol the generation of the compare, rollover and FIFO empty interrupts.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_pwm_pwmir
    },
    {
        "PWMSAR",
        "The PWM sample register (PWM_PWMSAR) is the input to the FIFO.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pwm_pwmsar
    },
    {
        "PWMPR",
        "The PWM period register (PWM_PWMPR) determines the period of the PWM o"
        "utput signal.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pwm_pwmpr
    },
    {
        "PWMCNR",
        "The read-only pulse-width modulator counter register (PWM_PWMCNR) cont"
        "ains the current count value and can be read at any time without distu"
        "rbing the counter.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_pwm_pwmcnr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark PXP
#endif

// Bitfields in register PXP_CTRL.
static const field_t hw_pxp_ctrl[] =
{
    {
        "ENABLE",
        "Enables PXP operation with specified parameters.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRQ_ENABLE",
        "Interrupt enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NEXT_IRQ_ENABLE",
        "Next command interrupt enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_DMA_IRQ_ENABLE",
        "LUT DMA interrupt enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_LCD_HANDSHAKE",
        "Enable handshake with LCD controller.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROTATE",
        "Indicates the clockwise rotation to be applied at the output buffer.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HFLIP",
        "Indicates that the output buffer should be flipped horizontally (effec"
        "t applied before rotation).",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VFLIP",
        "Indicates that the output buffer should be flipped vertically (effect "
        "applied before rotation).",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROT_POS",
        "This bit controls where rotation will occur in the PXP datapath.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BLOCK_SIZE",
        "Select the block size to process.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_REPEAT",
        "Enable the PXP to run continuously.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE",
        "This bit must be set to zero for normal operation.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFTRST",
        "Set this bit to zero to enable normal PXP operation.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_STAT.
static const field_t hw_pxp_stat[] =
{
    {
        "IRQ",
        "Indicates current PXP interrupt status.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AXI_WRITE_ERROR",
        "Indicates PXP encountered an AXI write error and processing has been t"
        "erminated.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AXI_READ_ERROR",
        "Indicates PXP encountered an AXI read error and processing has been te"
        "rminated.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NEXT_IRQ",
        "Indicates that a command issued with the \"Next Command\" functionalit"
        "y has been issued and that a new command may be initiated with a write"
        " to the PXP_NEXT register.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AXI_ERROR_ID",
        "Indicates the AXI ID of the failing bus operation.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LUT_DMA_LOAD_DONE_IRQ",
        "Indicates that the LUT DMA transfer has completed.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BLOCKY",
        "Indicates the X coordinate of the block currently being rendered.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BLOCKX",
        "Indicates the X coordinate of the block currently being rendered.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_CTRL.
static const field_t hw_pxp_out_ctrl[] =
{
    {
        "FORMAT",
        "Output framebuffer format.",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INTERLACED_OUTPUT",
        "Determines how the PXP writes it's output data.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ALPHA_OUTPUT",
        "Indicates that alpha component in output buffer pixels should be overw"
        "ritten by PXP_OUT_CTRL[ALPHA].",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ALPHA",
        "When generating an output buffer with an alpha component, the value in"
        " this field will be used when enabled to override the alpha passed thr"
        "ough the pixel data pipeline.",
        24, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_BUF.
static const field_t hw_pxp_out_buf[] =
{
    {
        "ADDR",
        "Current address pointer for the output frame buffer.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_BUF2.
static const field_t hw_pxp_out_buf2[] =
{
    {
        "ADDR",
        "Current address pointer for the output frame buffer.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_PITCH.
static const field_t hw_pxp_out_pitch[] =
{
    {
        "PITCH",
        "Indicates the number of bytes in memory between two vertically adjacen"
        "t pixels.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_LRC.
static const field_t hw_pxp_out_lrc[] =
{
    {
        "Y",
        "Indicates the number of vertical PIXELS in the output surface (non-rot"
        "ated).",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "X",
        "Indicates number of horizontal PIXELS in the output surface (non-rotat"
        "ed).",
        16, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_PS_ULC.
static const field_t hw_pxp_out_ps_ulc[] =
{
    {
        "Y",
        "This field indicates the upper left Y-coordinate (in pixels) of the pr"
        "ocessed surface in the output buffer.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "X",
        "This field indicates the upper left X-coordinate (in pixels) of the pr"
        "ocessed surface (PS) in the output buffer.",
        16, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_PS_LRC.
static const field_t hw_pxp_out_ps_lrc[] =
{
    {
        "Y",
        "This field indicates the lower right Y-coordinate (in pixels) of the p"
        "rocessed surface in the output frame buffer.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "X",
        "This field indicates the lower right X-coordinate (in pixels) of the p"
        "rocessed surface (PS) in the output frame buffer.",
        16, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_AS_ULC.
static const field_t hw_pxp_out_as_ulc[] =
{
    {
        "Y",
        "This field indicates the upper left Y-coordinate (in pixels) of the al"
        "pha surface in the output frame buffer.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "X",
        "This field indicates the upper left X-coordinate (in pixels) of the al"
        "pha surface (AS) in the output frame buffer.",
        16, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_OUT_AS_LRC.
static const field_t hw_pxp_out_as_lrc[] =
{
    {
        "Y",
        "This field indicates the lower right Y-coordinate (in pixels) of the a"
        "lpha surface in the output frame buffer.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "X",
        "This field indicates the lower right X-coordinate (in pixels) of the a"
        "lpha surface (AS) in the output frame buffer.",
        16, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_CTRL.
static const field_t hw_pxp_ps_ctrl[] =
{
    {
        "FORMAT",
        "PS buffer format.",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WB_SWAP",
        "Swap bytes in words.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DECY",
        "Verticle pre decimation filter control.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DECX",
        "Horizontal pre decimation filter control.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_BUF.
static const field_t hw_pxp_ps_buf[] =
{
    {
        "ADDR",
        "Address pointer for the PS RGB or Y (luma) input buffer.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_UBUF.
static const field_t hw_pxp_ps_ubuf[] =
{
    {
        "ADDR",
        "Address pointer for the PS U/Cb or 2 plane UV Chroma input buffer.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_VBUF.
static const field_t hw_pxp_ps_vbuf[] =
{
    {
        "ADDR",
        "Address pointer for the PS V/Cr Chroma input buffer.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_PITCH.
static const field_t hw_pxp_ps_pitch[] =
{
    {
        "PITCH",
        "Indicates the number of bytes in memory between two vertically adjacen"
        "t pixels.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_BACKGROUND.
static const field_t hw_pxp_ps_background[] =
{
    {
        "COLOR",
        "Background color (in 24bpp format) for any pixels not within the buffe"
        "r range specified by the PS ULC/LRC.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_SCALE.
static const field_t hw_pxp_ps_scale[] =
{
    {
        "XSCALE",
        "This is a two bit integer and 12 bit fractional representation (##.###"
        "#_####_####) of the X scaling factor for the PS source buffer.",
        0, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "YSCALE",
        "This is a two bit integer and 12 bit fractional representation (##.###"
        "#_####_####) of the Y scaling factor for the PS source buffer.",
        16, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_OFFSET.
static const field_t hw_pxp_ps_offset[] =
{
    {
        "XOFFSET",
        "This is a 12 bit fractional representation (0.####_####_####) of the X"
        " scaling offset.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "YOFFSET",
        "This is a 12 bit fractional representation (0.####_####_####) of the Y"
        " scaling offset.",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_CLRKEYLOW.
static const field_t hw_pxp_ps_clrkeylow[] =
{
    {
        "PIXEL",
        "Low range of color key applied to PS buffer.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_PS_CLRKEYHIGH.
static const field_t hw_pxp_ps_clrkeyhigh[] =
{
    {
        "PIXEL",
        "High range of color key applied to PS buffer.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_AS_CTRL.
static const field_t hw_pxp_as_ctrl[] =
{
    {
        "ALPHA_CTRL",
        "Determines how the alpha value is constructed for this alpha surface.",
        1, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_COLORKEY",
        "Indicates that colorkey functionality is enabled for this alpha surfac"
        "e.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FORMAT",
        "Indicates the input buffer format for AS.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ALPHA",
        "Alpha modifier used when the ALPHA_MULTIPLY or ALPHA_OVERRIDE values a"
        "re programmed in PXP_AS_CTRL[ALPHA_CTRL].",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROP",
        "Indicates a raster operation to perform when enabled.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ALPHA_INVERT",
        "Setting this bit to logic 0 will not alter the alpha value.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_AS_BUF.
static const field_t hw_pxp_as_buf[] =
{
    {
        "ADDR",
        "Address pointer for the alpha surface 0 buffer.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_AS_PITCH.
static const field_t hw_pxp_as_pitch[] =
{
    {
        "PITCH",
        "Indicates the number of bytes in memory between two vertically adjacen"
        "t pixels.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_AS_CLRKEYLOW.
static const field_t hw_pxp_as_clrkeylow[] =
{
    {
        "PIXEL",
        "Low range of RGB color key applied to AS buffer.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_AS_CLRKEYHIGH.
static const field_t hw_pxp_as_clrkeyhigh[] =
{
    {
        "PIXEL",
        "High range of RGB color key applied to AS buffer.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC1_COEF0.
static const field_t hw_pxp_csc1_coef0[] =
{
    {
        "Y_OFFSET",
        "Two's compliment amplitude offset implicit in the Y data.",
        0, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UV_OFFSET",
        "Two's compliment phase offset implicit for CbCr data.",
        9, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C0",
        "Two's compliment Y multiplier coefficient.",
        18, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Bypass the CSC unit in the scaling engine.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "YCBCR_MODE",
        "Set to 1 when performing YCbCr conversion to RGB.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC1_COEF1.
static const field_t hw_pxp_csc1_coef1[] =
{
    {
        "C4",
        "Two's compliment Blue U/Cb multiplier coefficient.",
        0, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C1",
        "Two's compliment Red V/Cr multiplier coefficient.",
        16, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC1_COEF2.
static const field_t hw_pxp_csc1_coef2[] =
{
    {
        "C3",
        "Two's complement Green U/Cb multiplier coefficient.",
        0, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C2",
        "Two's complement Green V/Cr multiplier coefficient.",
        16, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC2_CTRL.
static const field_t hw_pxp_csc2_ctrl[] =
{
    {
        "BYPASS",
        "This bit controls whether the pixels entering the CSC2 unit get conver"
        "ted or not.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSC_MODE",
        "This field controls how the CSC unit operates on pixels when the CSC i"
        "s not bypassed.",
        1, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC2_COEF0.
static const field_t hw_pxp_csc2_coef0[] =
{
    {
        "A1",
        "Two's complement coefficient offset.",
        0, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "A2",
        "Two's complement coefficient offset.",
        16, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC2_COEF1.
static const field_t hw_pxp_csc2_coef1[] =
{
    {
        "A3",
        "Two's complement coefficient offset.",
        0, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "B1",
        "Two's complement coefficient offset.",
        16, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC2_COEF2.
static const field_t hw_pxp_csc2_coef2[] =
{
    {
        "B2",
        "Two's complement coefficient offset.",
        0, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "B3",
        "Two's complement coefficient offset.",
        16, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC2_COEF3.
static const field_t hw_pxp_csc2_coef3[] =
{
    {
        "C1",
        "Two's complement coefficient offset.",
        0, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "C2",
        "Two's complement coefficient offset.",
        16, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC2_COEF4.
static const field_t hw_pxp_csc2_coef4[] =
{
    {
        "C3",
        "Two's complement coefficient offset.",
        0, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "D1",
        "Two's complement coefficient integer offset to be added.",
        16, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CSC2_COEF5.
static const field_t hw_pxp_csc2_coef5[] =
{
    {
        "D2",
        "Two's complement D1 coefficient integer offset to be added.",
        0, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "D3",
        "Two's complement coefficient integer offset to be added.",
        16, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_LUT_CTRL.
static const field_t hw_pxp_lut_ctrl[] =
{
    {
        "DMA_START",
        "Setting this bit will result in the DMA operation to load the PXP LUT "
        "memory based on PXP_LUT_ADDR_NUM_BYTES, PXP_LUT_ADDR_ADDR, and PXP_LUT"
        "_MEM_ADDR.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INVALID",
        "Invalidate the cache LRU and valid bits.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LRU_UPD",
        "Least Recently Used Policy Update Control: 1=> block LRU update for hi"
        "t after miss.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEL_8KB",
        "Selects which 8KB bank of memory to use for direct 12bpp lookup modes.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OUT_MODE",
        "Select the output mode of operation for  the LUT resource.",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOOKUP_MODE",
        "Configure the input address for the 16KB  LUT memory.",
        24, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BYPASS",
        "Setting this bit will bypass the LUT memory resource completely.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_LUT_ADDR.
static const field_t hw_pxp_lut_addr[] =
{
    {
        "ADDR",
        "LUT indexed address pointer.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM_BYTES",
        "Indicates the number of bytes to load via a DMA operation.",
        16, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_LUT_DATA.
static const field_t hw_pxp_lut_data[] =
{
    {
        "DATA",
        "Writing this field will load 4 bytes, aligned to four byte boundaries,"
        " of data indexed by the ADDR field of the PXP_LUT_CTRL register.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_LUT_EXTMEM.
static const field_t hw_pxp_lut_extmem[] =
{
    {
        "ADDR",
        "This register contains the external memory address used for LUT memory"
        " operation.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_CFA.
static const field_t hw_pxp_cfa[] =
{
    {
        "DATA",
        "This register contains the Color Filter  Array pattern for decimation "
        "of RGBW4444 16 bit pixels  to individual R, G, B, W values.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST_CTRL.
static const field_t hw_pxp_hist_ctrl[] =
{
    {
        "STATUS",
        "Indicates which histogram matched the processed bitmap.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PANEL_MODE",
        "Specifies the EPDC panel grayscale depth.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST2_PARAM.
static const field_t hw_pxp_hist2_param[] =
{
    {
        "VALUE0",
        "Black value for 2-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "White value for 2-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST4_PARAM.
static const field_t hw_pxp_hist4_param[] =
{
    {
        "VALUE0",
        "GRAY0 (Black) value for 4-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "GRAY1 value for 4-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE2",
        "GRAY2 value for 4-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE3",
        "GRAY3 (White) value for 4-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST8_PARAM0.
static const field_t hw_pxp_hist8_param0[] =
{
    {
        "VALUE0",
        "GRAY0 (Black) value for 8-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "GRAY1 value for 8-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE2",
        "GRAY2 value for 8-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE3",
        "GRAY3 value for 8-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST8_PARAM1.
static const field_t hw_pxp_hist8_param1[] =
{
    {
        "VALUE4",
        "GRAY4 value for 8-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE5",
        "GRAY5 value for 8-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE6",
        "GRAY6 value for 8-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE7",
        "GRAY7 (White) value for 8-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST16_PARAM0.
static const field_t hw_pxp_hist16_param0[] =
{
    {
        "VALUE0",
        "GRAY0 (Black) value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE1",
        "GRAY1 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE2",
        "GRAY2 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE3",
        "GRAY3 value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST16_PARAM1.
static const field_t hw_pxp_hist16_param1[] =
{
    {
        "VALUE4",
        "GRAY4 value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE5",
        "GRAY5 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE6",
        "GRAY6 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE7",
        "GRAY7 value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST16_PARAM2.
static const field_t hw_pxp_hist16_param2[] =
{
    {
        "VALUE8",
        "GRAY8 value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE9",
        "GRAY9 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE10",
        "GRAY10 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE11",
        "GRAY11 value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_HIST16_PARAM3.
static const field_t hw_pxp_hist16_param3[] =
{
    {
        "VALUE12",
        "GRAY12 value for 16-level histogram",
        0, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE13",
        "GRAY13 value for 16-level histogram",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE14",
        "GRAY14 value for 16-level histogram",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALUE15",
        "GRAY15 (White) value for 16-level histogram",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_POWER.
static const field_t hw_pxp_power[] =
{
    {
        "LUT_LP_STATE_WAY0_BANK0",
        "Select the low power state of the LUT's WAY0-BANK0 memory.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_LP_STATE_WAY0_BANKN",
        "Select the low power state of the LUT's WAY0-BANK1,2,3 memory.",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_LP_STATE_WAY1_BANKN",
        "Select the low power state of the LUT's WAY0-BANK0,1,2,3 memory.",
        6, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROT_MEM_LP_STATE",
        "Select the low power state of the ROT memory.",
        9, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CTRL",
        "This register contains power control for  the PXP.",
        12, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register PXP_NEXT.
static const field_t hw_pxp_next[] =
{
    {
        "ENABLED",
        "Indicates that the \"next frame\" functionality has been enabled.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "POINTER",
        "A pointer to a data structure containing register values to be used wh"
        "en processing the next frame.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a PXP module.
static const reg_t hw_pxp[] =
{
    {
        "CTRL",
        "The CTRL register contains controls for the PXP module.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        13, // Number of bitfields
        hw_pxp_ctrl
    },
    {
        "STAT",
        "The PXP Interrupt Status register provides interrupt status informatio"
        "n.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_pxp_stat
    },
    {
        "OUT_CTRL",
        "The OUT_CTRL register contains controls for the Output Buffer.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_out_ctrl
    },
    {
        "OUT_BUF",
        "Output Framebuffer Pointer.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_out_buf
    },
    {
        "OUT_BUF2",
        "Output Framebuffer Pointer #2.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_out_buf2
    },
    {
        "OUT_PITCH",
        "This register contains the output buffer pitch in bytes.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_out_pitch
    },
    {
        "OUT_LRC",
        "This register contains the size, or lower right coordinate, of the out"
        "put buffer NOT rotated.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_out_lrc
    },
    {
        "OUT_PS_ULC",
        "This register contains the upper left pixel coordinate for the Process"
        "ed Surface in the OUTPUT buffer.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_out_ps_ulc
    },
    {
        "OUT_PS_LRC",
        "This register contains the lower right extent for the Processed Surfac"
        "e in the OUTPUT buffer.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_out_ps_lrc
    },
    {
        "OUT_AS_ULC",
        "This register contains the upper left location for the Alpha Surface i"
        "n the output buffer.",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_out_as_ulc
    },
    {
        "OUT_AS_LRC",
        "This register contains the lower right extent for Alpha Surface in the"
        " output buffer.",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_out_as_lrc
    },
    {
        "PS_CTRL",
        "The PS_CTRL register contains controls for the Processed Surface Buffe"
        "r.",
        4, // Width in bytes
        0x000000b0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_ps_ctrl
    },
    {
        "PS_BUF",
        "PS Input Buffer Address.",
        4, // Width in bytes
        0x000000c0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_ps_buf
    },
    {
        "PS_UBUF",
        "PS Chroma (U/Cb/UV) Input Buffer Address.",
        4, // Width in bytes
        0x000000d0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_ps_ubuf
    },
    {
        "PS_VBUF",
        "PS Chroma (V/Cr) Input Buffer Address.",
        4, // Width in bytes
        0x000000e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_ps_vbuf
    },
    {
        "PS_PITCH",
        "This register contains the processed surface pitch in bytes.",
        4, // Width in bytes
        0x000000f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_ps_pitch
    },
    {
        "PS_BACKGROUND",
        "PS Background Pixel Color.",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_ps_background
    },
    {
        "PS_SCALE",
        "PS Scale Factor.",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_ps_scale
    },
    {
        "PS_OFFSET",
        "PS Scale Offset.",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_ps_offset
    },
    {
        "PS_CLRKEYLOW",
        "This register contains the color key low value for the PS buffer.",
        4, // Width in bytes
        0x00000130, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_ps_clrkeylow
    },
    {
        "PS_CLRKEYHIGH",
        "This register contains the color key high value for the PS buffer.",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_ps_clrkeyhigh
    },
    {
        "AS_CTRL",
        "This register contains buffer control for the Alpha Surface 0 input bu"
        "ffer.",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_pxp_as_ctrl
    },
    {
        "AS_BUF",
        "Alpha Surface 0 Buffer Address Pointer.",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_as_buf
    },
    {
        "AS_PITCH",
        "This register contains the alpha surface pitch in bytes.",
        4, // Width in bytes
        0x00000170, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_as_pitch
    },
    {
        "AS_CLRKEYLOW",
        "This register contains the color key low value for the AS buffer.",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_as_clrkeylow
    },
    {
        "AS_CLRKEYHIGH",
        "This register contains the color key high value for the AS buffer.",
        4, // Width in bytes
        0x00000190, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_as_clrkeyhigh
    },
    {
        "CSC1_COEF0",
        "This register contains color space conversion coefficients in two's co"
        "mpliment notation.",
        4, // Width in bytes
        0x000001a0, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_pxp_csc1_coef0
    },
    {
        "CSC1_COEF1",
        "This register contains color space conversion coefficients in two's co"
        "mpliment notation.",
        4, // Width in bytes
        0x000001b0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc1_coef1
    },
    {
        "CSC1_COEF2",
        "This register contains color space conversion coefficients in two's co"
        "mpliment notation.",
        4, // Width in bytes
        0x000001c0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc1_coef2
    },
    {
        "CSC2_CTRL",
        "This register contains the control registers to configure the CSC modu"
        "le.",
        4, // Width in bytes
        0x000001d0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc2_ctrl
    },
    {
        "CSC2_COEF0",
        "This register contains color space conversion coefficients in two's co"
        "mplement notation.",
        4, // Width in bytes
        0x000001e0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc2_coef0
    },
    {
        "CSC2_COEF1",
        "This register contains color space conversion coefficients in two's co"
        "mplement notation.",
        4, // Width in bytes
        0x000001f0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc2_coef1
    },
    {
        "CSC2_COEF2",
        "This register contains color space conversion coefficients in two's co"
        "mplement notation.",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc2_coef2
    },
    {
        "CSC2_COEF3",
        "This register contains color space conversion coefficients in two's co"
        "mplement notation.",
        4, // Width in bytes
        0x00000210, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc2_coef3
    },
    {
        "CSC2_COEF4",
        "This register contains color space conversion coefficients in two's co"
        "mplement notation.",
        4, // Width in bytes
        0x00000220, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc2_coef4
    },
    {
        "CSC2_COEF5",
        "This register contains color space conversion coefficients in two's co"
        "mplement notation.",
        4, // Width in bytes
        0x00000230, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_csc2_coef5
    },
    {
        "LUT_CTRL",
        "This register is used to access/control the Monochrome Lookup table.",
        4, // Width in bytes
        0x00000240, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_pxp_lut_ctrl
    },
    {
        "LUT_ADDR",
        "This register is used to access/control the Monochrome Lookup table.",
        4, // Width in bytes
        0x00000250, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_lut_addr
    },
    {
        "LUT_DATA",
        "This register is used to load data into the lookup table.",
        4, // Width in bytes
        0x00000260, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_lut_data
    },
    {
        "LUT_EXTMEM",
        "For DMA LUT memory loads, this is the base address from which data wil"
        "l be sourced to store into the LUT memory array.",
        4, // Width in bytes
        0x00000270, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_lut_extmem
    },
    {
        "CFA",
        "There are sixteen 2 bit values in this register each mapping a selecte"
        "d component to the output pixel.",
        4, // Width in bytes
        0x00000280, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_pxp_cfa
    },
    {
        "HIST_CTRL",
        "Provides control and status registers for the PXP's histogram classifi"
        "cation algorithm.",
        4, // Width in bytes
        0x00000290, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_hist_ctrl
    },
    {
        "HIST2_PARAM",
        "This register specifies the valid values for a 2-level histogram.",
        4, // Width in bytes
        0x000002a0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_hist2_param
    },
    {
        "HIST4_PARAM",
        "This register specifies the valid values for a 4-level histogram.",
        4, // Width in bytes
        0x000002b0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_hist4_param
    },
    {
        "HIST8_PARAM0",
        "This register specifies four of the valid values for an 8-level histog"
        "ram.",
        4, // Width in bytes
        0x000002c0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_hist8_param0
    },
    {
        "HIST8_PARAM1",
        "This register specifies four of the valid values for an 8-level histog"
        "ram.",
        4, // Width in bytes
        0x000002d0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_hist8_param1
    },
    {
        "HIST16_PARAM0",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x000002e0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_hist16_param0
    },
    {
        "HIST16_PARAM1",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x000002f0, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_hist16_param1
    },
    {
        "HIST16_PARAM2",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x00000300, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_hist16_param2
    },
    {
        "HIST16_PARAM3",
        "This register specifies four of the valid values for a 16-level histog"
        "ram.",
        4, // Width in bytes
        0x00000310, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_pxp_hist16_param3
    },
    {
        "POWER",
        "",
        4, // Width in bytes
        0x00000320, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_pxp_power
    },
    {
        "NEXT",
        "This register contains a pointer to a data structure used to reload th"
        "e PXP registers at the end of the current frame.",
        4, // Width in bytes
        0x00000400, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_pxp_next
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark QOS
#endif

// Bitfields in register QOS_CTRL.
static const field_t hw_qos_ctrl[] =
{
    {
        "EMI_PRIORITY_MODE",
        "Set this bit to one to enable QoS and priority values to be set separa"
        "tely for the AXI fabric and EMI.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "XLATE_AXI_MODE",
        "This bit selects the way in which AXI QoS information is translated in"
        "to EMI priority values.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPDC_PRIORITY_BOOST",
        "Set this field to enable a feedback path that allows the EPDC to reque"
        "st higher priority for its accesses when needed.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LCDIF_PRIORITY_BOOST",
        "Set this field to enable a feedback path that allows the eLCDIF to req"
        "uest higher priority for its accesses when needed.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE",
        "This bit must be set to zero for normal operation.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFTRST",
        "Set to zero for normal operation.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_AXI_QOS0.
static const field_t hw_qos_axi_qos0[] =
{
    {
        "M0_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 0.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M0_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 0.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_0_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_0_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 1.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_1_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 2.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_1_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 2.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_2_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 3.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_2_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 3.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_AXI_QOS1.
static const field_t hw_qos_axi_qos1[] =
{
    {
        "M2_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 4.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M2_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 4.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M3_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 5.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M3_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 5.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M4_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 6.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M4_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 6.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M5_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 7.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M5_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 7.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_AXI_QOS2.
static const field_t hw_qos_axi_qos2[] =
{
    {
        "M6_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 8.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M6_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 8.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M7_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 9.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M7_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 9.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M8_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 10.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M8_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 10.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M9_AWQOS",
        "Specifies the QOS level for the write commands on MasterID 10.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M9_ARQOS",
        "Specifies the QOS level for the read commands on MasterID 11.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_EMI_PRIORITY0.
static const field_t hw_qos_emi_priority0[] =
{
    {
        "M0_WR",
        "Specifies the priority level for the write commands on MasterID 0.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M0_RD",
        "Specifies the priority level for the read commands on MasterID 0.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_0_WR",
        "Specifies the priority level for the write commands on MasterID 1.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_0_RD",
        "Specifies the priority level for the read commands on MasterID 1.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_1_WR",
        "Specifies the priority level for the write commands on MasterID 2.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_1_RD",
        "Specifies the priority level for the read commands on MasterID 2.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_2_WR",
        "Specifies the priority level for the write commands on MasterID 3.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_2_RD",
        "Specifies the priority level for the read commands on MasterID 3.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_EMI_PRIORITY1.
static const field_t hw_qos_emi_priority1[] =
{
    {
        "M2_WR",
        "Specifies the priority level for the write commands on MasterID 4.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M2_RD",
        "Specifies the priority level for the read commands on MasterID 4.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M3_WR",
        "Specifies the priority level for the write commands on MasterID 5.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M3_RD",
        "Specifies the priority level for the read commands on MasterID 5.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M4_WR",
        "Specifies the priority level for the write commands on MasterID 6.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M4_RD",
        "Specifies the priority level for the read commands on MasterID 6.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M5_WR",
        "Specifies the priority level for the write commands on MasterID 7.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M5_RD",
        "Specifies the priority level for the read commands on MasterID 7.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_EMI_PRIORITY2.
static const field_t hw_qos_emi_priority2[] =
{
    {
        "M6_WR",
        "Specifies the priority level for the write commands on MasterID 8.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M6_RD",
        "Specifies the priority level for the read commands on MasterID 8.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M7_WR",
        "Specifies the priority level for the write commands on MasterID 9.",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M7_RD",
        "Specifies the priority level for the read commands on MasterID 9.",
        12, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M8_WR",
        "Specifies the priority level for the write commands on MasterID 10.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M8_RD",
        "Specifies the priority level for the read commands on MasterID 10.",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M9_WR",
        "Specifies the priority level for the write commands on MasterID 10.",
        24, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M9_RD",
        "Specifies the priority level for the read commands on MasterID 11.",
        28, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_DISABLE.
static const field_t hw_qos_disable[] =
{
    {
        "M1_0_DIS",
        "Setting this bit disables the interface between master1_0 and the AXI "
        "fabric.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_1_DIS",
        "Setting this bit disables the interface between master1_1 and the AXI "
        "fabric.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_2_DIS",
        "Setting this bit disables the interface between master1_2 and the AXI "
        "fabric.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M2_DIS",
        "Setting this bit disables the interface between master2 and the AXI fa"
        "bric.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M3_DIS",
        "Setting this bit disables the interface between master3 and the AXI fa"
        "bric.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M4_DIS",
        "Setting this bit disables the interface between master4 and the AXI fa"
        "bric.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M5_DIS",
        "Setting this bit disables the interface between master5 and the AXI fa"
        "bric.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M6_DIS",
        "Setting this bit disables the interface between master6 and the AXI fa"
        "bric.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M7_DIS",
        "Setting this bit disables the interface between master7 and the AXI fa"
        "bric.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M8_DIS",
        "Setting this bit disables the interface between master8 and the AXI fa"
        "bric.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M9_DIS",
        "Setting this bit disables the interface between master9 and the AXI fa"
        "bric.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "M1_0_DIS_STAT",
        "This bit indicates the enable/disable status for master port1_0 on the"
        " AXI fabric.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M1_1_DIS_STAT",
        "This bit indicates the enable/disable status for master port1_1 on the"
        " AXI fabric.",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M1_2_DIS_STAT",
        "This bit indicates the enable/disable status for master port1_2 on the"
        " AXI fabric.",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M2_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M3_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        21, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M4_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M5_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M6_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M7_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M8_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    {
        "M9_DIS_STAT",
        "This bit indicates the enable/disable status for master port0 on the A"
        "XI fabric.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register QOS_VERSION.
static const field_t hw_qos_version[] =
{
    {
        "STEP",
        "Fixed read-only value reflecting the stepping of the RTL version.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MINOR",
        "Fixed read-only value reflecting the MINOR field of the RTL version.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-only value reflecting the MAJOR field of the RTL version.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a QOS module.
static const reg_t hw_qos[] =
{
    {
        "CTRL",
        "The Control Register specifies the reset state and the interrupt contr"
        "ols for the Performance Monitor.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_qos_ctrl
    },
    {
        "AXI_QOS0",
        "This register specifies the AWQOS and ARQOS parameters for masters 0-3"
        ".",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_qos_axi_qos0
    },
    {
        "AXI_QOS1",
        "This register specifies the AWQOS and ARQOS parameters for masters 4-7"
        ".",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_qos_axi_qos1
    },
    {
        "AXI_QOS2",
        "This register specifies the AWQOS and ARQOS parameters for masters 8-1"
        "1.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_qos_axi_qos2
    },
    {
        "EMI_PRIORITY0",
        "For EMI manual priority mode, this register specifies the AWPRIORITY a"
        "nd ARPRIORITY parameters for masters 0-3.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_qos_emi_priority0
    },
    {
        "EMI_PRIORITY1",
        "For EMI manual priority mode, this register specifies the AWPRIORITY a"
        "nd ARPRIORITY parameters for masters 4-7.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_qos_emi_priority1
    },
    {
        "EMI_PRIORITY2",
        "For EMI manual priority mode, this register specifies the AWPRIORITY a"
        "nd ARPRIORITY parameters for masters 8-11.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_qos_emi_priority2
    },
    {
        "DISABLE",
        "This register allows disabling of the individual master ports of the A"
        "XI fabric.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        22, // Number of bitfields
        hw_qos_disable
    },
    {
        "VERSION",
        "This register always returns a known read value for debug purposes it "
        "indicates the version of the block.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_qos_version
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark RNG
#endif

// Bitfields in register RNG_VER.
static const field_t hw_rng_ver[] =
{
    {
        "MINOR",
        "Subjiect to change.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "This field is always set to 0x02.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TYPE",
        "",
        28, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register RNG_CMD.
static const field_t hw_rng_cmd[] =
{
    {
        "ST",
        "Initiates a self test of the RNGB's internal logic.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GS",
        "Initiates the seed generation process.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CI",
        "Clears the RNGB interrupt if an error is not present.",
        4, // LSB
        4, // MSB
        false, // Readable
        true // Writable
    },
    {
        "CE",
        "Clears the errors in the RNG_ESR register and the RNGB interrupt.This "
        "bit is self-clearing.",
        5, // LSB
        5, // MSB
        false, // Readable
        true // Writable
    },
    {
        "SR",
        "Performs a software reset of the RNGB.",
        6, // LSB
        6, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register RNG_CR.
static const field_t hw_rng_cr[] =
{
    {
        "FUFMOD",
        "Controls the RNGB's response to a FIFO underflow condition.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AR",
        "Setting this bit allows the RNGB to automatically generate a new seed "
        "whenever one is needed.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASKDONE",
        "Masks interrupts generated upon completion of seed and self test modes"
        ".",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASKERR",
        "Masks interrupts generated by errors in the RNGB.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register RNG_SR.
static const field_t hw_rng_sr[] =
{
    {
        "BUSY",
        "Reflects the current state of RNGB.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SLP",
        "Indicates if the RNGB is in sleep mode.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RS",
        "Indicates the RNGB needs to be reseeded.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "STDN",
        "Indicates the self test is complete.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SDN",
        "Indicates the RNG has generated the first seed.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NSDN",
        "Indicates that a new seed is ready for use during the next seed genera"
        "tion process.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FIFO_LVL",
        "Indicates the number of random words currently in the output FIFO.",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FIFO_SIZE",
        "Size of the FIFO, and maximum possible FIFO level.",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ERR",
        "Indicates an error was detected in the RNGB.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ST_PF",
        "Indicates Pass or Fail status of the TRNG, PRNG, and RESEED self tests"
        ",  Bit 23 - TRNG self test pass/fail  Bit 22 - PRNG self test pass/fai"
        "l  Bit 21 - RESEED self test pass/fail",
        21, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "STATPF",
        "Indicates pass or fail status of the various statistics tests on the l"
        "ast seed generated.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register RNG_ESR.
static const field_t hw_rng_esr[] =
{
    {
        "LFE",
        "When this bit is set, the interrupt generated was caused by a failure "
        "of one of the LFSRs in one of the RNGB's three entropy sources.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OSCE",
        "Indicates the oscillator in the RNG may be broken.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "STE",
        "Indicates the RNGB has failed the most recent self test.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SATE",
        "Indicates if RNGB has failed the statistical tests for the last genera"
        "ted seed.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FUFE",
        "Indicates the RNGB has experienced a FIFO underflow condition resultin"
        "g in the last random data read being unreliable.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register RNG_OUT.
static const field_t hw_rng_out[] =
{
    {
        "RANDOUT",
        "",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a RNG module.
static const reg_t hw_rng[] =
{
    {
        "VER",
        "The read-only RNG_VER register contains the current version of the RNG"
        "B.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_rng_ver
    },
    {
        "CMD",
        "RNG_CMD controls the RNG's operating modes and interrupt status.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_rng_cmd
    },
    {
        "CR",
        "Through use of this register, the RNGB can be programmed to provide sl"
        "ightly different functionality based on its desired use.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_rng_cr
    },
    {
        "SR",
        "The RNGBSR is a read-only register which reflects the internal status "
        "of the RNGB.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        false, // Writable
        11, // Number of bitfields
        hw_rng_sr
    },
    {
        "ESR",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        false, // Writable
        5, // Number of bitfields
        hw_rng_esr
    },
    {
        "OUT",
        "The RNGBOUT provides temporary storage for random data generated by th"
        "e RNGB.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_rng_out
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark ROMC
#endif

// Bitfields in register ROMC_ROMPATCH0D.
static const field_t hw_romc_rompatch0d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH1D.
static const field_t hw_romc_rompatch1d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH2D.
static const field_t hw_romc_rompatch2d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH3D.
static const field_t hw_romc_rompatch3d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH4D.
static const field_t hw_romc_rompatch4d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH5D.
static const field_t hw_romc_rompatch5d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH6D.
static const field_t hw_romc_rompatch6d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH7D.
static const field_t hw_romc_rompatch7d[] =
{
    {
        "DATAX",
        "Data Fix Registers - Stores the data used for 1-word data fix operatio"
        "ns.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCHCNTL.
static const field_t hw_romc_rompatchcntl[] =
{
    {
        "DATAFIX",
        "Data Fix Enable - Controls the use of the first 8 address comparators "
        "for 1-word data fix or for code patch routine.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIS",
        "ROMC Disable -- This bit, when set, disables all ROMC operations.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCHENH.
static const field_t hw_romc_rompatchenh[] =
{
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCHENL.
static const field_t hw_romc_rompatchenl[] =
{
    {
        "ENABLE",
        "Enable Address Comparator - This bit enables the corresponding address"
        " comparator to trigger an event.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH0A.
static const field_t hw_romc_rompatch0a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH1A.
static const field_t hw_romc_rompatch1a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH2A.
static const field_t hw_romc_rompatch2a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH3A.
static const field_t hw_romc_rompatch3a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH4A.
static const field_t hw_romc_rompatch4a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH5A.
static const field_t hw_romc_rompatch5a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH6A.
static const field_t hw_romc_rompatch6a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH7A.
static const field_t hw_romc_rompatch7a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH8A.
static const field_t hw_romc_rompatch8a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH9A.
static const field_t hw_romc_rompatch9a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH10A.
static const field_t hw_romc_rompatch10a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH11A.
static const field_t hw_romc_rompatch11a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH12A.
static const field_t hw_romc_rompatch12a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH13A.
static const field_t hw_romc_rompatch13a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH14A.
static const field_t hw_romc_rompatch14a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCH15A.
static const field_t hw_romc_rompatch15a[] =
{
    {
        "THUMBX",
        "THUMB Comparator Select - Indicates that this address will trigger a T"
        "HUMB opcode patch or an ARM opcode patch.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADDRX",
        "Address Comparator Registers - Indicates the memory address to be watc"
        "hed.",
        1, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register ROMC_ROMPATCHSR.
static const field_t hw_romc_rompatchsr[] =
{
    {
        "SOURCE",
        "ROMC Source Number - Binary encoding of the number of the address comp"
        "arator which has an address match in the most recent patch event on RO"
        "MC AHB.",
        0, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SW",
        "ROMC AHB Multiple Address Comparator matches Indicator - Indicates tha"
        "t multiple address comparator matches occurred.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a ROMC module.
static const reg_t hw_romc[] =
{
    {
        "ROMPATCH0D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000d4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch0d
    },
    {
        "ROMPATCH1D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000d8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch1d
    },
    {
        "ROMPATCH2D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000dc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch2d
    },
    {
        "ROMPATCH3D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000e0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch3d
    },
    {
        "ROMPATCH4D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000e4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch4d
    },
    {
        "ROMPATCH5D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000e8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch5d
    },
    {
        "ROMPATCH6D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000ec, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch6d
    },
    {
        "ROMPATCH7D",
        "The ROMC data registers (ROMC_ROMPATCHD7 through ROMC_ROMPATCHD0) stor"
        "e the data to use for the 8 1-word data fix events.",
        4, // Width in bytes
        0x000000f0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatch7d
    },
    {
        "ROMPATCHCNTL",
        "The ROMC control register (ROMC_ROMPATCHCNTL) contains the block disab"
        "le bit and the data fix enable bits.",
        4, // Width in bytes
        0x000000f4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatchcntl
    },
    {
        "ROMPATCHENH",
        "The ROMC enable register high (ROMC_ROMPATCHENH) and ROMC enable regis"
        "ter low (ROMC_ROMPATCHENL) control whether or not the associated addre"
        "ss comparator can trigger a opcode patch or data fix event.",
        4, // Width in bytes
        0x000000f8, // Base address offset
        true, // Readable
        false, // Writable
        0, // Number of bitfields
        hw_romc_rompatchenh
    },
    {
        "ROMPATCHENL",
        "The ROMC enable register high (ROMC_ROMPATCHENH) and ROMC enable regis"
        "ter low (ROMC_ROMPATCHENL) control whether or not the associated addre"
        "ss comparator can trigger a opcode patch or data fix event.",
        4, // Width in bytes
        0x000000fc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_romc_rompatchenl
    },
    {
        "ROMPATCH0A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch0a
    },
    {
        "ROMPATCH1A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000104, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch1a
    },
    {
        "ROMPATCH2A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000108, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch2a
    },
    {
        "ROMPATCH3A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x0000010c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch3a
    },
    {
        "ROMPATCH4A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch4a
    },
    {
        "ROMPATCH5A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000114, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch5a
    },
    {
        "ROMPATCH6A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000118, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch6a
    },
    {
        "ROMPATCH7A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x0000011c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch7a
    },
    {
        "ROMPATCH8A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch8a
    },
    {
        "ROMPATCH9A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000124, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch9a
    },
    {
        "ROMPATCH10A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000128, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch10a
    },
    {
        "ROMPATCH11A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x0000012c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch11a
    },
    {
        "ROMPATCH12A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000130, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch12a
    },
    {
        "ROMPATCH13A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000134, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch13a
    },
    {
        "ROMPATCH14A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x00000138, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch14a
    },
    {
        "ROMPATCH15A",
        "The ROMC address registers (ROMC_ROMPATCHA0 through ROMC_ROMPATCHA15) "
        "store the memory addresses where opcode patching begins and data fixin"
        "g occurs.",
        4, // Width in bytes
        0x0000013c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatch15a
    },
    {
        "ROMPATCHSR",
        "The ROMC status register (ROMC_ROMPATCHSR) indicates the current state"
        " of the ROMC and the source number of the most recent address comparat"
        "or event.",
        4, // Width in bytes
        0x00000208, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_romc_rompatchsr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SDMAARM
#endif

// Bitfields in register SDMAARM_MC0PTR.
static const field_t hw_sdmaarm_mc0ptr[] =
{
    {
        "MC0PTR",
        "Channel 0 Pointer contains the 32-bit address, in ARM platform memory,"
        " of channel 0 control block (the boot channel).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_INTR.
static const field_t hw_sdmaarm_intr[] =
{
    {
        "HI",
        "The ARM platform Interrupts register contains the 32 HI[i] bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_STOP_STAT.
static const field_t hw_sdmaarm_stop_stat[] =
{
    {
        "HE",
        "This 32-bit register gives access to the ARM platform Enable bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_HSTART.
static const field_t hw_sdmaarm_hstart[] =
{
    {
        "HSTART",
        "The HSTART/HE registers are 32 bits wide with one bit for every channe"
        "l.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_EVTOVR.
static const field_t hw_sdmaarm_evtovr[] =
{
    {
        "EO",
        "The Channel Event Override register contains the 32 EO[i] bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_DSPOVR.
static const field_t hw_sdmaarm_dspovr[] =
{
    {
        "DO",
        "This register is reserved.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_HOSTOVR.
static const field_t hw_sdmaarm_hostovr[] =
{
    {
        "HO",
        "The Channel ARM platform Override register contains the 32 HO[i] bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_EVTPEND.
static const field_t hw_sdmaarm_evtpend[] =
{
    {
        "EP",
        "The Channel Event Pending register contains the 32 EP[i] bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_RESET.
static const field_t hw_sdmaarm_reset[] =
{
    {
        "RESET",
        "When set, this bit causes the SDMA to be held in a software reset.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RESCHED",
        "When set, this bit forces the SDMA to reschedule as if a script had ex"
        "ecuted a done instruction.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_EVTERR.
static const field_t hw_sdmaarm_evterr[] =
{
    {
        "CHNERR",
        "This register is used by the SDMA to warn the ARM platform when an inc"
        "oming DMA request was detected and it triggers a channel that is alrea"
        "dy pending or being serviced.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_INTRMASK.
static const field_t hw_sdmaarm_intrmask[] =
{
    {
        "HIMASK",
        "The Interrupt Mask Register contains 32 interrupt generation mask bits"
        ".",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_PSW.
static const field_t hw_sdmaarm_psw[] =
{
    {
        "CCR",
        "The Current Channel Register indicates the number of the channel that "
        "is being executed by the SDMA.",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CCP",
        "The Current Channel Priority indicates the priority of the current act"
        "ive channel.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NCR",
        "The Next Channel Register indicates the number of the next scheduled p"
        "ending channel with the highest priority.",
        8, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NCP",
        "The Next Channel Priority gives the next pending channel priority.",
        13, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_EVTERRDBG.
static const field_t hw_sdmaarm_evterrdbg[] =
{
    {
        "CHNERR",
        "This register is the same as EVTERR, except reading it does not clear "
        "its contents.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_CONFIG.
static const field_t hw_sdmaarm_config[] =
{
    {
        "CSM",
        "Selects the Context Switch Mode.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACR",
        "ARM platform DMA / SDMA Core Clock Ratio.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTDOBS",
        "Indicates if Real-Time Debug pins are used: They do not toggle by defa"
        "ult in order to reduce power consumption.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSPDMA",
        "This bit's function is reserved and should be configured as zero.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_LOCK.
static const field_t hw_sdmaarm_sdma_lock[] =
{
    {
        "LOCK",
        "The LOCK bit is used to restrict access to update SDMA script memory t"
        "hrough ROM channel zero scripts and through the OnCE interface under A"
        "RM platform control.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRESET_LOCK_CLR",
        "The SRESET_LOCK_CLR bit determine if the LOCK bit is cleared on a soft"
        "ware reset triggered by writing to the RESET register.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_ONCE_ENB.
static const field_t hw_sdmaarm_once_enb[] =
{
    {
        "ENB",
        "The OnCE Enable register selects the OnCE control source: When cleared"
        " (0), the OnCE registers are accessed through the JTAG interface; when"
        " set (1), the OnCE registers may be accessed by the ARM platform throu"
        "gh the addresses described, as follows.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_ONCE_DATA.
static const field_t hw_sdmaarm_once_data[] =
{
    {
        "DATA",
        "Data register of the OnCE JTAG controller.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_ONCE_INSTR.
static const field_t hw_sdmaarm_once_instr[] =
{
    {
        "INSTR",
        "Instruction register of the OnCE JTAG controller.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_ONCE_STAT.
static const field_t hw_sdmaarm_once_stat[] =
{
    {
        "ECDR",
        "Event Cell Debug Request.",
        0, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MST",
        "This flag is raised when the OnCE is controlled from the ARM platform "
        "peripheral interface.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SWB",
        "This flag is raised when the SDMA has entered debug mode after a softw"
        "are breakpoint.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ODR",
        "This flag is raised when the SDMA has entered debug mode after a OnCE "
        "debug request.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EDR",
        "This flag is raised when the SDMA has entered debug mode after an exte"
        "rnal debug request.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RCV",
        "After each write access to the real time buffer (RTB), the RCV bit is "
        "set.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PST",
        "The Processor Status bits reflect the state of the SDMA RISC engine.",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_ONCE_CMD.
static const field_t hw_sdmaarm_once_cmd[] =
{
    {
        "CMD",
        "Writing to this register will cause the OnCE to execute the command th"
        "at is written.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_ILLINSTADDR.
static const field_t hw_sdmaarm_illinstaddr[] =
{
    {
        "ILLINSTADDR",
        "The Illegal Instruction Trap Address is the address where the SDMA jum"
        "ps when an illegal instruction is executed.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_CHN0ADDR.
static const field_t hw_sdmaarm_chn0addr[] =
{
    {
        "CHN0ADDR",
        "This 14-bit register is used by the boot code of the SDMA.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SMSZ",
        "The bit 14 (Scratch Memory Size) determines if scratch memory must be "
        "available after every channel context.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_EVT_MIRROR.
static const field_t hw_sdmaarm_evt_mirror[] =
{
    {
        "EVENTS",
        "This register reflects the DMA requests received by the SDMA for event"
        "s 31-0.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_EVT_MIRROR2.
static const field_t hw_sdmaarm_evt_mirror2[] =
{
    {
        "EVENTS",
        "This register reflects the DMA requests received by the SDMA for event"
        "s 47-32.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_XTRIG_CONF1.
static const field_t hw_sdmaarm_xtrig_conf1[] =
{
    {
        "NUM0",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF0",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM1",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF1",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM2",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF2",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM3",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF3",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_XTRIG_CONF2.
static const field_t hw_sdmaarm_xtrig_conf2[] =
{
    {
        "NUM4",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF4",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM5",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        8, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF5",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM6",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF6",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NUM7",
        "Contains the number of the DMA request or channel that triggers the pu"
        "lse on the cross-trigger event line number i .",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNF7",
        "Configuration of the SDMA event line number i that is connected to the"
        " cross-trigger.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI0.
static const field_t hw_sdmaarm_sdma_chnpri0[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI1.
static const field_t hw_sdmaarm_sdma_chnpri1[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI2.
static const field_t hw_sdmaarm_sdma_chnpri2[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI3.
static const field_t hw_sdmaarm_sdma_chnpri3[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI4.
static const field_t hw_sdmaarm_sdma_chnpri4[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI5.
static const field_t hw_sdmaarm_sdma_chnpri5[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI6.
static const field_t hw_sdmaarm_sdma_chnpri6[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI7.
static const field_t hw_sdmaarm_sdma_chnpri7[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI8.
static const field_t hw_sdmaarm_sdma_chnpri8[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI9.
static const field_t hw_sdmaarm_sdma_chnpri9[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI10.
static const field_t hw_sdmaarm_sdma_chnpri10[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI11.
static const field_t hw_sdmaarm_sdma_chnpri11[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI12.
static const field_t hw_sdmaarm_sdma_chnpri12[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI13.
static const field_t hw_sdmaarm_sdma_chnpri13[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI14.
static const field_t hw_sdmaarm_sdma_chnpri14[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI15.
static const field_t hw_sdmaarm_sdma_chnpri15[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI16.
static const field_t hw_sdmaarm_sdma_chnpri16[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI17.
static const field_t hw_sdmaarm_sdma_chnpri17[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI18.
static const field_t hw_sdmaarm_sdma_chnpri18[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI19.
static const field_t hw_sdmaarm_sdma_chnpri19[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI20.
static const field_t hw_sdmaarm_sdma_chnpri20[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI21.
static const field_t hw_sdmaarm_sdma_chnpri21[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI22.
static const field_t hw_sdmaarm_sdma_chnpri22[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI23.
static const field_t hw_sdmaarm_sdma_chnpri23[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI24.
static const field_t hw_sdmaarm_sdma_chnpri24[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI25.
static const field_t hw_sdmaarm_sdma_chnpri25[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI26.
static const field_t hw_sdmaarm_sdma_chnpri26[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI27.
static const field_t hw_sdmaarm_sdma_chnpri27[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI28.
static const field_t hw_sdmaarm_sdma_chnpri28[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI29.
static const field_t hw_sdmaarm_sdma_chnpri29[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI30.
static const field_t hw_sdmaarm_sdma_chnpri30[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNPRI31.
static const field_t hw_sdmaarm_sdma_chnpri31[] =
{
    {
        "CHNPRIN",
        "This contains the priority of channel number n .",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL0.
static const field_t hw_sdmaarm_sdma_chnenbl0[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL1.
static const field_t hw_sdmaarm_sdma_chnenbl1[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL2.
static const field_t hw_sdmaarm_sdma_chnenbl2[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL3.
static const field_t hw_sdmaarm_sdma_chnenbl3[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL4.
static const field_t hw_sdmaarm_sdma_chnenbl4[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL5.
static const field_t hw_sdmaarm_sdma_chnenbl5[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL6.
static const field_t hw_sdmaarm_sdma_chnenbl6[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL7.
static const field_t hw_sdmaarm_sdma_chnenbl7[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL8.
static const field_t hw_sdmaarm_sdma_chnenbl8[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL9.
static const field_t hw_sdmaarm_sdma_chnenbl9[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL10.
static const field_t hw_sdmaarm_sdma_chnenbl10[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL11.
static const field_t hw_sdmaarm_sdma_chnenbl11[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL12.
static const field_t hw_sdmaarm_sdma_chnenbl12[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL13.
static const field_t hw_sdmaarm_sdma_chnenbl13[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL14.
static const field_t hw_sdmaarm_sdma_chnenbl14[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL15.
static const field_t hw_sdmaarm_sdma_chnenbl15[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL16.
static const field_t hw_sdmaarm_sdma_chnenbl16[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL17.
static const field_t hw_sdmaarm_sdma_chnenbl17[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL18.
static const field_t hw_sdmaarm_sdma_chnenbl18[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL19.
static const field_t hw_sdmaarm_sdma_chnenbl19[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL20.
static const field_t hw_sdmaarm_sdma_chnenbl20[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL21.
static const field_t hw_sdmaarm_sdma_chnenbl21[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL22.
static const field_t hw_sdmaarm_sdma_chnenbl22[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL23.
static const field_t hw_sdmaarm_sdma_chnenbl23[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL24.
static const field_t hw_sdmaarm_sdma_chnenbl24[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL25.
static const field_t hw_sdmaarm_sdma_chnenbl25[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL26.
static const field_t hw_sdmaarm_sdma_chnenbl26[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL27.
static const field_t hw_sdmaarm_sdma_chnenbl27[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL28.
static const field_t hw_sdmaarm_sdma_chnenbl28[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL29.
static const field_t hw_sdmaarm_sdma_chnenbl29[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL30.
static const field_t hw_sdmaarm_sdma_chnenbl30[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL31.
static const field_t hw_sdmaarm_sdma_chnenbl31[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL32.
static const field_t hw_sdmaarm_sdma_chnenbl32[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL33.
static const field_t hw_sdmaarm_sdma_chnenbl33[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL34.
static const field_t hw_sdmaarm_sdma_chnenbl34[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL35.
static const field_t hw_sdmaarm_sdma_chnenbl35[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL36.
static const field_t hw_sdmaarm_sdma_chnenbl36[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL37.
static const field_t hw_sdmaarm_sdma_chnenbl37[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL38.
static const field_t hw_sdmaarm_sdma_chnenbl38[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL39.
static const field_t hw_sdmaarm_sdma_chnenbl39[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL40.
static const field_t hw_sdmaarm_sdma_chnenbl40[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL41.
static const field_t hw_sdmaarm_sdma_chnenbl41[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL42.
static const field_t hw_sdmaarm_sdma_chnenbl42[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL43.
static const field_t hw_sdmaarm_sdma_chnenbl43[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL44.
static const field_t hw_sdmaarm_sdma_chnenbl44[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL45.
static const field_t hw_sdmaarm_sdma_chnenbl45[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL46.
static const field_t hw_sdmaarm_sdma_chnenbl46[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMAARM_SDMA_CHNENBL47.
static const field_t hw_sdmaarm_sdma_chnenbl47[] =
{
    {
        "ENBLN",
        "This 32-bit value selects the channels that are triggered by the DMA r"
        "equest number n .",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a SDMAARM module.
static const reg_t hw_sdmaarm[] =
{
    {
        "MC0PTR",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_mc0ptr
    },
    {
        "INTR",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_intr
    },
    {
        "STOP_STAT",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_stop_stat
    },
    {
        "HSTART",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_hstart
    },
    {
        "EVTOVR",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_evtovr
    },
    {
        "DSPOVR",
        "",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_dspovr
    },
    {
        "HOSTOVR",
        "",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_hostovr
    },
    {
        "EVTPEND",
        "",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_evtpend
    },
    {
        "RESET",
        "",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_sdmaarm_reset
    },
    {
        "EVTERR",
        "",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmaarm_evterr
    },
    {
        "INTRMASK",
        "",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_intrmask
    },
    {
        "PSW",
        "",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_sdmaarm_psw
    },
    {
        "EVTERRDBG",
        "",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmaarm_evterrdbg
    },
    {
        "CONFIG",
        "",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_sdmaarm_config
    },
    {
        "SDMA_LOCK",
        "",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_sdmaarm_sdma_lock
    },
    {
        "ONCE_ENB",
        "",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_once_enb
    },
    {
        "ONCE_DATA",
        "",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_once_data
    },
    {
        "ONCE_INSTR",
        "",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_once_instr
    },
    {
        "ONCE_STAT",
        "",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_sdmaarm_once_stat
    },
    {
        "ONCE_CMD",
        "",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_once_cmd
    },
    {
        "ILLINSTADDR",
        "",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_illinstaddr
    },
    {
        "CHN0ADDR",
        "",
        4, // Width in bytes
        0x0000005c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_sdmaarm_chn0addr
    },
    {
        "EVT_MIRROR",
        "",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmaarm_evt_mirror
    },
    {
        "EVT_MIRROR2",
        "",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmaarm_evt_mirror2
    },
    {
        "XTRIG_CONF1",
        "",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_sdmaarm_xtrig_conf1
    },
    {
        "XTRIG_CONF2",
        "",
        4, // Width in bytes
        0x00000074, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_sdmaarm_xtrig_conf2
    },
    {
        "SDMA_CHNPRI0",
        "",
        4, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri0
    },
    {
        "SDMA_CHNPRI1",
        "",
        4, // Width in bytes
        0x00000104, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri1
    },
    {
        "SDMA_CHNPRI2",
        "",
        4, // Width in bytes
        0x00000108, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri2
    },
    {
        "SDMA_CHNPRI3",
        "",
        4, // Width in bytes
        0x0000010c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri3
    },
    {
        "SDMA_CHNPRI4",
        "",
        4, // Width in bytes
        0x00000110, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri4
    },
    {
        "SDMA_CHNPRI5",
        "",
        4, // Width in bytes
        0x00000114, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri5
    },
    {
        "SDMA_CHNPRI6",
        "",
        4, // Width in bytes
        0x00000118, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri6
    },
    {
        "SDMA_CHNPRI7",
        "",
        4, // Width in bytes
        0x0000011c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri7
    },
    {
        "SDMA_CHNPRI8",
        "",
        4, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri8
    },
    {
        "SDMA_CHNPRI9",
        "",
        4, // Width in bytes
        0x00000124, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri9
    },
    {
        "SDMA_CHNPRI10",
        "",
        4, // Width in bytes
        0x00000128, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri10
    },
    {
        "SDMA_CHNPRI11",
        "",
        4, // Width in bytes
        0x0000012c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri11
    },
    {
        "SDMA_CHNPRI12",
        "",
        4, // Width in bytes
        0x00000130, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri12
    },
    {
        "SDMA_CHNPRI13",
        "",
        4, // Width in bytes
        0x00000134, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri13
    },
    {
        "SDMA_CHNPRI14",
        "",
        4, // Width in bytes
        0x00000138, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri14
    },
    {
        "SDMA_CHNPRI15",
        "",
        4, // Width in bytes
        0x0000013c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri15
    },
    {
        "SDMA_CHNPRI16",
        "",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri16
    },
    {
        "SDMA_CHNPRI17",
        "",
        4, // Width in bytes
        0x00000144, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri17
    },
    {
        "SDMA_CHNPRI18",
        "",
        4, // Width in bytes
        0x00000148, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri18
    },
    {
        "SDMA_CHNPRI19",
        "",
        4, // Width in bytes
        0x0000014c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri19
    },
    {
        "SDMA_CHNPRI20",
        "",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri20
    },
    {
        "SDMA_CHNPRI21",
        "",
        4, // Width in bytes
        0x00000154, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri21
    },
    {
        "SDMA_CHNPRI22",
        "",
        4, // Width in bytes
        0x00000158, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri22
    },
    {
        "SDMA_CHNPRI23",
        "",
        4, // Width in bytes
        0x0000015c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri23
    },
    {
        "SDMA_CHNPRI24",
        "",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri24
    },
    {
        "SDMA_CHNPRI25",
        "",
        4, // Width in bytes
        0x00000164, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri25
    },
    {
        "SDMA_CHNPRI26",
        "",
        4, // Width in bytes
        0x00000168, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri26
    },
    {
        "SDMA_CHNPRI27",
        "",
        4, // Width in bytes
        0x0000016c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri27
    },
    {
        "SDMA_CHNPRI28",
        "",
        4, // Width in bytes
        0x00000170, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri28
    },
    {
        "SDMA_CHNPRI29",
        "",
        4, // Width in bytes
        0x00000174, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri29
    },
    {
        "SDMA_CHNPRI30",
        "",
        4, // Width in bytes
        0x00000178, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri30
    },
    {
        "SDMA_CHNPRI31",
        "",
        4, // Width in bytes
        0x0000017c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnpri31
    },
    {
        "SDMA_CHNENBL0",
        "",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl0
    },
    {
        "SDMA_CHNENBL1",
        "",
        4, // Width in bytes
        0x00000204, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl1
    },
    {
        "SDMA_CHNENBL2",
        "",
        4, // Width in bytes
        0x00000208, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl2
    },
    {
        "SDMA_CHNENBL3",
        "",
        4, // Width in bytes
        0x0000020c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl3
    },
    {
        "SDMA_CHNENBL4",
        "",
        4, // Width in bytes
        0x00000210, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl4
    },
    {
        "SDMA_CHNENBL5",
        "",
        4, // Width in bytes
        0x00000214, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl5
    },
    {
        "SDMA_CHNENBL6",
        "",
        4, // Width in bytes
        0x00000218, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl6
    },
    {
        "SDMA_CHNENBL7",
        "",
        4, // Width in bytes
        0x0000021c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl7
    },
    {
        "SDMA_CHNENBL8",
        "",
        4, // Width in bytes
        0x00000220, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl8
    },
    {
        "SDMA_CHNENBL9",
        "",
        4, // Width in bytes
        0x00000224, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl9
    },
    {
        "SDMA_CHNENBL10",
        "",
        4, // Width in bytes
        0x00000228, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl10
    },
    {
        "SDMA_CHNENBL11",
        "",
        4, // Width in bytes
        0x0000022c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl11
    },
    {
        "SDMA_CHNENBL12",
        "",
        4, // Width in bytes
        0x00000230, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl12
    },
    {
        "SDMA_CHNENBL13",
        "",
        4, // Width in bytes
        0x00000234, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl13
    },
    {
        "SDMA_CHNENBL14",
        "",
        4, // Width in bytes
        0x00000238, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl14
    },
    {
        "SDMA_CHNENBL15",
        "",
        4, // Width in bytes
        0x0000023c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl15
    },
    {
        "SDMA_CHNENBL16",
        "",
        4, // Width in bytes
        0x00000240, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl16
    },
    {
        "SDMA_CHNENBL17",
        "",
        4, // Width in bytes
        0x00000244, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl17
    },
    {
        "SDMA_CHNENBL18",
        "",
        4, // Width in bytes
        0x00000248, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl18
    },
    {
        "SDMA_CHNENBL19",
        "",
        4, // Width in bytes
        0x0000024c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl19
    },
    {
        "SDMA_CHNENBL20",
        "",
        4, // Width in bytes
        0x00000250, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl20
    },
    {
        "SDMA_CHNENBL21",
        "",
        4, // Width in bytes
        0x00000254, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl21
    },
    {
        "SDMA_CHNENBL22",
        "",
        4, // Width in bytes
        0x00000258, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl22
    },
    {
        "SDMA_CHNENBL23",
        "",
        4, // Width in bytes
        0x0000025c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl23
    },
    {
        "SDMA_CHNENBL24",
        "",
        4, // Width in bytes
        0x00000260, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl24
    },
    {
        "SDMA_CHNENBL25",
        "",
        4, // Width in bytes
        0x00000264, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl25
    },
    {
        "SDMA_CHNENBL26",
        "",
        4, // Width in bytes
        0x00000268, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl26
    },
    {
        "SDMA_CHNENBL27",
        "",
        4, // Width in bytes
        0x0000026c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl27
    },
    {
        "SDMA_CHNENBL28",
        "",
        4, // Width in bytes
        0x00000270, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl28
    },
    {
        "SDMA_CHNENBL29",
        "",
        4, // Width in bytes
        0x00000274, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl29
    },
    {
        "SDMA_CHNENBL30",
        "",
        4, // Width in bytes
        0x00000278, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl30
    },
    {
        "SDMA_CHNENBL31",
        "",
        4, // Width in bytes
        0x0000027c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl31
    },
    {
        "SDMA_CHNENBL32",
        "",
        4, // Width in bytes
        0x00000280, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl32
    },
    {
        "SDMA_CHNENBL33",
        "",
        4, // Width in bytes
        0x00000284, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl33
    },
    {
        "SDMA_CHNENBL34",
        "",
        4, // Width in bytes
        0x00000288, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl34
    },
    {
        "SDMA_CHNENBL35",
        "",
        4, // Width in bytes
        0x0000028c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl35
    },
    {
        "SDMA_CHNENBL36",
        "",
        4, // Width in bytes
        0x00000290, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl36
    },
    {
        "SDMA_CHNENBL37",
        "",
        4, // Width in bytes
        0x00000294, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl37
    },
    {
        "SDMA_CHNENBL38",
        "",
        4, // Width in bytes
        0x00000298, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl38
    },
    {
        "SDMA_CHNENBL39",
        "",
        4, // Width in bytes
        0x0000029c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl39
    },
    {
        "SDMA_CHNENBL40",
        "",
        4, // Width in bytes
        0x000002a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl40
    },
    {
        "SDMA_CHNENBL41",
        "",
        4, // Width in bytes
        0x000002a4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl41
    },
    {
        "SDMA_CHNENBL42",
        "",
        4, // Width in bytes
        0x000002a8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl42
    },
    {
        "SDMA_CHNENBL43",
        "",
        4, // Width in bytes
        0x000002ac, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl43
    },
    {
        "SDMA_CHNENBL44",
        "",
        4, // Width in bytes
        0x000002b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl44
    },
    {
        "SDMA_CHNENBL45",
        "",
        4, // Width in bytes
        0x000002b4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl45
    },
    {
        "SDMA_CHNENBL46",
        "",
        4, // Width in bytes
        0x000002b8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl46
    },
    {
        "SDMA_CHNENBL47",
        "",
        4, // Width in bytes
        0x000002bc, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmaarm_sdma_chnenbl47
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SDMABP
#endif

// Bitfields in register SDMABP_DC0PTR.
static const field_t hw_sdmabp_dc0ptr[] =
{
    {
        "DC0PTR",
        "Channel 0 Pointer contains the 32-bit address, in BP memory, of the ar"
        "ray of channel control blocks starting with the one for channel 0 (the"
        " control channel).",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMABP_INTR.
static const field_t hw_sdmabp_intr[] =
{
    {
        "DI",
        "The BP Interrupts register contains the 32 DI[i] bits.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMABP_STOP_STAT.
static const field_t hw_sdmabp_stop_stat[] =
{
    {
        "DE",
        "This 32-bit register gives access to the BP (DSP) Enable bits, DE.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMABP_DSTART.
static const field_t hw_sdmabp_dstart[] =
{
    {
        "DSTART",
        "The DSTART/DE registers are 32 bits wide with one bit for every channe"
        "l.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMABP_EVTERR.
static const field_t hw_sdmabp_evterr[] =
{
    {
        "CHNERR",
        "This register is used by the SDMA to warn the BP when an incoming DMA "
        "request was detected; it then triggers a channel that is already pendi"
        "ng or being serviced, which may mean there is an overflow of data for "
        "that channel.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMABP_INTRMASK.
static const field_t hw_sdmabp_intrmask[] =
{
    {
        "DIMASK",
        "The Interrupt Mask Register contains 32 interrupt generation mask bits"
        ".",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMABP_EVTERRDBG.
static const field_t hw_sdmabp_evterrdbg[] =
{
    {
        "CHNERR",
        "This register is the same as EVTERR except reading it does not clear i"
        "ts contents.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a SDMABP module.
static const reg_t hw_sdmabp[] =
{
    {
        "DC0PTR",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmabp_dc0ptr
    },
    {
        "INTR",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmabp_intr
    },
    {
        "STOP_STAT",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmabp_stop_stat
    },
    {
        "DSTART",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmabp_dstart
    },
    {
        "EVTERR",
        "",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmabp_evterr
    },
    {
        "INTRMASK",
        "",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmabp_intrmask
    },
    {
        "EVTERRDBG",
        "",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmabp_evterrdbg
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SDMACORE
#endif

// Bitfields in register SDMACORE_MC0PTR.
static const field_t hw_sdmacore_mc0ptr[] =
{
    {
        "MC0PTR",
        "Contains the address-in the ARM platform memory space-of the initial S"
        "DMA context and scripts that are loaded by the SDMA boot script runnin"
        "g on channel 0.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_CCPTR.
static const field_t hw_sdmacore_ccptr[] =
{
    {
        "CCPTR",
        "Contains the start address of the context data for the current channel"
        ": Its value is CONTEXT_BASE + 24* CCR or CONTEXT_BASE + 32* CCR where "
        "CONTEXT_BASE = 0x0800.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_CCR.
static const field_t hw_sdmacore_ccr[] =
{
    {
        "CCR",
        "Contains the number of the current running channel whose context is in"
        "stalled.",
        0, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_NCR.
static const field_t hw_sdmacore_ncr[] =
{
    {
        "NCR",
        "Contains the number of the pending channel that the scheduler has sele"
        "cted to run next.",
        0, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_EVENTS.
static const field_t hw_sdmacore_events[] =
{
    {
        "EVENTS",
        "Reflects the status of the SDMA's external DMA requests.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_CCPRI.
static const field_t hw_sdmacore_ccpri[] =
{
    {
        "CCPRI",
        "Contains the 3-bit priority of the channel whose context is installed.",
        0, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_NCPRI.
static const field_t hw_sdmacore_ncpri[] =
{
    {
        "NCPRI",
        "Contains the 3-bit priority of the channel the scheduler has selected "
        "to run next.",
        0, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_ECOUNT.
static const field_t hw_sdmacore_ecount[] =
{
    {
        "ECOUNT",
        "The event cell counter contains the number of times minus one that an "
        "event detection must occur before generating a debug request.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_ECTL.
static const field_t hw_sdmacore_ectl[] =
{
    {
        "ATS",
        "The access type select bits define the memory access type required on "
        "the SDMA memory bus.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AATC",
        "The Address A Trigger Condition (AATC) controls the operations perform"
        "ed by address comparator A.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ABTC",
        "The Address B Trigger Condition (ABTC) controls the operations perform"
        "ed by address comparator B.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ATC",
        "The address trigger condition bits select how the two address conditio"
        "ns (addressA and addressB) are combined to define the global address m"
        "atching condition.",
        6, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTC",
        "The data trigger condition bits define when data is considered matchin"
        "g after comparison with the data register of the event detection unit.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ECTC",
        "The event cell trigger condition bits select the combination of addres"
        "s and data matching conditions that generate the final address/data co"
        "ndition.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNT",
        "Event Counter Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN",
        "Event Cell Enable.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_EAA.
static const field_t hw_sdmacore_eaa[] =
{
    {
        "EAA",
        "Event Cell Address Register A computes an address A condition.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_EAB.
static const field_t hw_sdmacore_eab[] =
{
    {
        "EAB",
        "Event Cell Address Register B computes an address B condition.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_EAM.
static const field_t hw_sdmacore_eam[] =
{
    {
        "EAM",
        "The Event Cell Address Mask contains a user-defined address mask value"
        ".",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_ED.
static const field_t hw_sdmacore_ed[] =
{
    {
        "ED",
        "The event cell data register contains a user defined data value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_EDM.
static const field_t hw_sdmacore_edm[] =
{
    {
        "EDM",
        "The event cell data mask register contains the user-defined data mask "
        "value.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_RTB.
static const field_t hw_sdmacore_rtb[] =
{
    {
        "RTB",
        "The Real Time Buffer register stores and retrieves run time informatio"
        "n without putting the SDMA in debug mode.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_TB.
static const field_t hw_sdmacore_tb[] =
{
    {
        "CHFADDR",
        "The change of flow address is the address where the change of flow is "
        "taken when executing a change of flow instruction.",
        0, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TADDR",
        "The target address is the address taken after the execution of the cha"
        "nge of flow instruction.",
        14, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TBF",
        "The Trace Buffer Flag is set when the buffer contains the addresses of"
        " a valid change of flow.",
        28, // LSB
        28, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_OSTAT.
static const field_t hw_sdmacore_ostat[] =
{
    {
        "ECDR",
        "Event Cell Debug Request.",
        0, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MST",
        "This flag is raised when the OnCE is controlled from the ARM platform "
        "peripheral interface.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SWB",
        "This flag is raised when the SDMA has entered debug mode after a softw"
        "are breakpoint.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ODR",
        "This flag is raised when the SDMA has entered debug mode after a OnCE "
        "debug request.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EDR",
        "This flag is raised when the SDMA has entered debug mode after an exte"
        "rnal debug request.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RCV",
        "After each write access to the real time buffer (RTB), the RCV bit is "
        "set.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PST",
        "The Processor Status bits reflect the state of the SDMA RISC engine.",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_MCHN0ADDR.
static const field_t hw_sdmacore_mchn0addr[] =
{
    {
        "CHN0ADDR",
        "Contains the address of the channel 0 routine programmed by the ARM pl"
        "atform; it is loaded into a general register at the very start of the "
        "boot and the SDMA jumps to the address it contains.",
        0, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SMSZ",
        "The bit 14 (Scratch Memory Size) determines if scratch memory must be "
        "available after every channel context.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_ENDIANNESS.
static const field_t hw_sdmacore_endianness[] =
{
    {
        "APEND",
        "APEND indicates the endian mode of the Peripheral and Burst DMA interf"
        "aces.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_SDMA_LOCK.
static const field_t hw_sdmacore_sdma_lock[] =
{
    {
        "LOCK",
        "The LOCK bit reports the value of the LOCK bit in the SDMA_LOCK status"
        " register.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SDMACORE_EVENTS2.
static const field_t hw_sdmacore_events2[] =
{
    {
        "EVENTS",
        "Reflects the status of the SDMA's external DMA requests.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a SDMACORE module.
static const reg_t hw_sdmacore[] =
{
    {
        "MC0PTR",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_mc0ptr
    },
    {
        "CCPTR",
        "",
        4, // Width in bytes
        0x00000002, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_ccptr
    },
    {
        "CCR",
        "",
        4, // Width in bytes
        0x00000003, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_ccr
    },
    {
        "NCR",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_ncr
    },
    {
        "EVENTS",
        "This register is very useful in the case of DMA requests that are acti"
        "ve when a peripheral FIFO level is above the programmed watermark.",
        4, // Width in bytes
        0x00000005, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_events
    },
    {
        "CCPRI",
        "",
        4, // Width in bytes
        0x00000006, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_ccpri
    },
    {
        "NCPRI",
        "",
        4, // Width in bytes
        0x00000007, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_ncpri
    },
    {
        "ECOUNT",
        "",
        4, // Width in bytes
        0x00000009, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmacore_ecount
    },
    {
        "ECTL",
        "",
        4, // Width in bytes
        0x0000000a, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_sdmacore_ectl
    },
    {
        "EAA",
        "",
        4, // Width in bytes
        0x0000000b, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmacore_eaa
    },
    {
        "EAB",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmacore_eab
    },
    {
        "EAM",
        "",
        4, // Width in bytes
        0x0000000d, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmacore_eam
    },
    {
        "ED",
        "",
        4, // Width in bytes
        0x0000000e, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmacore_ed
    },
    {
        "EDM",
        "",
        4, // Width in bytes
        0x0000000f, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmacore_edm
    },
    {
        "RTB",
        "",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_sdmacore_rtb
    },
    {
        "TB",
        "",
        4, // Width in bytes
        0x00000019, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_sdmacore_tb
    },
    {
        "OSTAT",
        "",
        4, // Width in bytes
        0x0000001a, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_sdmacore_ostat
    },
    {
        "MCHN0ADDR",
        "",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_sdmacore_mchn0addr
    },
    {
        "ENDIANNESS",
        "",
        4, // Width in bytes
        0x0000001d, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_endianness
    },
    {
        "SDMA_LOCK",
        "",
        4, // Width in bytes
        0x0000001e, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_sdma_lock
    },
    {
        "EVENTS2",
        "",
        4, // Width in bytes
        0x0000001f, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sdmacore_events2
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SJC
#endif

// Bitfields in register SJC_GPUSR1.
static const field_t hw_sjc_gpusr1[] =
{
    {
        "A_DBG",
        "ARM core debug status bit  Bit 0 is the ARM core DBGACK (debug acknowl"
        "edge)  DBGACK can be overwritten in the ARM core DCR to force a partic"
        "ular DBGACK value.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "A_WFI",
        "ARM core wait-for interrupt bit  Bit 1 is the ARM core standbywfi (sta"
        "nd by wait-for interrupt).",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "S_STAT",
        "3 LSBits of SDMA core statusH.",
        2, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PLL_LOCK",
        "PLL_LOCK  A Combined PLL-Lock flag indicator, for all the PLL's.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SJC_GPUSR2.
static const field_t hw_sjc_gpusr2[] =
{
    {
        "STBYWFI",
        "STBYWFI[3:0]  These bits provide status of \"Standby Wait-For-Interrup"
        "t\" state of all ARM cores.",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "S_STAT",
        "S_STAT[3:0]  SDMA debug status bits: debug_core_state[3:0]",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "STBYWFE",
        "STBYWFE[3:0]  Reflecting the \"Standby Wait For Event\" signals of all"
        " cores.",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SJC_GPUSR3.
static const field_t hw_sjc_gpusr3[] =
{
    {
        "IPG_WAIT",
        "IPG_WAIT  CCM's \"ipg_wait\" signal indication",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IPG_STOP",
        "IPG_STOP  CCM's \"ipg_stop\" signal indication",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SYS_WAIT",
        "System In wait  Indication on System in wait mode (from CCM).",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SJC_GPSSR.
static const field_t hw_sjc_gpssr[] =
{
    {
        "GPSSR",
        "General Purpose Secured Status Register  Register is used for testing "
        "and debug.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SJC_DCR.
static const field_t hw_sjc_dcr[] =
{
    {
        "DE_TO_ARM",
        "ARM platform debug request input propagation  This bit controls the pr"
        "opagation of debug request to ARM platform (\"dbgreq\"), when the JTAG"
        " state machine is put in \"ENTER_DEBUG\" IR instruction.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DE_TO_SDMA",
        "SDMA debug request input propagation  This bit controls the propagatio"
        "n of debug request to SDMA, when the JTAG state machine is put in \"EN"
        "TER_DEBUG\" IR instruction..",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEBUG_OBS",
        "Debug observability  This bit controls the propagation of the \"system"
        " debug\" input to SJC  For i.MX 6x, the SJC's \"system_debug\" input i"
        "s tied to logic HIGH value, therefore, set of \"debug_obs\" bit, will "
        "result in unconditional assertion of DE_B pad.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIRECT_SDMA_REQ_EN",
        "Debug enable of the sdma debug request  This bit controls the propagat"
        "ion of debug request DE_B to the sdma.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIRECT_ARM_REQ_EN",
        "Pass Debug Enable event from DE_B pin to ARM platform debug request si"
        "gnal(s).",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SJC_SSR.
static const field_t hw_sjc_ssr[] =
{
    {
        "KTF",
        "Kill Trace Enable fuse value",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "KTA",
        "Kill Trace is active",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SWF",
        "Software JTAG enable fuse  Status of the no SW disable JTAG fuse",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SWE",
        "SW enable  SW JTAG enable status",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EBF",
        "External Boot fuse  Status of the external boot disable fuse",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EBG",
        "External boot granted  External boot enabled, requested and granted",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FT",
        "Fuse type  Fuse type bit - e-fuse or laser fuse",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SJM",
        "SJC Secure mode  Secure JTAG mode, as set by external fuses.",
        9, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RSSTAT",
        "Response status  Response status bits",
        11, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BOOTIND",
        "Boot Indication  Inverted Internal Boot indication, i.e inverse of SRC"
        ": \"src_int_boot\" signal",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SJC_GPCCR.
static const field_t hw_sjc_gpccr[] =
{
    {
        "SCLKR",
        "SDMA Clock ON Register - This bit forces the clock on of the SDMA",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACLKOFFDIS",
        "Disable/prevent ARM platform clock/power shutdown",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a SJC module.
static const reg_t hw_sjc[] =
{
    {
        "GPUSR1",
        "The General Purpose Unsecured Status Register 1 is a read only registe"
        "rs used to check the status of the different Cores and of the PLL.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_sjc_gpusr1
    },
    {
        "GPUSR2",
        "",
        4, // Width in bytes
        0x00000001, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_sjc_gpusr2
    },
    {
        "GPUSR3",
        "",
        4, // Width in bytes
        0x00000002, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_sjc_gpusr3
    },
    {
        "GPSSR",
        "The General Purpose Secured Status Register is a read-only register us"
        "ed to check the status of the different critical information in the So"
        "C.",
        4, // Width in bytes
        0x00000003, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_sjc_gpssr
    },
    {
        "DCR",
        "This register is used to control propagation of debug request from DE_"
        "B pad to the cores and debug signals from internal logic to the DE_B p"
        "ad.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_sjc_dcr
    },
    {
        "SSR",
        "",
        4, // Width in bytes
        0x00000005, // Base address offset
        true, // Readable
        false, // Writable
        10, // Number of bitfields
        hw_sjc_ssr
    },
    {
        "GPCCR",
        "This register is used to configure clock related modes in SOC, see Sys"
        "tem Configuration chapter for more information.",
        4, // Width in bytes
        0x00000007, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_sjc_gpccr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SPBA
#endif

// Bitfields in register SPBA_PRR0.
static const field_t hw_spba_prr0[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR1.
static const field_t hw_spba_prr1[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR2.
static const field_t hw_spba_prr2[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR3.
static const field_t hw_spba_prr3[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR4.
static const field_t hw_spba_prr4[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR5.
static const field_t hw_spba_prr5[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR6.
static const field_t hw_spba_prr6[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR7.
static const field_t hw_spba_prr7[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR8.
static const field_t hw_spba_prr8[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR9.
static const field_t hw_spba_prr9[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR10.
static const field_t hw_spba_prr10[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR11.
static const field_t hw_spba_prr11[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR12.
static const field_t hw_spba_prr12[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR13.
static const field_t hw_spba_prr13[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR14.
static const field_t hw_spba_prr14[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR15.
static const field_t hw_spba_prr15[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR16.
static const field_t hw_spba_prr16[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR17.
static const field_t hw_spba_prr17[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR18.
static const field_t hw_spba_prr18[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR19.
static const field_t hw_spba_prr19[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR20.
static const field_t hw_spba_prr20[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR21.
static const field_t hw_spba_prr21[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR22.
static const field_t hw_spba_prr22[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR23.
static const field_t hw_spba_prr23[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR24.
static const field_t hw_spba_prr24[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR25.
static const field_t hw_spba_prr25[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR26.
static const field_t hw_spba_prr26[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR27.
static const field_t hw_spba_prr27[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR28.
static const field_t hw_spba_prr28[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR29.
static const field_t hw_spba_prr29[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR30.
static const field_t hw_spba_prr30[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPBA_PRR31.
static const field_t hw_spba_prr31[] =
{
    {
        "RARA",
        "Resource Access Right.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARB",
        "Resource Access Right.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RARC",
        "Resource Access Right.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROI",
        "Resource Owner ID.",
        16, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RMO",
        "Requesting Master Owner.",
        30, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a SPBA module.
static const reg_t hw_spba[] =
{
    {
        "PRR0",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr0
    },
    {
        "PRR1",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr1
    },
    {
        "PRR2",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr2
    },
    {
        "PRR3",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr3
    },
    {
        "PRR4",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr4
    },
    {
        "PRR5",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr5
    },
    {
        "PRR6",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr6
    },
    {
        "PRR7",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr7
    },
    {
        "PRR8",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr8
    },
    {
        "PRR9",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr9
    },
    {
        "PRR10",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr10
    },
    {
        "PRR11",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr11
    },
    {
        "PRR12",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr12
    },
    {
        "PRR13",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr13
    },
    {
        "PRR14",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr14
    },
    {
        "PRR15",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr15
    },
    {
        "PRR16",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr16
    },
    {
        "PRR17",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr17
    },
    {
        "PRR18",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr18
    },
    {
        "PRR19",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr19
    },
    {
        "PRR20",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr20
    },
    {
        "PRR21",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000054, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr21
    },
    {
        "PRR22",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr22
    },
    {
        "PRR23",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000005c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr23
    },
    {
        "PRR24",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr24
    },
    {
        "PRR25",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr25
    },
    {
        "PRR26",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000068, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr26
    },
    {
        "PRR27",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000006c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr27
    },
    {
        "PRR28",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr28
    },
    {
        "PRR29",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000074, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr29
    },
    {
        "PRR30",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x00000078, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr30
    },
    {
        "PRR31",
        "This register controls master ownership and access for a peripheral.",
        4, // Width in bytes
        0x0000007c, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spba_prr31
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SPDC
#endif

// Bitfields in register SPDC_DISP_TRIGGER.
static const field_t hw_spdc_disp_trigger[] =
{
    {
        "DISP_TRIG",
        "EPD screen update trigger  When set to \"1\" it will trigger TCON star"
        "t to display image on panel.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPD_MODE",
        "EPD update mode setting  0x0 MODE_0 - It will be displayed with mode 0"
        " waveform  0x1 MODE_1 - It will be displayed with mode 1 waveform  0x2"
        " MODE_2 - It will be displayed with mode 2 waveform  0x3 MODE_3 - It w"
        "ill be displayed with mode 3 waveform  0x4 MODE_4 - It will be display"
        "ed with mode 4 waveform (High Speed Handwriting Mode)  0x5 MODE_5 - It"
        " will be displayed with mode 5 waveform  0x7 MODE_7 - It will be displ"
        "ayed with mode 7 waveform",
        1, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FLASH_SET",
        "Judge EPD screen update will be flashed or not in mode 7  0x0 NO_FLASH"
        " - It will not be flashed screen update in mode 7  0x1 FLASH - It will"
        " be flashed screen update in mode 7",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_UPDATE_X_Y.
static const field_t hw_spdc_update_x_y[] =
{
    {
        "COORDINATE_Y",
        "Vertical coordinate of update region (minimum: 1)  If image have rotat"
        "ed counterclockwise 90  Y should be 4n + 1 format where n is integer",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "COORDINATE_X",
        "Horizontal coordinate of update region (minimum: 1)  X should be 4n + "
        "1 format where n is integer",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_UPDATE_W_H.
static const field_t hw_spdc_update_w_h[] =
{
    {
        "HEIGHT",
        "Height of update region  If image have rotated counterclockwise 90  L "
        "should be 4n format where n is integer",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WIDTH",
        "Width of update region  W should be 4n format where n is integer",
        16, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_LUT_PARA_UPDATE.
static const field_t hw_spdc_lut_para_update[] =
{
    {
        "LUT_PARAMETER_UPDATED_VALUE",
        "The value of update LUT parameter",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_PARAMETER_UPDATED_ADDR",
        "The address of updated LUT parameter",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_OPERATE.
static const field_t hw_spdc_operate[] =
{
    {
        "DISP_REFRESH",
        "Display refresh  When set to \"1\" it will trigger TCON start to refre"
        "sh panel screen base on DDR image data (current frame memory) with mod"
        "e 0 waveform.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEEP_REFRESH",
        "Deep refresh (combine display reset and display refresh function)  Whe"
        "n set to \"1\" it will trigger TCON start to reset panel screen and au"
        "tomatic refresh panel screen base on DDR image data (current frame mem"
        "ory) with mode 0 waveform.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISP_RESET",
        "Display reset  When set to \"1\" it will trigger TCON start to reset p"
        "anel screen and automatic update next/current/previous frame memories "
        "according to flash LUT setting gray level.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_TCON_RESET",
        "Software TCON register reset  When set to \"1\" it will trigger TCON s"
        "tart to reset all internal registers in TCON.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_PANEL_INIT_SET.
static const field_t hw_spdc_panel_init_set[] =
{
    {
        "YOE_POL",
        "Set the YOE signal's polarity of gate driver IC  0x0 LOW_ENABLE -Gate "
        "pulse outputs low enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DUAL_GATE",
        "Set to enable the panel type of dual gate driver  0x0 SINGLE_GATE -Sin"
        "gle gate driver (support gate driver K7900).",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RESOLUTION",
        "Set the panel resolution  GRAY_MODE - Please reference the Table 45-12"
        "  RGBW_MODE -Please reference the Table 45-13",
        2, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UD",
        "Select gate driver IC scanning direction up or down.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RL",
        "Select source driver IC scanning direction right or left.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_FILTER_N",
        "Automatic filter input image pixel data base on the display mode  0x0 "
        "ENABLE_AUTO_DATA_FILTER -Filter input data 16-step gray to 4-step gray"
        " data for mode 2 and mode 3.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "POWER_READY",
        "Driver IC power ready signal  0x0 POWER_NOT_READY - The TCON will be h"
        "old wait for driver IC power turn on finish  0x1 POWER_READY - The TCO"
        "N will into IDLE state and user cloud send the display trigger",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RGBW_MODE_EN",
        "RGBW color mode enable  0x0 DISABLE_RGBW_MODE -It will enable TCON int"
        "o gray mode  0x1 ENABLE_ RGBW_MODE - It will enable TCON into RGBW mod"
        "e",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HBURST_LEN_EN",
        "HBURST length enable  0x0 DISABLE_HBURST - AHB2AXI gasket ignores the "
        "HBURST_LEN[3:0].",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_TEMPER_SETTING.
static const field_t hw_spdc_temper_setting[] =
{
    {
        "TEMPERATURE",
        "Update TCON temperature information  Temperature Information is used t"
        "o indicate the temperature of operating environment.",
        0, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_NEXT_BUF.
static const field_t hw_spdc_next_buf[] =
{
    {
        "NEXT_FRAME_MEMORY_ADDR",
        "Update next frame memory address of DDR memory  The next frame memory "
        "address should be word-align value, it means EPD TCON wills automatic "
        "truncate two bits of LSB.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_CURRENT_BUF.
static const field_t hw_spdc_current_buf[] =
{
    {
        "CURRENT_FRAME_MEMORY_ADDR",
        "Update current frame memory address of DDR memory  The current frame m"
        "emory address should be word-align value, it means EPD TCON wills auto"
        "matic truncate two bits of LSB.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_PREVIOUS_BUFF.
static const field_t hw_spdc_previous_buff[] =
{
    {
        "PREVIOUS_FRAME_MEMORY_ADDR",
        "Update previous frame memory address of DDR memory  The previous frame"
        " memory address should be word-align value, it means EPD TCON wills au"
        "tomatic truncate two bits of LSB.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_FRM_CNT_BUFF.
static const field_t hw_spdc_frm_cnt_buff[] =
{
    {
        "COUNTER_FRAME_MEMORY_ADDR",
        "Update counter frame memory address of DDR memory  The counter frame m"
        "emory address should be word-align value, it means EPD TCON wills auto"
        "matic truncate two bits of LSB.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_LUT_BUFF.
static const field_t hw_spdc_lut_buff[] =
{
    {
        "LUT_MEMORY_ADDR",
        "Update LUT memory address of DDR memory  The LUT memory address should"
        " be word-align value, it means EPD TCON wills automatic truncate two b"
        "its of LSB.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_INT_EN.
static const field_t hw_spdc_int_en[] =
{
    {
        "UPDATE_FINISH_INT_EN",
        "Screen update finish interrupt signal mask  0x0 DISABLE_UPDATE_FINISH_"
        "INTERRUPT - Disable Frame Update Finish Interrupt  0x1 ENABLE_UPDATE_F"
        "INISH_INTERRUPT - Enable Frame Update Finish Interrupt",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCON_INIT_FINISH_INT_EN",
        "TCON initialization finish interrupt signal mask  0x0 DISABLE_TCON_INI"
        "T_FINISH_INTERRUPT - Disable TCON Initialization Finish Interrupt  0x1"
        " ENABLE_TCON_INIT_FINISH_INTERRUPT - Enable TCON Initialization Interr"
        "upt",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_DOWNLOAD_FINISH_INT_EN",
        "LUT initial download finish interrupt signal mask  0x0 DISABLE_LUT_DOW"
        "NLOAD_FINISH_INTERRUPT - Disable LUT initial Download Finish Interrupt"
        "  0x1 ENABLE_LUT_DOWNLOAD_FINISH_INTERRUPT - Enable LUT initial Downlo"
        "ad Finish Interrupt",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCON_ERROR_INT_EN",
        "Error interrupt signal mask  0x0 DISABLE_ERRPR_INTERRUPT - Disable Err"
        "or Interrupt  0x1 ENABLE_ERRPR_INTERRUPT - Enable Error Interrupt",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_INT_ST_CLR.
static const field_t hw_spdc_int_st_clr[] =
{
    {
        "UPDATE_FINISH_INT_CLR",
        "Screen update finish interrupt signal clear  When set to \"1\" it will"
        " trigger TCON start to clear screen update finish interrupt.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCON_INIT_FINISH_INT_CLR",
        "TCON initialization finish interrupt signal clear  When set to \"1\" i"
        "t will trigger TCON start to clear TCON initialization finish interrup"
        "t.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LUT_DOWNLOAD_FINISH_INT_CLR",
        "LUT initial download finish interrupt signal clear  When set to \"1\" "
        "it will trigger TCON start to clear LUT download finish interrupt.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCON_ERROR_INT_CLR",
        "Error interrupt signal clear  When set to \"1\" it will trigger TCON s"
        "tart to clear error interrupt.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_STATUS.
static const field_t hw_spdc_status[] =
{
    {
        "EPD_TCON_BUSY_N",
        "EPD TCON busy status  0x0 BUSY_STATUS - Panel is under updating  0x1 N"
        "OT_BUSY_STATUS - Panel update finished",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LUT_CHECKSUM",
        "LUT checksum status  0x0 CHECKSUM_FAIL - LUT checksum fail  0x1 CHECKS"
        "UM_SUCCESS - LUT checksum pass",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EPD_TCON_STATUS",
        "EPD TCON IP status  0x0 RST_STATUS - EPD TCON during reset phase  0x1 "
        "WAIT_FOR_CHECKSUM_AND_POWER_READY_STATUS - EPD TCON is hold for waitin"
        "g checksum pass and driver IC power ready  0x2 CHECKSUM_FAIL_BUT_POWER"
        "_READY_STATUS - EPD TCON checksum fail but driver IC power ready.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_PANEL_TYPE_VER.
static const field_t hw_spdc_panel_type_ver[] =
{
    {
        "PRODUCT_AND_DRIVER_ID",
        "Product ID and driver ID, they are stand for different type of panel  "
        "0x0 A060SE02 - Panel type (AUO-K7900)  0x1 A090XE01 - Panel type (AUO-"
        "K7900)  Others need to be defined",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LUT_VERSION",
        "LUT version, it stands for different version of driving waveform  0x01"
        "01 V308 - Driving waveform type  0x0102 V312 - Driving waveform type  "
        "Others need to be defined",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EPD_PANEL_TYPE",
        "EPD panel type, it stands for different material type of film.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_TCON_VER.
static const field_t hw_spdc_tcon_ver[] =
{
    {
        "EPD_TCON_VERSION",
        "EPD TCON version, it stands for different version of EPD TCON.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDC_SW_GATE_CLK.
static const field_t hw_spdc_sw_gate_clk[] =
{
    {
        "GATING_ALL_CLK_EN",
        "All clocks gating enable  0x0 ENABLE_ALL_CLOCKS - All clocks free run "
        " 0x1 GATING_ALL_CLOCKS - Gating all clocks for low power",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a SPDC module.
static const reg_t hw_spdc[] =
{
    {
        "DISP_TRIGGER",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_spdc_disp_trigger
    },
    {
        "UPDATE_X_Y",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdc_update_x_y
    },
    {
        "UPDATE_W_H",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdc_update_w_h
    },
    {
        "LUT_PARA_UPDATE",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdc_lut_para_update
    },
    {
        "OPERATE",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_spdc_operate
    },
    {
        "PANEL_INIT_SET",
        "Resolution Mapping Table of Gray Mode        Resolution [4:0] (binary)"
        "  Resolution Setting (Width x Height)  Portrait / Landscape    11000  "
        "Display resolution is 600 x 800.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_spdc_panel_init_set
    },
    {
        "TEMPER_SETTING",
        "Temperature Mapping Table       TEMPERATURE [8:0] (binary)  Thermal Se"
        "nsor Temperature (oC)    110010010  -55    :  :    111111110  -1    11"
        "1111111  -0.5    000000000  0    000000001  0.5    000000010  1    :  "
        ":    001111111  63.5    :  :",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_spdc_temper_setting
    },
    {
        "NEXT_BUF",
        "",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_spdc_next_buf
    },
    {
        "CURRENT_BUF",
        "",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_spdc_current_buf
    },
    {
        "PREVIOUS_BUFF",
        "",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_spdc_previous_buff
    },
    {
        "FRM_CNT_BUFF",
        "",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_spdc_frm_cnt_buff
    },
    {
        "LUT_BUFF",
        "",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_spdc_lut_buff
    },
    {
        "INT_EN",
        "",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_spdc_int_en
    },
    {
        "INT_ST_CLR",
        "",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_spdc_int_st_clr
    },
    {
        "STATUS",
        "",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_spdc_status
    },
    {
        "PANEL_TYPE_VER",
        "",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_spdc_panel_type_ver
    },
    {
        "TCON_VER",
        "",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_spdc_tcon_ver
    },
    {
        "SW_GATE_CLK",
        "",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_spdc_sw_gate_clk
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SPDIF
#endif

// Bitfields in register SPDIF_SCR.
static const field_t hw_spdif_scr[] =
{
    {
        "USRC_SEL",
        "",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXSEL",
        "",
        2, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALCTRL",
        "",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_TX_EN",
        "DMA Transmit Request Enable  (Tx FIFO empty)",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMA_RX_EN",
        "DMA Receive Request Enable  (RX FIFO full)",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXFIFO_CTRL",
        "",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SOFT_RESET",
        "When write 1 to this bit, it will cause SPDIF software reset.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOW_POWER",
        "When write 1 to this bit, it will cause SPDIF enter low-power mode.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXFIFOEMPTY_SEL",
        "",
        15, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXAUTOSYNC",
        "",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXAUTOSYNC",
        "",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFIFOFULL_SEL",
        "",
        19, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFIFO_RST",
        "",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFIFO_OFF",
        "",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFIFO_CTRL",
        "",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRCD.
static const field_t hw_spdif_srcd[] =
{
    {
        "USYNCMODE",
        "",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRPC.
static const field_t hw_spdif_srpc[] =
{
    {
        "GAINSEL",
        "Gain selection:",
        3, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "LOCK bit to show that the internal DPLL is locked, read only",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CLKSRC_SEL",
        "Clock source selection, all other settings not shown are reserved:",
        7, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SIE.
static const field_t hw_spdif_sie[] =
{
    {
        "RXFIFOFUL",
        "SPDIF Rx FIFO full, can't be cleared with reg.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXEM",
        "SPDIF Tx FIFO empty, can't be cleared with reg.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCKLOSS",
        "SPDIF receiver loss of lock",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFIFORESYN",
        "Rx FIFO resync",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFIFOUNOV",
        "Rx FIFO underrun/overrun",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UQERR",
        "U/Q Channel framing error",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UQSYNC",
        "U/Q Channel sync found",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "QRXOV",
        "Q Channel receive register overrun",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "QRXFUL",
        "Q Channel receive register full, can't be cleared with reg.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URXOV",
        "U Channel receive register overrun",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URXFUL",
        "U Channel receive register full, can't be cleared with reg.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BITERR",
        "SPDIF receiver found parity bit error",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SYMERR",
        "SPDIF receiver found illegal symbol",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VALNOGOOD",
        "SPDIF validity flag no good",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CNEW",
        "SPDIF receive change in value of control channel",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXRESYN",
        "SPDIF Tx FIFO resync",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXUNOV",
        "SPDIF Tx FIFO under/overrun",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOCK",
        "SPDIF receiver's DPLL is locked",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SIS.
static const field_t hw_spdif_sis[] =
{
    {
        "RXFIFOFUL",
        "SPDIF Rx FIFO full, can't be cleared with reg.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXEM",
        "SPDIF Tx FIFO empty, can't be cleared with reg.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LOCKLOSS",
        "SPDIF receiver loss of lock",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RXFIFORESYN",
        "Rx FIFO resync",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RXFIFOUNOV",
        "Rx FIFO underrun/overrun",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UQERR",
        "U/Q Channel framing error",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UQSYNC",
        "U/Q Channel sync found",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "QRXOV",
        "Q Channel receive register overrun",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "QRXFUL",
        "Q Channel receive register full, can't be cleared with reg.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "URXOV",
        "U Channel receive register overrun",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "URXFUL",
        "U Channel receive register full, can't be cleared with reg.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BITERR",
        "SPDIF receiver found parity bit error",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SYMERR",
        "SPDIF receiver found illegal symbol",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VALNOGOOD",
        "SPDIF validity flag no good",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CNEW",
        "SPDIF receive change in value of control channel",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXRESYN",
        "SPDIF Tx FIFO resync",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXUNOV",
        "SPDIF Tx FIFO under/overrun",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LOCK",
        "SPDIF receiver's DPLL is locked",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRL.
static const field_t hw_spdif_srl[] =
{
    {
        "RXDATALEFT",
        "Processor receive SPDIF data left",
        0, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRR.
static const field_t hw_spdif_srr[] =
{
    {
        "RXDATARIGHT",
        "Processor receive SPDIF data right",
        0, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRCSH.
static const field_t hw_spdif_srcsh[] =
{
    {
        "RXCCHANNEL_H",
        "SPDIF receive C channel register, contains first 24 bits of C channel "
        "without interpretation",
        0, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRCSL.
static const field_t hw_spdif_srcsl[] =
{
    {
        "RXCCHANNEL_L",
        "SPDIF receive C channel register, contains next 24 bits of C channel w"
        "ithout interpretation",
        0, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRU.
static const field_t hw_spdif_sru[] =
{
    {
        "RXUCHANNEL",
        "SPDIF receive U channel register, contains next 3 U channel bytes",
        0, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "This is a 24-bit register the upper byte is unimplemented.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRQ.
static const field_t hw_spdif_srq[] =
{
    {
        "RXQCHANNEL",
        "SPDIF receive Q channel register, contains next 3 Q channel bytes",
        0, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_STL.
static const field_t hw_spdif_stl[] =
{
    {
        "TXDATALEFT",
        "SPDIF transmit left channel data.",
        0, // LSB
        23, // MSB
        false, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "This is a 24-bit register the upper byte is unimplemented.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_STR.
static const field_t hw_spdif_str[] =
{
    {
        "TXDATARIGHT",
        "SPDIF transmit right channel data.",
        0, // LSB
        23, // MSB
        false, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "This is a 24-bit register the upper byte is unimplemented.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_STCSCH.
static const field_t hw_spdif_stcsch[] =
{
    {
        "TXCCHANNELCONS_H",
        "SPDIF transmit Cons.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_STCSCL.
static const field_t hw_spdif_stcscl[] =
{
    {
        "TXCCHANNELCONS_L",
        "SPDIF transmit Cons.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_SRFM.
static const field_t hw_spdif_srfm[] =
{
    {
        "FREQMEAS",
        "Frequency measurement data",
        0, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SPDIF_STC.
static const field_t hw_spdif_stc[] =
{
    {
        "TXCLK_DF",
        "Divider factor (1-128)",
        0, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TX_ALL_CLK_EN",
        "Spdif transfer clock enable.When data is going to be transfered, this "
        "bit should be set to1.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXCLK_SOURCE",
        "",
        8, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SYSCLK_DF",
        "system clock divider factor, 2~512.",
        11, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UNIMPLEMENTED",
        "",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a SPDIF module.
static const reg_t hw_spdif[] =
{
    {
        "SCR",
        "",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_spdif_scr
    },
    {
        "SRCD",
        "",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdif_srcd
    },
    {
        "SRPC",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_spdif_srpc
    },
    {
        "SIE",
        "The InterruptEn register (SPDIF_SIE) provides control over the enablin"
        "g of interrupts.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        19, // Number of bitfields
        hw_spdif_sie
    },
    {
        "SIS",
        "The InterruptStat (SPDIF_SIS) register is a read only register that pr"
        "ovides the status on interrupt operations.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        false, // Writable
        19, // Number of bitfields
        hw_spdif_sis
    },
    {
        "SRL",
        "SPDIFRxLeft register is an audio data reception register.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_spdif_srl
    },
    {
        "SRR",
        "SPDIFRxRight register is an audio data reception register.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_spdif_srr
    },
    {
        "SRCSH",
        "SPDIFRxCChannel_h register is a channel status reception register.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_spdif_srcsh
    },
    {
        "SRCSL",
        "SPDIFRxCChannel_l register is a channel status reception register.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_spdif_srcsl
    },
    {
        "SRU",
        "UChannelRx register is a user bits reception register.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_spdif_sru
    },
    {
        "SRQ",
        "QChannelRx register is a user bits reception register.",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_spdif_srq
    },
    {
        "STL",
        "SPDIFTxLeft register is an audio data transmission register.",
        4, // Width in bytes
        0x0000002c, // Base address offset
        false, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdif_stl
    },
    {
        "STR",
        "SPDIFTxRight register is an audio data transmission register.",
        4, // Width in bytes
        0x00000030, // Base address offset
        false, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdif_str
    },
    {
        "STCSCH",
        "SPDIFTxCChannelCons_h register is a channel status transmission regist"
        "er.",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdif_stcsch
    },
    {
        "STCSCL",
        "SPDIFTxCChannelCons_l register is a channel status transmission regist"
        "er.",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_spdif_stcscl
    },
    {
        "SRFM",
        "",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_spdif_srfm
    },
    {
        "STC",
        "The SPDIFTxClk Control register includes the means to select the trans"
        "mit clock and frequency division.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_spdif_stc
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SRC
#endif

// Bitfields in register SRC_SCR.
static const field_t hw_src_scr[] =
{
    {
        "WARM_RESET_ENABLE",
        "WARM reset enable bit.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_GPU_RST",
        "Software reset for gpu  this is a self clearing bit.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_VPU_RST",
        "Software reset for vpu  this is a self clearing bit.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SW_OPEN_VG_RST",
        "Software reset for open_vg  This is a self clearing bit.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WARM_RST_BYPASS_COUNT",
        "Defines the ckil cycles to count before bypassing the MMDC acknowledge"
        " for WARM reset.",
        5, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_WDOG_RST",
        "Mask wdog_rst_b source.",
        7, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EIM_RST",
        "EIM reset is needed in order to reconfigure the eim chip select.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CORE0_RST",
        "Software reset for core0 only.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CORE0_DBG_RST",
        "Software reset for core0 debug only.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CORES_DBG_RST",
        "Software reset for debug of arm platform only.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DBG_RST_MSK_PG",
        "Do not assert debug resets after power gating event of cpu",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SRC_SBMR1.
static const field_t hw_src_sbmr1[] =
{
    {
        "BOOT_CFG1",
        "Please refer to fuse map.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BOOT_CFG2",
        "Please refer to fuse map.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BOOT_CFG3",
        "Please refer to fuse map.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BOOT_CFG4",
        "Please refer to fuse map.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SRC_SRSR.
static const field_t hw_src_srsr[] =
{
    {
        "IPP_RESET_B",
        "Indicates whether reset was the result of ipp_reset_b pin (Power-up se"
        "quence)",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CSU_RESET_B",
        "Indicates whether the reset was the result of the csu_reset_b input.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IPP_USER_RESET_B",
        "Indicates whether the reset was the result of the ipp_user_reset_b qua"
        "lified reset.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDOG_RST_B",
        "IC Watchdog Time-out reset.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "JTAG_RST_B",
        "HIGH - Z JTAG reset.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "JTAG_SW_RST",
        "JTAG SW reset.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WARM_BOOT",
        "WARM boot indication shows that WARM boot was initiated by software.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SRC_SISR.
static const field_t hw_src_sisr[] =
{
    {
        "GPU_PASSED_RESET",
        "Interrupt generated to indicate that gpu passed software reset and is "
        "ready to be used",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VPU_PASSED_RESET",
        "Interrupt generated to indicate that vpu passed software reset and is "
        "ready to be used",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OPEN_VG_PASSED_RESET",
        "Interrupt generated to indicate that open_vg passed software reset and"
        " is ready to be used",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CORE0_WDOG_RST_REQ",
        "WDOG reset request from CPU core0.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SRC_SIMR.
static const field_t hw_src_simr[] =
{
    {
        "MASK_GPU_PASSED_RESET",
        "mask interrupt generation due to gpu passed reset",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_VPU_PASSED_RESET",
        "mask interrupt generation due to vpu passed reset",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MASK_OPEN_VG_PASSED_RESET",
        "mask interrupt generation due to open_vg passed reset",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SRC_SBMR2.
static const field_t hw_src_sbmr2[] =
{
    {
        "SEC_CONFIG",
        "Please refer to fuse map.",
        0, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DIR_BT_DIS",
        "Please refer to fuse map.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BT_FUSE_SEL",
        "BT_FUSE_SEL (connected to gpio bt_fuse_sel)",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BMOD",
        "Please refer to fuse map.",
        24, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TEST_MODE",
        "Please refer to fuse map.",
        27, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a SRC module.
static const reg_t hw_src[] =
{
    {
        "SCR",
        "The Reset control register (SCR), contains bits that control operation"
        " of the reset controller.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_src_scr
    },
    {
        "SBMR1",
        "The Boot Mode register (SBMR) contains bits that reflect the status of"
        " Boot Mode Pins of the chip.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_src_sbmr1
    },
    {
        "SRSR",
        "The SRSR is a write to one clear register which records the source of "
        "the reset events for the chip.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_src_srsr
    },
    {
        "SISR",
        "",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_src_sisr
    },
    {
        "SIMR",
        "",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_src_simr
    },
    {
        "SBMR2",
        "The Boot Mode register (SBMR), contains bits that reflect the status o"
        "f Boot Mode Pins of the chip.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        false, // Writable
        5, // Number of bitfields
        hw_src_sbmr2
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark SSI
#endif

// Bitfields in register SSI_SSI_STX0.
static const field_t hw_ssi_ssi_stx0[] =
{
    {
        "STXN",
        "SSI Transmit Data.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_STX1.
static const field_t hw_ssi_ssi_stx1[] =
{
    {
        "STXN",
        "SSI Transmit Data.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SRX0.
static const field_t hw_ssi_ssi_srx0[] =
{
    {
        "SRXN",
        "SSI Receive Data.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SRX1.
static const field_t hw_ssi_ssi_srx1[] =
{
    {
        "SRXN",
        "SSI Receive Data.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SCR.
static const field_t hw_ssi_ssi_scr[] =
{
    {
        "SSIEN",
        "SSIEN - SSI Enable  This bit is used to enable/disable the SSI.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TE",
        "Transmit Enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RE",
        "Receive Enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NET",
        "Network Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SYN",
        "Synchronous Mode.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "I2S_MODE",
        "I2S Mode Select.",
        5, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SYS_CLK_EN",
        "Network Clock (Oversampling Clock) Enable.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCH_EN",
        "Two-Channel Operation Enable.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLK_IST",
        "Clock Idle State.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFR_CLK_DIS",
        "Transmit Frame Clock Disable.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFR_CLK_DIS",
        "Receive Frame Clock Disable.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SYNC_TX_FS",
        "SYNC_FS_TX bit provides a safe window for TE to be visible to the inte"
        "rnal circuit which is just after FS occurrence.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SISR.
static const field_t hw_ssi_ssi_sisr[] =
{
    {
        "TFE0",
        "Transmit FIFO Empty 0.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TFE1",
        "Transmit FIFO Empty 1.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RFF0",
        "Receive FIFO Full 0.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RFF1",
        "Receive FIFO Full 1.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RLS",
        "Receive Last Time Slot.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TLS",
        "Transmit Last Time Slot.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RFS",
        "Receive Frame Sync.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TFS",
        "Transmit Frame Sync.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TUE0",
        "Transmitter Underrun Error 0.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TUE1",
        "Transmitter Underrun Error 1.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROE0",
        "Receiver Overrun Error 0.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ROE1",
        "Receiver Overrun Error 1.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TDE0",
        "Transmit Data Register Empty 0.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TDE1",
        "Transmit Data Register Empty 1.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RDR0",
        "Receive Data Ready 0.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RDR1",
        "Receive Data Ready 1.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RXT",
        "Receive Tag Updated.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CMDDU",
        "Command Data Register Updated.",
        17, // LSB
        17, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CMDAU",
        "Command Address Register Updated.",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TFRC",
        "Transmit Frame Complete.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RFRC",
        "Receive Frame Complete.",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SIER.
static const field_t hw_ssi_sier[] =
{
    {
        "ENABLE_BITS1",
        "Enable Bit.",
        0, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIE",
        "Transmit Interrupt Enable.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TDMAE",
        "Transmit DMA Enable.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RIE",
        "Receive Interrupt Enable.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RDMAE",
        "Receive DMA Enable.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENABLE_BITS",
        "Enable Bit.",
        23, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_STCR.
static const field_t hw_ssi_ssi_stcr[] =
{
    {
        "TEFS",
        "Transmit Early Frame Sync.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSL",
        "Transmit Frame Sync Length.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFSI",
        "Transmit Frame Sync Invert.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TSCKP",
        "Transmit Clock Polarity.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TSHFD",
        "Transmit Shift Direction.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXDIR",
        "Transmit Clock Direction.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFDIR",
        "Transmit Frame Direction.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFEN0",
        "Transmit FIFO Enable 0.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFEN1",
        "Transmit FIFO Enable 1.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXBIT0",
        "Transmit Bit 0.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SRCR.
static const field_t hw_ssi_ssi_srcr[] =
{
    {
        "REFS",
        "Receive Early Frame Sync.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSL",
        "Receive Frame Sync Length.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFSI",
        "Receive Frame Sync Invert.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RSCKP",
        "Receive Clock Polarity.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RSHFD",
        "Receive Shift Direction.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDIR",
        "Receive Clock Direction.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFDIR",
        "Receive Frame Direction.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFEN0",
        "Receive FIFO Enable 0.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFEN1",
        "Receive FIFO Enable 1.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXBIT0",
        "Receive Bit 0.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXEXT",
        "Receive Data Extension.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_STCCR.
static const field_t hw_ssi_ssi_stccr[] =
{
    {
        "PM7_PM0",
        "Prescaler Modulus Select.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DC4_DC0",
        "Frame Rate Divider Control.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL3_WL0",
        "Word Length Control.",
        13, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSR",
        "Prescaler Range.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIV2",
        "Divide By 2.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SRCCR.
static const field_t hw_ssi_srccr[] =
{
    {
        "PM7_PM0",
        "Prescaler Modulus Select.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DC4_DC0",
        "Frame Rate Divider Control.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WL3_WL0",
        "Word Length Control.",
        13, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSR",
        "Prescaler Range.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DIV2",
        "Divide By 2.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SFCSR.
static const field_t hw_ssi_ssi_sfcsr[] =
{
    {
        "TFWM0",
        "Transmit FIFO Empty WaterMark 0.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFWM0",
        "Receive FIFO Full WaterMark 0.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFCNT0",
        "Transmit FIFO Counter 0.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFCNT0",
        "Receive FIFO Counter 0.",
        12, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFWM1",
        "Transmit FIFO Empty WaterMark 1.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFWM1",
        "Receive FIFO Full WaterMark 1.",
        20, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TFCNT1",
        "Transmit FIFO Counter1.",
        24, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFCNT1",
        "Receive FIFO Counter1.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SACNT.
static const field_t hw_ssi_ssi_sacnt[] =
{
    {
        "AC97EN",
        "AC97 Mode Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FV",
        "Fixed/Variable Operation.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIF",
        "Tag in FIFO.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD",
        "Read Command.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR",
        "Write Command.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRDIV",
        "Frame Rate Divider.",
        5, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SACADD.
static const field_t hw_ssi_ssi_sacadd[] =
{
    {
        "SACADD",
        "AC97 Command Address.",
        0, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SACDAT.
static const field_t hw_ssi_ssi_sacdat[] =
{
    {
        "SACDAT",
        "AC97 Command Data.",
        0, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SATAG.
static const field_t hw_ssi_satag[] =
{
    {
        "SATAG",
        "AC97 Tag Value.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_STMSK.
static const field_t hw_ssi_ssi_stmsk[] =
{
    {
        "STMSK",
        "Transmit Mask.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SRMSK.
static const field_t hw_ssi_ssi_srmsk[] =
{
    {
        "SRMSK",
        "Receive Mask.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SACCST.
static const field_t hw_ssi_ssi_saccst[] =
{
    {
        "SACCST",
        "AC97 Channel Status.",
        0, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SACCEN.
static const field_t hw_ssi_ssi_saccen[] =
{
    {
        "SACCEN",
        "AC97 Channel Enable.",
        0, // LSB
        9, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register SSI_SSI_SACCDIS.
static const field_t hw_ssi_ssi_saccdis[] =
{
    {
        "SACCDIS",
        "AC97 Channel Disable.",
        0, // LSB
        9, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a SSI module.
static const reg_t hw_ssi[] =
{
    {
        "SSI_STX0",
        "Enable SSI (SSIEN=1) before writing to SSI Transmit Data Registers.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_stx0
    },
    {
        "SSI_STX1",
        "Enable SSI (SSIEN=1) before writing to SSI Transmit Data Registers.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_stx1
    },
    {
        "SSI_SRX0",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_srx0
    },
    {
        "SSI_SRX1",
        "",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_srx1
    },
    {
        "SSI_SCR",
        "The SSI Control Register (SSI_SCR) sets up the SSI reset is controlled"
        " by bit 0 in the SSI_SCR.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_ssi_ssi_scr
    },
    {
        "SSI_SISR",
        "The SSI Interrupt Status Register (SSI_SISR) is used to monitor the SS"
        "I.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        true, // Writable
        21, // Number of bitfields
        hw_ssi_ssi_sisr
    },
    {
        "SIER",
        "The SSI Interrupt Enable Register (SIER) is a 25-bit register used to "
        "set up the SSI interrupts and DMA requests.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_ssi_sier
    },
    {
        "SSI_STCR",
        "The SSI Transmit Configuration Register (SSI_STCR) is a read/write con"
        "trol register used to direct the transmit operation of the STCR contro"
        "ls the direction of the bit clock and frame sync ports, STCK and STFS.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_ssi_ssi_stcr
    },
    {
        "SSI_SRCR",
        "The SSI Receive Configuration Register (SSI_SRCR) is a read/write cont"
        "rol register used to direct the receive operation of the SSI.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_ssi_ssi_srcr
    },
    {
        "SSI_STCCR",
        "The SSI Transmit and Receive Control (SSI_STCCR and SSI_SRCCR) registe"
        "rs are 19-bit, read/write control registers used to direct the operati"
        "on of the SSI.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_ssi_ssi_stccr
    },
    {
        "SRCCR",
        "The SSI Transmit and Receive Control (SSI_STCCR and SSI_SRCCR) registe"
        "rs are 19-bit, read/write control registers used to direct the operati"
        "on of the SSI.",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_ssi_srccr
    },
    {
        "SSI_SFCSR",
        "The SSI FIFO Control / Status Register indicates the status of the Tra"
        "nsmit FIFO Empty flag, with different settings of the Transmit FIFO Wa"
        "terMark bits and varying amounts of data in the Tx FIFO  .",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_ssi_ssi_sfcsr
    },
    {
        "SSI_SACNT",
        "",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_ssi_ssi_sacnt
    },
    {
        "SSI_SACADD",
        "",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_sacadd
    },
    {
        "SSI_SACDAT",
        "",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_sacdat
    },
    {
        "SATAG",
        "",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_satag
    },
    {
        "SSI_STMSK",
        "",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_stmsk
    },
    {
        "SSI_SRMSK",
        "",
        4, // Width in bytes
        0x0000004c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_srmsk
    },
    {
        "SSI_SACCST",
        "",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_saccst
    },
    {
        "SSI_SACCEN",
        "",
        4, // Width in bytes
        0x00000054, // Base address offset
        false, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_saccen
    },
    {
        "SSI_SACCDIS",
        "",
        4, // Width in bytes
        0x00000058, // Base address offset
        false, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_ssi_ssi_saccdis
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark TEMPMON
#endif

// Bitfields in register TEMPMON_TEMPSENSE0.
static const field_t hw_tempmon_tempsense0[] =
{
    {
        "POWER_DOWN",
        "This bit powers down the temperature sensor.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MEASURE_TEMP",
        "Starts the measurement process.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FINISHED",
        "Indicates that the latest temp is valid.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TEMP_CNT",
        "This bit field contains the last measured temperature count.",
        8, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ALARM_VALUE",
        "This bit field contains the temperature count (raw sensor output) that"
        " will generate an alarm interrupt.",
        20, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register TEMPMON_TEMPSENSE1.
static const field_t hw_tempmon_tempsense1[] =
{
    {
        "MEASURE_FREQ",
        "This bits determines how many RTC clocks to wait before automatically "
        "repeating a temperature measurement.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a TEMPMON module.
static const reg_t hw_tempmon[] =
{
    {
        "TEMPSENSE0",
        "This register defines the basic controls for the temperature sensor mi"
        "nus the frequency of automatic sampling which is defined in the tempse"
        "nsor.",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_tempmon_tempsense0
    },
    {
        "TEMPSENSE1",
        "This register defines the automatic repeat time of the temperature sen"
        "sor.",
        4, // Width in bytes
        0x00000190, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_tempmon_tempsense1
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark UART
#endif

// Bitfields in register UART_URXD.
static const field_t hw_uart_urxd[] =
{
    {
        "RX_DATA",
        "Received Data .",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PRERR",
        "In RS-485 mode, it holds the ninth data bit (bit [8]) of received 9-bi"
        "t RS-485 data In RS232/IrDA mode, it is the Parity Error flag .",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BRK",
        "BREAK Detect.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FRMERR",
        "Frame Error.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OVRRUN",
        "Receiver Overrun.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ERR",
        "Error Detect.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CHARRDY",
        "Character Ready.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UTXD.
static const field_t hw_uart_utxd[] =
{
    {
        "TX_DATA",
        "Transmit Data .",
        0, // LSB
        7, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UCR1.
static const field_t hw_uart_ucr1[] =
{
    {
        "UARTEN",
        "UART Enable .",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DOZE",
        "DOZE .",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ATDMAEN",
        "Aging DMA Timer Enable .",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXDMAEN",
        "Transmitter Ready DMA Enable .",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SNDBRK",
        "Send BREAK .",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTSDEN",
        "RTS Delta Interrupt Enable .",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXMPTYEN",
        "Transmitter Empty Interrupt Enable .",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IREN",
        "Infrared Interface Enable .",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDMAEN",
        "Receive Ready DMA Enable .",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RRDYEN",
        "Receiver Ready Interrupt Enable .",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ICD",
        "Idle Condition Detect .",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDEN",
        "Idle Condition Detected Interrupt Enable .",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRDYEN",
        "Transmitter Ready Interrupt Enable .",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADBR",
        "Automatic Detection of Baud Rate .",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADEN",
        "Automatic Baud Rate Detection Interrupt Enable .",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UCR2.
static const field_t hw_uart_ucr2[] =
{
    {
        "SRST",
        "Software Reset .",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXEN",
        "Receiver Enable .",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXEN",
        "Transmitter Enable .",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ATEN",
        "Aging Timer Enable .",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTSEN",
        "Request to Send Interrupt Enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WS",
        "Word Size .",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STPB",
        "Stop .",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PROE",
        "Parity Odd/Even .",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PREN",
        "Parity Enable .",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTEC",
        "Request to Send Edge Control .",
        9, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ESCEN",
        "Escape Enable .",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CTS",
        "Clear to Send .",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CTSC",
        "CTS Pin Control .",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRTS",
        "Ignore RTS Pin .",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ESCI",
        "Escape Sequence Interrupt Enable .",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UCR3.
static const field_t hw_uart_ucr3[] =
{
    {
        "ACIEN",
        "Autobaud Counter Interrupt Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INVT",
        "Invert TXD output in RS-232/RS-485 mode, set TXD active level in IrDA "
        "mode.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDMUXSEL",
        "RXD Muxed Input Selected.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTRDEN",
        "Data Terminal Ready Delta Enable .",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AWAKEN",
        "Asynchronous WAKE Interrupt Enable.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AIRINTEN",
        "Asynchronous IR WAKE Interrupt Enable.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDSEN",
        "Receive Status Interrupt Enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ADNIMP",
        "Autobaud Detection Not Improved-.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RI",
        "Ring Indicator .",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCD",
        "Data Carrier Detect .",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DSR",
        "Data Set Ready .",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRAERREN",
        "Frame Error Interrupt Enable.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PARERREN",
        "Parity Error Interrupt Enable.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTREN",
        "Data Terminal Ready Interrupt Enable .",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DPEC",
        "DTR/DSR Interrupt Edge Control .",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UCR4.
static const field_t hw_uart_ucr4[] =
{
    {
        "DREN",
        "Receive Data Ready Interrupt Enable .",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OREN",
        "Receiver Overrun Interrupt Enable .",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BKEN",
        "BREAK Condition Detected Interrupt Enable .",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCEN",
        "Transmit Complete Interrupt Enable .",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPBYP",
        "Low Power Bypass .",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRSC",
        "IR Special Case .",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDDMAEN",
        "DMA IDLE Condition Detected Interrupt Enable Enables/Disables the rece"
        "ive DMA request dma_req_rx for the IDLE interrupt (triggered with IDLE"
        " flag in USR2[12]).",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKEN",
        "WAKE Interrupt Enable .",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENIRI",
        "Serial Infrared Interrupt Enable .",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INVR",
        "Invert RXD input in RS-232/RS-485 Mode, d etermine RXD input logic lev"
        "el being sampled in In IrDA mode.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CTSTL",
        "CTS Trigger Level .",
        10, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UFCR.
static const field_t hw_uart_ufcr[] =
{
    {
        "RXTL",
        "Receiver Trigger Level .",
        0, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCEDTE",
        "DCE/DTE mode select .",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RFDIV",
        "Reference Frequency Divider.",
        7, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXTL",
        "Transmitter Trigger Level .",
        10, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_USR1.
static const field_t hw_uart_usr1[] =
{
    {
        "SAD",
        "RS-485 Slave Address Detected Interrupt Flag.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AWAKE",
        "Asynchronous WAKE Interrupt Flag.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AIRINT",
        "Asynchronous IR WAKE Interrupt Flag.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDS",
        "Receiver IDLE Interrupt Flag.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DTRD",
        "DTR Delta.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AGTIM",
        "Ageing Timer Interrupt Flag.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RRDY",
        "Receiver Ready Interrupt / DMA Flag .",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "FRAMERR",
        "Frame Error Interrupt Flag .",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ESCF",
        "Escape Sequence Interrupt Flag .",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTSD",
        "RTS Delta.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TRDY",
        "Transmitter Ready Interrupt / DMA Flag .",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RTSS",
        "RTS Pin Status .",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PARITYERR",
        "Parity Error Interrupt Flag .",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_USR2.
static const field_t hw_uart_usr2[] =
{
    {
        "RDR",
        "Receive Data Ready -Indicates that at least 1 character is received an"
        "d written to the RxFIFO.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ORE",
        "Overrun Error .",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BRCD",
        "BREAK Condition Detected .",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXDC",
        "Transmitter Complete .",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RTSF",
        "RTS Edge Triggered Interrupt Flag.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCDIN",
        "Data Carrier Detect Input .",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DCDDELT",
        "Data Carrier Detect Delta .",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAKE",
        "Wake .",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IRINT",
        "Serial Infrared Interrupt Flag.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RIIN",
        "Ring Indicator Input .",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RIDELT",
        "Ring Indicator Delta .",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ACST",
        "Autobaud Counter Stopped .",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDLE",
        "Idle Condition .",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTRF",
        "DTR edge triggered interrupt flag .",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXFE",
        "Transmit Buffer FIFO Empty .",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ADET",
        "Automatic Baud Rate Detect Complete .",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UESC.
static const field_t hw_uart_uesc[] =
{
    {
        "ESC_CHAR",
        "UART Escape Character .",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UTIM.
static const field_t hw_uart_utim[] =
{
    {
        "TIM",
        "UART Escape Timer.",
        0, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UBIR.
static const field_t hw_uart_ubir[] =
{
    {
        "INC",
        "Incremental Numerator.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UBMR.
static const field_t hw_uart_ubmr[] =
{
    {
        "MOD",
        "Modulator Denominator.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UBRC.
static const field_t hw_uart_ubrc[] =
{
    {
        "BCNT",
        "Baud Rate Count Register.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_ONEMS.
static const field_t hw_uart_onems[] =
{
    {
        "ONEMS",
        "One Millisecond Register.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UTS.
static const field_t hw_uart_uts[] =
{
    {
        "SOFTRST",
        "Software Reset.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXFULL",
        "RxFIFO FULL.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXFULL",
        "TxFIFO FULL.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXEMPTY",
        "RxFIFO Empty.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXEMPTY",
        "TxFIFO Empty.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDBG",
        "RX_fifo_debug_mode.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOOPIR",
        "Loop TX and RX for IR Test (LOOPIR) .",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DBGEN",
        "debug_enable .",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LOOP",
        "Loop TX and RX for Test.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRCPERR",
        "Force Parity Error.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register UART_UMCR.
static const field_t hw_uart_umcr[] =
{
    {
        "MDEN",
        "9-bit data or Multidrop Mode (RS-485) Enable.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLAM",
        "RS-485 Slave Address Detect Mode Selection.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXB8",
        "Transmit RS-485 bit 8 (the ninth bit or 9 th bit).",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SADEN",
        "RS-485 Slave Address Detected Interrupt Enable.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLADDR",
        "RS-485 Slave Address Character.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a UART module.
static const reg_t hw_uart[] =
{
    {
        "URXD",
        "The UART will yield a transfer error on the peripheral bus when core i"
        "s reading URXD register with receive interface disabled (RXEN=0 or UAR"
        "TEN=0).",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_uart_urxd
    },
    {
        "UTXD",
        "The UART will yield a transfer error on the peripheral bus when core i"
        "s writing into UART_URXD register with transmit interface disabled (TX"
        "EN=0 or UARTEN=0).",
        4, // Width in bytes
        0x00000040, // Base address offset
        false, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_uart_utxd
    },
    {
        "UCR1",
        "",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_uart_ucr1
    },
    {
        "UCR2",
        "",
        4, // Width in bytes
        0x00000084, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_uart_ucr2
    },
    {
        "UCR3",
        "",
        4, // Width in bytes
        0x00000088, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_uart_ucr3
    },
    {
        "UCR4",
        "",
        4, // Width in bytes
        0x0000008c, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_uart_ucr4
    },
    {
        "UFCR",
        "",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_uart_ufcr
    },
    {
        "USR1",
        "",
        4, // Width in bytes
        0x00000094, // Base address offset
        true, // Readable
        true, // Writable
        13, // Number of bitfields
        hw_uart_usr1
    },
    {
        "USR2",
        "",
        4, // Width in bytes
        0x00000098, // Base address offset
        true, // Readable
        true, // Writable
        16, // Number of bitfields
        hw_uart_usr2
    },
    {
        "UESC",
        "",
        4, // Width in bytes
        0x0000009c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_uart_uesc
    },
    {
        "UTIM",
        "",
        4, // Width in bytes
        0x000000a0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_uart_utim
    },
    {
        "UBIR",
        "This register can be written by both software and hardware.",
        4, // Width in bytes
        0x000000a4, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_uart_ubir
    },
    {
        "UBMR",
        "This register can be written by both software and hardware.",
        4, // Width in bytes
        0x000000a8, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_uart_ubmr
    },
    {
        "UBRC",
        "",
        4, // Width in bytes
        0x000000ac, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_uart_ubrc
    },
    {
        "ONEMS",
        "This register has been expanded from 16 bits to 24 bits.",
        4, // Width in bytes
        0x000000b0, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_uart_onems
    },
    {
        "UTS",
        "",
        4, // Width in bytes
        0x000000b4, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_uart_uts
    },
    {
        "UMCR",
        "",
        4, // Width in bytes
        0x000000b8, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_uart_umcr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark USB_ANALOG
#endif

// Bitfields in register USB_ANALOG_USB1_VBUS_DETECT.
static const field_t hw_usb_analog_usb1_vbus_detect[] =
{
    {
        "VBUSVALID_THRESH",
        "Set the threshold for the VBUSVALID comparator.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VBUSVALID_PWRUP_CMPS",
        "Powers up comparators for vbus_valid detector.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISCHARGE_VBUS",
        "USB OTG discharge VBUS.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CHARGE_VBUS",
        "USB OTG charge VBUS.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_CHARGER_RESISTOR",
        "Enable 125k pullup on USB_DP and 375k on USB_DN to provide USB_CHARGER"
        " functionality for USB.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB1_CHRG_DETECT.
static const field_t hw_usb_analog_usb1_chrg_detect[] =
{
    {
        "CHK_CONTACT",
        "",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CHK_CHRG_B",
        "",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_B",
        "Control the charger detector.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB1_VBUS_DETECT_STAT.
static const field_t hw_usb_analog_usb1_vbus_detect_stat[] =
{
    {
        "SESSEND",
        "Session End for USB OTG.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BVALID",
        "Indicates VBus is valid for a B-peripheral.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AVALID",
        "Indicates VBus is valid for a A-peripheral.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VBUS_VALID",
        "VBus valid for USB OTG.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB1_CHRG_DETECT_STAT.
static const field_t hw_usb_analog_usb1_chrg_detect_stat[] =
{
    {
        "PLUG_CONTACT",
        "State of the USB plug contact detector.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CHRG_DETECTED",
        "State of charger detection.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DM_STATE",
        "DM line state output of the charger detector.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DP_STATE",
        "DP line state output of the charger detector.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB1_MISC.
static const field_t hw_usb_analog_usb1_misc[] =
{
    {
        "HS_USE_EXTERNAL_R",
        "Use external resistor to generate the current bias for the high speed "
        "transmitter.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_DEGLITCH",
        "Enable the deglitching circuit of the USB PLL output.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_CLK_UTMI",
        "Enables the clk to the UTMI block.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB2_VBUS_DETECT.
static const field_t hw_usb_analog_usb2_vbus_detect[] =
{
    {
        "VBUSVALID_THRESH",
        "Set the threshold for the VBUSVALID comparator.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VBUSVALID_PWRUP_CMPS",
        "Powers up comparators for vbus_valid detector.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISCHARGE_VBUS",
        "USB OTG discharge VBUS.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CHARGE_VBUS",
        "USB OTG charge VBUS.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_CHARGER_RESISTOR",
        "Enable 125k pullup on USB_DP and 375k on USB_DN to provide USB_CHARGER"
        " functionality for USB.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB2_CHRG_DETECT.
static const field_t hw_usb_analog_usb2_chrg_detect[] =
{
    {
        "CHK_CONTACT",
        "",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CHK_CHRG_B",
        "",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_B",
        "Control the charger detector.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB2_VBUS_DETECT_STAT.
static const field_t hw_usb_analog_usb2_vbus_detect_stat[] =
{
    {
        "SESSEND",
        "Session End for USB OTG.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BVALID",
        "Indicates VBus is valid for a B-peripheral.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AVALID",
        "Indicates VBus is valid for a A-peripheral.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VBUS_VALID",
        "VBus valid for USB OTG.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB2_CHRG_DETECT_STAT.
static const field_t hw_usb_analog_usb2_chrg_detect_stat[] =
{
    {
        "PLUG_CONTACT",
        "State of the USB plug contact detector.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CHRG_DETECTED",
        "State of charger detection.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DM_STATE",
        "DM line state output of the charger detector.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DP_STATE",
        "DP line state output of the charger detector.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_USB2_MISC.
static const field_t hw_usb_analog_usb2_misc[] =
{
    {
        "HS_USE_EXTERNAL_R",
        "Use external resistor to generate the current bias for the high speed "
        "transmitter.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_DEGLITCH",
        "Enable the deglitching circuit of the USB PLL output.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EN_CLK_UTMI",
        "Enables the clk to the UTMI block.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USB_ANALOG_DIGPROG.
static const field_t hw_usb_analog_digprog[] =
{
    {
        "MINOR",
        "Fixed read-only value reflecting the MINOR field of the RTL version.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-only value reflecting the MAJOR field of the RTL version.",
        8, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a USB_ANALOG module.
static const reg_t hw_usb_analog[] =
{
    {
        "USB1_VBUS_DETECT",
        "This register defines controls for USB VBUS detect.",
        4, // Width in bytes
        0x000001a0, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_usb_analog_usb1_vbus_detect
    },
    {
        "USB1_CHRG_DETECT",
        "This register defines controls for USB charger detect.",
        4, // Width in bytes
        0x000001b0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usb_analog_usb1_chrg_detect
    },
    {
        "USB1_VBUS_DETECT_STAT",
        "This register defines fields for USB VBUS Detect status.",
        4, // Width in bytes
        0x000001c0, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_usb_analog_usb1_vbus_detect_stat
    },
    {
        "USB1_CHRG_DETECT_STAT",
        "This register defines fields for USB charger detect status.",
        4, // Width in bytes
        0x000001d0, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_usb_analog_usb1_chrg_detect_stat
    },
    {
        "USB1_MISC",
        "This register defines controls for USB.",
        4, // Width in bytes
        0x000001f0, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usb_analog_usb1_misc
    },
    {
        "USB2_VBUS_DETECT",
        "This register defines controls for USB VBUS detect.",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_usb_analog_usb2_vbus_detect
    },
    {
        "USB2_CHRG_DETECT",
        "This register defines controls for USB charger detect.",
        4, // Width in bytes
        0x00000210, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usb_analog_usb2_chrg_detect
    },
    {
        "USB2_VBUS_DETECT_STAT",
        "This register defines fields for USB VBUS Detect status.",
        4, // Width in bytes
        0x00000220, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_usb_analog_usb2_vbus_detect_stat
    },
    {
        "USB2_CHRG_DETECT_STAT",
        "This register defines fields for USB charger detect status.",
        4, // Width in bytes
        0x00000230, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_usb_analog_usb2_chrg_detect_stat
    },
    {
        "USB2_MISC",
        "This register defines controls for USB.",
        4, // Width in bytes
        0x00000250, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usb_analog_usb2_misc
    },
    {
        "DIGPROG",
        "The DIGPROG register returns the digital program ID for the silicon.",
        4, // Width in bytes
        0x00000280, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usb_analog_digprog
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark USBC
#endif

// Bitfields in register USBC_UOG1_ID.
static const field_t hw_usbc_uog1_id[] =
{
    {
        "ID",
        "Configuration number.",
        0, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NID",
        "Complement version of ID",
        8, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REVISION",
        "Revision number of the controller core.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HWGENERAL.
static const field_t hw_usbc_uog1_hwgeneral[] =
{
    {
        "PHYW",
        "Data width of the transciever connected to the controller core.",
        4, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PHYM",
        "Transciever type  PHYM bit reset value: '0000b' for OTG controller cor"
        "e, '0100b' for Host-only controller core.",
        6, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SM",
        "Serial interface mode capability  SM bit reset value is '00b'",
        9, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HWHOST.
static const field_t hw_usbc_uog1_hwhost[] =
{
    {
        "HC",
        "Host Capable.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NPORT",
        "The Nmber of downstream ports supported by the host controller is (NPO"
        "RT+1).",
        1, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HWDEVICE.
static const field_t hw_usbc_uog1_hwdevice[] =
{
    {
        "DC",
        "Device Capable.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DEVEP",
        "Device Endpoint Number",
        1, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HWTXBUF.
static const field_t hw_usbc_uog1_hwtxbuf[] =
{
    {
        "TXBURST",
        "Default burst size for memory to TX buffer transfer.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXCHANADD",
        "TX FIFO Buffer size is: (2^TXCHANADD) * 4 Bytes.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HWRXBUF.
static const field_t hw_usbc_uog1_hwrxbuf[] =
{
    {
        "RXBURST",
        "Default burst size for memory to RX buffer transfer.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RXADD",
        "Buffer total size for all receive endpoints is (2^RXADD).",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_GPTIMER0LD.
static const field_t hw_usbc_uog1_gptimer0ld[] =
{
    {
        "GPTLD",
        "General Purpose Timer Load Value  These bit fields are loaded to GPTCN"
        "T bits when GPTRST bit is set '1b'.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_GPTIMER0CTRL.
static const field_t hw_usbc_uog1_gptimer0ctrl[] =
{
    {
        "GPTCNT",
        "General Purpose Timer Counter.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTMODE",
        "General Purpose Timer Mode  In one shot mode, the timer will count dow"
        "n to zero, generate an interrupt, and stop until the counter is reset "
        "by software;  In repeat mode, the timer will count down to zero, gener"
        "ate an interrupt and automatically reload the counter value from GPTLD"
        " bits to start again.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRST",
        "General Purpose Timer Reset",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRUN",
        "General Purpose Timer Run  GPTCNT bits are not effected when setting o"
        "r clearing this bit.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_GPTIMER1LD.
static const field_t hw_usbc_uog1_gptimer1ld[] =
{
    {
        "GPTLD",
        "General Purpose Timer Load Value  These bit fields are loaded to GPTCN"
        "T bits when GPTRST bit is set '1b'.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_GPTIMER1CTRL.
static const field_t hw_usbc_uog1_gptimer1ctrl[] =
{
    {
        "GPTCNT",
        "General Purpose Timer Counter.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTMODE",
        "General Purpose Timer Mode  In one shot mode, the timer will count dow"
        "n to zero, generate an interrupt, and stop until the counter is reset "
        "by software.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRST",
        "General Purpose Timer Reset",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRUN",
        "General Purpose Timer Run  GPTCNT bits are not effected when setting o"
        "r clearing this bit.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_SBUSCFG.
static const field_t hw_usbc_uog1_sbuscfg[] =
{
    {
        "AHBBRST",
        "AHB master interface Burst configuration  These bits control AHB maste"
        "r transfer type sequence (or priority).",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_CAPLENGTH.
static const field_t hw_usbc_uog1_caplength[] =
{
    {
        "CAPLENGTH",
        "These bits are used as an offset to add to register base to find the b"
        "eginning of the Operational Register.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HCIVERSION.
static const field_t hw_usbc_uog1_hciversion[] =
{
    {
        "HCIVERSION",
        "Host Controller Interface Version Number  Default value is '10h', whic"
        "h means EHCI rev1.0.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HCSPARAMS.
static const field_t hw_usbc_uog1_hcsparams[] =
{
    {
        "N_PORTS",
        "Number of downstream ports.",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PPC",
        "Port Power Control  This field indicates whether the host controller i"
        "mplementation includes port power control.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_PCC",
        "Number of Ports per Companion Controller  This field indicates the num"
        "ber of ports supported per internal Companion Controller.",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_CC",
        "Number of Companion Controller (N_CC).",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PI",
        "Port Indicators (P INDICATOR)  This bit indicates whether the ports su"
        "pport port indicator control.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_PTT",
        "Number of Ports per Transaction Translator (N_PTT).",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_TT",
        "Number of Transaction Translators (N_TT).",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_HCCPARAMS.
static const field_t hw_usbc_uog1_hccparams[] =
{
    {
        "ADC",
        "64-bit Addressing Capability  This bit is set '0b' in all controller c"
        "ore, no 64-bit addressing capability is supported.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFL",
        "Programmable Frame List Flag  If this bit is set to zero, then the sys"
        "tem software must use a frame list length of 1024 elements with this h"
        "ost controller.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ASP",
        "Asynchronous Schedule Park Capability  If this bit is set to a one, th"
        "en the host controller supports the park feature for high-speed queue "
        "heads in the Asynchronous Schedule.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IST",
        "Isochronous Scheduling Threshold.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EECP",
        "EHCI Extended Capabilities Pointer.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_DCIVERSION.
static const field_t hw_usbc_uog1_dciversion[] =
{
    {
        "DCIVERSION",
        "Device Controller Interface Version Number  Default value is '01h', wh"
        "ich means rev0.1.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_DCCPARAMS.
static const field_t hw_usbc_uog1_dccparams[] =
{
    {
        "DEN",
        "Device Endpoint Number  This field indicates the number of endpoints b"
        "uilt into the device controller.",
        0, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DC",
        "Device Capable  When this bit is 1, this controller is capable of oper"
        "ating as a USB 2.0 device.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HC",
        "Host Capable  When this bit is 1, this controller is capable of operat"
        "ing as an EHCI compatible USB 2.0 host controller.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_USBCMD.
static const field_t hw_usbc_uog1_usbcmd[] =
{
    {
        "RS",
        "Run/Stop (RS) - Read/Write.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RST",
        "Controller Reset (RESET) - Read/Write.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FS",
        "See description at bit 15",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSE",
        "Periodic Schedule Enable- Read/Write.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASE",
        "Asynchronous Schedule Enable - Read/Write.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IAA",
        "Interrupt on Async Advance Doorbell - Read/Write.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASP",
        "Asynchronous Schedule Park Mode Count - Read/Write.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASPE",
        "Asynchronous Schedule Park Mode Enable - Read/Write.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUTW",
        "Setup TripWire - Read/Write.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ATDTW",
        "Add dTD TripWire - Read/Write.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FS1",
        "See also bits 3-2  Frame List Size - (Read/Write or Read Only).",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ITC",
        "Interrupt Threshold Control -Read/Write.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_USBSTS.
static const field_t hw_usbc_uog1_usbsts[] =
{
    {
        "UI",
        "USB Interrupt (USBINT) - R/WC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UEI",
        "USB Error Interrupt (USBERRINT) - R/WC.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PCI",
        "Port Change Detect - R/WC.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRI",
        "Frame List Rollover - R/WC.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEI",
        "System Error- R/WC.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AAI",
        "Interrupt on Async Advance - R/WC.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URI",
        "USB Reset Received - R/WC.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRI",
        "SOF Received - R/WC.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLI",
        "DCSuspend - R/WC.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ULPII",
        "ULPI Interrupt - R/WC.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HCH",
        "HCHaIted - Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCL",
        "Reclamation - Read Only.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PS",
        "Periodic Schedule Status - Read Only.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AS",
        "Asynchronous Schedule Status - Read Only.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NAKI",
        "NAK Interrupt Bit--RO.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TI0",
        "General Purpose Timer Interrupt 0(GPTINT0)--R/WC.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TI1",
        "General Purpose Timer Interrupt 1(GPTINT1)--R/WC.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_USBINTR.
static const field_t hw_usbc_uog1_usbintr[] =
{
    {
        "UE",
        "USB Interrupt Enalbe  When this bit is one and the UI bit in n_USBSTS "
        "register is a one the controller will issue an interrupt.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UEE",
        "USB Error Interrupt Enable  When this bit is one and the UEI bit in n_"
        "USBSTS register is a one the controller will issue an interrupt.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PCE",
        "Port Change Detect Interrupt Enable  When this bit is one and the PCI "
        "bit in n_USBSTS register is a one the controller will issue an interru"
        "pt.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRE",
        "Frame List Rollover Interrupt Enable  When this bit is one and the FRI"
        " bit in n_USBSTS register is a one the controller will issue an interr"
        "upt.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEE",
        "System Error Interrupt Enable  When this bit is one and the SEI bit in"
        " n_USBSTS register is a one the controller will issue an interrupt.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AAE",
        "Async Advance Interrupt Enable  When this bit is one and the AAI bit i"
        "n n_USBSTS register is a one the controller will issue an interrupt.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URE",
        "USB Reset Interrupt Enable  When this bit is one and the URI bit in n_"
        "USBSTS register is a one the controller will issue an interrupt.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRE",
        "SOF Received Interrupt Enable  When this bit is one and the SRI bit in"
        " n_USBSTS register is a one the controller will issue an interrupt.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLE",
        "Sleep Interrupt Enable  When this bit is one and the SLI bit in n_n_US"
        "BSTS register is a one the controller will issue an interrupt.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ULPIE",
        "ULPI Interrupt Enable  When this bit is one and the UPLII bit in n_USB"
        "STS register is a one the controller will issue an interrupt.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NAKE",
        "NAK Interrupt Enable  When this bit is one and the NAKI bit in n_USBST"
        "S register is a one the controller will issue an interrupt.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UAIE",
        "USB Host Asynchronous Interrupt Enable  When this bit is one, and the "
        "UAI bit in the n_USBSTS register is one, host controller will issue an"
        " interrupt at the next interrupt threshold.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPIE",
        "USB Host Periodic Interrupt Enable  When this bit is one, and the UPI "
        "bit in the n_USBSTS register is one, host controller will issue an int"
        "errupt at the next interrupt threshold.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIE0",
        "General Purpose Timer #0 Interrupt Enable  When this bit is one and th"
        "e TI0 bit in n_USBSTS register is a one the controller will issue an i"
        "nterrupt.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIE1",
        "General Purpose Timer #1 Interrupt Enable  When this bit is one and th"
        "e TI1 bit in n_USBSTS register is a one the controller will issue an i"
        "nterrupt.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_FRINDEX.
static const field_t hw_usbc_uog1_frindex[] =
{
    {
        "FRINDEX",
        "Frame Index.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_PERIODICLISTBASE.
static const field_t hw_usbc_uog1_periodiclistbase[] =
{
    {
        "BASEADR",
        "Base Address (Low).",
        12, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ASYNCLISTADDR.
static const field_t hw_usbc_uog1_asynclistaddr[] =
{
    {
        "ASYBASE",
        "Link Pointer Low (LPL).",
        5, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_BURSTSIZE.
static const field_t hw_usbc_uog1_burstsize[] =
{
    {
        "RXPBURST",
        "Programmable RX Burst Size.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXPBURST",
        "Programmable TX Burst Size.",
        8, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_TXFILLTUNING.
static const field_t hw_usbc_uog1_txfilltuning[] =
{
    {
        "TXSCHOH",
        "Scheduler Overhead.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXSCHHEALTH",
        "Scheduler Health Counter.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXFIFOTHRES",
        "FIFO Burst Threshold.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTNAK.
static const field_t hw_usbc_uog1_endptnak[] =
{
    {
        "EPRN",
        "RX Endpoint NAK - R/WC.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPTN",
        "TX Endpoint NAK - R/WC.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTNAKEN.
static const field_t hw_usbc_uog1_endptnaken[] =
{
    {
        "EPRNE",
        "RX Endpoint NAK Enable - R/W.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPTNE",
        "TX Endpoint NAK Enable - R/W.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_CONFIGFLAG.
static const field_t hw_usbc_uog1_configflag[] =
{
    {
        "CF",
        "Configure Flag  Host software sets this bit as the last action in its "
        "process of configuring the Host Controller.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_PORTSC1.
static const field_t hw_usbc_uog1_portsc1[] =
{
    {
        "CCS",
        "Current Connect Status-Read Only.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CSC",
        "Connect Status Change-R/WC.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PE",
        "Port Enabled/Disabled-Read/Write.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PEC",
        "Port Enable/Disable Change-R/WC.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCA",
        "Over-current Active-Read Only.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OCC",
        "Over-current Change-R/WC.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FPR",
        "Force Port Resume -Read/Write.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUSP",
        "Suspend - Read/Write or Read Only.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PR",
        "Port Reset - Read/Write or Read Only.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSP",
        "High-Speed Port - Read Only.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LS",
        "Line Status-Read Only.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PP",
        "Port Power (PP)-Read/Write or Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PO",
        "Port Owner-Read/Write.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PIC",
        "Port Indicator Control - Read/Write.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTC",
        "Port Test Control - Read/Write.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKCN",
        "Wake on Connect Enable (WKCNNT_E) - Read/Write.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKDC",
        "Wake on Disconnect Enable (WKDSCNNT_E) - Read/Write.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKOC",
        "Wake on Over-current Enable (WKOC_E) - Read/Write.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PHCD",
        "PHY Low Power Suspend - Clock Disable (PLPSCD) - Read/Write.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFSC",
        "Port Force Full Speed Connect - Read/Write.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTS",
        "See description at bits 31-30",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSPD",
        "Port Speed - Read Only.",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTW",
        "Parallel Transceiver Width  This bit has no effect if serial interface"
        " engine is used.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STS",
        "Serial Transceiver Select - Read Only  Serial Transceiver Select  1 Se"
        "rial Interface Engine is selected  0 Parallel Interface signals is sel"
        "ected  Serial Interface Engine can be used in combination with UTMI+/U"
        "LPI physical interface to provide FS/LS signaling instead of the paral"
        "lel interface signals.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTS1",
        "Bit field {bit25, bit31, bit30}:  \"000b\" UTMI/UTMI+  \"001b\" Reserv"
        "ed  \"010b\" ULPI  \"011b\" Serial/USB 1.1 PHY/IC-USB (FS Only)  \"100"
        "b\" HSIC  All USB port interface modes are listed in this field descri"
        "ption, but not all are supported.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_OTGSC.
static const field_t hw_usbc_uog1_otgsc[] =
{
    {
        "VD",
        "VBUS_Discharge - Read/Write.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VC",
        "VBUS Charge - Read/Write.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OT",
        "OTG Termination - Read/Write.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DP",
        "Data Pulsing - Read/Write.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDPU",
        "ID Pullup - Read/Write  This bit provide control over the ID pull-up r"
        "esister; 0 = off, 1 = on [default].",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ID",
        "USB ID - Read Only.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AVV",
        "A VBus Valid - Read Only.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ASV",
        "A Session Valid - Read Only.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BSV",
        "B Session Valid - Read Only.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BSE",
        "B Session End - Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "_1MST",
        "1 millisecond timer toggle - Read Only.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DPS",
        "Data Bus Pulsing Status - Read Only.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDIS",
        "USB ID Interrupt Status - Read/Write.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AVVIS",
        "A VBus Valid Interrupt Status - Read/Write to Clear.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASVIS",
        "A Session Valid Interrupt Status - Read/Write to Clear.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSVIS",
        "B Session Valid Interrupt Status - Read/Write to Clear.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSEIS",
        "B Session End Interrupt Status - Read/Write to Clear.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "_1MSS",
        "1 millisecond timer Interrupt Status - Read/Write to Clear.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DPIS",
        "Data Pulse Interrupt Status - Read/Write to Clear.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDIE",
        "USB ID Interrupt Enable - Read/Write.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AVVIE",
        "A VBus Valid Interrupt Enable - Read/Write.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASVIE",
        "A Session Valid Interrupt Enable - Read/Write.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSVIE",
        "B Session Valid Interrupt Enable - Read/Write.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSEIE",
        "B Session End Interrupt Enable - Read/Write.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "_1MSE",
        "1 millisecond timer Interrupt Enable - Read/Write",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DPIE",
        "Data Pulse Interrupt Enable",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_USBMODE.
static const field_t hw_usbc_uog1_usbmode[] =
{
    {
        "CM",
        "Controller Mode - R/WO.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ES",
        "Endian Select - Read/Write.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLOM",
        "Setup Lockout Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDIS",
        "Stream Disable Mode.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTSETUPSTAT.
static const field_t hw_usbc_uog1_endptsetupstat[] =
{
    {
        "ENDPTSETUPSTAT",
        "Setup Endpoint Status.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTPRIME.
static const field_t hw_usbc_uog1_endptprime[] =
{
    {
        "PERB",
        "Prime Endpoint Receive Buffer - R/WS.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PETB",
        "Prime Endpoint Transmit Buffer - R/WS.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTFLUSH.
static const field_t hw_usbc_uog1_endptflush[] =
{
    {
        "FERB",
        "Flush Endpoint Receive Buffer - R/WS.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FETB",
        "Flush Endpoint Transmit Buffer - R/WS.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTSTAT.
static const field_t hw_usbc_uog1_endptstat[] =
{
    {
        "ERBR",
        "Endpoint Receive Buffer Ready -- Read Only.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ETBR",
        "Endpoint Transmit Buffer Ready -- Read Only.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCOMPLETE.
static const field_t hw_usbc_uog1_endptcomplete[] =
{
    {
        "ERCE",
        "Endpoint Receive Complete Event - RW/C.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ETCE",
        "Endpoint Transmit Complete Event - R/WC.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL0.
static const field_t hw_usbc_uog1_endptctrl0[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  Endpoint0 is fixed as a Con"
        "trol End Point.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  1 Enabled  Endpoint0 is always enabled.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK [Default]  1 End Point "
        "Stalled  Software can write a one to this bit to force the endpoint to"
        " return a STALL handshake to the Host.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 - Control  Endpoint0 is fixed as a C"
        "ontrol End Point.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  1 Enabled  Endpoint0 is always enabled.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL1.
static const field_t hw_usbc_uog1_endptctrl1[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL2.
static const field_t hw_usbc_uog1_endptctrl2[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL3.
static const field_t hw_usbc_uog1_endptctrl3[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL4.
static const field_t hw_usbc_uog1_endptctrl4[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL5.
static const field_t hw_usbc_uog1_endptctrl5[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL6.
static const field_t hw_usbc_uog1_endptctrl6[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG1_ENDPTCTRL7.
static const field_t hw_usbc_uog1_endptctrl7[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ID.
static const field_t hw_usbc_uog2_id[] =
{
    {
        "ID",
        "Configuration number.",
        0, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NID",
        "Complement version of ID",
        8, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REVISION",
        "Revision number of the controller core.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HWGENERAL.
static const field_t hw_usbc_uog2_hwgeneral[] =
{
    {
        "PHYW",
        "Data width of the transciever connected to the controller core.",
        4, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PHYM",
        "Transciever type  PHYM bit reset value: '0000b' for OTG controller cor"
        "e, '0100b' for Host-only controller core.",
        6, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SM",
        "Serial interface mode capability  SM bit reset value is '00b'",
        9, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HWHOST.
static const field_t hw_usbc_uog2_hwhost[] =
{
    {
        "HC",
        "Host Capable.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NPORT",
        "The Nmber of downstream ports supported by the host controller is (NPO"
        "RT+1).",
        1, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HWDEVICE.
static const field_t hw_usbc_uog2_hwdevice[] =
{
    {
        "DC",
        "Device Capable.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DEVEP",
        "Device Endpoint Number",
        1, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HWTXBUF.
static const field_t hw_usbc_uog2_hwtxbuf[] =
{
    {
        "TXBURST",
        "Default burst size for memory to TX buffer transfer.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXCHANADD",
        "TX FIFO Buffer size is: (2^TXCHANADD) * 4 Bytes.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HWRXBUF.
static const field_t hw_usbc_uog2_hwrxbuf[] =
{
    {
        "RXBURST",
        "Default burst size for memory to RX buffer transfer.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RXADD",
        "Buffer total size for all receive endpoints is (2^RXADD).",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_GPTIMER0LD.
static const field_t hw_usbc_uog2_gptimer0ld[] =
{
    {
        "GPTLD",
        "General Purpose Timer Load Value  These bit fields are loaded to GPTCN"
        "T bits when GPTRST bit is set '1b'.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_GPTIMER0CTRL.
static const field_t hw_usbc_uog2_gptimer0ctrl[] =
{
    {
        "GPTCNT",
        "General Purpose Timer Counter.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTMODE",
        "General Purpose Timer Mode  In one shot mode, the timer will count dow"
        "n to zero, generate an interrupt, and stop until the counter is reset "
        "by software;  In repeat mode, the timer will count down to zero, gener"
        "ate an interrupt and automatically reload the counter value from GPTLD"
        " bits to start again.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRST",
        "General Purpose Timer Reset",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRUN",
        "General Purpose Timer Run  GPTCNT bits are not effected when setting o"
        "r clearing this bit.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_GPTIMER1LD.
static const field_t hw_usbc_uog2_gptimer1ld[] =
{
    {
        "GPTLD",
        "General Purpose Timer Load Value  These bit fields are loaded to GPTCN"
        "T bits when GPTRST bit is set '1b'.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_GPTIMER1CTRL.
static const field_t hw_usbc_uog2_gptimer1ctrl[] =
{
    {
        "GPTCNT",
        "General Purpose Timer Counter.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTMODE",
        "General Purpose Timer Mode  In one shot mode, the timer will count dow"
        "n to zero, generate an interrupt, and stop until the counter is reset "
        "by software.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRST",
        "General Purpose Timer Reset",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRUN",
        "General Purpose Timer Run  GPTCNT bits are not effected when setting o"
        "r clearing this bit.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_SBUSCFG.
static const field_t hw_usbc_uog2_sbuscfg[] =
{
    {
        "AHBBRST",
        "AHB master interface Burst configuration  These bits control AHB maste"
        "r transfer type sequence (or priority).",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_CAPLENGTH.
static const field_t hw_usbc_uog2_caplength[] =
{
    {
        "CAPLENGTH",
        "These bits are used as an offset to add to register base to find the b"
        "eginning of the Operational Register.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HCIVERSION.
static const field_t hw_usbc_uog2_hciversion[] =
{
    {
        "HCIVERSION",
        "Host Controller Interface Version Number  Default value is '10h', whic"
        "h means EHCI rev1.0.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HCSPARAMS.
static const field_t hw_usbc_uog2_hcsparams[] =
{
    {
        "N_PORTS",
        "Number of downstream ports.",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PPC",
        "Port Power Control  This field indicates whether the host controller i"
        "mplementation includes port power control.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_PCC",
        "Number of Ports per Companion Controller  This field indicates the num"
        "ber of ports supported per internal Companion Controller.",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_CC",
        "Number of Companion Controller (N_CC).",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PI",
        "Port Indicators (P INDICATOR)  This bit indicates whether the ports su"
        "pport port indicator control.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_PTT",
        "Number of Ports per Transaction Translator (N_PTT).",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_TT",
        "Number of Transaction Translators (N_TT).",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_HCCPARAMS.
static const field_t hw_usbc_uog2_hccparams[] =
{
    {
        "ADC",
        "64-bit Addressing Capability  This bit is set '0b' in all controller c"
        "ore, no 64-bit addressing capability is supported.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFL",
        "Programmable Frame List Flag  If this bit is set to zero, then the sys"
        "tem software must use a frame list length of 1024 elements with this h"
        "ost controller.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ASP",
        "Asynchronous Schedule Park Capability  If this bit is set to a one, th"
        "en the host controller supports the park feature for high-speed queue "
        "heads in the Asynchronous Schedule.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IST",
        "Isochronous Scheduling Threshold.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EECP",
        "EHCI Extended Capabilities Pointer.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_DCIVERSION.
static const field_t hw_usbc_uog2_dciversion[] =
{
    {
        "DCIVERSION",
        "Device Controller Interface Version Number  Default value is '01h', wh"
        "ich means rev0.1.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_DCCPARAMS.
static const field_t hw_usbc_uog2_dccparams[] =
{
    {
        "DEN",
        "Device Endpoint Number  This field indicates the number of endpoints b"
        "uilt into the device controller.",
        0, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DC",
        "Device Capable  When this bit is 1, this controller is capable of oper"
        "ating as a USB 2.0 device.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HC",
        "Host Capable  When this bit is 1, this controller is capable of operat"
        "ing as an EHCI compatible USB 2.0 host controller.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_USBCMD.
static const field_t hw_usbc_uog2_usbcmd[] =
{
    {
        "RS",
        "Run/Stop (RS) - Read/Write.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RST",
        "Controller Reset (RESET) - Read/Write.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FS",
        "See description at bit 15",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSE",
        "Periodic Schedule Enable- Read/Write.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASE",
        "Asynchronous Schedule Enable - Read/Write.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IAA",
        "Interrupt on Async Advance Doorbell - Read/Write.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASP",
        "Asynchronous Schedule Park Mode Count - Read/Write.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASPE",
        "Asynchronous Schedule Park Mode Enable - Read/Write.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUTW",
        "Setup TripWire - Read/Write.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ATDTW",
        "Add dTD TripWire - Read/Write.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FS1",
        "See also bits 3-2  Frame List Size - (Read/Write or Read Only).",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ITC",
        "Interrupt Threshold Control -Read/Write.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_USBSTS.
static const field_t hw_usbc_uog2_usbsts[] =
{
    {
        "UI",
        "USB Interrupt (USBINT) - R/WC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UEI",
        "USB Error Interrupt (USBERRINT) - R/WC.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PCI",
        "Port Change Detect - R/WC.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRI",
        "Frame List Rollover - R/WC.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEI",
        "System Error- R/WC.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AAI",
        "Interrupt on Async Advance - R/WC.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URI",
        "USB Reset Received - R/WC.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRI",
        "SOF Received - R/WC.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLI",
        "DCSuspend - R/WC.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ULPII",
        "ULPI Interrupt - R/WC.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HCH",
        "HCHaIted - Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCL",
        "Reclamation - Read Only.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PS",
        "Periodic Schedule Status - Read Only.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AS",
        "Asynchronous Schedule Status - Read Only.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NAKI",
        "NAK Interrupt Bit--RO.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TI0",
        "General Purpose Timer Interrupt 0(GPTINT0)--R/WC.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TI1",
        "General Purpose Timer Interrupt 1(GPTINT1)--R/WC.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_USBINTR.
static const field_t hw_usbc_uog2_usbintr[] =
{
    {
        "UE",
        "USB Interrupt Enalbe  When this bit is one and the UI bit in n_USBSTS "
        "register is a one the controller will issue an interrupt.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UEE",
        "USB Error Interrupt Enable  When this bit is one and the UEI bit in n_"
        "USBSTS register is a one the controller will issue an interrupt.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PCE",
        "Port Change Detect Interrupt Enable  When this bit is one and the PCI "
        "bit in n_USBSTS register is a one the controller will issue an interru"
        "pt.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRE",
        "Frame List Rollover Interrupt Enable  When this bit is one and the FRI"
        " bit in n_USBSTS register is a one the controller will issue an interr"
        "upt.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEE",
        "System Error Interrupt Enable  When this bit is one and the SEI bit in"
        " n_USBSTS register is a one the controller will issue an interrupt.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AAE",
        "Async Advance Interrupt Enable  When this bit is one and the AAI bit i"
        "n n_USBSTS register is a one the controller will issue an interrupt.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URE",
        "USB Reset Interrupt Enable  When this bit is one and the URI bit in n_"
        "USBSTS register is a one the controller will issue an interrupt.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRE",
        "SOF Received Interrupt Enable  When this bit is one and the SRI bit in"
        " n_USBSTS register is a one the controller will issue an interrupt.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLE",
        "Sleep Interrupt Enable  When this bit is one and the SLI bit in n_n_US"
        "BSTS register is a one the controller will issue an interrupt.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ULPIE",
        "ULPI Interrupt Enable  When this bit is one and the UPLII bit in n_USB"
        "STS register is a one the controller will issue an interrupt.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NAKE",
        "NAK Interrupt Enable  When this bit is one and the NAKI bit in n_USBST"
        "S register is a one the controller will issue an interrupt.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UAIE",
        "USB Host Asynchronous Interrupt Enable  When this bit is one, and the "
        "UAI bit in the n_USBSTS register is one, host controller will issue an"
        " interrupt at the next interrupt threshold.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPIE",
        "USB Host Periodic Interrupt Enable  When this bit is one, and the UPI "
        "bit in the n_USBSTS register is one, host controller will issue an int"
        "errupt at the next interrupt threshold.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIE0",
        "General Purpose Timer #0 Interrupt Enable  When this bit is one and th"
        "e TI0 bit in n_USBSTS register is a one the controller will issue an i"
        "nterrupt.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIE1",
        "General Purpose Timer #1 Interrupt Enable  When this bit is one and th"
        "e TI1 bit in n_USBSTS register is a one the controller will issue an i"
        "nterrupt.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_FRINDEX.
static const field_t hw_usbc_uog2_frindex[] =
{
    {
        "FRINDEX",
        "Frame Index.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_PERIODICLISTBASE.
static const field_t hw_usbc_uog2_periodiclistbase[] =
{
    {
        "BASEADR",
        "Base Address (Low).",
        12, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ASYNCLISTADDR.
static const field_t hw_usbc_uog2_asynclistaddr[] =
{
    {
        "ASYBASE",
        "Link Pointer Low (LPL).",
        5, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_BURSTSIZE.
static const field_t hw_usbc_uog2_burstsize[] =
{
    {
        "RXPBURST",
        "Programmable RX Burst Size.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXPBURST",
        "Programmable TX Burst Size.",
        8, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_TXFILLTUNING.
static const field_t hw_usbc_uog2_txfilltuning[] =
{
    {
        "TXSCHOH",
        "Scheduler Overhead.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXSCHHEALTH",
        "Scheduler Health Counter.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXFIFOTHRES",
        "FIFO Burst Threshold.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTNAK.
static const field_t hw_usbc_uog2_endptnak[] =
{
    {
        "EPRN",
        "RX Endpoint NAK - R/WC.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPTN",
        "TX Endpoint NAK - R/WC.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTNAKEN.
static const field_t hw_usbc_uog2_endptnaken[] =
{
    {
        "EPRNE",
        "RX Endpoint NAK Enable - R/W.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EPTNE",
        "TX Endpoint NAK Enable - R/W.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_CONFIGFLAG.
static const field_t hw_usbc_uog2_configflag[] =
{
    {
        "CF",
        "Configure Flag  Host software sets this bit as the last action in its "
        "process of configuring the Host Controller.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_PORTSC1.
static const field_t hw_usbc_uog2_portsc1[] =
{
    {
        "CCS",
        "Current Connect Status-Read Only.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CSC",
        "Connect Status Change-R/WC.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PE",
        "Port Enabled/Disabled-Read/Write.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PEC",
        "Port Enable/Disable Change-R/WC.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCA",
        "Over-current Active-Read Only.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OCC",
        "Over-current Change-R/WC.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FPR",
        "Force Port Resume -Read/Write.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUSP",
        "Suspend - Read/Write or Read Only.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PR",
        "Port Reset - Read/Write or Read Only.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSP",
        "High-Speed Port - Read Only.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LS",
        "Line Status-Read Only.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PP",
        "Port Power (PP)-Read/Write or Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PO",
        "Port Owner-Read/Write.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PIC",
        "Port Indicator Control - Read/Write.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTC",
        "Port Test Control - Read/Write.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKCN",
        "Wake on Connect Enable (WKCNNT_E) - Read/Write.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKDC",
        "Wake on Disconnect Enable (WKDSCNNT_E) - Read/Write.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKOC",
        "Wake on Over-current Enable (WKOC_E) - Read/Write.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PHCD",
        "PHY Low Power Suspend - Clock Disable (PLPSCD) - Read/Write.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFSC",
        "Port Force Full Speed Connect - Read/Write.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTS",
        "See description at bits 31-30",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSPD",
        "Port Speed - Read Only.",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTW",
        "Parallel Transceiver Width  This bit has no effect if serial interface"
        " engine is used.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STS",
        "Serial Transceiver Select - Read Only  Serial Transceiver Select  1 Se"
        "rial Interface Engine is selected  0 Parallel Interface signals is sel"
        "ected  Serial Interface Engine can be used in combination with UTMI+/U"
        "LPI physical interface to provide FS/LS signaling instead of the paral"
        "lel interface signals.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTS1",
        "Bit field {bit25, bit31, bit30}:  \"000b\" UTMI/UTMI+  \"001b\" Reserv"
        "ed  \"010b\" ULPI  \"011b\" Serial/USB 1.1 PHY/IC-USB (FS Only)  \"100"
        "b\" HSIC  All USB port interface modes are listed in this field descri"
        "ption, but not all are supported.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_OTGSC.
static const field_t hw_usbc_uog2_otgsc[] =
{
    {
        "VD",
        "VBUS_Discharge - Read/Write.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VC",
        "VBUS Charge - Read/Write.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OT",
        "OTG Termination - Read/Write.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DP",
        "Data Pulsing - Read/Write.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDPU",
        "ID Pullup - Read/Write  This bit provide control over the ID pull-up r"
        "esister; 0 = off, 1 = on [default].",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ID",
        "USB ID - Read Only.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AVV",
        "A VBus Valid - Read Only.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ASV",
        "A Session Valid - Read Only.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BSV",
        "B Session Valid - Read Only.",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BSE",
        "B Session End - Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "_1MST",
        "1 millisecond timer toggle - Read Only.",
        13, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DPS",
        "Data Bus Pulsing Status - Read Only.",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IDIS",
        "USB ID Interrupt Status - Read/Write.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AVVIS",
        "A VBus Valid Interrupt Status - Read/Write to Clear.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASVIS",
        "A Session Valid Interrupt Status - Read/Write to Clear.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSVIS",
        "B Session Valid Interrupt Status - Read/Write to Clear.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSEIS",
        "B Session End Interrupt Status - Read/Write to Clear.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "_1MSS",
        "1 millisecond timer Interrupt Status - Read/Write to Clear.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DPIS",
        "Data Pulse Interrupt Status - Read/Write to Clear.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IDIE",
        "USB ID Interrupt Enable - Read/Write.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AVVIE",
        "A VBus Valid Interrupt Enable - Read/Write.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASVIE",
        "A Session Valid Interrupt Enable - Read/Write.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSVIE",
        "B Session Valid Interrupt Enable - Read/Write.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BSEIE",
        "B Session End Interrupt Enable - Read/Write.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "_1MSE",
        "1 millisecond timer Interrupt Enable - Read/Write",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DPIE",
        "Data Pulse Interrupt Enable",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_USBMODE.
static const field_t hw_usbc_uog2_usbmode[] =
{
    {
        "CM",
        "Controller Mode - R/WO.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ES",
        "Endian Select - Read/Write.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLOM",
        "Setup Lockout Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDIS",
        "Stream Disable Mode.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTSETUPSTAT.
static const field_t hw_usbc_uog2_endptsetupstat[] =
{
    {
        "ENDPTSETUPSTAT",
        "Setup Endpoint Status.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTPRIME.
static const field_t hw_usbc_uog2_endptprime[] =
{
    {
        "PERB",
        "Prime Endpoint Receive Buffer - R/WS.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PETB",
        "Prime Endpoint Transmit Buffer - R/WS.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTFLUSH.
static const field_t hw_usbc_uog2_endptflush[] =
{
    {
        "FERB",
        "Flush Endpoint Receive Buffer - R/WS.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FETB",
        "Flush Endpoint Transmit Buffer - R/WS.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTSTAT.
static const field_t hw_usbc_uog2_endptstat[] =
{
    {
        "ERBR",
        "Endpoint Receive Buffer Ready -- Read Only.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ETBR",
        "Endpoint Transmit Buffer Ready -- Read Only.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCOMPLETE.
static const field_t hw_usbc_uog2_endptcomplete[] =
{
    {
        "ERCE",
        "Endpoint Receive Complete Event - RW/C.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ETCE",
        "Endpoint Transmit Complete Event - R/WC.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL0.
static const field_t hw_usbc_uog2_endptctrl0[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  Endpoint0 is fixed as a Con"
        "trol End Point.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  1 Enabled  Endpoint0 is always enabled.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK [Default]  1 End Point "
        "Stalled  Software can write a one to this bit to force the endpoint to"
        " return a STALL handshake to the Host.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 - Control  Endpoint0 is fixed as a C"
        "ontrol End Point.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  1 Enabled  Endpoint0 is always enabled.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL1.
static const field_t hw_usbc_uog2_endptctrl1[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL2.
static const field_t hw_usbc_uog2_endptctrl2[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL3.
static const field_t hw_usbc_uog2_endptctrl3[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL4.
static const field_t hw_usbc_uog2_endptctrl4[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL5.
static const field_t hw_usbc_uog2_endptctrl5[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL6.
static const field_t hw_usbc_uog2_endptctrl6[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UOG2_ENDPTCTRL7.
static const field_t hw_usbc_uog2_endptctrl7[] =
{
    {
        "RXS",
        "RX Endpoint Stall - Read/Write  0 End Point OK.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXD",
        "RX Endpoint Data Sink - Read/Write - TBD  0 Dual Port Memory Buffer/DM"
        "A Engine [Default]  Should always be written as zero.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXT",
        "RX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Reserved",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXI",
        "RX Data Toggle Inhibit  0 Disabled [Default]  1 Enabled  This bit is o"
        "nly used for test and should always be written as zero.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXR",
        "RX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the ho"
        "st and device.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXE",
        "RX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXS",
        "TX Endpoint Stall - Read/Write  0 End Point OK  1 End Point Stalled  T"
        "his bit is set automatically upon receipt of a SETUP request if this E"
        "ndpoint is not configured as a Control Endpoint.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXD",
        "TX Endpoint Data Source - Read/Write  0 Dual Port Memory Buffer/DMA En"
        "gine [DEFAULT]  Should always be written as 0.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXT",
        "TX Endpoint Type - Read/Write  00 Control  01 Isochronous  10 Bulk  11"
        " Interrupt",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXI",
        "TX Data Toggle Inhibit  0 PID Sequencing Enabled.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXR",
        "TX Data Toggle Reset (WS)  Write 1 - Reset PID Sequence  Whenever a co"
        "nfiguration event is received for this Endpoint, software must write a"
        " one to this bit in order to synchronize the data PID's between the Ho"
        "st and device.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXE",
        "TX Endpoint Enable  0 Disabled [Default]  1 Enabled  An Endpoint shoul"
        "d be enabled only after it has been configured.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_ID.
static const field_t hw_usbc_uh1_id[] =
{
    {
        "ID",
        "Configuration number.",
        0, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NID",
        "Complement version of ID",
        8, // LSB
        13, // MSB
        true, // Readable
        false // Writable
    },
    {
        "REVISION",
        "Revision number of the controller core.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_HWGENERAL.
static const field_t hw_usbc_uh1_hwgeneral[] =
{
    {
        "PHYW",
        "Data width of the transciever connected to the controller core.",
        4, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PHYM",
        "Transciever type  PHYM bit reset value: '0000b' for OTG controller cor"
        "e, '0100b' for Host-only controller core.",
        6, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SM",
        "Serial interface mode capability  SM bit reset value is '00b'",
        9, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_HWHOST.
static const field_t hw_usbc_uh1_hwhost[] =
{
    {
        "HC",
        "Host Capable.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "NPORT",
        "The Nmber of downstream ports supported by the host controller is (NPO"
        "RT+1).",
        1, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_HWTXBUF.
static const field_t hw_usbc_uh1_hwtxbuf[] =
{
    {
        "TXBURST",
        "Default burst size for memory to TX buffer transfer.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TXCHANADD",
        "TX FIFO Buffer size is: (2^TXCHANADD) * 4 Bytes.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_HWRXBUF.
static const field_t hw_usbc_uh1_hwrxbuf[] =
{
    {
        "RXBURST",
        "Default burst size for memory to RX buffer transfer.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RXADD",
        "Buffer total size for all receive endpoints is (2^RXADD).",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_GPTIMER0LD.
static const field_t hw_usbc_uh1_gptimer0ld[] =
{
    {
        "GPTLD",
        "General Purpose Timer Load Value  These bit fields are loaded to GPTCN"
        "T bits when GPTRST bit is set '1b'.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_GPTIMER0CTRL.
static const field_t hw_usbc_uh1_gptimer0ctrl[] =
{
    {
        "GPTCNT",
        "General Purpose Timer Counter.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTMODE",
        "General Purpose Timer Mode  In one shot mode, the timer will count dow"
        "n to zero, generate an interrupt, and stop until the counter is reset "
        "by software;  In repeat mode, the timer will count down to zero, gener"
        "ate an interrupt and automatically reload the counter value from GPTLD"
        " bits to start again.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRST",
        "General Purpose Timer Reset",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRUN",
        "General Purpose Timer Run  GPTCNT bits are not effected when setting o"
        "r clearing this bit.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_GPTIMER1LD.
static const field_t hw_usbc_uh1_gptimer1ld[] =
{
    {
        "GPTLD",
        "General Purpose Timer Load Value  These bit fields are loaded to GPTCN"
        "T bits when GPTRST bit is set '1b'.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_GPTIMER1CTRL.
static const field_t hw_usbc_uh1_gptimer1ctrl[] =
{
    {
        "GPTCNT",
        "General Purpose Timer Counter.",
        0, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTMODE",
        "General Purpose Timer Mode  In one shot mode, the timer will count dow"
        "n to zero, generate an interrupt, and stop until the counter is reset "
        "by software.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRST",
        "General Purpose Timer Reset",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "GPTRUN",
        "General Purpose Timer Run  GPTCNT bits are not effected when setting o"
        "r clearing this bit.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_SBUSCFG.
static const field_t hw_usbc_uh1_sbuscfg[] =
{
    {
        "AHBBRST",
        "AHB master interface Burst configuration  These bits control AHB maste"
        "r transfer type sequence (or priority).",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_CAPLENGTH.
static const field_t hw_usbc_uh1_caplength[] =
{
    {
        "CAPLENGTH",
        "These bits are used as an offset to add to register base to find the b"
        "eginning of the Operational Register.",
        0, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_HCIVERSION.
static const field_t hw_usbc_uh1_hciversion[] =
{
    {
        "HCIVERSION",
        "Host Controller Interface Version Number  Default value is '10h', whic"
        "h means EHCI rev1.0.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_HCSPARAMS.
static const field_t hw_usbc_uh1_hcsparams[] =
{
    {
        "N_PORTS",
        "Number of downstream ports.",
        0, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PPC",
        "Port Power Control  This field indicates whether the host controller i"
        "mplementation includes port power control.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_PCC",
        "Number of Ports per Companion Controller  This field indicates the num"
        "ber of ports supported per internal Companion Controller.",
        8, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_CC",
        "Number of Companion Controller (N_CC).",
        12, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PI",
        "Port Indicators (P INDICATOR)  This bit indicates whether the ports su"
        "pport port indicator control.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_PTT",
        "Number of Ports per Transaction Translator (N_PTT).",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "N_TT",
        "Number of Transaction Translators (N_TT).",
        24, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_HCCPARAMS.
static const field_t hw_usbc_uh1_hccparams[] =
{
    {
        "ADC",
        "64-bit Addressing Capability  This bit is set '0b' in all controller c"
        "ore, no 64-bit addressing capability is supported.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PFL",
        "Programmable Frame List Flag  If this bit is set to zero, then the sys"
        "tem software must use a frame list length of 1024 elements with this h"
        "ost controller.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ASP",
        "Asynchronous Schedule Park Capability  If this bit is set to a one, th"
        "en the host controller supports the park feature for high-speed queue "
        "heads in the Asynchronous Schedule.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IST",
        "Isochronous Scheduling Threshold.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EECP",
        "EHCI Extended Capabilities Pointer.",
        8, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_USBCMD.
static const field_t hw_usbc_uh1_usbcmd[] =
{
    {
        "RS",
        "Run/Stop (RS) - Read/Write.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RST",
        "Controller Reset (RESET) - Read/Write.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FS",
        "See description at bit 15",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSE",
        "Periodic Schedule Enable- Read/Write.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASE",
        "Asynchronous Schedule Enable - Read/Write.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IAA",
        "Interrupt on Async Advance Doorbell - Read/Write.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASP",
        "Asynchronous Schedule Park Mode Count - Read/Write.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ASPE",
        "Asynchronous Schedule Park Mode Enable - Read/Write.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUTW",
        "Setup TripWire - Read/Write.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ATDTW",
        "Add dTD TripWire - Read/Write.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FS1",
        "See also bits 3-2  Frame List Size - (Read/Write or Read Only).",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ITC",
        "Interrupt Threshold Control -Read/Write.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_USBSTS.
static const field_t hw_usbc_uh1_usbsts[] =
{
    {
        "UI",
        "USB Interrupt (USBINT) - R/WC.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UEI",
        "USB Error Interrupt (USBERRINT) - R/WC.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PCI",
        "Port Change Detect - R/WC.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRI",
        "Frame List Rollover - R/WC.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEI",
        "System Error- R/WC.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AAI",
        "Interrupt on Async Advance - R/WC.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URI",
        "USB Reset Received - R/WC.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRI",
        "SOF Received - R/WC.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLI",
        "DCSuspend - R/WC.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ULPII",
        "ULPI Interrupt - R/WC.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HCH",
        "HCHaIted - Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCL",
        "Reclamation - Read Only.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PS",
        "Periodic Schedule Status - Read Only.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AS",
        "Asynchronous Schedule Status - Read Only.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NAKI",
        "NAK Interrupt Bit--RO.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TI0",
        "General Purpose Timer Interrupt 0(GPTINT0)--R/WC.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TI1",
        "General Purpose Timer Interrupt 1(GPTINT1)--R/WC.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_USBINTR.
static const field_t hw_usbc_uh1_usbintr[] =
{
    {
        "UE",
        "USB Interrupt Enalbe  When this bit is one and the UI bit in n_USBSTS "
        "register is a one the controller will issue an interrupt.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UEE",
        "USB Error Interrupt Enable  When this bit is one and the UEI bit in n_"
        "USBSTS register is a one the controller will issue an interrupt.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PCE",
        "Port Change Detect Interrupt Enable  When this bit is one and the PCI "
        "bit in n_USBSTS register is a one the controller will issue an interru"
        "pt.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRE",
        "Frame List Rollover Interrupt Enable  When this bit is one and the FRI"
        " bit in n_USBSTS register is a one the controller will issue an interr"
        "upt.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SEE",
        "System Error Interrupt Enable  When this bit is one and the SEI bit in"
        " n_USBSTS register is a one the controller will issue an interrupt.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AAE",
        "Async Advance Interrupt Enable  When this bit is one and the AAI bit i"
        "n n_USBSTS register is a one the controller will issue an interrupt.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "URE",
        "USB Reset Interrupt Enable  When this bit is one and the URI bit in n_"
        "USBSTS register is a one the controller will issue an interrupt.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRE",
        "SOF Received Interrupt Enable  When this bit is one and the SRI bit in"
        " n_USBSTS register is a one the controller will issue an interrupt.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLE",
        "Sleep Interrupt Enable  When this bit is one and the SLI bit in n_n_US"
        "BSTS register is a one the controller will issue an interrupt.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ULPIE",
        "ULPI Interrupt Enable  When this bit is one and the UPLII bit in n_USB"
        "STS register is a one the controller will issue an interrupt.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NAKE",
        "NAK Interrupt Enable  When this bit is one and the NAKI bit in n_USBST"
        "S register is a one the controller will issue an interrupt.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UAIE",
        "USB Host Asynchronous Interrupt Enable  When this bit is one, and the "
        "UAI bit in the n_USBSTS register is one, host controller will issue an"
        " interrupt at the next interrupt threshold.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UPIE",
        "USB Host Periodic Interrupt Enable  When this bit is one, and the UPI "
        "bit in the n_USBSTS register is one, host controller will issue an int"
        "errupt at the next interrupt threshold.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIE0",
        "General Purpose Timer #0 Interrupt Enable  When this bit is one and th"
        "e TI0 bit in n_USBSTS register is a one the controller will issue an i"
        "nterrupt.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TIE1",
        "General Purpose Timer #1 Interrupt Enable  When this bit is one and th"
        "e TI1 bit in n_USBSTS register is a one the controller will issue an i"
        "nterrupt.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_FRINDEX.
static const field_t hw_usbc_uh1_frindex[] =
{
    {
        "FRINDEX",
        "Frame Index.",
        0, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_PERIODICLISTBASE.
static const field_t hw_usbc_uh1_periodiclistbase[] =
{
    {
        "BASEADR",
        "Base Address (Low).",
        12, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_ASYNCLISTADDR.
static const field_t hw_usbc_uh1_asynclistaddr[] =
{
    {
        "ASYBASE",
        "Link Pointer Low (LPL).",
        5, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_BURSTSIZE.
static const field_t hw_usbc_uh1_burstsize[] =
{
    {
        "RXPBURST",
        "Programmable RX Burst Size.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXPBURST",
        "Programmable TX Burst Size.",
        8, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_TXFILLTUNING.
static const field_t hw_usbc_uh1_txfilltuning[] =
{
    {
        "TXSCHOH",
        "Scheduler Overhead.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXSCHHEALTH",
        "Scheduler Health Counter.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXFIFOTHRES",
        "FIFO Burst Threshold.",
        16, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_CONFIGFLAG.
static const field_t hw_usbc_uh1_configflag[] =
{
    {
        "CF",
        "Configure Flag  Host software sets this bit as the last action in its "
        "process of configuring the Host Controller.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_PORTSC1.
static const field_t hw_usbc_uh1_portsc1[] =
{
    {
        "CCS",
        "Current Connect Status-Read Only.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CSC",
        "Connect Status Change-R/WC.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PE",
        "Port Enabled/Disabled-Read/Write.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PEC",
        "Port Enable/Disable Change-R/WC.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OCA",
        "Over-current Active-Read Only.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OCC",
        "Over-current Change-R/WC.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FPR",
        "Force Port Resume -Read/Write.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUSP",
        "Suspend - Read/Write or Read Only.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PR",
        "Port Reset - Read/Write or Read Only.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSP",
        "High-Speed Port - Read Only.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "LS",
        "Line Status-Read Only.",
        10, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PP",
        "Port Power (PP)-Read/Write or Read Only.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PO",
        "Port Owner-Read/Write.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PIC",
        "Port Indicator Control - Read/Write.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTC",
        "Port Test Control - Read/Write.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKCN",
        "Wake on Connect Enable (WKCNNT_E) - Read/Write.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKDC",
        "Wake on Disconnect Enable (WKDSCNNT_E) - Read/Write.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKOC",
        "Wake on Over-current Enable (WKOC_E) - Read/Write.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PHCD",
        "PHY Low Power Suspend - Clock Disable (PLPSCD) - Read/Write.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PFSC",
        "Port Force Full Speed Connect - Read/Write.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTS",
        "See description at bits 31-30",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PSPD",
        "Port Speed - Read Only.",
        26, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTW",
        "Parallel Transceiver Width  This bit has no effect if serial interface"
        " engine is used.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STS",
        "Serial Transceiver Select - Read Only  Serial Transceiver Select  1 Se"
        "rial Interface Engine is selected  0 Parallel Interface signals is sel"
        "ected  Serial Interface Engine can be used in combination with UTMI+/U"
        "LPI physical interface to provide FS/LS signaling instead of the paral"
        "lel interface signals.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PTS1",
        "Bit field {bit25, bit31, bit30}:  \"000b\" UTMI/UTMI+  \"001b\" Reserv"
        "ed  \"010b\" ULPI  \"011b\" Serial/USB 1.1 PHY/IC-USB (FS Only)  \"100"
        "b\" HSIC  All USB port interface modes are listed in this field descri"
        "ption, but not all are supported.",
        30, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBC_UH1_USBMODE.
static const field_t hw_usbc_uh1_usbmode[] =
{
    {
        "CM",
        "Controller Mode - R/WO.",
        0, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ES",
        "Endian Select - Read/Write.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SLOM",
        "Setup Lockout Mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDIS",
        "Stream Disable Mode.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a USBC module.
static const reg_t hw_usbc[] =
{
    {
        "UOG1_ID",
        "The ID register identifies the USB 2.0 High-Speed core and its revisio"
        "n.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uog1_id
    },
    {
        "UOG1_HWGENERAL",
        "General hardware parameters as defined in System Level Issues and Core"
        " Configuration.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uog1_hwgeneral
    },
    {
        "UOG1_HWHOST",
        "",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_hwhost
    },
    {
        "UOG1_HWDEVICE",
        "This register is only available in OTG core.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_hwdevice
    },
    {
        "UOG1_HWTXBUF",
        "",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_hwtxbuf
    },
    {
        "UOG1_HWRXBUF",
        "",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_hwrxbuf
    },
    {
        "UOG1_GPTIMER0LD",
        "This register controls load value of the count timer in register n_GPT"
        "IMER0CTRL.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_gptimer0ld
    },
    {
        "UOG1_GPTIMER0CTRL",
        "This register contains the control for this countdown timer and a data"
        " field can be queried to determine the running count value.",
        4, // Width in bytes
        0x00000084, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uog1_gptimer0ctrl
    },
    {
        "UOG1_GPTIMER1LD",
        "This register controls load value of the count timer in register n_GPT"
        "IMER1CTRL.",
        4, // Width in bytes
        0x00000088, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_gptimer1ld
    },
    {
        "UOG1_GPTIMER1CTRL",
        "This register contains the control for this countdown timer and a data"
        " field can be queried to determine the running count value.",
        4, // Width in bytes
        0x0000008c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uog1_gptimer1ctrl
    },
    {
        "UOG1_SBUSCFG",
        "",
        4, // Width in bytes
        0x00000090, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_sbuscfg
    },
    {
        "UOG1_CAPLENGTH",
        "The Capability Registers Length register contains the address offset t"
        "o the Operational registers relative to the CAPLENGTH register.",
        1, // Width in bytes
        0x00000100, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_caplength
    },
    {
        "UOG1_HCIVERSION",
        "This is a 2-byte register containing a BCD encoding of the EHCI revisi"
        "on number supported by this host controller.",
        2, // Width in bytes
        0x00000102, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_hciversion
    },
    {
        "UOG1_HCSPARAMS",
        "The following figure shows the port steering logic capabilities of Hos"
        "t Control Structural Parameters (n_HCSPARAMS).",
        4, // Width in bytes
        0x00000104, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_usbc_uog1_hcsparams
    },
    {
        "UOG1_HCCPARAMS",
        "This register identifies multiple mode control (time-base bit function"
        "ality), addressing capability.",
        4, // Width in bytes
        0x00000108, // Base address offset
        true, // Readable
        false, // Writable
        5, // Number of bitfields
        hw_usbc_uog1_hccparams
    },
    {
        "UOG1_DCIVERSION",
        "This register indicates the two-byte BCD encoding of the device contro"
        "ller interface version number.",
        2, // Width in bytes
        0x00000120, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_dciversion
    },
    {
        "UOG1_DCCPARAMS",
        "These fields describe the overall device capability of the controller.",
        4, // Width in bytes
        0x00000124, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uog1_dccparams
    },
    {
        "UOG1_USBCMD",
        "The Command Register indicates the command to be executed by the seria"
        "l bus host/device controller.",
        4, // Width in bytes
        0x00000140, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_usbcmd
    },
    {
        "UOG1_USBSTS",
        "This register indicates various states of the Host/Device Controller a"
        "nd any pending interrupts.",
        4, // Width in bytes
        0x00000144, // Base address offset
        true, // Readable
        true, // Writable
        17, // Number of bitfields
        hw_usbc_uog1_usbsts
    },
    {
        "UOG1_USBINTR",
        "The interrupts to software are enabled with this register.",
        4, // Width in bytes
        0x00000148, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_usbc_uog1_usbintr
    },
    {
        "UOG1_FRINDEX",
        "This register is used by the host controller to index the periodic fra"
        "me list.",
        4, // Width in bytes
        0x0000014c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_frindex
    },
    {
        "UOG1_PERIODICLISTBASE",
        "Host Controller only  This 32-bit register contains the beginning addr"
        "ess of the Periodic Frame List in the system memory.",
        4, // Width in bytes
        0x00000154, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_periodiclistbase
    },
    {
        "UOG1_ASYNCLISTADDR",
        "Host Controller only  This 32-bit register contains the address of the"
        " next asynchronous queue head to be executed by the host.",
        4, // Width in bytes
        0x00000158, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_asynclistaddr
    },
    {
        "UOG1_BURSTSIZE",
        "This register is used to control the burst size used during data movem"
        "ent on the AHB master interface.",
        4, // Width in bytes
        0x00000160, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_burstsize
    },
    {
        "UOG1_TXFILLTUNING",
        "The fields in this register control performance tuning associated with"
        " how the host controller posts data to the TX latency FIFO before movi"
        "ng the data onto the USB bus.",
        4, // Width in bytes
        0x00000164, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usbc_uog1_txfilltuning
    },
    {
        "UOG1_ENDPTNAK",
        "",
        4, // Width in bytes
        0x00000178, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_endptnak
    },
    {
        "UOG1_ENDPTNAKEN",
        "",
        4, // Width in bytes
        0x0000017c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_endptnaken
    },
    {
        "UOG1_CONFIGFLAG",
        "",
        4, // Width in bytes
        0x00000180, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_configflag
    },
    {
        "UOG1_PORTSC1",
        "Host Controller  A host controller could implement one to eight port s"
        "tatus and control registers.",
        4, // Width in bytes
        0x00000184, // Base address offset
        true, // Readable
        true, // Writable
        25, // Number of bitfields
        hw_usbc_uog1_portsc1
    },
    {
        "UOG1_OTGSC",
        "This register is availabe only in OTG controller core.",
        4, // Width in bytes
        0x000001a4, // Base address offset
        true, // Readable
        true, // Writable
        26, // Number of bitfields
        hw_usbc_uog1_otgsc
    },
    {
        "UOG1_USBMODE",
        "",
        4, // Width in bytes
        0x000001a8, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uog1_usbmode
    },
    {
        "UOG1_ENDPTSETUPSTAT",
        "",
        4, // Width in bytes
        0x000001ac, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog1_endptsetupstat
    },
    {
        "UOG1_ENDPTPRIME",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000001b0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_endptprime
    },
    {
        "UOG1_ENDPTFLUSH",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000001b4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_endptflush
    },
    {
        "UOG1_ENDPTSTAT",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000001b8, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_endptstat
    },
    {
        "UOG1_ENDPTCOMPLETE",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000001bc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog1_endptcomplete
    },
    {
        "UOG1_ENDPTCTRL0",
        "Every Device implements Endpoint 0 as a control endpoint.",
        4, // Width in bytes
        0x000001c0, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_usbc_uog1_endptctrl0
    },
    {
        "UOG1_ENDPTCTRL1",
        "This is endpoint control register for endpoint 1 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000001c4, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_endptctrl1
    },
    {
        "UOG1_ENDPTCTRL2",
        "This is endpoint control register for endpoint 2 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000001c8, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_endptctrl2
    },
    {
        "UOG1_ENDPTCTRL3",
        "This is endpoint control register for endpoint 3 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000001cc, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_endptctrl3
    },
    {
        "UOG1_ENDPTCTRL4",
        "This is endpoint control register for endpoint 4 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000001d0, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_endptctrl4
    },
    {
        "UOG1_ENDPTCTRL5",
        "This is endpoint control register for endpoint 5 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000001d4, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_endptctrl5
    },
    {
        "UOG1_ENDPTCTRL6",
        "This is endpoint control register for endpoint 6 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000001d8, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_endptctrl6
    },
    {
        "UOG1_ENDPTCTRL7",
        "This is endpoint control register for endpoint 7 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000001dc, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog1_endptctrl7
    },
    {
        "UOG2_ID",
        "The ID register identifies the USB 2.0 High-Speed core and its revisio"
        "n.",
        4, // Width in bytes
        0x00000200, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uog2_id
    },
    {
        "UOG2_HWGENERAL",
        "General hardware parameters as defined in System Level Issues and Core"
        " Configuration.",
        4, // Width in bytes
        0x00000204, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uog2_hwgeneral
    },
    {
        "UOG2_HWHOST",
        "",
        4, // Width in bytes
        0x00000208, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_hwhost
    },
    {
        "UOG2_HWDEVICE",
        "This register is only available in OTG core.",
        4, // Width in bytes
        0x0000020c, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_hwdevice
    },
    {
        "UOG2_HWTXBUF",
        "",
        4, // Width in bytes
        0x00000210, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_hwtxbuf
    },
    {
        "UOG2_HWRXBUF",
        "",
        4, // Width in bytes
        0x00000214, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_hwrxbuf
    },
    {
        "UOG2_GPTIMER0LD",
        "This register controls load value of the count timer in register n_GPT"
        "IMER0CTRL.",
        4, // Width in bytes
        0x00000280, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_gptimer0ld
    },
    {
        "UOG2_GPTIMER0CTRL",
        "This register contains the control for this countdown timer and a data"
        " field can be queried to determine the running count value.",
        4, // Width in bytes
        0x00000284, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uog2_gptimer0ctrl
    },
    {
        "UOG2_GPTIMER1LD",
        "This register controls load value of the count timer in register n_GPT"
        "IMER1CTRL.",
        4, // Width in bytes
        0x00000288, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_gptimer1ld
    },
    {
        "UOG2_GPTIMER1CTRL",
        "This register contains the control for this countdown timer and a data"
        " field can be queried to determine the running count value.",
        4, // Width in bytes
        0x0000028c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uog2_gptimer1ctrl
    },
    {
        "UOG2_SBUSCFG",
        "",
        4, // Width in bytes
        0x00000290, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_sbuscfg
    },
    {
        "UOG2_CAPLENGTH",
        "The Capability Registers Length register contains the address offset t"
        "o the Operational registers relative to the CAPLENGTH register.",
        1, // Width in bytes
        0x00000300, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_caplength
    },
    {
        "UOG2_HCIVERSION",
        "This is a 2-byte register containing a BCD encoding of the EHCI revisi"
        "on number supported by this host controller.",
        2, // Width in bytes
        0x00000302, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_hciversion
    },
    {
        "UOG2_HCSPARAMS",
        "The following figure shows the port steering logic capabilities of Hos"
        "t Control Structural Parameters (n_HCSPARAMS).",
        4, // Width in bytes
        0x00000304, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_usbc_uog2_hcsparams
    },
    {
        "UOG2_HCCPARAMS",
        "This register identifies multiple mode control (time-base bit function"
        "ality), addressing capability.",
        4, // Width in bytes
        0x00000308, // Base address offset
        true, // Readable
        false, // Writable
        5, // Number of bitfields
        hw_usbc_uog2_hccparams
    },
    {
        "UOG2_DCIVERSION",
        "This register indicates the two-byte BCD encoding of the device contro"
        "ller interface version number.",
        2, // Width in bytes
        0x00000320, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_dciversion
    },
    {
        "UOG2_DCCPARAMS",
        "These fields describe the overall device capability of the controller.",
        4, // Width in bytes
        0x00000324, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uog2_dccparams
    },
    {
        "UOG2_USBCMD",
        "The Command Register indicates the command to be executed by the seria"
        "l bus host/device controller.",
        4, // Width in bytes
        0x00000340, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_usbcmd
    },
    {
        "UOG2_USBSTS",
        "This register indicates various states of the Host/Device Controller a"
        "nd any pending interrupts.",
        4, // Width in bytes
        0x00000344, // Base address offset
        true, // Readable
        true, // Writable
        17, // Number of bitfields
        hw_usbc_uog2_usbsts
    },
    {
        "UOG2_USBINTR",
        "The interrupts to software are enabled with this register.",
        4, // Width in bytes
        0x00000348, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_usbc_uog2_usbintr
    },
    {
        "UOG2_FRINDEX",
        "This register is used by the host controller to index the periodic fra"
        "me list.",
        4, // Width in bytes
        0x0000034c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_frindex
    },
    {
        "UOG2_PERIODICLISTBASE",
        "Host Controller only  This 32-bit register contains the beginning addr"
        "ess of the Periodic Frame List in the system memory.",
        4, // Width in bytes
        0x00000354, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_periodiclistbase
    },
    {
        "UOG2_ASYNCLISTADDR",
        "Host Controller only  This 32-bit register contains the address of the"
        " next asynchronous queue head to be executed by the host.",
        4, // Width in bytes
        0x00000358, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_asynclistaddr
    },
    {
        "UOG2_BURSTSIZE",
        "This register is used to control the burst size used during data movem"
        "ent on the AHB master interface.",
        4, // Width in bytes
        0x00000360, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_burstsize
    },
    {
        "UOG2_TXFILLTUNING",
        "The fields in this register control performance tuning associated with"
        " how the host controller posts data to the TX latency FIFO before movi"
        "ng the data onto the USB bus.",
        4, // Width in bytes
        0x00000364, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usbc_uog2_txfilltuning
    },
    {
        "UOG2_ENDPTNAK",
        "",
        4, // Width in bytes
        0x00000378, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_endptnak
    },
    {
        "UOG2_ENDPTNAKEN",
        "",
        4, // Width in bytes
        0x0000037c, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_endptnaken
    },
    {
        "UOG2_CONFIGFLAG",
        "",
        4, // Width in bytes
        0x00000380, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_configflag
    },
    {
        "UOG2_PORTSC1",
        "Host Controller  A host controller could implement one to eight port s"
        "tatus and control registers.",
        4, // Width in bytes
        0x00000384, // Base address offset
        true, // Readable
        true, // Writable
        25, // Number of bitfields
        hw_usbc_uog2_portsc1
    },
    {
        "UOG2_OTGSC",
        "This register is availabe only in OTG controller core.",
        4, // Width in bytes
        0x000003a4, // Base address offset
        true, // Readable
        true, // Writable
        26, // Number of bitfields
        hw_usbc_uog2_otgsc
    },
    {
        "UOG2_USBMODE",
        "",
        4, // Width in bytes
        0x000003a8, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uog2_usbmode
    },
    {
        "UOG2_ENDPTSETUPSTAT",
        "",
        4, // Width in bytes
        0x000003ac, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uog2_endptsetupstat
    },
    {
        "UOG2_ENDPTPRIME",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000003b0, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_endptprime
    },
    {
        "UOG2_ENDPTFLUSH",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000003b4, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_endptflush
    },
    {
        "UOG2_ENDPTSTAT",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000003b8, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_endptstat
    },
    {
        "UOG2_ENDPTCOMPLETE",
        "This register is only used in device mode.",
        4, // Width in bytes
        0x000003bc, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uog2_endptcomplete
    },
    {
        "UOG2_ENDPTCTRL0",
        "Every Device implements Endpoint 0 as a control endpoint.",
        4, // Width in bytes
        0x000003c0, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_usbc_uog2_endptctrl0
    },
    {
        "UOG2_ENDPTCTRL1",
        "This is endpoint control register for endpoint 1 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000003c4, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_endptctrl1
    },
    {
        "UOG2_ENDPTCTRL2",
        "This is endpoint control register for endpoint 2 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000003c8, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_endptctrl2
    },
    {
        "UOG2_ENDPTCTRL3",
        "This is endpoint control register for endpoint 3 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000003cc, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_endptctrl3
    },
    {
        "UOG2_ENDPTCTRL4",
        "This is endpoint control register for endpoint 4 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000003d0, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_endptctrl4
    },
    {
        "UOG2_ENDPTCTRL5",
        "This is endpoint control register for endpoint 5 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000003d4, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_endptctrl5
    },
    {
        "UOG2_ENDPTCTRL6",
        "This is endpoint control register for endpoint 6 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000003d8, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_endptctrl6
    },
    {
        "UOG2_ENDPTCTRL7",
        "This is endpoint control register for endpoint 7 in device operation m"
        "ode.",
        4, // Width in bytes
        0x000003dc, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uog2_endptctrl7
    },
    {
        "UH1_ID",
        "The ID register identifies the USB 2.0 High-Speed core and its revisio"
        "n.",
        4, // Width in bytes
        0x00000400, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uh1_id
    },
    {
        "UH1_HWGENERAL",
        "General hardware parameters as defined in System Level Issues and Core"
        " Configuration.",
        4, // Width in bytes
        0x00000404, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbc_uh1_hwgeneral
    },
    {
        "UH1_HWHOST",
        "",
        4, // Width in bytes
        0x00000408, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uh1_hwhost
    },
    {
        "UH1_HWTXBUF",
        "",
        4, // Width in bytes
        0x00000410, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uh1_hwtxbuf
    },
    {
        "UH1_HWRXBUF",
        "",
        4, // Width in bytes
        0x00000414, // Base address offset
        true, // Readable
        false, // Writable
        2, // Number of bitfields
        hw_usbc_uh1_hwrxbuf
    },
    {
        "UH1_GPTIMER0LD",
        "This register controls load value of the count timer in register n_GPT"
        "IMER0CTRL.",
        4, // Width in bytes
        0x00000480, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_gptimer0ld
    },
    {
        "UH1_GPTIMER0CTRL",
        "This register contains the control for this countdown timer and a data"
        " field can be queried to determine the running count value.",
        4, // Width in bytes
        0x00000484, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uh1_gptimer0ctrl
    },
    {
        "UH1_GPTIMER1LD",
        "This register controls load value of the count timer in register n_GPT"
        "IMER1CTRL.",
        4, // Width in bytes
        0x00000488, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_gptimer1ld
    },
    {
        "UH1_GPTIMER1CTRL",
        "This register contains the control for this countdown timer and a data"
        " field can be queried to determine the running count value.",
        4, // Width in bytes
        0x0000048c, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uh1_gptimer1ctrl
    },
    {
        "UH1_SBUSCFG",
        "",
        4, // Width in bytes
        0x00000490, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_sbuscfg
    },
    {
        "UH1_CAPLENGTH",
        "The Capability Registers Length register contains the address offset t"
        "o the Operational registers relative to the CAPLENGTH register.",
        1, // Width in bytes
        0x00000500, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_caplength
    },
    {
        "UH1_HCIVERSION",
        "This is a 2-byte register containing a BCD encoding of the EHCI revisi"
        "on number supported by this host controller.",
        2, // Width in bytes
        0x00000502, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_hciversion
    },
    {
        "UH1_HCSPARAMS",
        "The following figure shows the port steering logic capabilities of Hos"
        "t Control Structural Parameters (n_HCSPARAMS).",
        4, // Width in bytes
        0x00000504, // Base address offset
        true, // Readable
        false, // Writable
        7, // Number of bitfields
        hw_usbc_uh1_hcsparams
    },
    {
        "UH1_HCCPARAMS",
        "This register identifies multiple mode control (time-base bit function"
        "ality), addressing capability.",
        4, // Width in bytes
        0x00000508, // Base address offset
        true, // Readable
        false, // Writable
        5, // Number of bitfields
        hw_usbc_uh1_hccparams
    },
    {
        "UH1_USBCMD",
        "The Command Register indicates the command to be executed by the seria"
        "l bus host/device controller.",
        4, // Width in bytes
        0x00000540, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usbc_uh1_usbcmd
    },
    {
        "UH1_USBSTS",
        "This register indicates various states of the Host/Device Controller a"
        "nd any pending interrupts.",
        4, // Width in bytes
        0x00000544, // Base address offset
        true, // Readable
        true, // Writable
        17, // Number of bitfields
        hw_usbc_uh1_usbsts
    },
    {
        "UH1_USBINTR",
        "The interrupts to software are enabled with this register.",
        4, // Width in bytes
        0x00000548, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_usbc_uh1_usbintr
    },
    {
        "UH1_FRINDEX",
        "This register is used by the host controller to index the periodic fra"
        "me list.",
        4, // Width in bytes
        0x0000054c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_frindex
    },
    {
        "UH1_PERIODICLISTBASE",
        "Host Controller only  This 32-bit register contains the beginning addr"
        "ess of the Periodic Frame List in the system memory.",
        4, // Width in bytes
        0x00000554, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_periodiclistbase
    },
    {
        "UH1_ASYNCLISTADDR",
        "Host Controller only  This 32-bit register contains the address of the"
        " next asynchronous queue head to be executed by the host.",
        4, // Width in bytes
        0x00000558, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_asynclistaddr
    },
    {
        "UH1_BURSTSIZE",
        "This register is used to control the burst size used during data movem"
        "ent on the AHB master interface.",
        4, // Width in bytes
        0x00000560, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usbc_uh1_burstsize
    },
    {
        "UH1_TXFILLTUNING",
        "The fields in this register control performance tuning associated with"
        " how the host controller posts data to the TX latency FIFO before movi"
        "ng the data onto the USB bus.",
        4, // Width in bytes
        0x00000564, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usbc_uh1_txfilltuning
    },
    {
        "UH1_CONFIGFLAG",
        "",
        4, // Width in bytes
        0x00000580, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbc_uh1_configflag
    },
    {
        "UH1_PORTSC1",
        "Host Controller  A host controller could implement one to eight port s"
        "tatus and control registers.",
        4, // Width in bytes
        0x00000584, // Base address offset
        true, // Readable
        true, // Writable
        25, // Number of bitfields
        hw_usbc_uh1_portsc1
    },
    {
        "UH1_USBMODE",
        "",
        4, // Width in bytes
        0x000005a8, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbc_uh1_usbmode
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark USBNC
#endif

// Bitfields in register USBNC_USB_OTG1_CTRL.
static const field_t hw_usbnc_usb_otg1_ctrl[] =
{
    {
        "OVER_CUR_DIS",
        "Disable OTG1 Overcurrent Detection",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OVER_CUR_POL",
        "OTG1 Polarity of Overcurrent  The polarity of OTG1 port overcurrent ev"
        "ent",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWR_POL",
        "OTG1 Power Polarity  This bit should be set according to PMIC Power Pi"
        "n polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WIE",
        "OTG1 Wake-up Interrupt Enable  This bit enables or disables the OTG1 w"
        "ake-up interrupt.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_SW_EN",
        "OTG1 Software Wake-up Enable",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_SW",
        "OTG1 Software Wake-up",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_ID_EN",
        "OTG1 Wake-up on ID change enable",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_VBUS_EN",
        "OTG1 wake-up on VBUS change enable",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WIR",
        "OTG1 Wake-up Interrupt Request  This bit indicates that a wake-up inte"
        "rrupt request is received on the OTG1 port.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBNC_USB_OTG2_CTRL.
static const field_t hw_usbnc_usb_otg2_ctrl[] =
{
    {
        "OVER_CUR_DIS",
        "Disable OTG2 Overcurrent Detection",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OVER_CUR_POL",
        "OTG2 Polarity of Overcurrent  The polarity of OTG2 port overcurrent ev"
        "ent",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "PWR_POL",
        "OTG2 Power Polarity  This bit should be set according to PMIC Power Pi"
        "n polarity.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WIE",
        "OTG2 Wake-up Interrupt Enable  This bit enables or disables the OTG2 w"
        "ake-up interrupt.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_SW_EN",
        "OTG2 Software Wake-up Enable",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_SW",
        "OTG2 Software Wake-up",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_ID_EN",
        "OTG2 Wake-up on ID change enable",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_VBUS_EN",
        "OTG2 wake-up on VBUS change enable",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WIR",
        "OTG2 Wake-up Interrupt Request  This bit indicates that a wake-up inte"
        "rrupt request is received on the OTG port.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBNC_USB_UH_CTRL.
static const field_t hw_usbnc_usb_uh_ctrl[] =
{
    {
        "WIE",
        "Host Wake-up Interrupt Enable  This bit enables or disables the Host w"
        "ake-up interrupt.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RESET",
        "Force Host UTMI PHY Reset  This bit is used to force a reset to the UT"
        "MI PHY.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SUSPENDM",
        "Force Host UTMI PHY Suspend  This bit is used to put PHY into suspend "
        "mode.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "_480M_CLK_ON",
        "Force OTG UTMI PHY 480M clock output on when Host is not in suspend mo"
        "de.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_SW_EN",
        "Host Software Wake-up Enable",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WKUP_SW",
        "Host Software Wake-up",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WIR",
        "Host Wake-up Interrupt Request  This bit indicates that a wake-up inte"
        "rrupt request is received on the Host port.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBNC_USB_UH_HSIC_CTRL.
static const field_t hw_usbnc_usb_uh_hsic_ctrl[] =
{
    {
        "HSIC_CLK_ON",
        "Force Host HSIC module 480M clock on, even when in Host is in suspend "
        "mode.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSIC_EN",
        "Host HSIC enable",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLK_VLD",
        "Indicating whether Host HSIC clock is valid.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBNC_USB_OTG1_PHY_CTRL_0.
static const field_t hw_usbnc_usb_otg1_phy_ctrl_0[] =
{
    {
        "UTMI_CLK_VLD",
        "Indicating whether OTG1 UTMI PHY clock is valid",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBNC_USB_OTG2_PHY_CTRL_0.
static const field_t hw_usbnc_usb_otg2_phy_ctrl_0[] =
{
    {
        "UTMI_CLK_VLD",
        "Indicating whether OTG2 UTMI PHY clock is valid",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a USBNC module.
static const reg_t hw_usbnc[] =
{
    {
        "USB_OTG1_CTRL",
        "The USB OTG1 control register controls the integration specific featur"
        "es of the USB OTG1 module.",
        4, // Width in bytes
        0x00000800, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_usbnc_usb_otg1_ctrl
    },
    {
        "USB_OTG2_CTRL",
        "The USB OTG2 control register controls the integration specific featur"
        "es of the USB OTG2 module.",
        4, // Width in bytes
        0x00000804, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_usbnc_usb_otg2_ctrl
    },
    {
        "USB_UH_CTRL",
        "The USB Host control register controls the integration specific featur"
        "es of the USB host module.",
        4, // Width in bytes
        0x00000808, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_usbnc_usb_uh_ctrl
    },
    {
        "USB_UH_HSIC_CTRL",
        "The USB Host HSIC control register controls Host high speed IC configu"
        "ration.",
        4, // Width in bytes
        0x00000810, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usbnc_usb_uh_hsic_ctrl
    },
    {
        "USB_OTG1_PHY_CTRL_0",
        "USB OTG1 UTMI PHY control register 0 is used to control the on-chip OT"
        "G1 UTMI PHY.",
        4, // Width in bytes
        0x00000818, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbnc_usb_otg1_phy_ctrl_0
    },
    {
        "USB_OTG2_PHY_CTRL_0",
        "USB OTG2 UTMI PHY Control Register 0 are used to control the on-chip O"
        "TG2 UTMI PHY.",
        4, // Width in bytes
        0x0000081c, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbnc_usb_otg2_phy_ctrl_0
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark USBPHY
#endif

// Bitfields in register USBPHY_PWD.
static const field_t hw_usbphy_pwd[] =
{
    {
        "TXPWDFS",
        "0 = Normal operation.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXPWDIBIAS",
        "0 = Normal operation.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXPWDV2I",
        "0 = Normal operation.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXPWDENV",
        "0 = Normal operation.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXPWD1PT1",
        "0 = Normal operation.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXPWDDIFF",
        "0 = Normal operation.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXPWDRX",
        "0 = Normal operation.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_TX.
static const field_t hw_usbphy_tx[] =
{
    {
        "D_CAL",
        "Resistor Trimming Code:  0000 = 0.16%  0111 = Nominal  1111 = +25%",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXCAL45DN",
        "Decode to select a 45-Ohm resistance to the USB_DN output pin.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TXCAL45DP",
        "Decode to select a 45-Ohm resistance to the USB_DP output pin.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_RX.
static const field_t hw_usbphy_rx[] =
{
    {
        "ENVADJ",
        "The ENVADJ field adjusts the trip point for the envelope detector.",
        0, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISCONADJ",
        "The DISCONADJ field adjusts the trip point for the disconnect detector"
        ":  000 = Trip-Level Voltage is 0.57500 V  001 = Trip-Level Voltage is "
        "0.56875 V  010 = Trip-Level Voltage is 0.58125 V  011 = Trip-Level Vol"
        "tage is 0.58750 V  1XX = Reserved",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RXDBYPASS",
        "0 = Normal operation.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_CTRL.
static const field_t hw_usbphy_ctrl[] =
{
    {
        "ENOTG_ID_CHG_IRQ",
        "Enable OTG_ID_CHG_IRQ.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENHOSTDISCONDETECT",
        "For host mode, enables high-speed disconnect detector.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENIRQHOSTDISCON",
        "Enables interrupt for detection of disconnection to Device when in hig"
        "h-speed host mode.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HOSTDISCONDETECT_IRQ",
        "Indicates that the device has disconnected in high-speed mode.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENDEVPLUGINDETECT",
        "For device mode, enables 200-KOhm pullups for detecting connectivity t"
        "o the host.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEVPLUGIN_POLARITY",
        "For device mode, if this bit is cleared to 0, then it trips the interr"
        "upt if the device is plugged in.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OTG_ID_CHG_IRQ",
        "OTG ID change interrupt.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENOTGIDDETECT",
        "Enables circuit to detect resistance of MiniAB ID pin.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RESUMEIRQSTICKY",
        "Set to 1 will make RESUME_IRQ bit a sticky bit until software clear it"
        ".",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENIRQRESUMEDETECT",
        "Enables interrupt for detection of a non-J state on the USB line.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RESUME_IRQ",
        "Indicates that the host is sending a wake-up after suspend.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENIRQDEVPLUGIN",
        "Enables interrupt for the detection of connectivity to the USB line.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEVPLUGIN_IRQ",
        "Indicates that the device is connected.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DATA_ON_LRADC",
        "Enables the LRADC to monitor USB_DP and USB_DM.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENUTMILEVEL2",
        "Enables UTMI+ Level2.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENUTMILEVEL3",
        "Enables UTMI+ Level3.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENIRQWAKEUP",
        "Enables interrupt for the wakeup events.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WAKEUP_IRQ",
        "Indicates that there is a wakeup event.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENAUTOCLR_CLKGATE",
        "Enables the feature to auto-clear the CLKGATE bit if there is wakeup e"
        "vent while USB is suspended.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENAUTOCLR_PHY_PWD",
        "Enables the feature to auto-clear the PWD register bits in USBPHYx_PWD"
        " if there is wakeup event while USB is suspended.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENDPDMCHG_WKUP",
        "Enables the feature to wakeup USB if DP/DM is toggled when USB is susp"
        "ended.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENIDCHG_WKUP",
        "Enables the feature to wakeup USB if ID is toggled when USB is suspend"
        "ed.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENVBUSCHG_WKUP",
        "Enables the feature to wakeup USB if VBUS is toggled when USB is suspe"
        "nded.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FSDLL_RST_EN",
        "Enables the feature to reset the FSDLL lock detection logic at the end"
        " of each TX packet.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OTG_ID_VALUE",
        "Almost same as OTGID_STATUS in USBPHYx_STATUS Register.",
        27, // LSB
        27, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HOST_FORCE_LS_SE0",
        "Forces the next FS packet that is transmitted to have a EOP with LS ti"
        "ming.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "UTMI_SUSPENDM",
        "Used by the PHY to indicate a powered-down state.",
        29, // LSB
        29, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CLKGATE",
        "Gate UTMI Clocks.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SFTRST",
        "Writing a 1 to this bit will soft-reset the USBPHYx_PWD, USBPHYx_TX, U"
        "SBPHYx_RX, and USBPHYx_CTRL registers.",
        31, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_STATUS.
static const field_t hw_usbphy_status[] =
{
    {
        "HOSTDISCONDETECT_STATUS",
        "Indicates that the device has disconnected while in high-speed host mo"
        "de.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DEVPLUGIN_STATUS",
        "Indicates that the device has been connected on the USB_DP and USB_DM "
        "lines.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OTGID_STATUS",
        "Indicates the results of ID pin on MiniAB plug.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RESUME_STATUS",
        "Indicates that the host is sending a wake-up after suspend and has tri"
        "ggered an interrupt.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_DEBUG.
static const field_t hw_usbphy_debug[] =
{
    {
        "OTGIDPIOLOCK",
        "Once OTG ID from USBPHYx_STATUS_OTGID_STATUS, use this to hold the val"
        "ue.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEBUG_INTERFACE_HOLD",
        "Use holding registers to assist in timing for external UTMI interface.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HSTPULLDOWN",
        "Set bit 3 to 1 to pull down 15-KOhm on USB_DP line.",
        2, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENHSTPULLDOWN",
        "Set bit 5 to 1 to override the control of the USB_DP 15-KOhm pulldown.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TX2RXCOUNT",
        "Delay in between the end of transmit to the beginning of receive.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENTX2RXCOUNT",
        "Set this bit to allow a countdown to transition in between TX and RX.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SQUELCHRESETCOUNT",
        "Delay in between the detection of squelch to the reset of high-speed R"
        "X.",
        16, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "ENSQUELCHRESET",
        "Set bit to allow squelch to reset high-speed receive.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SQUELCHRESETLENGTH",
        "Duration of RESET in terms of the number of 480-MHz cycles.",
        25, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HOST_RESUME_DEBUG",
        "Choose to trigger the host resume SE0 with HOST_FORCE_LS_SE0 = 0 or UT"
        "MI_SUSPEND = 1.",
        29, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE",
        "Gate Test Clocks.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_DEBUG0_STATUS.
static const field_t hw_usbphy_debug0_status[] =
{
    {
        "LOOP_BACK_FAIL_COUNT",
        "Running count of the failed pseudo-random generator loopback.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "UTMI_RXERROR_FAIL_COUNT",
        "Running count of the UTMI_RXERROR.",
        16, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SQUELCH_COUNT",
        "Running count of the squelch reset instead of normal end for HS RX.",
        26, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_DEBUG1.
static const field_t hw_usbphy_debug1[] =
{
    {
        "ENTAILADJVD",
        "Delay increment of the rise of squelch:  00 = Delay is nominal  01 = D"
        "elay is +20%  10 = Delay is -20%  11 = Delay is -40%",
        13, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USBPHY_VERSION.
static const field_t hw_usbphy_version[] =
{
    {
        "STEP",
        "Fixed read-only value reflecting the stepping of the RTL version.",
        0, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MINOR",
        "Fixed read-only value reflecting the MINOR field of the RTL version.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MAJOR",
        "Fixed read-only value reflecting the MAJOR field of the RTL version.",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a USBPHY module.
static const reg_t hw_usbphy[] =
{
    {
        "PWD",
        "The USB PHY Power-Down Register provides overall control of the PHY po"
        "wer state.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_usbphy_pwd
    },
    {
        "TX",
        "The USB PHY Transmitter Control Register handles the transmit controls"
        ".",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usbphy_tx
    },
    {
        "RX",
        "The USB PHY Receiver Control Register handles receive path controls.",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_usbphy_rx
    },
    {
        "CTRL",
        "The USB PHY General Control Register handles OTG and Host controls.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        29, // Number of bitfields
        hw_usbphy_ctrl
    },
    {
        "STATUS",
        "The USB PHY Status Register holds results of IRQ and other detects.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usbphy_status
    },
    {
        "DEBUG",
        "This register is used to debug the USB PHY.",
        4, // Width in bytes
        0x00000050, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_usbphy_debug
    },
    {
        "DEBUG0_STATUS",
        "The UTMI Debug Status Register 0 holds multiple views for counters and"
        " status of state machines.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbphy_debug0_status
    },
    {
        "DEBUG1",
        "Chooses the muxing of the debug register to be shown in USBPHYx_DEBUG0"
        "_STATUS.",
        4, // Width in bytes
        0x00000070, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usbphy_debug1
    },
    {
        "VERSION",
        "Fields for RTL Version.",
        4, // Width in bytes
        0x00000080, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usbphy_version
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark USDHC
#endif

// Bitfields in register USDHC_DS_ADDR.
static const field_t hw_usdhc_ds_addr[] =
{
    {
        "DS_ADDR",
        "DMA System Address: This register contains the 32-bit system memory ad"
        "dress for a DMA transfer.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_BLK_ATT.
static const field_t hw_usdhc_blk_att[] =
{
    {
        "BLKSIZE",
        "Transfer Block Size: This register specifies the block size for block "
        "data transfers.",
        0, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BLKCNT",
        "Blocks Count For Current Transfer: This register is enabled when the B"
        "lock Count Enable bit in the Transfer Mode register is set to 1 and is"
        " valid only for multiple block transfers.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_CMD_ARG.
static const field_t hw_usdhc_cmd_arg[] =
{
    {
        "CMDARG",
        "Command Argument: The SD/MMC Command Argument is specified as bits 39-"
        "8 of the Command Format in the SD or MMC Specification.This register i"
        "s write protected when the Command Inhibit (CMD) bit in the Present St"
        "ate register is set.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_CMD_XFR_TYP.
static const field_t hw_usdhc_cmd_xfr_typ[] =
{
    {
        "RSPTYP",
        "Response Type Select:",
        16, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CCCEN",
        "Command CRC Check Enable: If this bit is set to 1, the uSDHC shall che"
        "ck the CRC field in the response.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CICEN",
        "Command Index Check Enable: If this bit is set to 1, the uSDHC will ch"
        "eck the Index field in the response to see if it has the same value as"
        " the command index.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DPSEL",
        "Data Present Select: This bit is set to 1 to indicate that data is pre"
        "sent and shall be transferred using the DAT line.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMDTYP",
        "Command Type: There are three types of special commands: Suspend, Resu"
        "me and Abort.",
        22, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CMDINX",
        "Command Index: These bits shall be set to the command number that is s"
        "pecified in bits 45-40 of the Command-Format in the SD Memory Card Phy"
        "sical Layer Specification and SDIO Card Specification.",
        24, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_CMD_RSP0.
static const field_t hw_usdhc_cmd_rsp0[] =
{
    {
        "CMDRSP0",
        "Command Response 0: Refer to for the mapping of command responses from"
        " the SD Bus to this register for each response type.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_CMD_RSP1.
static const field_t hw_usdhc_cmd_rsp1[] =
{
    {
        "CMDRSP1",
        "Command Response 1: Refer to for the mapping of command responses from"
        " the SD Bus to this register for each response type.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_CMD_RSP2.
static const field_t hw_usdhc_cmd_rsp2[] =
{
    {
        "CMDRSP2",
        "Command Response 2: Refer to for the mapping of command responses from"
        " the SD Bus to this register for each response type.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_CMD_RSP3.
static const field_t hw_usdhc_cmd_rsp3[] =
{
    {
        "CMDRSP3",
        "Command Response 3: Refer to for the mapping of command responses from"
        " the SD Bus to this register for each response type.",
        0, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_DATA_BUFF_ACC_PORT.
static const field_t hw_usdhc_data_buff_acc_port[] =
{
    {
        "DATCONT",
        "Data Content: The Buffer Data Port register is for 32-bit data access "
        "by the ARM platform or the external DMA.",
        0, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_PRES_STATE.
static const field_t hw_usdhc_pres_state[] =
{
    {
        "CIHB",
        "Command Inhibit (CMD): If this status bit is 0, it indicates that the "
        "CMD line is not in use and the uSDHC can issue a SD/MMC Command using "
        "the CMD line.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CDIHB",
        "Command Inhibit (DAT): This status bit is generated if either the DAT "
        "Line Active or the Read Transfer Active is set to 1.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DLA",
        "Data Line Active This status bit indicates whether one of the DAT line"
        "s on the SD Bus is in use.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SDSTB",
        "SD Clock Stable This status bit indicates that the internal card clock"
        " is stable.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "IPGOFF",
        "ipg_clk Gated Off Internally: This status bit indicates that the ipg_c"
        "lk is internally gated off.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HCKOFF",
        "hclk Gated Off Internally: This status bit indicates that the hclk is "
        "internally gated off.",
        5, // LSB
        5, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PEROFF",
        "ipg_perclk Gated Off Internally: This status bit indicates that the ip"
        "g_perclk is internally gated off.",
        6, // LSB
        6, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SDOFF",
        "SD Clock Gated Off Internally: This status bit indicates that the SD C"
        "lock is internally gated off, because of buffer over/under-run or read"
        " pause without read wait assertion, or the driver set FRC_SDCLK_ON bit"
        " is 0 to stop the SD clock in idle status.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WTA",
        "Write Transfer Active: This status bit indicates a write transfer is a"
        "ctive.",
        8, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RTA",
        "Read Transfer Active: This status bit is used for detecting completion"
        " of a read transfer.",
        9, // LSB
        9, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BWEN",
        "Buffer Write Enable: This status bit is used for non-DMA write transfe"
        "rs.",
        10, // LSB
        10, // MSB
        true, // Readable
        false // Writable
    },
    {
        "BREN",
        "Buffer Read Enable: This status bit is used for non-DMA read transfers"
        ".",
        11, // LSB
        11, // MSB
        true, // Readable
        false // Writable
    },
    {
        "RTR",
        "Re-Tuning Request: (only for SD3.0 SDR104 mode) Host Controller may re"
        "quest Host Driver to execute re-tuning sequence by setting this bit wh"
        "en the data window is shifted by temperature drift and a tuned samplin"
        "g point does not have a good margin to receive correct data.",
        12, // LSB
        12, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TSCD",
        "Tape Select Change Done : This bit indicates the dealy setting is effe"
        "ctive after write CLK_TUNE_CTRL_STATUS register.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CINST",
        "Card Inserted: This bit indicates whether a card has been inserted.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CDPL",
        "Card Detect Pin Level: This bit reflects the inverse value of the CD# "
        "pin for the card socket.",
        18, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "WPSPL",
        "Write Protect Switch Pin Level: The Write Protect Switch is supported "
        "for memory and combo cards.This bit reflects the inverted value of the"
        " WP pin of the card socket.",
        19, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CLSL",
        "CMD Line Signal Level: This status is used to check the CMD line level"
        " to recover from errors, and for debugging.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DLSL",
        "DAT[7:0] Line Signal Level: This status is used to check the DAT line "
        "level to recover from errors, and for debugging.This is especially use"
        "ful in detecting the busy signal level from DAT[0].",
        24, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_PROT_CTRL.
static const field_t hw_usdhc_prot_ctrl[] =
{
    {
        "LCTL",
        "LED Control: This bit, fully controlled by the Host Driver, is used to"
        " caution the user not to remove the card while the card is being acces"
        "sed.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTW",
        "Data Transfer Width: This bit selects the data width of the SD bus for"
        " a data transfer.",
        1, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "D3CD",
        "DAT3 as Card Detection Pin: If this bit is set, DAT3 should be pulled "
        "down to act as a card detection pin.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EMODE",
        "Endian Mode: The uSDHC supports all three endian modes in data transfe"
        "r.",
        4, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CDTL",
        "Card Detect Test Level: This is bit is enabled while the Card Detectio"
        "n Signal Selection is set to 1 and it indicates card insertion.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CDSS",
        "Card Detect Signal Selection: This bit selects the source for the card"
        " detection.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMASEL",
        "DMA Select: This field is valid while DMA (SDMA or ADMA) is enabled an"
        "d selects the DMA operation.",
        8, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SABGREQ",
        "Stop At Block Gap Request: This bit is used to stop executing a transa"
        "ction at the next block gap for both DMA and non-DMA transfers.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CREQ",
        "Continue Request: This bit is used to restart a transaction which was "
        "stopped using the Stop At Block Gap Request.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWCTL",
        "Read Wait Control: The read wait function is optional for SDIO cards.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IABG",
        "Interrupt At Block Gap: This bit is valid only in 4-bit mode, of the S"
        "DIO card, and selects a sample point in the interrupt cycle.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_DONE_NO_8CLK",
        "Read done no 8 clock: According to the SD/MMC spec, for read data tran"
        "saction, 8 clocks are needed after the end bit of the last data block.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WECINT",
        "Wakeup Event Enable On Card Interrupt: This bit enables a wakeup event"
        ", via a Card Interrupt, in the Interrupt Status register.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WECINS",
        "Wakeup Event Enable On SD Card Insertion: This bit enables a wakeup ev"
        "ent, via a Card Insertion, in the Interrupt Status register.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WECRM",
        "Wakeup Event Enable On SD Card Removal: This bit enables a wakeup even"
        "t, via a Card Removal, in the Interrupt Status register.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BURST_LEN_EN",
        "BURST length enable for INCR, INCR4/INCR8/INCR16, INCR4-WRAP/INCR8-WRA"
        "P/INCR16-WRAP This is used to enable/disable the burst length for the "
        "external AHB2AXI bridge.",
        27, // LSB
        29, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NON_EXACT_BLK_RD",
        "Current block read is non-exact block read.",
        30, // LSB
        30, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_SYS_CTRL.
static const field_t hw_usdhc_sys_ctrl[] =
{
    {
        "DVS",
        "Divisor: This register is used to provide a more exact divisor to gene"
        "rate the desired SD clock frequency.",
        4, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SDCLKFS",
        "SDCLK Frequency Select: This register is used to select the frequency "
        "of the SDCLK pin.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTOCV",
        "Data Timeout Counter Value: This value determines the interval by whic"
        "h DAT line timeouts are detected.",
        16, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IPP_RST_N",
        "This register's value will be output to CARD from pad directly for har"
        "dware reset of the card if the card supports this feature.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RSTA",
        "Software Reset For ALL: This reset effects the entire Host Controller "
        "except for the card detection circuit.",
        24, // LSB
        24, // MSB
        false, // Readable
        true // Writable
    },
    {
        "RSTC",
        "Software Reset For CMD Line: Only part of the command circuit is reset"
        ".",
        25, // LSB
        25, // MSB
        false, // Readable
        true // Writable
    },
    {
        "RSTD",
        "Software Reset For DAT Line: Only part of the data circuit is reset.",
        26, // LSB
        26, // MSB
        false, // Readable
        true // Writable
    },
    {
        "INITA",
        "Initialization Active: When this bit is set, 80 SD-Clocks are sent to "
        "the card.",
        27, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RSTT",
        "Reset Tuning: When set this bit to 1, it will reset tuning circuit.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_INT_STATUS.
static const field_t hw_usdhc_int_status[] =
{
    {
        "CC",
        "Command Complete: This bit is set when you receive the end bit of the "
        "command response (except Auto CMD12).",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TC",
        "Transfer Complete: This bit is set when a read or write transfer is co"
        "mpleted.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BGE",
        "Block Gap Event: If the Stop At Block Gap Request bit in the Protocol "
        "Control register is set, this bit is set when a read or write transact"
        "ion is stopped at a block gap.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DINT",
        "DMA Interrupt: Occurs only when the internal DMA finishes the data tra"
        "nsfer successfully.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BWR",
        "Buffer Write Ready: This status bit is set if the Buffer Write Enable "
        "bit, in the Present State register, changes from 0 to 1.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BRR",
        "Buffer Read Ready: This status bit is set if the Buffer Read Enable bi"
        "t, in the Present State register, changes from 0 to 1.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CINS",
        "Card Insertion: This status bit is set if the Card Inserted bit in the"
        " Present State register changes from 0 to 1.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRM",
        "Card Removal: This status bit is set if the Card Inserted bit in the P"
        "resent State register changes from 1 to 0.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CINT",
        "Card Interrupt: This status bit is set when an interrupt signal is det"
        "ected from the external card.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTE",
        "Re-Tuning Event: (only for SD3.0 SDR104 mode) This status is set if Re"
        "-Tuning Request in the Present State register changes from 0 to 1.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TP",
        "Tuning Pass:(only for SD3.0 SDR104 mode) Current CMD19 transfer is don"
        "e successfully.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CTOE",
        "Command Timeout Error: Occurs only if no response is returned within 6"
        "4 SDCLK cycles from the end bit of the command.",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CCE",
        "Command CRC Error: Command CRC Error is generated in two cases.",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CEBE",
        "Command End Bit Error: Occurs when detecting that the end bit of a com"
        "mand response is 0.",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CIE",
        "Command Index Error: Occurs if a Command Index error occurs in the com"
        "mand response.",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTOE",
        "Data Timeout Error: Occurs when detecting one of following time-out co"
        "nditions.",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCE",
        "Data CRC Error: Occurs when detecting a CRC error when transferring re"
        "ad data, which uses the DAT line, or when detecting the Write CRC stat"
        "us having a value other than 010.",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEBE",
        "Data End Bit Error: Occurs either when detecting 0 at the end bit posi"
        "tion of read data, which uses the DAT line, or at the end bit position"
        " of the CRC.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AC12E",
        "Auto CMD12 Error: Occurs when detecting that one of the bits in the Au"
        "to CMD12 Error Status register has changed from 0 to 1.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TNE",
        "Tuning Error: (only for SD3.0 SDR104 mode) This bit is set when an unr"
        "ecoverable error is detected in a tuning circuit.",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAE",
        "DMA Error: Occurs when an Internal DMA transfer has failed.",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_INT_STATUS_EN.
static const field_t hw_usdhc_int_status_en[] =
{
    {
        "CCSEN",
        "Command Complete Status Enable: 1 Enabled 0 Masked",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCSEN",
        "Transfer Complete Status Enable:",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BGESEN",
        "Block Gap Event Status Enable:",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DINTSEN",
        "DMA Interrupt Status Enable:",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BWRSEN",
        "Buffer Write Ready Status Enable:",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BRRSEN",
        "Buffer Read Ready Status Enable:",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CINSSEN",
        "Card Insertion Status Enable:",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRMSEN",
        "Card Removal Status Enable:",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CINTSEN",
        "Card Interrupt Status Enable: If this bit is set to 0, the uSDHC will "
        "clear the interrupt request to the System.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTESEN",
        "Re-Tuning Event Status Enable",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TPSEN",
        "Tuning Pass Status Enable",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CTOESEN",
        "Command Timeout Error Status Enable:",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CCESEN",
        "Command CRC Error Status Enable:",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CEBESEN",
        "Command End Bit Error Status Enable:",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CIESEN",
        "Command Index Error Status Enable:",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTOESEN",
        "Data Timeout Error Status Enable:",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCESEN",
        "Data CRC Error Status Enable:",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEBESEN",
        "Data End Bit Error Status Enable:",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AC12ESEN",
        "Auto CMD12 Error Status Enable:",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TNESEN",
        "Tuning Error Status Enable:",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAESEN",
        "DMA Error Status Enable:",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_INT_SIGNAL_EN.
static const field_t hw_usdhc_int_signal_en[] =
{
    {
        "CCIEN",
        "Command Complete Interrupt Enable:",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TCIEN",
        "Transfer Complete Interrupt Enable:",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BGEIEN",
        "Block Gap Event Interrupt Enable:",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DINTIEN",
        "DMA Interrupt Enable:",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BWRIEN",
        "Buffer Write Ready Interrupt Enable:",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BRRIEN",
        "Buffer Read Ready Interrupt Enable:",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CINSIEN",
        "Card Insertion Interrupt Enable:",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CRMIEN",
        "Card Removal Interrupt Enable:",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CINTIEN",
        "Card Interrupt Interrupt Enable:",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RTEIEN",
        "Re-Tuning Event Interrupt Enable",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TPIEN",
        "Tuning Pass Interrupt Enable",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CTOEIEN",
        "Command Timeout Error Interrupt Enable",
        16, // LSB
        16, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CCEIEN",
        "Command CRC Error Interrupt Enable:",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CEBEIEN",
        "Command End Bit Error Interrupt Enable:",
        18, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CIEIEN",
        "Command Index Error Interrupt Enable:",
        19, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTOEIEN",
        "Data Timeout Error Interrupt Enable:",
        20, // LSB
        20, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DCEIEN",
        "Data CRC Error Interrupt Enable:",
        21, // LSB
        21, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DEBEIEN",
        "Data End Bit Error Interrupt Enable:",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AC12EIEN",
        "Auto CMD12 Error Interrupt Enable:",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TNEIEN",
        "Tuning Error Interrupt Enable",
        26, // LSB
        26, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DMAEIEN",
        "DMA Error Interrupt Enable:",
        28, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_AUTOCMD12_ERR_STATUS.
static const field_t hw_usdhc_autocmd12_err_status[] =
{
    {
        "AC12NE",
        "Auto CMD12 Not Executed: If memory multiple block data transfer is not"
        " started, due to a command error, this bit is not set because it is no"
        "t necessary to issue an Auto CMD12.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AC12TOE",
        "Auto CMD12 Timeout Error: Occurs if no response is returned within 64 "
        "SDCLK cycles from the end bit of the command.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AC12EBE",
        "Auto CMD12 End Bit Error: Occurs when detecting that the end bit of co"
        "mmand response is 0 which should be 1.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AC12CE",
        "Auto CMD12 CRC Error: Occurs when detecting a CRC error in the command"
        " response.",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    {
        "AC12IE",
        "Auto CMD12 Index Error: Occurs if the Command Index error occurs in re"
        "sponse to a command.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CNIBAC12E",
        "Command Not Issued By Auto CMD12 Error: Setting this bit to 1 means CM"
        "D_wo_DAT is not executed due to an Auto CMD12 Error (D04-D01) in this "
        "register.",
        7, // LSB
        7, // MSB
        true, // Readable
        false // Writable
    },
    {
        "EXECUTE_TUNING",
        "Execute Tuning: When std_tuning_en bit is set, this bit is used to sta"
        "rt tuning procedure.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SMP_CLK_SEL",
        "Sample Clock Select: When std_tuning_en bit is set, this bit is used t"
        "o select sampling clock to receive CMD and DAT.",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_HOST_CTRL_CAP.
static const field_t hw_usdhc_host_ctrl_cap[] =
{
    {
        "SDR50_SUPPORT",
        "SDR50 support: This bit indicates support of SDR50 mode.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SDR104_SUPPORT",
        "SDR104 support: This bit indicates support of SDR104 mode.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DDR50_SUPPORT",
        "DDR50 support: This bit indicates support of DDR50 mode.",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TIME_COUNT_RETUNING",
        "Time counter for retuning: This bit indicates an initial value of the "
        "Retuning Timer for Re-Tuning Mode1 and 3.Setting to 0 disables Retunin"
        "g Timer.",
        8, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "USE_TUNING_SDR50",
        "Use Tuning for SDR50: This bit is set to 1.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RETUNING_MODE",
        "Retuning Mode: This bit selects retuning method.",
        14, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "MBL",
        "Max Block Length: This value indicates the maximum block size that the"
        " Host Driver can read and write to the buffer in the uSDHC.",
        16, // LSB
        18, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ADMAS",
        "ADMA Support: This bit indicates whether the uSDHC supports the ADMA f"
        "eature.",
        20, // LSB
        20, // MSB
        true, // Readable
        false // Writable
    },
    {
        "HSS",
        "High Speed Support: This bit indicates whether the uSDHC supports High"
        " Speed mode and the Host System can supply a SD Clock frequency from 2"
        "5 MHz to 50 MHz.",
        21, // LSB
        21, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DMAS",
        "DMA Support: This bit indicates whether the uSDHC is capable of using "
        "the internal DMA to transfer data between system memory and the data b"
        "uffer directly.",
        22, // LSB
        22, // MSB
        true, // Readable
        false // Writable
    },
    {
        "SRS",
        "Suspend / Resume Support: This bit indicates whether the uSDHC support"
        "s Suspend / Resume functionality.",
        23, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VS33",
        "Voltage Support 3.3V: This bit shall depend on the Host System ability"
        ".",
        24, // LSB
        24, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VS30",
        "Voltage Support 3.0V: This bit shall depend on the Host System ability"
        ".",
        25, // LSB
        25, // MSB
        true, // Readable
        false // Writable
    },
    {
        "VS18",
        "Voltage Support 1.8V: This bit shall depend on the Host System ability"
        ".",
        26, // LSB
        26, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_WTMK_LVL.
static const field_t hw_usdhc_wtmk_lvl[] =
{
    {
        "RD_WML",
        "Read Watermark Level: The number of words used as the watermark level "
        "(FIFO threshold) in a DMA read operation.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RD_BRST_LEN",
        "Read Burst Length: Due to system restriction, the actual burst length "
        "may not exceed 16.",
        8, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_WML",
        "Write Watermark Level: The number of words used as the watermark level"
        " (FIFO threshold) in a DMA write operation.",
        16, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WR_BRST_LEN",
        "Write Burst Length: Due to system restriction, the actual burst length"
        " may not exceed 16.",
        24, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_MIX_CTRL.
static const field_t hw_usdhc_mix_ctrl[] =
{
    {
        "DMAEN",
        "DMA Enable: This bit enables DMA functionality.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BCEN",
        "Block Count Enable: This bit is used to enable the Block Count registe"
        "r, which is only relevant for multiple block transfers.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AC12EN",
        "Auto CMD12 Enable: Multiple block transfers for memory require a CMD12"
        " to stop the transaction.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DDR_EN",
        "Dual Data Rate mode selection",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DTDSEL",
        "Data Transfer Direction Select: This bit defines the direction of DAT "
        "line data transfers.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "MSBSEL",
        "Multi / Single Block Select: This bit enables multiple block DAT line "
        "data transfers.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NIBBLE_POS",
        "In DDR 4-bit mode nibble position indictation.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AC23EN",
        "Auto CMD23 Enable When this bit is set to 1, the Host Controller issue"
        "s a CMD23 automatically before issuing a command specified in the Comm"
        "and Register.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "EXE_TUNE",
        "Execute Tuning: (Only used for SD3.0, SDR104 mode) When std_tuning_en "
        "is 0, this bit is set to 1 to indicate the Host Driver is starting tun"
        "ing procedure.",
        22, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SMP_CLK_SEL",
        "When std_tuning_en is 0, this bit is used to select Tuned clock or Fix"
        "ed clock to sample data/cmd (Only used for SD3.0, SDR104 mode)",
        23, // LSB
        23, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUTO_TUNE_EN",
        "Auto tuning enable (Only used for SD3.0, SDR104 mode)",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FBCLK_SEL",
        "Feedback clock source selection (Only used for SD3.0, SDR104 mode)",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_FORCE_EVENT.
static const field_t hw_usdhc_force_event[] =
{
    {
        "FEVTAC12NE",
        "Force Event Auto Command 12 Not Executed: Forces the AC12NE bit in the"
        " Auto Command12 Error Status Register to be set",
        0, // LSB
        0, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTAC12TOE",
        "Force Event Auto Command 12 Time Out Error: Forces the AC12TOE bit in "
        "the Auto Command12 Error Status Register to be set",
        1, // LSB
        1, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTAC12CE",
        "Force Event Auto Command 12 CRC Error: Forces the AC12CE bit in the Au"
        "to Command12 Error Status Register to be set",
        2, // LSB
        2, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTAC12EBE",
        "Force Event Auto Command 12 End Bit Error: Forces the AC12EBE bit in t"
        "he Auto Command12 Error Status Register to be set",
        3, // LSB
        3, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTAC12IE",
        "Force Event Auto Command 12 Index Error: Forces the AC12IE bit in the "
        "Auto Command12 Error Status Register to be set",
        4, // LSB
        4, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTCNIBAC12E",
        "Force Event Command Not Executed By Auto Command 12 Error: Forces the "
        "CNIBAC12E bit in the Auto Command12 Error Status Register to be set",
        7, // LSB
        7, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTCCE",
        "Force Event Command Time Out Error: Forces the CTOE bit of Interrupt S"
        "tatus Register to be set",
        16, // LSB
        16, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTCCE1",
        "Force Event Command CRC Error: Forces the CCE bit of Interrupt Status "
        "Register to be set",
        17, // LSB
        17, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTCEBE",
        "Force Event Command End Bit Error: Forces the CEBE bit of Interrupt St"
        "atus Register to be set",
        18, // LSB
        18, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTCIE",
        "Force Event Command Index Error: Forces the CCE bit of Interrupt Statu"
        "s Register to be set",
        19, // LSB
        19, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTDTOE",
        "Force Event Data Time Out Error: Force the DTOE bit of Interrupt Statu"
        "s Register to be set",
        20, // LSB
        20, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTDCE",
        "Force Event Data CRC Error: Forces the DCE bit of Interrupt Status Reg"
        "ister to be set",
        21, // LSB
        21, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTDEBE",
        "Force Event Data End Bit Error: Forces the DEBE bit of Interrupt Statu"
        "s Register to be set",
        22, // LSB
        22, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTAC12E",
        "Force Event Auto Command 12 Error: Forces the AC12E bit of Interrupt S"
        "tatus Register to be set",
        24, // LSB
        24, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTTNE",
        "Force Tuning Error: Forces the TNE bit of Interrupt Status Register to"
        " be set",
        26, // LSB
        26, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTDMAE",
        "Force Event DMA Error: Forces the DMAE bit of Interrupt Status Registe"
        "r to be set",
        28, // LSB
        28, // MSB
        false, // Readable
        true // Writable
    },
    {
        "FEVTCINT",
        "Force Event Card Interrupt: Writing 1 to this bit generates a short lo"
        "w-level pulse on the internal DAT[1] line, as if a self clearing inter"
        "rupt was received from the external card.",
        31, // LSB
        31, // MSB
        false, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_ADMA_ERR_STATUS.
static const field_t hw_usdhc_adma_err_status[] =
{
    {
        "ADMAES",
        "ADMA Error State (when ADMA Error is occurred.): This field indicates "
        "the state of the ADMA when an error has occurred during an ADMA data t"
        "ransfer.",
        0, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ADMALME",
        "ADMA Length Mismatch Error: This error occurs in the following 2 cases"
        ": While the Block Count Enable is being set, the total data length spe"
        "cified by the Descriptor table is different from that specified by the"
        " Block Count and Block Length Total data length can not be divided by "
        "the block length",
        2, // LSB
        2, // MSB
        true, // Readable
        false // Writable
    },
    {
        "ADMADCE",
        "ADMA Descritor Error: This error occurs when invalid descriptor fetche"
        "d by ADMA:",
        3, // LSB
        3, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_ADMA_SYS_ADDR.
static const field_t hw_usdhc_adma_sys_addr[] =
{
    {
        "ADS_ADDR",
        "ADMA System Address: This register holds the word address of the execu"
        "ting command in the Descriptor table.",
        2, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_DLL_CTRL.
static const field_t hw_usdhc_dll_ctrl[] =
{
    {
        "DLL_CTRL_ENABLE",
        "Set this bit to 1 to enable the DLL and delay chain; otherwise; set to"
        " 0 to bypasses DLL.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_RESET",
        "Setting this bit to 1 force a reset on DLL.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_FORCE_UPD",
        "Setting this bit to 1, forces the slave delay line to update to the DL"
        "L calibrated value immediately.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_DLY_TARGET",
        "The delay target for the uSDHC loopback read clock can be programmed i"
        "n 1/16th increments of an ref_clock half-period.",
        3, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_GATE_UPDATE",
        "Set this bit to 1 to prevent the DLL from updating (since when clock_i"
        "n exists, glitches may appear during DLL updates).",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_OVERRIDE",
        "Set this bit to 1 to Enable manual override for slave delay chain usin"
        "g SLV_OVERRIDE_VAL; to set 0 to disable manual override.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_OVERRIDE_VAL",
        "When SLV_OVERRIDE=1 This field is used to select 1 of 128 physical tap"
        "s manually.",
        9, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_DLY_TARGET1",
        "Refer to DLL_CTRL_SLV_DLY_TARGET[3:0] below.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_SLV_UPDATE_INT",
        "Slave delay line update interval.",
        20, // LSB
        27, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLL_CTRL_REF_UPDATE_INT",
        "DLL control loop update interval.",
        28, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_DLL_STATUS.
static const field_t hw_usdhc_dll_status[] =
{
    {
        "DLL_STS_SLV_LOCK",
        "Slave delay-line lock status.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DLL_STS_REF_LOCK",
        "Reference DLL lock status.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DLL_STS_SLV_SEL",
        "Slave delay line select status.",
        2, // LSB
        8, // MSB
        true, // Readable
        false // Writable
    },
    {
        "DLL_STS_REF_SEL",
        "Reference delay line select taps.",
        9, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_CLK_TUNE_CTRL_STATUS.
static const field_t hw_usdhc_clk_tune_ctrl_status[] =
{
    {
        "DLY_CELL_SET_POST",
        "Set the number of delay cells on the feedback clock between CLK_OUT an"
        "d CLK_POST.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLY_CELL_SET_OUT",
        "Set the number of delay cells on the feedback clock between CLK_PRE an"
        "d CLK_OUT.",
        4, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DLY_CELL_SET_PRE",
        "Set the number of delay cells on the feedback clock between the feedba"
        "ck clock and CLK_PRE.",
        8, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "NXT_ERR",
        "NXT error which means the number of delay cells added on the feedback "
        "clock is too large.",
        15, // LSB
        15, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TAP_SEL_POST",
        "Reflect the number of delay cells added on the feedback clock between "
        "CLK_OUT and CLK_POST.",
        16, // LSB
        19, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TAP_SEL_OUT",
        "Reflect the number of delay cells added on the feedback clock between "
        "CLK_PRE and CLK_OUT.",
        20, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TAP_SEL_PRE",
        "Reflects the number of delay cells added on the feedback clock between"
        " the feedback clock and CLK_PRE.",
        24, // LSB
        30, // MSB
        true, // Readable
        false // Writable
    },
    {
        "PRE_ERR",
        "PRE error which means the number of delay cells added on the feedback "
        "clock is too small.",
        31, // LSB
        31, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_VEND_SPEC.
static const field_t hw_usdhc_vend_spec[] =
{
    {
        "EXT_DMA_EN",
        "External DMA Request Enable Enable the request to external DMA.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "VSELECT",
        "Voltage Selection Change the value of output signal VSELECT, to contro"
        "l the voltage on pads for external card.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CONFLICT_CHK_EN",
        "It's not implemented in uSDHC IP.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AC12_WR_CHKBUSY_EN",
        "Check busy enable after auto CMD12 for write data packet",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DAT3_CD_POL",
        "Only for debug.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CD_POL",
        "Only for debug.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WP_POL",
        "Only for debug.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKONJ_IN_ABORT",
        "Only for debug.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "FRC_SDCLK_ON",
        "Force CLK output active:",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IPG_CLK_SOFT_EN",
        "IPG_CLK software enable",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "HCLK_SOFT_EN",
        "Please note, hardware auto-enables the AHB clock when the internal DMA"
        " is enabled even if HCLK_SOFT_EN is 0.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "IPG_PERCLK_SOFT_EN",
        "ipg_perclk software enable",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CARD_CLK_SOFT_EN",
        "card clock software enable",
        14, // LSB
        14, // MSB
        true, // Readable
        false // Writable
    },
    {
        "CRC_CHK_DIS",
        "CRC check disable",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "INT_ST_VAL",
        "Internal State Value Internal state value, reflecting the correspondin"
        "g state value selected by Debug Select field.",
        16, // LSB
        23, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_MMC_BOOT.
static const field_t hw_usdhc_mmc_boot[] =
{
    {
        "DTOCV_ACK",
        "Boot ACK time out counter value.",
        0, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BOOT_ACK",
        "Boot ack mode select.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BOOT_MODE",
        "Boot mode select.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BOOT_EN",
        "Boot mode enable.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "AUTO_SABG_EN",
        "During boot, enable auto stop at block gap function.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISABLE_TIME_OUT",
        "Please note, when this bit is set, there is no timeout check no matter"
        " whether boot_en is set or not.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "BOOT_BLK_CNT",
        "The value defines the Stop At Block Gap value of automatic mode.",
        16, // LSB
        31, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_VEND_SPEC2.
static const field_t hw_usdhc_vend_spec2[] =
{
    {
        "SDR104_TIMING_DIS",
        "Timeout counter test.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDR104_OE_DIS",
        "CMD_OE/DAT_OE logic generation test.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SDR104_NSD_DIS",
        "Interrupt window after abort command is sent.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CARD_INT_D3_TEST",
        "Card interrupt detection test.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TUNING_8BIT_EN",
        "Enable the auto tuning circuit to check the DAT[7:0].",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TUNING_1BIT_EN",
        "Enable the auto tuning circuit to check the DAT0 only.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TUNING_CMD_EN",
        "Enable the auto tuning circuit to check the CMD line.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CARD_INT_AUTO_CLR_DIS",
        "Disable the feature to clear the Card interrupt status bit when Card I"
        "nterrupt status enable bit is cleared.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register USDHC_TUNING_CTRL.
static const field_t hw_usdhc_tuning_ctrl[] =
{
    {
        "TUNING_START_TAP",
        "The start dealy cell point when send first CMD19 in tuning procedure.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TUNING_COUNTER",
        "The MAX repeat CMD19 times in tuning procedure.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TUNING_STEP",
        "The increasing delay cell step in tuning procedure.",
        16, // LSB
        18, // MSB
        true, // Readable
        true // Writable
    },
    {
        "TUNING_WINDOW",
        "Select data window value for auto tuning",
        20, // LSB
        22, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STD_TUNING_EN",
        "Standard tuning circuit and procedure enable: This bit is used to enab"
        "le standard tuning circuit and procedure.",
        24, // LSB
        24, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a USDHC module.
static const reg_t hw_usdhc[] =
{
    {
        "DS_ADDR",
        "This register contains the physical system memory address used for DMA"
        " transfers.",
        4, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usdhc_ds_addr
    },
    {
        "BLK_ATT",
        "This register is used to configure the number of data blocks and the n"
        "umber of bytes in each block.",
        4, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        true, // Writable
        2, // Number of bitfields
        hw_usdhc_blk_att
    },
    {
        "CMD_ARG",
        "This register contains the SD/MMC Command Argument.",
        4, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usdhc_cmd_arg
    },
    {
        "CMD_XFR_TYP",
        "This register is used to control the operation of data transfers.",
        4, // Width in bytes
        0x0000000c, // Base address offset
        true, // Readable
        true, // Writable
        6, // Number of bitfields
        hw_usdhc_cmd_xfr_typ
    },
    {
        "CMD_RSP0",
        "This register is used to store part 0 of the response bits from the ca"
        "rd.",
        4, // Width in bytes
        0x00000010, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usdhc_cmd_rsp0
    },
    {
        "CMD_RSP1",
        "This register is used to store part 1 of the response bits from the ca"
        "rd.",
        4, // Width in bytes
        0x00000014, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usdhc_cmd_rsp1
    },
    {
        "CMD_RSP2",
        "This register is used to store part 2 of the response bits from the ca"
        "rd.",
        4, // Width in bytes
        0x00000018, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usdhc_cmd_rsp2
    },
    {
        "CMD_RSP3",
        "This register is used to store part 3 of the response bits from the ca"
        "rd.",
        4, // Width in bytes
        0x0000001c, // Base address offset
        true, // Readable
        false, // Writable
        1, // Number of bitfields
        hw_usdhc_cmd_rsp3
    },
    {
        "DATA_BUFF_ACC_PORT",
        "This is a 32-bit data port register used to access the internal buffer"
        ".",
        4, // Width in bytes
        0x00000020, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usdhc_data_buff_acc_port
    },
    {
        "PRES_STATE",
        "The Host Driver can get status of the uSDHC from this 32-bit read only"
        " register.",
        4, // Width in bytes
        0x00000024, // Base address offset
        true, // Readable
        false, // Writable
        19, // Number of bitfields
        hw_usdhc_pres_state
    },
    {
        "PROT_CTRL",
        "There are three cases to restart the transfer after stop at the block "
        "gap.",
        4, // Width in bytes
        0x00000028, // Base address offset
        true, // Readable
        true, // Writable
        17, // Number of bitfields
        hw_usdhc_prot_ctrl
    },
    {
        "SYS_CTRL",
        "",
        4, // Width in bytes
        0x0000002c, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_usdhc_sys_ctrl
    },
    {
        "INT_STATUS",
        "An interrupt is generated when the Normal Interrupt Signal Enable is e"
        "nabled and at least one of the status bits is set to 1.",
        4, // Width in bytes
        0x00000030, // Base address offset
        true, // Readable
        true, // Writable
        21, // Number of bitfields
        hw_usdhc_int_status
    },
    {
        "INT_STATUS_EN",
        "Setting the bits in this register to 1 enables the corresponding Inter"
        "rupt Status to be set by the specified event.",
        4, // Width in bytes
        0x00000034, // Base address offset
        true, // Readable
        true, // Writable
        21, // Number of bitfields
        hw_usdhc_int_status_en
    },
    {
        "INT_SIGNAL_EN",
        "This register is used to select which interrupt status is indicated to"
        " the Host System as the interrupt.",
        4, // Width in bytes
        0x00000038, // Base address offset
        true, // Readable
        true, // Writable
        21, // Number of bitfields
        hw_usdhc_int_signal_en
    },
    {
        "AUTOCMD12_ERR_STATUS",
        "When the Auto CMD12 Error Status bit in the Status register is set, th"
        "e Host Driver shall check this register to identify what kind of error"
        " the Auto CMD12 indicated.",
        4, // Width in bytes
        0x0000003c, // Base address offset
        true, // Readable
        false, // Writable
        8, // Number of bitfields
        hw_usdhc_autocmd12_err_status
    },
    {
        "HOST_CTRL_CAP",
        "This register provides the Host Driver with information specific to th"
        "e uSDHC implementation.",
        4, // Width in bytes
        0x00000040, // Base address offset
        true, // Readable
        false, // Writable
        14, // Number of bitfields
        hw_usdhc_host_ctrl_cap
    },
    {
        "WTMK_LVL",
        "Both write and read watermark levels (FIFO threshold) are configurable"
        ".",
        4, // Width in bytes
        0x00000044, // Base address offset
        true, // Readable
        true, // Writable
        4, // Number of bitfields
        hw_usdhc_wtmk_lvl
    },
    {
        "MIX_CTRL",
        "This register is used to DMA and data transfer.",
        4, // Width in bytes
        0x00000048, // Base address offset
        true, // Readable
        true, // Writable
        12, // Number of bitfields
        hw_usdhc_mix_ctrl
    },
    {
        "FORCE_EVENT",
        "The Force Event Register is not a physically implemented register.",
        4, // Width in bytes
        0x00000050, // Base address offset
        false, // Readable
        true, // Writable
        17, // Number of bitfields
        hw_usdhc_force_event
    },
    {
        "ADMA_ERR_STATUS",
        "When an ADMA Error Interrupt has occurred, the ADMA Error States field"
        " in this register holds the ADMA state and the ADMA System Address reg"
        "ister holds the address around the error descriptor.",
        4, // Width in bytes
        0x00000054, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_usdhc_adma_err_status
    },
    {
        "ADMA_SYS_ADDR",
        "This register contains the physical system memory address used for ADM"
        "A transfers.",
        4, // Width in bytes
        0x00000058, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_usdhc_adma_sys_addr
    },
    {
        "DLL_CTRL",
        "This register contains control bits for DLL.",
        4, // Width in bytes
        0x00000060, // Base address offset
        true, // Readable
        true, // Writable
        10, // Number of bitfields
        hw_usdhc_dll_ctrl
    },
    {
        "DLL_STATUS",
        "This register contains the DLL status information.",
        4, // Width in bytes
        0x00000064, // Base address offset
        true, // Readable
        false, // Writable
        4, // Number of bitfields
        hw_usdhc_dll_status
    },
    {
        "CLK_TUNE_CTRL_STATUS",
        "This register contains the Clock Tuning Control status information.",
        4, // Width in bytes
        0x00000068, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_usdhc_clk_tune_ctrl_status
    },
    {
        "VEND_SPEC",
        "This register contains the vendor specific control/status register.",
        4, // Width in bytes
        0x000000c0, // Base address offset
        true, // Readable
        true, // Writable
        15, // Number of bitfields
        hw_usdhc_vend_spec
    },
    {
        "MMC_BOOT",
        "This register contains the MMC Fast Boot control register.",
        4, // Width in bytes
        0x000000c4, // Base address offset
        true, // Readable
        true, // Writable
        7, // Number of bitfields
        hw_usdhc_mmc_boot
    },
    {
        "VEND_SPEC2",
        "This register contains the vendor specific control 2 register.",
        4, // Width in bytes
        0x000000c8, // Base address offset
        true, // Readable
        true, // Writable
        8, // Number of bitfields
        hw_usdhc_vend_spec2
    },
    {
        "TUNING_CTRL",
        "The register contains configuration of tunning circuit.",
        4, // Width in bytes
        0x000000cc, // Base address offset
        true, // Readable
        true, // Writable
        5, // Number of bitfields
        hw_usdhc_tuning_ctrl
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark WDOG
#endif

// Bitfields in register WDOG_WCR.
static const field_t hw_wdog_wcr[] =
{
    {
        "WDZST",
        "Watchdog Low Power.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDBG",
        "Watchdog DEBUG Enable.",
        1, // LSB
        1, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDE",
        "Watchdog Enable.",
        2, // LSB
        2, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDT",
        "WDOG Time-out assertion.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRS",
        "Software Reset Signal.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDA",
        "WDOG assertion.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "SRE",
        "adopt a new way to generate a more robust software reset.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WDW",
        "Watchdog Disable for Wait.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WT",
        "Watchdog Time-out Field.",
        8, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register WDOG_WSR.
static const field_t hw_wdog_wsr[] =
{
    {
        "WSR",
        "Watchdog Service Register.",
        0, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register WDOG_WRSR.
static const field_t hw_wdog_wrsr[] =
{
    {
        "SFTW",
        "Software Reset.",
        0, // LSB
        0, // MSB
        true, // Readable
        false // Writable
    },
    {
        "TOUT",
        "Timeout.",
        1, // LSB
        1, // MSB
        true, // Readable
        false // Writable
    },
    {
        "POR",
        "Power On Reset.",
        4, // LSB
        4, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register WDOG_WICR.
static const field_t hw_wdog_wicr[] =
{
    {
        "WICT",
        "Watchdog Interrupt Count Time-out (WICT) field determines, how long be"
        "fore the counter time-out must the interrupt occur.",
        0, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WTIS",
        "Watchdog TImer Interrupt Status bit will reflect the timer interrupt s"
        "tatus, whether interrupt has occurred or not.",
        14, // LSB
        14, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WIE",
        "Watchdog Timer Interrupt enable bit.",
        15, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register WDOG_WMCR.
static const field_t hw_wdog_wmcr[] =
{
    {
        "PDE",
        "Power Down Enable bit.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Registers in a WDOG module.
static const reg_t hw_wdog[] =
{
    {
        "WCR",
        "The Watchdog Control Register (WDOG_WCR) controls the WDOG operation.",
        2, // Width in bytes
        0x00000000, // Base address offset
        true, // Readable
        true, // Writable
        9, // Number of bitfields
        hw_wdog_wcr
    },
    {
        "WSR",
        "When enabled, the WDOG requires that a service sequence be written to "
        "the Watchdog Service Register (WSR) to prevent the timeout condition.",
        2, // Width in bytes
        0x00000002, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_wdog_wsr
    },
    {
        "WRSR",
        "The WRSR is a read-only register that records the source of the output"
        " reset assertion.",
        2, // Width in bytes
        0x00000004, // Base address offset
        true, // Readable
        false, // Writable
        3, // Number of bitfields
        hw_wdog_wrsr
    },
    {
        "WICR",
        "The WDOG_WICR controls the WDOG interrupt generation.",
        2, // Width in bytes
        0x00000006, // Base address offset
        true, // Readable
        true, // Writable
        3, // Number of bitfields
        hw_wdog_wicr
    },
    {
        "WMCR",
        "WDOG_WMCR Controls the Power Down counter operation.",
        2, // Width in bytes
        0x00000008, // Base address offset
        true, // Readable
        true, // Writable
        1, // Number of bitfields
        hw_wdog_wmcr
    },
    { 0 } // Terminator
};

//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark XTALOSC24M
#endif

// Bitfields in register XTALOSC24M_MISC0.
static const field_t hw_xtalosc24m_misc0[] =
{
    {
        "REFTOP_PWD",
        "Control bit to power-down the analog bandgap reference circuitry.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_SELFBIASOFF",
        "Control bit to disable the self-bias circuit in the analog bandgap.",
        3, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_VBGADJ",
        "Not related to oscillator.",
        4, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_VBGUP",
        "Status bit which signals that the analog bandgap voltage is up and sta"
        "ble.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "STOP_MODE_CONFIG",
        "Configure the analog behavior in stop mode.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OSC_I",
        "This bit field determines the bias current in the 24MHz oscillator.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OSC_XTALOK",
        "Status bit which signals that the output of the 24MHz crystal oscillat"
        "or is stable.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    {
        "OSC_XTALOK_EN",
        "Enable bit for the xtal_ok module(24 MHz)",
        17, // LSB
        17, // MSB
        true, // Readable
        true // Writable
    },
    {
        "WBCP_VPW_THRESH",
        "This signal alters the voltage that the pwell is charged pumped to.",
        18, // LSB
        19, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE_CTRL",
        "This bit allows disabling the clock gate (always un-gated) for the xta"
        "l 24MHz clock that clocks the digital logic in the analog block.",
        25, // LSB
        25, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CLKGATE_DELAY",
        "This field specifies the delay between powering up the XTAL 24MHz cloc"
        "k and release the clock to the digital logic inside the analog block.",
        26, // LSB
        28, // MSB
        true, // Readable
        true // Writable
    },
    { 0 } // Terminator
};

// Bitfields in register XTALOSC24M_LPC_MISC1.
static const field_t hw_xtalosc24m_lpc_misc1[] =
{
    {
        "RC_OSC_EN",
        "RC Osc.",
        0, // LSB
        0, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RC_OSC_PROG",
        "RC osc.",
        1, // LSB
        3, // MSB
        true, // Readable
        true // Writable
    },
    {
        "OSC_SEL",
        "Select the source for the 24MHz clock.",
        4, // LSB
        4, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPBG_SEL",
        "Bandgap select.",
        5, // LSB
        5, // MSB
        true, // Readable
        true // Writable
    },
    {
        "LPBG_TEST",
        "Low power bandgap test bit.",
        6, // LSB
        6, // MSB
        true, // Readable
        true // Writable
    },
    {
        "REFTOP_IBIAS_OFF",
        "Low power reftop ibias disable.",
        7, // LSB
        7, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L1_PWRGATE",
        "L1 power gate control.",
        8, // LSB
        8, // MSB
        true, // Readable
        true // Writable
    },
    {
        "L2_PWRGATE",
        "L2 power gate control.",
        9, // LSB
        9, // MSB
        true, // Readable
        true // Writable
    },
    {
        "CPU_PWRGATE",
        "CPU power gate control.",
        10, // LSB
        10, // MSB
        true, // Readable
        true // Writable
    },
    {
        "DISPLAY_PWRGATE",
        "Display logic power gate control.",
        11, // LSB
        11, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RWB_EN",
        "Reverse well bias enable control.",
        12, // LSB
        12, // MSB
        true, // Readable
        true // Writable
    },
    {
        "RCOSC_CG_OVERRIDE",
        "For debug purposes only.",
        13, // LSB
        13, // MSB
        true, // Readable
        true // Writable
    },
    {
        "XTALOSC_PWRUP_DELAY",
        "Specifies the time delay between when the 24MHz xtal is powered up unt"
        "il it is stable and ready to use.",
        14, // LSB
        15, // MSB
        true, // Readable
        true // Writable
    },
    {
        "XTALOSC_PWRUP_STAT",
        "Status of the 24MHz xtal oscillator.",
        16, // LSB
        16, // MSB
        true, // Readable
        false // Writable
    },
    { 0 } // Terminator
};

// Registers in a XTALOSC24M module.
static const reg_t hw_xtalosc24m[] =
{
    {
        "MISC0",
        "This register defines the control and status bits for miscellaneous an"
        "alog blocks.",
        4, // Width in bytes
        0x00000150, // Base address offset
        true, // Readable
        true, // Writable
        11, // Number of bitfields
        hw_xtalosc24m_misc0
    },
    {
        "LPC_MISC1",
        "XTAL OSC miscellaneous register 1.",
        4, // Width in bytes
        0x00000260, // Base address offset
        true, // Readable
        true, // Writable
        14, // Number of bitfields
        hw_xtalosc24m_lpc_misc1
    },
    { 0 } // Terminator
};


//------------------------------------------------------------------------------
#if __nonexistant__
#pragma mark mx6
#endif

const module_t mx6sl[] =
{
    // Module name, instance number, base address, register count, register definitions
    { "AIPSTZ1",         1, 0x02000000, 6,    hw_aipstz },
    { "AIPSTZ2",         2, 0x02100000, 6,    hw_aipstz },
    { "ARMGLOBALTIMER",  1, 0x00a00000, 7,    hw_armglobaltimer },
    { "AUDMUX",          1, 0x021d8000, 14,   hw_audmux },
    { "CCM",             1, 0x020c4000, 32,   hw_ccm },
    { "CCM_ANALOG",      1, 0x020c8000, 15,   hw_ccm_analog },
    { "CSI",             1, 0x020e4000, 13,   hw_csi },
    { "CSU",             1, 0x021c0000, 43,   hw_csu },
    { "DBGMON",          1, 0x02090000, 10,   hw_dbgmon },
    { "DCP",             1, 0x020fc000, 35,   hw_dcp },
    { "DVFSC",           1, 0x020dc000, 17,   hw_dvfsc },
    { "ECSPI1",          1, 0x02008000, 10,   hw_ecspi },
    { "ECSPI2",          2, 0x0200c000, 10,   hw_ecspi },
    { "ECSPI3",          3, 0x02010000, 10,   hw_ecspi },
    { "ECSPI4",          4, 0x02014000, 10,   hw_ecspi },
    { "EIM",             1, 0x021b8000, 41,   hw_eim },
    { "EPDC",            1, 0x020f4000, 101,  hw_epdc },
    { "EPIT1",           1, 0x020d0000, 5,    hw_epit },
    { "EPIT2",           2, 0x020d4000, 5,    hw_epit },
    { "FEC",             1, 0x02188000, 23,   hw_fec },
    { "GPC",             1, 0x020dc000, 10,   hw_gpc },
    { "GPIO1",           1, 0x0209c000, 8,    hw_gpio },
    { "GPIO2",           2, 0x020a0000, 8,    hw_gpio },
    { "GPIO3",           3, 0x020a4000, 8,    hw_gpio },
    { "GPIO4",           4, 0x020a8000, 8,    hw_gpio },
    { "GPIO5",           5, 0x020ac000, 8,    hw_gpio },
    { "GPIO6",           6, 0x020b0000, 8,    hw_gpio },
    { "GPIO7",           7, 0x020b4000, 8,    hw_gpio },
    { "GPT",             1, 0x02098000, 10,   hw_gpt },
    { "GPU2D",           1, 0x00134000, 36,   hw_gpu2d },
    { "I2C1",            1, 0x021a0000, 5,    hw_i2c },
    { "I2C2",            2, 0x021a4000, 5,    hw_i2c },
    { "I2C3",            3, 0x021a8000, 5,    hw_i2c },
    { "I2C4",            4, 0x021f8000, 5,    hw_i2c },
    { "IOMUXC",          1, 0x020e0000, 535,  hw_iomuxc },
    { "KPP",             1, 0x020b8000, 4,    hw_kpp },
    { "LCDIF",           1, 0x020f8000, 33,   hw_lcdif },
    { "MMDC",            1, 0x021b0000, 79,   hw_mmdc },
    { "OCOTP",           1, 0x021bc000, 75,   hw_ocotp },
    { "PGC",             1, 0x020dc000, 12,   hw_pgc },
    { "PMU",             1, 0x020c8000, 8,    hw_pmu },
    { "PWM1",            1, 0x02080000, 6,    hw_pwm },
    { "PWM2",            2, 0x02084000, 6,    hw_pwm },
    { "PWM3",            3, 0x02088000, 6,    hw_pwm },
    { "PWM4",            4, 0x0208c000, 6,    hw_pwm },
    { "PXP",             1, 0x020f0000, 52,   hw_pxp },
    { "QOS",             1, 0x02094000, 9,    hw_qos },
    { "RNG",             1, 0x021b4000, 6,    hw_rng },
    { "ROMC",            1, 0x021ac000, 28,   hw_romc },
    { "SDMAARM",         1, 0x020ec000, 106,  hw_sdmaarm },
    { "SDMABP",          1, 0x020ec000, 7,    hw_sdmabp },
    { "SDMACORE",        1, 0x020ec000, 21,   hw_sdmacore },
    { "SJC",             1, 0x00000000, 7,    hw_sjc },
    { "SPBA",            1, 0x0203c000, 32,   hw_spba },
    { "SPDC",            1, 0x020e8000, 18,   hw_spdc },
    { "SPDIF",           1, 0x02004000, 17,   hw_spdif },
    { "SRC",             1, 0x020d8000, 6,    hw_src },
    { "SSI1",            1, 0x02028000, 21,   hw_ssi },
    { "SSI2",            2, 0x0202c000, 21,   hw_ssi },
    { "SSI3",            3, 0x02030000, 21,   hw_ssi },
    { "TEMPMON",         1, 0x020c8000, 2,    hw_tempmon },
    { "UART1",           1, 0x02020000, 17,   hw_uart },
    { "UART2",           2, 0x021e8000, 17,   hw_uart },
    { "UART3",           3, 0x021ec000, 17,   hw_uart },
    { "UART4",           4, 0x021f0000, 17,   hw_uart },
    { "UART5",           5, 0x021f4000, 17,   hw_uart },
    { "USBC",            1, 0x02184000, 113,  hw_usbc },
    { "USBNC",           1, 0x02184000, 6,    hw_usbnc },
    { "USBPHY1",         1, 0x020c9000, 9,    hw_usbphy },
    { "USBPHY2",         2, 0x020ca000, 9,    hw_usbphy },
    { "USB_ANALOG",      1, 0x020c8000, 11,   hw_usb_analog },
    { "USDHC1",          1, 0x02190000, 29,   hw_usdhc },
    { "USDHC2",          2, 0x02194000, 29,   hw_usdhc },
    { "USDHC3",          3, 0x02198000, 29,   hw_usdhc },
    { "USDHC4",          4, 0x0219c000, 29,   hw_usdhc },
    { "WDOG1",           1, 0x020bc000, 5,    hw_wdog },
    { "WDOG2",           2, 0x020c0000, 5,    hw_wdog },
    { "XTALOSC24M",      1, 0x020c8000, 2,    hw_xtalosc24m },
    { 0 } // Terminator
};

////////////////////////////////////////////////////////////////////////////////
